
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%%
       subroutine CODENS (irec)
c      %%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use prepvar
       USE H2_DATA
       USE HD_DATA
       USE SP_DATA
       USE CHEM_DATA
       USE PROFIL

c---------------------------------------------------------------------
c       calcul des colonnes densites dans le nuage
c---------------------------------------------------------------------

       implicit double precision (a-h, o-z)

c  ENEP: LOG10(E)
c  CDUNIT: proportion gaz poussiere: NH = CDUNIT * E(B-V)
c  RRR: proportion indice de couleur extinction: Av = RRR * E(B-V)

       data  enep / 0.434294482 /

c   Black J.H., 1987, in "Spectroscopy of astrophysical plasmas" p 279 (eq 11.3)
c      cdunit = 4.77d21
c   Bohlin et al., 1978, ApJ, 224, 132
c      cdunit = 5.8d21
c   valeur usuelle: rrr = 3.1
c   valeur demandee par David:   rrr = 3.0
c  Nuage de Magellan: from Bouchet et al 1985, AA149, 330
c      cdunit = 4.5d22
c      rrr = 2.7d0

       codesp(1:nspec,0) = zero
       codeh2(1:l0h2,0) = zero
       cdoph2(:,0) = zero
       codeco(:,0) = zero
       cdeco1(:,0) = zero
       cdeco2(:,0) = zero
       cdeco3(:,0) = zero
       codecs(:,0) = zero
       codehcop(:,0) = zero
       codeh2b(:,0:njbc,0) = zero
       codeh2c(:,0:njbc,0) = zero
       codecat(:,0) = zero
       codenat(:,0) = zero
       codeoat(:,0) = zero
       codesat(:,0) = zero
       codesiat(:,0) = zero
       codecpl(:,0)  = zero
       codenpl(:,0)  = zero
       codeopl(:,0)  = zero
       codespl(:,0)  = zero
       codesip(:,0) = zero

       do 10 iopt=1,npo

c  calcul de la colonne densite dans la couche iopt
c         de toutes les especes:

       deltau = tau(iopt) - tau(iopt-1)

       do isp=1,nspec
         cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1     * ((abnua(isp,iopt) / densh(iopt))
     2     + (abnua(isp,iopt-1) / densh(iopt-1))) / deux
         codesp(isp,iopt) = codesp(isp,iopt-1) + cden
       end do

c----------------------------------------------------------------------
c       calcul de la colonne densite de h2 pour chaque niveau
c----------------------------------------------------------------------

       h2v(:,iopt) = 0.0_DP
       do lev=1,l0h2
         nv = nvlev(lev)
         h2v(nv,iopt) = h2v(nv,iopt) + xreh2(lev,iopt)
       end do

       do lev=1,l0h2
         nj = njlev(lev)
         nv = nvlev(lev)
         if (nj > njh2) cycle
         cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1     * ((abnua(i_h2,iopt) * xreh2(lev,iopt) / densh(iopt))
     2     + (abnua(i_h2,iopt-1) * xreh2(lev,iopt-1) / densh(iopt-1)))
     3     / deux
         codeh2(lev,iopt) = codeh2(lev,iopt-1) + cden
       end do

       para_1 = codeh2(1,iopt)
       para_2 = 0.0d0
       para_3 = 0.0d0
       orth_1 = codeh2(2,iopt)
       orth_2 = 0.0d0
       orth_3 = 0.0d0
       do lev=3,l0h2
         ij = njlev(lev)
         if (ij > njh2) cycle
         iv = nvlev(lev)
         if (mod(ij,2) == 0) then
           para_1 = para_1 + codeh2(lev,iopt)
           para_2 = para_2 + codeh2(lev,iopt)
           if (iv >= 1) then
             para_3 = para_3 + codeh2(lev,iopt)
           endif
         else
           orth_1 = orth_1 + codeh2(lev,iopt)
           orth_2 = orth_2 + codeh2(lev,iopt)
           if (iv >= 1) then
             orth_3 = orth_3 + codeh2(lev,iopt)
           endif
         endif
       end do
       cdoph2(1,iopt) = orth_1 / para_1
       cdoph2(2,iopt) = orth_2 / para_2
       cdoph2(3,iopt) = orth_3 / para_3

       do nvl=0,nvxh2
         cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1     * ((abnua(i_h2,iopt) * h2v(nvl,iopt) / densh(iopt))
     2     + (abnua(i_h2,iopt-1) * h2v(nvl,iopt-1) / densh(iopt-1)))
     3     / deux
         codh2v(nvl,iopt) = codh2v(nvl,iopt-1) + cden
       end do

       do nv=0,nvbc
        do nj=0,njbc
          cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1     * ((abnua(i_h2,iopt) * h2bnua(nv,nj,iopt) / densh(iopt))
     2     + (abnua(i_h2,iopt-1) * h2bnua(nv,nj,iopt-1)
     3     / densh(iopt-1))) / deux
          codeh2b(nv,nj,iopt) = codeh2b(nv,nj,iopt-1) + cden
          cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1     * ((abnua(i_h2,iopt) * h2cnua(nv,nj,iopt) / densh(iopt))
     2     + (abnua(i_h2,iopt-1) * h2cnua(nv,nj,iopt-1)
     3     / densh(iopt-1))) / deux
          codeh2c(nv,nj,iopt) = codeh2c(nv,nj,iopt-1) + cden
        enddo
       enddo

c----------------------------------------------------------------------
c       calcul de la colonne densite de co pour chaque j
c----------------------------------------------------------------------

       do j=0,njco
         cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1     * ((abnua(i_co0,iopt) * xreco0(j+1,iopt) / densh(iopt))
     2     + (abnua(i_co0,iopt-1) * xreco0(j+1,iopt-1)
     3     / densh(iopt-1))) / deux
         codeco(j,iopt) = codeco(j,iopt-1) + cden
       end do

c----------------------------------------------------------------------
c       calcul de la colonne densite de hd pour chaque niveau
c----------------------------------------------------------------------

       do 51 lev=1,l0hd
         nj = njhd(lev)
         if (nj .gt. mjhd) goto 51
         cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1     * ((abnua(i_hd,iopt) * xrehd(lev,iopt) / densh(iopt))
     2     + (abnua(i_hd,iopt-1) * xrehd(lev,iopt-1) / densh(iopt-1)))
     3     / deux
         codehd(lev,iopt) = codehd(lev,iopt-1) + cden
   51  continue

c----------------------------------------------------------------------
c       calcul de la colonne densite de c13o pour chaque j
c----------------------------------------------------------------------

       if (i_c13 .ne. 0) then
         do j=0,njco
          cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1     * ((abnua(i_co1,iopt) * xreco1(j+1,iopt) / densh(iopt))
     2     + (abnua(i_co1,iopt-1) * xreco1(j+1,iopt-1)
     3     / densh(iopt-1))) / deux
          cdeco1(j,iopt) = cdeco1(j,iopt-1) + cden
         end do
       endif

c----------------------------------------------------------------------
c       calcul de la colonne densite de co18 pour chaque j
c----------------------------------------------------------------------

       if (i_o18 .ne. 0) then
         do j=0,njco
          cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1     * ((abnua(i_co2,iopt) * xreco2(j+1,iopt) / densh(iopt))
     2     + (abnua(i_co2,iopt-1) * xreco2(j+1,iopt-1)
     3     / densh(iopt-1))) / deux
          cdeco2(j,iopt) = cdeco2(j,iopt-1) + cden
         end do
       endif

c----------------------------------------------------------------------
c       calcul de la colonne densite de c13o18 pour chaque j
c----------------------------------------------------------------------

       if (i_c13 .ne. 0 .and. i_o18 .ne. 0) then
         do j=0,njco
          cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1     * ((abnua(i_co3,iopt) * xreco3(j+1,iopt) / densh(iopt))
     2     + (abnua(i_co3,iopt-1) * xreco3(j+1,iopt-1)
     3     / densh(iopt-1))) / deux
          cdeco3(j,iopt) = cdeco3(j,iopt-1) + cden
         end do
       endif

c----------------------------------------------------------------------
c       calcul de la colonne densite de cs pour chaque j
c----------------------------------------------------------------------

       if (i_cs .ne. 0) then
         do j=0,njcs
          cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1     * ((abnua(i_cs,iopt) * xrecs0(j+1,iopt) / densh(iopt))
     2     + (abnua(i_cs,iopt-1) * xrecs0(j+1,iopt-1) / densh(iopt-1)))
     3     / deux
          codecs(j,iopt) = codecs(j,iopt-1) + cden
         end do
       endif

c--------------------------------------------------------------------------
c       calcul de la colonne densite de hcop pour chaque j
c--------------------------------------------------------------------------
       if (i_hcop .ne. 0) then
         do j=0,njhcop
          cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1     * ((abnua(i_hcop,iopt) * xrehcop(j+1,iopt) / densh(iopt))
     2     + (abnua(i_hcop,iopt-1) * xrehcop(j+1,iopt-1)
     3     / densh(iopt-1))) / deux
          codehcop(j,iopt) = codehcop(j,iopt-1) + cden
         enddo
       endif

c---------------------------------------------------------------------------
c       calcul de la colonne densite de C
c---------------------------------------------------------------------------
        do i=1,nlvcat
          cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1       * ((abnua(i_c,iopt) * xrecat(i,iopt) / densh(iopt))
     2       + (abnua(i_c,iopt-1) * xrecat(i,iopt-1)
     3       / densh(iopt-1))) / deux
          codecat(i,iopt) = codecat(i,iopt-1) + cden
        end do

c---------------------------------------------------------------------------
c       calcul de la colonne densite de N
c---------------------------------------------------------------------------
       if (i_n .ne. 0) then
         do i=1,nlvnat
           cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1        * ((abnua(i_n,iopt) * xrenat(i,iopt) / densh(iopt))
     2        + (abnua(i_n,iopt-1) * xrenat(i,iopt-1)
     3        / densh(iopt-1))) / deux
           codenat(i,iopt) = codenat(i,iopt-1) + cden
         end do
       endif

c---------------------------------------------------------------------------
c       calcul de la colonne densite de O
c---------------------------------------------------------------------------
        do i=1,nlvoat
          cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1       * ((abnua(i_o,iopt) * xreoat(i,iopt) / densh(iopt))
     2       + (abnua(i_o,iopt-1) * xreoat(i,iopt-1)
     3       / densh(iopt-1))) / deux
          codeoat(i,iopt) = codeoat(i,iopt-1) + cden
        end do

c---------------------------------------------------------------------------
c       calcul de la colonne densite de S
c---------------------------------------------------------------------------
       if (i_s .ne. 0) then
         do i=1,nlvsat
           cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1        * ((abnua(i_s,iopt) * xresat(i,iopt) / densh(iopt))
     2        + (abnua(i_s,iopt-1) * xresat(i,iopt-1)
     3        / densh(iopt-1))) / deux
           codesat(i,iopt) = codesat(i,iopt-1) + cden
         end do
       endif

c---------------------------------------------------------------------------
c       calcul de la colonne densite de Si
c---------------------------------------------------------------------------
       if (i_si .ne. 0) then
         do i=1,nlvsiat
           cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1        * ((abnua(i_si,iopt) * xresiat(i,iopt) / densh(iopt))
     2        + (abnua(i_si,iopt-1) * xresiat(i,iopt-1)
     3        / densh(iopt-1))) / deux
           codesiat(i,iopt) = codesiat(i,iopt-1) + cden
         end do
       endif

c---------------------------------------------------------------------------
c       calcul de la colonne densite de C+ 
c---------------------------------------------------------------------------
        do i=1,nlvcpl
          cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1       * ((abnua(i_cp,iopt) * xrecpl(i,iopt) / densh(iopt))
     2       + (abnua(i_cp,iopt-1) * xrecpl(i,iopt-1)
     3       / densh(iopt-1))) / deux
          codecpl(i,iopt) = codecpl(i,iopt-1) + cden
        end do

c---------------------------------------------------------------------------
c       calcul de la colonne densite de N+ 
c---------------------------------------------------------------------------
       if (i_np .ne. 0) then
         do i=1,nlvnpl
           cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1        * ((abnua(i_np,iopt) * xrenpl(i,iopt) / densh(iopt))
     2        + (abnua(i_np,iopt-1) * xrenpl(i,iopt-1)
     3        / densh(iopt-1))) / deux
           codenpl(i,iopt) = codenpl(i,iopt-1) + cden
         end do
       endif

c---------------------------------------------------------------------------
c       calcul de la colonne densite de O+ 
c---------------------------------------------------------------------------
        do i=1,nlvopl
          cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1       * ((abnua(i_op,iopt) * xreopl(i,iopt) / densh(iopt))
     2       + (abnua(i_op,iopt-1) * xreopl(i,iopt-1)
     3       / densh(iopt-1))) / deux
          codeopl(i,iopt) = codeopl(i,iopt-1) + cden
        end do

c---------------------------------------------------------------------------
c       calcul de la colonne densite de S+ 
c---------------------------------------------------------------------------
       if (i_sp .ne. 0) then
         do i=1,nlvspl
           cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1        * ((abnua(i_sp,iopt) * xrespl(i,iopt) / densh(iopt))
     2        + (abnua(i_sp,iopt-1) * xrespl(i,iopt-1)
     3        / densh(iopt-1))) / deux
           codespl(i,iopt) = codespl(i,iopt-1) + cden
         end do
       endif 

c---------------------------------------------------------------------------
c       calcul de la colonne densite de Si+ 
c---------------------------------------------------------------------------
       if (i_sip .ne. 0) then
         do i=1,nlvsip
           cden = cdunit * (2.5d0 * enep / rrr) * deltau
     1        * ((abnua(i_sip,iopt) * xresip(i,iopt) / densh(iopt))
     2        + (abnua(i_sip,iopt-1) * xresip(i,iopt-1)
     3        / densh(iopt-1))) / deux
           codesip(i,iopt) = codesip(i,iopt-1) + cden
         end do
       endif

c---------------------------------------------------------------------------

   10  continue

       return
       end
