
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%
       subroutine INIT1
c      %%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       use inoutput
       USE RF_TOOLS
       USE SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE H2_DATA
       USE HD_DATA
       USE PROFIL
       USE TRANSFER

c--------+---------+---------+---------+---------+---------+---------+-*-------+

       implicit double precision (a-h,o-z)

       real (kind=dp)     :: coefosc

c--------+---------+---------+---------+---------+---------+---------+-*-------+

c  Compute some constants

      tautav = 2.5_DP * log10(exp(1.0_DP))

      print *,'--------------------------------------------------------'
      print *,'                   INITIALISATION                       '
      print *,'--------------------------------------------------------'

      abnua = zero

c  Read input datafile: pdr2.in (see short comments in input file)

       fichier = TRIM(data_dir)//'pdr2.in'
       print *, "Ouverture de :", fichier

       open (iread1, file = fichier, status = 'old')
         read (iread1,1012) modele      ! name of the file to be created
         modele = TRIM(ADJUSTL(modele))
         read (iread1,1012) chimie      ! name of the chemistry file
         chimie = TRIM(ADJUSTL(chimie))

         read (iread1,*) Avmax
         read (iread1,*) nh_init
         read (iread1,*) radm
         read (iread1,*) radp
         read (iread1,1012) srcpp
         srcpp = TRIM(ADJUSTL(srcpp))
         read (iread1,*) d_sour
         read (iread1,*) fmrc
         read (iread1,*) ifeqth
         read (iread1,*) tg_init
         read (iread1,*) ifisob
         read (iread1,*) presse

         read (iread1,*) vturb
         read (iread1,*) itrh2
         read (iread1,*) mjh2
         read (iread1,*) l0h2
         read (iread1,*) ichh2
         read (iread1,*) jlyman
         read (iread1,*) jfgkh2
         read (iread1,*) itrhd
         read (iread1,*) mjhd
         read (iread1,*) l0hd
         read (iread1,*) itrco0
         read (iread1,*) itrco1
         read (iread1,*) itrco2
         read (iread1,*) itrco3
         read (iread1,*) ifafm

         read (iread1,1012) los_ext
         read (iread1,*) rrr
         read (iread1,*) cdunit

         read (iread1,*) alb
         read (iread1,*) gg
         read (iread1,*) gratio
         read (iread1,*) rhogr
         read (iread1,*) alpgr
         read (iread1,*) rgrmin
         read (iread1,*) rgrmax
         read (iread1,*) dsite
         read (iread1,*) iforh2
         read (iread1,*) istic
       close (iread1)

       call INIT_PROFIL

c  Initialize output files

       fichier = TRIM(out_dir)//TRIM(ADJUSTL(modele))//'.def'
       open (iwrit2, file = fichier, status = 'unknown')

       file_bin = TRIM(out_dir)//TRIM(ADJUSTL(modele))//'.bin'
       print *, file_bin

       call grparam

c--------+---------+---------+---------+---------+---------+---------+-*-------+

       write (iwrit2,2000)
       write (iwrit2,2002) modele
       write (iwrit2,2003) chimie
       write (iwrit2,2010) radm, radp, srcpp, d_sour, fmrc
       write (iwrit2,2009) ifeqth, ifisob, presse
       write (iwrit2,2005) itrh2, itrhd, itrco0, itrco1, itrco2, itrco3
       write (iwrit2,2011) l0h2, mjhd, l0hd, iforh2, istic
       write (iwrit2,2013) rrr, cdunit, vturb
       write (iwrit2,2014) gratio, rhogr, alpgr, rgrmin, rgrmax, dsite
       write (iwrit2,2015) lleg, alb, gg, taumax, Avmax
       write (iwrit2,2018) los_ext, cc1, cc2,cc3, cc4, xgam, y0
       write (iwrit2,2020) tgaz(0), densh(0)

c  read chemical species, initial abundances
c  and chemical equations

       call LECTUR

       call TRANSF

       tau(2:npo) = tau_o(2:npo)
       abnua(i_h2,0:npo) = abh2_o
       do i=1,l0h2
         xreh2(i,1:npo) = xreh2(i,0)
       end do
       if (i_hd /= 0) then
         abnua(i_hd,0:npo) = 1.0e-4_dp * abnua(i_h2,0:npo)
       endif
       abnua(i_co0,0:npo) = 1.0e-4_dp * abnua(i_h2,0:npo)
       if (i_co1 /= 0) then
         abnua(i_co1,0:npo) = abnua(i_co0,0:npo) / 70.0_dp
       endif
       if (i_co2 /= 0) then
         abnua(i_co2,0:npo) = abnua(i_co0,0:npo) / 500.0_dp
       endif
       if (i_co3 /= 0) then
         abnua(i_co3,0:npo) = abnua(i_co0,0:npo) / (70.0_dp * 500.0_dp)
       endif

       call CDT

       cdth2_o = cdth2(:,0:npo)
       cdthd_o = cdthd(:,0:npo)
       cdtc0_o = cdtc0(:,0:npo)
       cdtc1_o = cdtc1(:,0:npo)
       cdtc2_o = cdtc2(:,0:npo)
       cdtc3_o = cdtc3(:,0:npo)

c--------+---------+---------+---------+---------+---------+---------+-*-------+

c  Collision rates

       if (ichh2 .eq. 0) then
         nqc = 4
       else
         nqc = 3
       endif
       allocate (qh1h2(nqc,l0h2,l0h2))
       allocate (qheh2(3,l0h2,l0h2))
       allocate (qph2h2(3,l0h2,l0h2))
       allocate (qoh2h2(3,l0h2,l0h2))
       qh1h2  = 0.0_dp
       qheh2  = 0.0_dp
       qph2h2 = 0.0_dp
       qoh2h2 = 0.0_dp
       qh1h2(1,:,:)  = -50.0_dp
       qheh2(1,:,:)  = -50.0_dp
       qph2h2(1,:,:) = -50.0_dp
       qoh2h2(1,:,:) = -50.0_dp

       if (i_hd .ne. 0) then
       do i=0,njthd
         do j=0,njthd
           qh1hd(1,j,i) = -50.0d0
           qh1hd(2,j,i) = 0.0d0
           qh1hd(3,j,i) = 0.0d0
           qh1hd(4,j,i) = 0.0d0
           qhehd(1,j,i) = -50.0d0
           qhehd(2,j,i) = 0.0d0
           qhehd(3,j,i) = 0.0d0
           qhehd(4,j,i) = 0.0d0
           qph2hd(1,j,i) = -50.0d0
           qph2hd(2,j,i) = 0.0d0
           qph2hd(3,j,i) = 0.0d0
           qph2hd(4,j,i) = 0.0d0
           qoh2hd(1,j,i) = -50.0d0
           qoh2hd(2,j,i) = 0.0d0
           qoh2hd(3,j,i) = 0.0d0
           qoh2hd(4,j,i) = 0.0d0
         end do
       end do
       endif

      print *
      print *,'-------------- Niveaux de H2 ----------------'
      call lev_H2 (l0h2)
      print *,'Nbre niveaux de H2 (mlevh2):',mlevh2

c  Lecture des taux de collision H + H2

       if (ichh2 .eq. 0) then
         fichier = TRIM(data_dir)//'qh1h2_MM.dat'
       else
         fichier = TRIM(data_dir)//'qh1h2_DRF.dat'
       endif
       open(iread2, file = fichier, status = 'old')
       read (iread2,*) ntnr

       do j=1,ntnr
         read (iread2,*) levu, levl, q1, q2, q3, q4
         if ((levu <= l0h2) .AND. (levl <= l0h2)) then
           qh1h2(1,levu,levl) = q1
           qh1h2(2,levu,levl) = q2
           qh1h2(3,levu,levl) = q3
           if (nqc == 4) then
             qh1h2(4,levu,levl) = q4
           endif
         endif
       end do
       close (iread2)

c  5 jan 2000 - Patch grossier pour les transitions manquantes

       do levu=3,l0h2
         iju = njlev(levu)
         ivu = nvlev(levu)
         do levl=1,levu-1
           ijl = njlev(levl)
           if (mod(iju-ijl,2) .ne. 0) cycle
           ivl = nvlev(levl)

           if (qh1h2(1,levu,levl) .eq. -50.0d0) then
             if (ivl .eq. ivu .or. ivu .eq. 1) then
               if (ivl .ge. 4) then
                 lev2 = level(ivu-1,iju)
                 lev1 = level(ivl-1,ijl)
               else if (ijl .le. 1) then
                 lev2 = level(ivu,iju-2)
                 lev1 = levl
               else
                 lev2 = level(ivu,iju-2)
                 lev1 = level(ivl,ijl-2)
               endif
             else if (ivu .gt. ivl .and.
     1                (iju .ne. 0 .or. ijl .ne. 0)) then
               lev2 = level(ivu-1,iju)
               lev1 = level(max(ivl-1,0),ijl)
             else
               lev2 = levu
               lev1 = levl
             end if
             do j=1,nqc
               qh1h2(j,levu,levl) = qh1h2(j,lev2,lev1)
             end do
           end if

         end do
       end do

c  Lecture des taux de collision He + H2

       fichier = TRIM(data_dir)//'qheh2.dat'
       open(iread2, file = fichier, status = 'old')
       read (iread2,*) ntnr

       do j=1,ntnr
         read (iread2,*) levu, levl, q1, q2, q3
         if ((levu <= l0h2) .AND. (levl <= l0h2)) then
           qheh2(1,levu,levl) = q1
           qheh2(2,levu,levl) = q2
           qheh2(3,levu,levl) = q3
         endif
       end do
       close (iread2)

c  Lecture des taux de collision oH2 + H2

       fichier = TRIM(data_dir)//'qoh2h2.dat'
       open(iread2, file = fichier, status = 'old')
       read (iread2,*) ntnr

       do j=1,ntnr
         read (iread2,*) levu, levl, q1, q2, q3
         if ((levu <= l0h2) .AND. (levl <= l0h2)) then
           qoh2h2(1,levu,levl) = q1
           qoh2h2(2,levu,levl) = q2
           qoh2h2(3,levu,levl) = q3
         endif
       end do
       close (iread2)

c  Lecture des taux de collision pH2 + H2

       fichier = TRIM(data_dir)//'qph2h2.dat'
       open(iread2, file = fichier, status = 'old')
       read (iread2,*) ntnr

       do j=1,ntnr
         read (iread2,*) levu, levl, q1, q2, q3
         if ((levu <= l0h2) .AND. (levl <= l0h2)) then
           qph2h2(1,levu,levl) = q1
           qph2h2(2,levu,levl) = q2
           qph2h2(3,levu,levl) = q3
         endif
       end do
       close (iread2)

         if (i_hd .ne. 0) then
c  Lecture des taux de collision He + HD
         fichier = TRIM(data_dir)//'qhehd.dat'
         open (iread2, file = fichier, status = 'old')
         read(iread2,*) ntnd

         do 703 ntc=1,ntnd
           read(iread2,4001) levu, levl, (qhehd(i,levu,levl), i=1,4)
  703    continue
         close (iread2)

c  Lecture des taux de collision H + HD
         fichier = TRIM(data_dir)//'qh1hd.dat'
         open (iread2, file = fichier, status = 'old')
         read(iread2,*) ntnd

         do 704 ntc=1,ntnd
           read(iread2,4001) levu, levl, (qh1hd(i,levu,levl), i=1,4)
 704     continue
         close (iread2)

c  Lecture des taux de collision oH2 + HD
         fichier = TRIM(data_dir)//'qoh2hd.dat'
         open (iread2, file = fichier, status = 'old')
         read(iread2,*) ntnd

         do 705 ntc=1,ntnd
           read(iread2,4001) levu, levl, (qoh2hd(i,levu,levl), i=1,4)
 705     continue
         close (iread2)

c  Lecture des taux de collision pH2 + HD
         fichier = TRIM(data_dir)//'qph2hd.dat'
         open (iread2, file = fichier, status = 'old')
         read(iread2,*) ntnd

         do 706 ntc=1,ntnd
           read(iread2,4001) levu, levl, (qph2hd(i,levu,levl), i=1,4)
 706     continue
         close (iread2)

         endif

c  Lecture des taux de collision CS + H2

         fichier = TRIM(data_dir)//'ratcsh2'
         open (iread2, file = fichier, status = 'old')
         read(iread2,*) ntccs

         do nt=1,ntccs
           read(iread2,*) ji, jf, acs1, acs2, acs3, acs4
           ntc = ji * (ji-1) * 0.5_dp + jf + 1
           if (nt /= ntc) then
             print *, "  Pb coll CS!", nt, ntc
             stop
           endif
           cdocs(1,ntc) = acs1
           cdocs(2,ntc) = acs2
           cdocs(3,ntc) = acs3
           cdocs(4,ntc) = acs4
         end do

         close (iread2)

c  Lecture des taux de collision HCOp + H2
         fichier = TRIM(data_dir)//'rathcoph2'
         open (iread2, file = fichier, status = 'old')
         read(iread2,*) ntchcop

         do  ntc=1,ntchcop
           read(iread2,*) ju, jl, (colhcoph2(i,ju,jl), i=1,4)
         enddo
         close (iread2)

c  initialisation du tableau contenant les inverse des durees de vie
         tmunh2bc = 0.0_dp
c  initialisation du tableau des abond des bandes B et C
         h2bnua = 0.0_dp
         h2cnua = 0.0_dp

c  Calcul anticipe de wlth0 (pour rejeter les raies inutiles)

         hiop = 2.0_DP * xpi*xpi * xqe**4 * xme / (xhp*xhp)
         wlth0 = 1.0e8 * xhp * clum / hiop
         print *, "  wlth0:", wlth0

c  lecture du (des) fichier(s) de transitions radiatives de H2:
c
c              UVH2B.DAT contient les transitions des bandes de Lyman
c                 pour tous les v.
c              UVH2C.DAT contient les transitions des bandes de Werner
c                 pour tous les v.
c
c              le fichier "uvh2.dat" contient les transitions
c              radiatives de l hydrogne moleculaire (au 20 juin 88).
c              complete le 25 sept 90 pour inclure j -> 14.
c
c              les fichiers "uvh2b.dat" et "uvh2c.dat" contiennent
c              les transitions radiatives (lyman et werner)
c              de l hydrogne moleculaire (au 01 janvier 89).
c
c              on lit une raie par ligne, (les transitions de
c              differentes molecules peuvent etre melangees).
c              deux tableaux resument les caracteristiques
c              d une transition:
c              IRAIH2: parametres entiers (nombres quantiques, ...)
c              PRAIH2: parametres reels:
c
c     IRAIH2:   1:       transition (1: h2, lyman; 2: h2, werner; etc)
c     IRAH2V:   2:       v du niveau bas.
c               3:       j du niveau bas.
c               4:       v du niveau haut.
c               5:       delta j (jhaut = jbas + delj)
c
c     PRAIH2:   1:       force d oscillateur.
c     PRAH2V:   2:       longueur d onde (en angstrom).
c               3:       inverse de la duree de vie du niveau haut (s-1).
c               4:       probabilite de dissociation du niveau haut.

c  Allocate various matrix

         allocate (evrdbc(l0h2,l0h2))
         allocate (evcolh(l0h2,l0h2))
         allocate (evltot(l0h2,l0h2))
         allocate (evlqua(nlevh2,nlevh2))

c  coef numerique: 8 pi**2 e**2 / (me * c)   x 1e16 (pour les Angstrom)

         coefosc = 8.0e16_dp * xpi*xpi * xqe2 / (xme * clum)

c  First pass: count the number of lines to keep

         aeitb = zero
         aeitc = zero

         nrh2bt = 0
         fichier = TRIM(data_dir)//'uvh2b29.dat'
         open (iread2, file = fichier, status = 'old')
         read (iread2,4101) nraib

         do i=1,nraib
           read (iread2,4112) num, ira1, nvl, njl, nvu, ndj
     1                      , pra1, pra2, pra3, pra4
           if ((level(nvl,njl) <= l0h2) .AND.
     2         (pra2 >= wlth0)) then
             nrh2bt = nrh2bt + 1
           endif
         end do
         close (iread2)

c  Second pass: read data

         ALLOCATE (irah2b(nrh2bt,4), prah2b(nrh2bt,4))

         open (iread2, file = fichier, status = 'old')
         read (iread2,4101) nraib
         xlamax = zero
         j = 0
         do i=1,nraib
           read (iread2,4112) num, ira1, nvl, njl, nvu, ndj
     1                     , pra1, pra2, pra3, pra4
           lev = level(nvl,njl)
             nju = njl + ndj
           if ((lev <= l0h2) .AND.
     1         (pra2 >= wlth0)) then
             dumy =  coefosc * pra1 / pra2**2
     1            * ((deux*njl+un) / (deux*nju+un))
             aeinb(nvu,nju,lev) = dumy
             aeitb(nvu,nju) = aeitb(nvu,nju) + dumy
c            if (lev > l0h2) then
c              print *, lev, nvu, nju, njl
c            endif
           endif

           if ((lev <= l0h2) .AND.
     2         (pra2 >= wlth0)) then
             j = j + 1
             irah2b(j,1) = nvl
             irah2b(j,2) = njl
             irah2b(j,3) = nvu
             irah2b(j,4) = ndj
             prah2b(j,1) = pra1
             prah2b(j,2) = pra2
             prah2b(j,3) = pra3
             prah2b(j,4) = pra4

             xlamax = max(xlamax,pra2)
             tmunh2bc(ira1,nvu,nju)=pra3 !ira1 precise si Ly ou We
           endif
         end do
         close (iread2)

c  Now Werner band

         nrh2ct = 0
         fichier = TRIM(data_dir)//'uvh2c29.dat'
         open (iread2, file = fichier, status = 'old')
         read (iread2,4101) nraic

         do i=1,nraic
           read (iread2,4112) num, ira1, nvl, njl, nvu, ndj
     1                      , pra1, pra2, pra3, pra4
           if ((level(nvl,njl) <= l0h2) .AND.
     2         (pra2 >= wlth0)) then
             nrh2ct = nrh2ct + 1
           endif
         end do
         close (iread2)

         ALLOCATE (irah2c(nrh2ct,4), prah2c(nrh2ct,4))

         open (iread2, file = fichier, status = 'old')
         read (iread2,4101) nraic
         j = 0
         do i=1,nraic
           read (iread2,4112) num, ira1, nvl, njl, nvu, ndj
     1                     , pra1, pra2, pra3, pra4
           lev = level(nvl,njl)
             nju = njl + ndj
           if ((lev <= l0h2) .AND.
     1         (pra2 >= wlth0)) then
             dumy =  coefosc * pra1 / pra2**2
     1             * ((deux*njl+un) / (deux*nju+un))
             aeinc(nvu,nju,lev) = dumy
             if (ndj .eq. 0) then
               aeitc(nvu,nju,2) = aeitc(nvu,nju,2) + dumy
             else
               aeitc(nvu,nju,1) = aeitc(nvu,nju,1) + dumy
             endif
           endif

           if ((lev <= l0h2) .AND.
     2         (pra2 >= wlth0)) then
             j = j + 1
             irah2c(j,1) = nvl
             irah2c(j,2) = njl
             irah2c(j,3) = nvu
             irah2c(j,4) = ndj
             prah2c(j,1) = pra1
             prah2c(j,2) = pra2
             prah2c(j,3) = pra3
             prah2c(j,4) = pra4

             xlamax = max(xlamax,pra2)
             tmunh2bc(ira1,nvu,nju)=pra3 !ira1 precise si Ly ou We
           endif
         end do
         close (iread2)

         evlqua = zero

c       print *, '  nrh2bt:', nrh2bt, ' nrh2ct:', nrh2ct

       fichier = TRIM(data_dir)//'aquadh2.dat'
       open (iread2, file = fichier, status = 'old')
       read (iread2,4005)
       read (iread2,4101) nrh2q

         do i=1,nrh2q
           read (iread2,4004) nvu, nvl, nju, as, aq, ao
           if (nju > njh2) cycle
           levu = level(nvu,nju)
           njl = nju -2
           if (njl .ge. 0 .and. as .ne. zero) then
             levl = level(nvl,njl)
             evlqua(levl,levu) = - as
             aeitq(levu) = aeitq(levu) + as
           endif
           njl = nju
           if (aq .ne. zero) then
             levl = level(nvl,njl)
             evlqua(levl,levu) = - aq
             aeitq(levu) = aeitq(levu) + aq
           endif
           njl = nju + 2
           if (njl > njh2) cycle
           if (ao .ne. zero) then
             levl = level(nvl,njl)
             evlqua(levl,levu) = - ao
             aeitq(levu) = aeitq(levu) + ao
           endif
         end do
       close (iread2)

       do levu=1,nlevh2
         evlqua(levu,levu) = aeitq(levu)
       end do

c  Calcul des coefficients de cascade, CASHM, BHMBX, BHMCX

       call CACOEF

c  Excitation de H2 par les electrons secondaires

c  Population de H2 a l ETL a 30 K:
c     Utilise pour l'excitation par les electrons secondaires

       jmax = 7
       texcit = 30.0d0
       dummy = zero

       popetl = zero
       entry = zero

       do j=0,jmax
         lev = level(0,j)
         popetl(j) = (deux * j + un) * trois**mod(j,2)
     1             * exp(-elhm(lev) / texcit)
         dummy = dummy + popetl(j)
       end do

       paretl = zero
       ortetl = zero

       do j=0,jmax
         popetl(j) = popetl(j) / dummy
         if (mod(j,2) .eq. 0) then
           paretl = paretl + popetl(j)
         else
           ortetl = ortetl + popetl(j)
         endif
       end do

       write(iscrn,*)
       write(iscrn,*) 'H2 ortho et para'
       write(iscrn,*) 'paretl:',paretl
       write(iscrn,*) 'ortetl:',ortetl

       sentry = zero
       sentro = zero
       sentrp = zero

       do ij=0,jmax
         do iv=0,nvxh2
           sentry = sentry + entry(iv,ij)
           if (mod(ij,2) .eq. 0) then
             sentrp = sentrp + entry(iv,ij)
           else
             sentro = sentro + entry(iv,ij)
           endif
         end do
       end do

       if (sentry .ne. zero) then
         sentrp = sentrp / sentry
         sentro = sentro / sentry
       else
         sentrp = un
         sentro = un
       endif

       do ij=0,jmax
         do iv=0,nvxh2
           if (mod(ij,2) .eq. 0) then
             entry(iv,ij) = entry(iv,ij) * paretl / sentrp
           else
             entry(iv,ij) = entry(iv,ij) * ortetl / sentro
           endif
         end do
       end do

c--------+---------+---------+---------+---------+---------+---------+-*-------+

c  lecture du fichier UVH.DAT:
c
c     IRAIH:             n du niveau bas.
c     PRAIH:    1:       force d oscillateur.
c               2:       longueur d onde (en angstrom).
c               3:       inverse de la duree de vie du niveau haut (s-1).

       fichier = TRIM(data_dir)//'uvh.dat'
       nraih = 49
       open (iread2, file = fichier, status = 'old')
          do i=1,nraih
          read (iread2,4022) iraih(i), ( praih(i,m), m = 1,3)
          end do
       close (iread2)

c--------+---------+---------+---------+---------+---------+---------+-*-------+

c  lecture du fichier UVD.DAT:
c
c     IRAID:             n du niveau bas.
c     PRAID:    1:       force d oscillateur.
c               2:       longueur d onde (en angstrom).
c               3:       inverse de la duree de vie du niveau haut (s-1).

       fichier = TRIM(data_dir)//'uvd.dat'
       nraid = 12
       open (iread2, file = fichier, status = 'old')
          do 29 i=1,nraid
          read (iread2,4022) iraid(i), ( praid(i,m), m = 1,3)
   29     continue
       close (iread2)

c--------+---------+---------+---------+---------+---------+---------+-*-------+

       nrhdbt = 0
       nrhdct = 0
       if (i_hd .ne. 0) then

c  lecture du fichier UVHD.DAT
c
c     IRAIHD:   1:       transition (1: h2, lyman; 2: h2, werner; etc)
c               2:       v du niveau bas.
c               3:       j du niveau bas.
c               4:       v du niveau haut.
c               5:       delta j (jhaut = jbas + delj)
c
c     PRAIHD:   1:       force d oscillateur.
c               2:       longueur d onde (en angstrom).
c               3:       nombre d'onde (en cm-1)
c               4:       inverse de la duree de vie du niveau haut (s-1).
c               5:       probabilite de dissociation du niveau haut.
c
c     evdihd(levl,levu) : Proba d emission dipolaire et quadrupolaire dans X
c     ahdtq(lev)        : Proba d emission dipolaire et quadrupolaire totale dans X
c     ahdnb(nvu,nju,lev): Proba d emission B-X de (v,j) vers lev
c     ahdtb(nvu,nju)    : Proba d emission totale B-X a partir du niv (v,j)
c     ahdnc(nvu,nju,lev): Proba d emission C-X
c     ahdtc(nvu,nju,1:2): Proba d emission totale C-X   1: C+
c                                                       2: C-

         print *
         print *, '-------------- Niveaux de HD ----------------'

         fichier = TRIM(data_dir)//'uvhd.dat'
         open (iread2, file = fichier, status = 'old')

         ahdtb = zero
         ahdnb = zero

         ahdtc = zero
         ahdnc = zero

c  On ne stocke dans airahd(bc) et prahd(bc) que les raies dont j <= mjhd
c    si njl est > a mjhd, la transition doit etre negligee pour
c      converver le nombre total de molecules

         read (iread2,4101) nrai
         do ii=1,nrai
           read (iread2,4212) ira1, nvl, njl, nvu, ndj
     1                     , pra1, pra2, pra3, pra4, pra5
           lev = levhd(nvl,njl)
           if ((njl <= mjhd) .AND.  (lev <= l0hd) .AND.
     2         (pra2 >= wlth0)) then
             if (ira1 .eq. 1) then
               nrhdbt = nrhdbt + 1
              else
               nrhdct = nrhdct + 1
             endif
           endif
         end do
         close (iread2)

         ALLOCATE (irahdb(nrhdbt,4), prahdb(nrhdbt,4))
         ALLOCATE (irahdc(nrhdct,4), prahdc(nrhdct,4))

         ib = 0
         ic = 0
         open (iread2, file = fichier, status = 'old')
         read (iread2,4101) nrai
         do ii=1,nrai
           read (iread2,4212) ira1, nvl, njl, nvu, ndj
     1                     , pra1, pra2, pra3, pra4, pra5
           lev = levhd(nvl,njl)
           if ((njl <= mjhd) .AND.  (lev <= l0hd) .AND.
     2         (pra2 >= wlth0)) then
             if (ira1 .eq. 1) then
               ib = ib + 1
               irahdb(ib,1) = nvl
               irahdb(ib,2) = njl
               irahdb(ib,3) = nvu
               irahdb(ib,4) = ndj
               prahdb(ib,1) = pra1
               prahdb(ib,2) = pra2
               prahdb(ib,3) = pra4
               prahdb(ib,4) = pra5
             else
               ic = ic + 1
               irahdc(ic,1) = nvl
               irahdc(ic,2) = njl
               irahdc(ic,3) = nvu
               irahdc(ic,4) = ndj
               prahdc(ic,1) = pra1
               prahdc(ic,2) = pra2
               prahdc(ic,3) = pra4
               prahdc(ic,4) = pra5
             endif

           xlamax = max(xlamax,pra2)
           endif
         end do
         close (iread2)

c         print *, 'xlamax', xlamax,'  nrhdbt:', nrhdbt,
c     1            '  nrhdct:', nrhdct

c Lecture des proba d'emission

        jmaxi0 = njhd(l0hd)
        fichier = TRIM(data_dir)//'ahdnb.dat'
        open(iread2,file=fichier,status='old')
        read(iread2,4005)
        do 300 i=1,30000
          read(iread2,4400,end=310) nvu, nju, nvl, njl, dumy
          if (nvu .gt. nvbhd .or. nju .gt. jmaxi0+1
     1       .or. nvl .gt. nvxhd .or. njl .gt. mjhd) goto 300
          lev = levhd(nvl,njl)
          ahdnb(nvu,nju,lev) = dumy
          ahdtb(nvu,nju) = ahdtb(nvu,nju) + dumy
 300    continue
 310    continue

        fichier = TRIM(data_dir)//'ahdnc1.dat'
        open(iread2,file=fichier,status='old')
        read(iread2,4005)
        do 301 i=1,30000
          read(iread2,4400,end=311) nvu, nju, nvl, njl, dumy
          if (nvu .gt. nvchd .or. nju .gt. jmaxi0+1
     1       .or. nvl .gt. nvxhd .or. njl .gt. mjhd) goto 301
          lev = levhd(nvl,njl)
          ahdnc(nvu,nju,lev) = dumy
          ahdtc(nvu,nju,1) = ahdtc(nvu,nju,1) + dumy
 301    continue
 311    continue

        fichier = TRIM(data_dir)//'ahdnc2.dat'
        open(iread2,file=fichier,status='old')
        read(iread2,4005)
        do 302 i=1,30000
         read(iread2,4400,end=312) nvu, nju, nvl, njl, dumy
         if (nvu .gt. nvchd .or. nju .gt. jmaxi0
     1      .or. nvl .gt. nvxhd .or. njl .gt. mjhd) goto 302
         lev = levhd(nvl,njl)
         ahdnc(nvu,nju,lev) = dumy
         ahdtc(nvu,nju,2) = ahdtc(nvu,nju,2) + dumy
 302    continue
 312    continue

        ahdtq = zero
        evdihd = zero


c---- Proba d'emission dipolaire et quadrupolaire de HD -------------

       fichier = TRIM(data_dir)//'aquadhd.dat'
       open (iread2, file = fichier, status = 'old')
       read (iread2,4005)
       read (iread2,4101) nrhdq

         do 293 i=1,nrhdq
           read (iread2,4006) nvu, nvl, nju, as, ar, aq, ap, ao
           if (nju .gt. mjhd) goto 293
           levu = levhd(nvu,nju)
           njl = nju -2
           if (njl .ge. 0 .and. as .ne. zero) then
             levl = levhd(nvl,njl)
             evdihd(levl,levu) = - as
             ahdtq(levu) = ahdtq(levu) + as
           endif
           njl = nju - 1
           if (njl .ge. 0 .and. ar .ne. zero) then
             levl = levhd(nvl,njl)
             evdihd(levl,levu) = - ar
             ahdtq(levu) = ahdtq(levu) + ar
           endif
           njl = nju
           if (aq .ne. zero) then
             levl = levhd(nvl,njl)
             evdihd(levl,levu) = - aq
             ahdtq(levu) = ahdtq(levu) + aq
           endif
           njl = nju + 1
           if (njl .le. mjhd .and. ap .ne. zero) then
             levl = levhd(nvl,njl)
             evdihd(levl,levu) = - ap
             ahdtq(levu) = ahdtq(levu) + ap
           endif
           njl = nju + 2
           if (njl .le. mjhd .and. ao .ne. zero) then
             levl = levhd(nvl,njl)
             evdihd(levl,levu) = - ao
             ahdtq(levu) = ahdtq(levu) + ao
           endif
  293    continue
       close (iread2)

       do levu=1,nlevhd
         evdihd(levu,levu) = ahdtq(levu)
       end do

c  Calcul des coefficients de cascade de HD,  CASHD, BHDBX, BHDCX

       call CACOEFHD

       write(iscrn,*)
       write(iscrn,*) "Longueur d'onde maximale:",xlamax
       write(iscrn,*) "Nbre de transitions Lyman (nrhdbt):",nrhdbt
       write(iscrn,*) "                    Werner(nrhdct):",nrhdct

       endif

c--------+---------+---------+---------+---------+---------+---------+-*-------+

       nrco0 = 0
       nrco1 = 0
       nrco2 = 0
       nrco3 = 0

c  lecture du fichier UVCO.DAT:
c
c     IRAICO:            j du niveau bas.
c     PRAICO:   1:       force d oscillateur.
c               2:       longueur d onde (en angstrom).
c               3:       inverse de la duree de vie du niveau haut (s-1).
c               4:       probabilite de dissociation du niveau haut.

       fichier = TRIM(data_dir)//'UVCO.NW1'
       open (iread2, file = fichier, status = 'old')

       read (iread2,4101) nrco0
       do i=1,nrco0
         read (iread2,4022) irco0(i), ( prco0(i,m), m = 1,4)
       end do
       close (iread2)

C
C --- LECTURE EVENTUELLE DES FICHIERS SPECTROSCOPIQUES DE C13O
C
       if (itrco1 .eq. 1) then

         fichier = TRIM(data_dir)//'UVC13O.NW1'
         open (iread2, file = fichier, status = 'old')

         read (iread2,4101) nrco1
         do i=1,nrco1
           read (iread2,4022) irco1(i), ( prco1(i,m), m = 1,4)
         end do
         close (iread2)

       endif
C
C --- LECTURE EVENTUELLE DES FICHIERS SPECTROSCOPIQUES DE CO18
C
       if (itrco2 .eq. 1)  then

         fichier = TRIM(data_dir)//'UVCO18.NW1'
         open (iread2, file = fichier, status = 'old')

         read (iread2,4101) nrco2
         do i=1,nrco2
           read (iread2,4022) irco2(i), ( prco2(i,m), m = 1,4)
         end do
         close (iread2)

       endif
C
C --- LECTURE EVENTUELLE DES FICHIERS SPECTROSCOPIQUES DE C13O18
C
       if (itrco3 .eq. 1)  then

         fichier = TRIM(data_dir)//'UVC13O18.NW1'
         open (iread2, file = fichier, status = 'old')

         read (iread2,4101) nrco3
         do i=1,nrco3
           read (iread2,4022) irco3(i), ( prco3(i,m), m = 1,4)
         end do
         close (iread2)

       endif

c  nombre de raies a prendre en compte:

         mfgk = nrh2bt + nrh2ct
     1        + nrhdbt + nrhdct
     2        + nrco0
     3        + nrco1 + nrco2 + nrco3

       allocate (fgkwl(mfgk), fgkrf(mfgk), fgkrfr(mfgk), fgkgrd(mfgk))

       ik = 0
       do i=1,nrh2bt
         ik = ik + 1
         fgkwl(ik) = prah2b(i,2)
       end do
       do i=1,nrh2ct
         ik = ik + 1
         fgkwl(ik) = prah2c(i,2)
       end do
       do i=1,nrhdbt
         ik = ik + 1
         fgkwl(ik) = prahdb(i,2)
       end do
       do i=1,nrhdct
         ik = ik + 1
         fgkwl(ik) = prahdb(i,2)
       end do
       do i=1,nrco0
         ik = ik + 1
         fgkwl(ik) = prco0(i,2)
       end do
       do i=1,nrco1
         ik = ik + 1
         fgkwl(ik) = prco1(i,2)
       end do
       do i=1,nrco2
         ik = ik + 1
         fgkwl(ik) = prco2(i,2)
       end do
       do i=1,nrco3
         ik = ik + 1
         fgkwl(ik) = prco3(i,2)
       end do

c--------+---------+---------+---------+---------+---------+---------+-*-------+

!   appel de la subroutine initialisation du champ de rayonnement

      CALL  RF_INIT

c      print *, '   fin de INIT1 !'
c--------+---------+---------+---------+---------+---------+---------+-*-------+

 1012 format (a35)

 2000 format (/,15x,'Model of Steady state Interstellar cloud (PDR)',//,
     3       15x,'JLB, GPF, ER and many others',
     4       5x,'VIII 91 to X 03',/,15x,28('*'))
 2002 format (/,5x,'Model name:     ',a35)
 2003 format (/,5x,'Chemistry file: ',a35)
 2010 format (/,5x,'Model definition :',//,
     1   5x,'Left  side radiation field intensity = ',1pe9.2,
     2      ' x Draine field',/,
     3   5x,'Right side radiation field intensity = ',e9.2,
     4      ' x Draine field',/,
     5   5x,'Illuminating star:                     ',a15,/,
     6   5x,'Star distance                      d = ',e10.3,
     7      ' pc',/,12x,'d < 0 : left side',/,
     8   12x,'d > 0 : right side',/,
     9   12x,'d = 0 : none',//,
     a   5x,'Cosmic rays ionisation flux: fmrc    = ',0pf9.2,
     b       ' (* 1e-17 s-1)',/)
 2009 format (
     1   5x,'Thermal balance (no: 0, yes: 1)      = ',i5,/,
     2   5x,'Equation of state: ifisob            = ',i5,/,
     3   12x,'0: Constant density nH',/,
     4   12x,'1: nH and T interpolated from a file',/,
     5   15x,'2: Isobaric (nH * T = Cte)',/,
     6   12x,'3: Analytical temperature profile',/,
     7   12x,'5: Fractal density profile (experimental)',/,
     8   5x,'external Pressure: (if needed)       = ',1pe10.2,
     9      ' K cm-3')
 2005 format (/,5x,'Photodissociations : ',/,
     1   5x,'(if ITRXX = 0: no photodissociation of XX)',/,
     3   10x,'itrh2  =',i2,10x,'itrhd  =',i2,/,
     5   10x,'itrco0 =',i2,10x,'itrco1 =',i2,
     6    5x,'itrco2 =',i2, 5x,'itrco3 =',i2,/)
c--------+---------+---------+---------+---------+---------+---------+-*-------+
 2011 format (5x,'H2 levels parameters:',/,
     1        5x,'l0h2 = ',i4,//,
     2        5x,'HD levels parameters:',/,
     3        5x,'Jmax = ',i8,5x,'l0hd = ',i4,//,
     4        5x,'H2 formation on grains: iforh2 = ',i4,/,
     5        15x,'0: Mean internal energy is 1/3 E(H2 formation)',/,
     6        15x,'1: Maxwell distribution at 1/3 E(H2 formation)',/,
     7        15x,'2: All on v = 14, J = 0 or 1 (obsolete)',/,
     8        15x,'3: Maxwell distrib at 65 K on v = 6 (obsolete)',//,
     9        5x,'H sticking on grains:    istic = ',i4,/
     a        15x,'1: Sticking read in chemistry file',/,
     b        15x,'2: Modified from Sternberg & Dalgarno',/,
     c        15x,'3: Modified from Andersson & Wannier',/,
     d        15x,'4: Modified from Flower & des Forets',/)
 2013 format (5x,'Extinction and gas to dust parameters:',/,
     1        5x,'   rrr    = ',f9.3,10x,'Av = rrr * E(B-V)',/,
     2        5x,'   cdunit = ',1pe9.2,10x,'NH = cdunit * E(B-V)',//,
     3        5x,'vitesse turbulente: vturb = ',e9.3,' cm s-1')
 2014  format (/,5x,'mass ratio (gas/grains):      gratio = ',1pe10.2,/,
     1        5x,'grains density :               rhogr = ',e10.2,
     2           ' g cm-3',/,
     3        5x,'Size distribution exponent:    alpgr = ',0pf6.2,/,
     4        5x,'grains min radius:            rgrmin = ',1pe10.2,
     5           ' cm',/,
     6        5x,'grains max radius:            rgrmax = ',e10.2,
     7           ' cm',/,
     8        5x,'mean dist between adsorption sites   = ',e10.2,
     7           ' cm')
 2015 format (/,
     1        5x,'Order of Legendre expansion:    lleg = ',i3,/,
     2        5x,'Dust albedo                      alb = ',1pe10.2,/,
     3        5x,'<cos(theta)> (g)                  gg = ',e10.2,//,
     4        5x,'Total optical depth:          taumax = ',e10.2,/,
     5        5x,'                         i.e.  Avmax = ',e10.2)
 2018  format (/,5x,'Courbe d extinction de :',a10,/,
     1           5x,'Fitzpatrick & Massa coefs:',1x,6f8.3,/)
 2020  format (5x,'Initial temperature:            ',1pe12.2,' K',/,
     1         5x,'Initial density:                ',e12.2,' cm-3')
 4001  format(1x,I2,1x,I2,4(1x,E10.3))
 4101  format (i8)
 4112  format (i8,i3,1x,2i3,1x,2i3,2x,1pd9.3,2x,1pd12.6,2(2x,1pd9.3))
 4212  format (6x,i1,1x,2i3,1x,2i3,2x,1pd9.3,2x,1pd12.6, 3x, 1pd13.7,
     1          2(2x,1pd9.3))
 4004  format (3i4,3d15.6)
 4005  format (//)
 4006  format (3i4,5d15.6)
 4022  format (2x,i3,2x,1pd9.3,2x,1pd12.6,2(2x,1pd9.3))
 4400  format (i3,5x,i3,5x,i3,5x,i3,5x,d13.6)
c--------+---------+---------+---------+---------+---------+---------+-*-------+

       return
       end

       subroutine CACOEF

       use constantes
       use auxiliaries
       USE H2_DATA
       USE STR_DATA
       USE HD_DATA
       USE PROFIL

       implicit double precision (a-h,o-z)

       dimension cscad(nlevh2,nlevh2)

c      debut du calcul des coefficients de cascade:

c      Il faut choisir a quel niveau du fondamental on s arrete
c      provisoirement passe par IFLVH2

       nj0 = njlev(l0h2)
       nv0 = nvlev(l0h2)
       write (iscrn,1001) l0h2, nv0, nj0, elhm(l0h2)
       write (iwrit2,1001) l0h2, nv0, nj0, elhm(l0h2)

       write(iscrn,*)
       write(iscrn,*) 'Nbre de transitions Lyman (nrh2bt)',nrh2bt
       write(iscrn,*) '                    Werner(nrh2ct)',nrh2ct

       cscad = zero

c      Calcul des coefficients de cascade par recurence
c        en commencant par le haut.

       do levl=nlevh2,l0h2+1,-1
         njl = njlev(levl)
         cscad(levl,levl) = un
         do levu=levl+1,nlevh2
           nju = njlev(levu)
           total = zero
           do k=levl+1,levu
             nj = njlev(k)
             if (aeitq(k) == zero) cycle
             total = total - cscad(levu,k) * evlqua(levl,k)
     1                     / aeitq(k)
           end do
           cscad(levu,levl) = total
         end do
       end do

c       print *, '    cscad calcule'

c      Calcul des CASHM (cascades vers les niveaux traites
c        completement)

       do levl=1,l0h2
         njl = njlev(levl)
         do levu=l0h2+1,nlevh2
           nju = njlev(levu)
           total = zero
           do k=l0h2+1,levu
             nj = njlev(k)
             if (aeitq(k) == zero) cycle
             total = total - cscad(levu,k) * evlqua(levl,k)
     1                     / aeitq(k)
           end do
           cashm(levu,levl) = total
         end do
       end do

c      Verification des CASHM:
c        la conservation des molecules impose
c        que la somme sur les niveaux inferieure soit 1

       do levu=l0h2+1,nlevh2
         nvu = nvlev(levu)
         nju = njlev(levu)
         total = un
         do levl=1,l0h2
           njl = njlev(levl)
           total = total - cashm(levu,levl)
         end do
         if (abs(total) .gt. 1.0d-8 .and.
     1       total .ne. un) then
           print *, 'pb CASHM!:',nvu,nju,total,levu
         endif
       end do

c       print *, '    cashm calcule'

c      calcul de bhmbx (transitions de Lyman)

       do nvu=0,nvbh2
         do nju=0,njh2+1
           dviei = aeitb(nvu,nju)
           if (dviei .eq. zero) cycle

           do levl=1,l0h2
             njl = njlev(levl)
             total = zero
             do levu=l0h2+1,nlevh2
               nj = njlev(levu)
               total = total + aeinb(nvu,nju,levu) * cashm(levu,levl)
             end do
             total = total + aeinb(nvu,nju,levl)
             bhmbx(nvu,nju,levl) = total / dviei
           end do

         end do
       end do

c      A fin de verifications...

c 510  print *, ' Quel BHMBX voulez-vous? (vu, Ju, vl, Jl)'
c      read *, nvu, nju, nvl, njl
c      levl = level(nvl,njl)
c      print *, nvu, nju, levl, bhmbx(nvu, nju, levl)
c      print *, '   Un autre? (oui: 1)'
c      read *, iii
c      if (iii .eq. 1) goto 510

c      test de bhmbx
         somme = 1

       do nvu=0,nvbh2
         do nju=0,njh2+1
           dviei = aeitb(nvu,nju)
           if (dviei .eq. zero) cycle
           total = un

           do levl=1,l0h2
             njl = njlev(levl)
             total = total - bhmbx(nvu,nju,levl)
           end do

           if (abs(total) .gt. 1.0d-8) then
             print *, 'pb BHMBX!:',nvu,nju,total
           endif

         end do
       end do

c       print *, '    bhmbx calcule'

c      calcul de BHMCX (transitions de Werner)
c        Attention au Lambda-doubling!
c        Chaque sous-niveau est identifie par la parite
c        du Delta-J

       do nvu=0,nvch2
         do nju=1,njh2+1
           dviei1 = aeitc(nvu,nju,1)
           dviei2 = aeitc(nvu,nju,2)
           if (dviei1 .eq. zero .and. dviei2 .eq. zero) cycle

           do levl=1,l0h2
             nvl = nvlev(levl)
             njl = njlev(levl)
             ndj0 = iabs(nju - njl)

             if (mod(ndj0,2) == 0
     1           .and. dviei2 /= zero) then

               total = zero
               do levu=l0h2+1,nlevh2
                 nv = nvlev(levu)
                 nj = njlev(levu)
                 ndj = iabs(nju - nj)
                 if (ndj .ne. 0) cycle
                 total = total + aeinc(nvu,nju,levu) * cashm(levu,levl)
               end do
               total = total + aeinc(nvu,nju,levl)
               bhmcx(nvu,nju,levl) = total / dviei2

             else if (mod(ndj0,2) == 1
     1                .and. dviei1 /= zero) then

               total = zero
               do levu=l0h2+1,nlevh2
                 nj = njlev(levu)
                 ndj = iabs(nju - nj)
                 if (ndj /= 1) cycle
                 total = total + aeinc(nvu,nju,levu) * cashm(levu,levl)
               end do
               total = total + aeinc(nvu,nju,levl)
               bhmcx(nvu,nju,levl) = total / dviei1
             endif

           end do

         end do
       end do

c      test des bhmcx
         Somme = 1

       do nvu=0,nvch2
         do nju=1,njh2+1
           dviei1 = aeitc(nvu,nju,1)
           dviei2 = aeitc(nvu,nju,2)
           if (dviei1 .eq. zero .and. dviei2 .eq. zero) cycle

           total1 = un
           total2 = un

           do levl=1,l0h2
             njl = njlev(levl)
             ndj = iabs(nju - njl)

             if (mod(ndj,2) == 0) then
               total2 = total2 - bhmcx(nvu,nju,levl)
             else if (mod(ndj,2) == 1) then
               total1 = total1 - bhmcx(nvu,nju,levl)
             endif

           end do

           if ((abs(total1) .gt. 1.0d-8
     1         .or. abs(total2) .gt. 1.0d-8)
     2         .and. nju .ne. njh2+1) then
             print *, 'CX!:',nvu, nju, total1, total2
           endif

         end do
         end do

c 520  print *, ' Quel BHMCX voulez-vous? (vu, Ju, vl, Jl)'
c      read *, nvu, nju, nvl, njl
c      levl = level(nvl,njl)
c      print *, nvu, nju, levl, bhmcx(nvu, nju, levl)
c      print *, '   Un autre? (oui: 1)'
c      read *, iii
c      if (iii .eq. 1) goto 520

c       print *, '    bhmcx calcule'

c--------+---------+---------+---------+---------+---------+---------+-*-------+

 1001  format(/,1x,'Levels from 1 to ',i4, ' are included',/
     1 ,1x,'Last level is: v:',i4,' J:',i4,/
     2 ,1x,'Last level energy is:',f15.3,' K',/)
c--------+---------+---------+---------+---------+---------+---------+-*-------+

       end

c--- Calcul des coefficients de cascade pour HD --------------------------------
c      voir Astron. Astrophys. 190, 215-236 (1988)

       Subroutine CACOEFHD

       use constantes
       use auxiliaries
       USE STR_DATA
       USE HD_DATA
       USE PROFIL

       implicit double precision (a-h,o-z)

       dimension cscad(nlevhd,nlevhd)

c      debut du calcul des coefficients de cascade:

       nj0 = njhd(l0hd)
       nv0 = nvhd(l0hd)
       write (iscrn,1001) l0hd, nv0, nj0, elhd(l0hd)
       write (iwrit2,1001) l0hd, nv0, nj0, elhd(l0hd)


c   initialisation

       do 80 levl=1,nlevhd
         do 70 levu=1,nlevhd
           cscad(levu,levl) = zero
   70    continue
   80  continue

c      Si mjhd n est pas assez grand, il peut y avoir des niveaux
c         "laisses pour compte", on les ecrit

       do 90 lev=1,l0hd
         nj=njhd(lev)
         if (nj .gt. mjhd) then
           write (iwrit2,2001) lev, nvhd(lev), nj
           write (iscrn,2001) lev, nvhd(lev), nj
         endif
   90  continue

c      Calcul des coefficients de cascade par recurence
c        en commencant par le haut.

       do 120 levl=nlevhd,l0hd+1,-1
         njl = njhd(levl)
         if (njl .gt. mjhd) goto 120
         cscad(levl,levl) = un

         do 110 levu=levl+1,nlevhd
           nju = njhd(levu)
           if (nju .gt. mjhd) goto 110
           total = zero
           do 100 k=levl+1,levu
             nj = njhd(k)
             if (nj .gt. mjhd .or. ahdtq(k) .eq. zero) goto 100
             total = total - cscad(levu,k) * evdihd(levl,k)
     1                     / ahdtq(k)
  100      continue
           cscad(levu,levl) = total
  110    continue

  120  continue

c       print *, '    cscad calcule'

c      Calcul des CASHD (cascades vers les niveaux traites
c        completement)

       do 150 levl=1,l0hd
         njl = njhd(levl)
         if (njl .gt. mjhd) goto 150

         do 140 levu=l0hd+1,nlevhd
           nju = njhd(levu)
           if (nju .gt. mjhd) goto 140
           total = zero
           do 130 k=l0hd+1,levu
             nj = njhd(k)
             if (nj .gt. mjhd .or. ahdtq(k) .eq. zero) goto 130
             total = total - cscad(levu,k) * evdihd(levl,k)
     1                     / ahdtq(k)
  130      continue
           cashd(levu,levl) = total
  140    continue

  150  continue

c      Verification des CASHD:
c        la conservation des molecules impose
c        que la somme sur les niveaux inferieure soit 1

       do 151 levu =l0hd+1,nlevhd
         nvu = nvhd(levu)
         nju = njhd(levu)
         if (nju .gt. mjhd) goto 151
         total = un

         do 141 levl=1,l0hd
           njl = njhd(levl)
           if (njl .gt. mjhd) goto 141
           total = total - cashd(levu,levl)
  141    continue

         if (abs(total) .gt. 1.0d-8 .and.
     1       total .ne. un) then
           print *, 'pb CASHD!:',nvu,nju,total,levu
         endif

  151  continue

c       print *, '    cashd calcule'


c      calcul de bhdbx (betas pour transitions de Lyman)

       jmaxi0 = njhd(l0hd)
       do 190 nvu=0,nvbhd
         do 180 nju=0,jmaxi0+1
           dviei = ahdtb(nvu,nju)
           if (dviei .eq. zero) goto 180
           do 170 levl=1,l0hd
             njl = njhd(levl)
             if (njl .gt. mjhd) goto 170
             total = zero
             do 160 levu=l0hd+1,nlevhd
               nj = njhd(levu)
               if (nj .gt. mjhd) goto 160
               total = total + ahdnb(nvu,nju,levu) * cashd(levu,levl)
  160        continue
             total = total + ahdnb(nvu,nju,levl)
             bhdbx(nvu,nju,levl) = total / dviei
  170      continue

  180    continue
  190  continue


c      A fin de verifications...

c 510  print *, ' Quel BHDBX voulez-vous? (vu, Ju, vl, Jl)'
c      read *, nvu, nju, nvl, njl
c      levl = levhd(nvl,njl)
c      print *, nvu, nju, levl, bhdbx(nvu, nju, levl)
c      print *, '   Un autre? (oui:1)'
c      read *, iii
c      if (iii .eq. 1) goto 510


c      test de bhdbx
         somme = 1

       do 191 nvu=0,nvbhd
         do 181 nju=1,jmaxi0+1
           dviei = ahdtb(nvu,nju)
           if (dviei .eq. zero) goto 181
           total = un

           do 171 levl=1,l0hd
             njl = njhd(levl)
             if (njl .gt. mjhd) goto 171
             total = total - bhdbx(nvu,nju,levl)
  171      continue

           if (abs(total) .gt. 1.0d-8) then
             print *, 'pb BHDBX!:',nvu,nju,total
           endif

  181    continue
  191  continue

c       print *, '    bhdbx calcule'

c      calcul de BHDCX (betas pour transitions de Werner)
c        Attention au Lambda-doubling!
c        Chaque sous-niveau est identifie par la parite
c        du Delta-J

c--------- A partir de C-  --------------

        do 230 nvu=0,nvchd
          do 220 nju=1,jmaxi0
           dviei2 = ahdtc(nvu,nju,2)
           if (dviei2 .eq. zero) goto 220

            do 210 levl=1,l0hd
             nvl = nvhd(levl)
             njl = njhd(levl)

             if (njl .le. mjhd
     1           .and. dviei2 .ne. zero) then

              total = zero
              do 200 levu=l0hd+1,nlevhd
               nj = njhd(levu)
               ndj = iabs(nju - nj)
               if (nj .gt. mjhd .or. ndj .ne. 0) goto 200
               total = total + ahdnc(nvu,nju,levu) * cashd(levu,levl)
  200         continue
              ndj = iabs(nju - njhd(levl))
              if (ndj .eq. 0) total = total + ahdnc(nvu,nju,levl)
              bhdcmx(nvu,nju,levl) = total / dviei2
             endif

  210     continue
  220    continue
  230   continue

c---------   A partir de C+ ----------

        do 232 nvu=0,nvchd
          do 222 nju=1,jmaxi0+1
           dviei1 = ahdtc(nvu,nju,1)
           if (dviei1 .eq. zero) goto 222

            do 212 levl=1,l0hd
             nvl = nvhd(levl)
             njl = njhd(levl)

              if (njl .le. mjhd
     1                .and. dviei1 .ne. zero) then

              total = zero
              do 206 levu=l0hd+1,nlevhd
               nj = njhd(levu)
               ndj = iabs(nju - nj)
               if (nj .gt. mjhd .or. ndj .ne. 1) goto 206
               total = total + ahdnc(nvu,nju,levu) * cashd(levu,levl)
  206         continue
              ndj = iabs(nju - njhd(levl))
              if (ndj .eq. 1)  total = total + ahdnc(nvu,nju,levl)
              bhdcpx(nvu,nju,levl) = total / dviei1
             endif

  212        continue

  222       continue
  232      continue


c      test des bhdcx
         Somme = 1

       do 231 nvu=0,nvchd
         do 221 nju=1,jmaxi0+1
           dviei1 = ahdtc(nvu,nju,1)
           dviei2 = ahdtc(nvu,nju,2)

           if (dviei1 .eq. zero .and. dviei2 .eq. zero) goto 221

           total1 = un

           total2 = un

           do 211 levl=1,l0hd
             njl = njhd(levl)
             ndj = iabs(nju - njl)

             if (njl .le. mjhd) then

               total2 = total2 - bhdcmx(nvu,nju,levl)

               total1 = total1 - bhdcpx(nvu,nju,levl)

             endif

 211               continue

           if ((abs(total1) .gt. 1.0d-8
     1         .or. abs(total2) .gt. 1.0d-8)
     1         .and. nju .ne. jmaxi0+1)
     2       print *, 'CX!:',nvu, nju, total1 , total2
 221           continue
 231               continue

c 520  print *, ' Quel BHDCX voulez-vous? (vu, Ju, vl, Jl)'
c      read *, nvu, nju, nvl, njl
c      levl = levhd(nvl,njl)
c      print *, nvu, nju, levl, bhdcx(nvu, nju, levl)
c      print *, '   Un autre? (oui: 1)'
c      read *, iii
c      if (iii .eq. 1) goto 520

c       print *, '    bhdcx calcule'

c--------+---------+---------+---------+---------+---------+---------+-*-------+

 1001    format(1x,'Les niveaux de 1 a ',i4, ' sont traites',/
     1 ,1x,'le dernier niveau est donc: v:',i4,' J:',i4,/
     2 ,1x,'energie du dernier niveau:',f15.3,' K')
 2001      format(3x,'niveau exclus:',i6,' v:',i3,' J:',i3)
c--------+---------+---------+---------+---------+---------+---------+-*-------+

       end

