
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%
       subroutine TRANSF
c      %%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE SP_DATA
       USE H2_DATA
       USE HD_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

       implicit double precision(a-h,o-z)

c      character (LEN=30) ::  le_fich

c      parameter (nwlgin = 45)
c      real wlang(nwlgin), ray(nwlgin)

       dimension wlnMMP(45), rayMMP(45)
       dimension ttetl(njco+1)

c---------------------------------------------------------------------
c   calcule la grille en longueur d onde et a chaque longueur d onde
c   1* le champ de rayonnement uv. 2* le coefficient d extinction des
c   poussieres. 3* l albedo des poussieres.
c   d apres mathis et al 1985, astron. astrophys. 128, 212
c   extinc=alambda/av
c   ray=u(lambda) en erg cm-3 a-1 -densite d energie par angstrom-
c---------------------------------------------------------------------

       real (kind=DP), dimension(5) :: tti

c  wlnMMP, rayMMP est le champ de rayonnement de
c       Mathis, Metzger & Panagia
c  d apres mathis et al 1985, astron. astrophys. 128, 212
c  ray=u(lambda) en erg cm-3 a-1 -densite d energie par angstrom-

       data wlnMMP / 5.0d2, 9.1211d+02, 9.12118d+02, 1.0d3, 1.1d3
     1            , 1.3d3, 1.43d3, 1.8d3, 2.0d3, 2.1d3, 2.16d3, 2.3d3
     2            , 2.5d3, 3.46d3, 4.35d3, 5.5d3, 7.0d3, 9.0d3, 1.2d4
     3            , 1.8d4, 2.2d4, 2.4d4, 3.4d4, 4.0d4, 5.0d4, 6.0d4
     4            , 8.0d4, 1.0d5, 1.2d5, 2.0d5, 2.5d5, 3.5d5, 4.0d5
     5            , 5.0d5, 6.0d5, 7.0d5, 8.0d5, 1.0d6, 1.5d6, 2.0d6
     6            , 3.0d6, 4.0d6, 6.0d6, 8.0d6, 1.0d7 /
       data rayMMP / 0.0d0, 0.0d0, 3.6d0, 4.9d0, 6.8d0, 6.83d0, 6.07d0
     1            , 4.13d0, 3.47d0, 3.20d0, 3.06d0, 2.75d0, 2.42d0
     2            , 4.33d0, 5.0d0, 5.23d0, 5.1d0, 4.4d0, 3.09d0
     3            , 1.35d0, 8.03d-1, 6.30d-1, 2.16d-1, 1.26d-1
     4            , 5.87d-2, 3.07d-2, 1.07d-2, 1.09d-2, 7.37d-3
     5            , 2.38d-3, 3.80d-3, 5.67d-3, 1.02d-2, 1.36d-2
     6            , 1.59d-2, 1.50d-2, 1.40d-2, 1.05d-2, 4.53d-3
     7            , 1.87d-3, 4.63d-4, 1.87d-4, 4.40d-5, 1.40d-5
     8            , 7.13d-6 /

c  abondances initiales

c  abondances initiales des especes chimiques et
c  abondances initiales des niveaux de rotation de H2 et CO
c  dans la premiere couche:

       temp = tgaz(1)

       do j=1,nspec
         abnua(j,0) = abin(j) * densh(0)
       end do

       h2para = 0.25d0
       h2orth = 0.75d0
       xreh2(1,0) = 0.25d0
       xreh2(2,0) = 0.75d0
       do lev=3,l0h2
         xj = njlev(lev)
         xreh2(lev,0) = dix**(-xj)
       end do
       cdth2 = zero
       cdth2_o = zero

c HD
       if (i_hd .ne. 0) then
         hdpara = 0.25d0
         hdorth = 0.75d0
         xrehd(1,0) = 0.25d0
         xrehd(2,0) = 0.75d0
         do lev=3,l0hd
           xj = njhd(lev)
           xrehd(lev,0) = dix**(-xj)
         end do
         cdthd = zero
         cdthd_o = zero
       endif

c carbone atomique

       emicat(:,0) = zero
       opdcat(:,0) = zero

       do i=1,nlvcat
         tti(i) = gstcat(i) * exp(-elkcat(i) / max(temp,500.0d0))
       end do
       do i=1,nlvcat
         xrecat(i,0) = tti(i) / SUM(tti(1:nlvcat))
       end do

c azote atomique

       eminat(:,0) = zero
       opdnat(:,0) = zero

       do i=1,nlvnat
         tti(i) = gstnat(i) * exp(-elknat(i) / max(temp,500.0d0))
       end do
       do i=1,nlvnat
         xrenat(i,0) = tti(i) / SUM(tti(1:nlvnat))
       end do

c oxygene atomique

       emioat(:,0) = zero
       opdoat(:,0) = zero

       do i=1,nlvoat
         tti(i) = gstoat(i) * exp(-elkoat(i) / max(temp,500.0d0))
       end do
       do i=1,nlvoat
         xreoat(i,0) = tti(i) / SUM(tti(1:nlvoat))
       end do

c souffre atomique

       emisat(:,0) = zero
       opdsat(:,0) = zero

       do i=1,nlvsat
         tti(i) = gstsat(i) * exp(-elksat(i) / max(temp,500.0d0))
       end do
       do i=1,nlvsat
         xresat(i,0) = tti(i) / SUM(tti(1:nlvsat))
       end do

c silicium atomique

       emisiat(:,0) = zero
       opdsiat(:,0) = zero

       do i=1,nlvsiat
         tti(i) = gstsiat(i) * exp(-elksiat(i) / max(temp,500.0d0))
       end do
       do i=1,nlvsiat
         xresiat(i,0) = tti(i) / SUM(tti(1:nlvsiat))
       end do

c carbone ionise

       emicpl(:,0) = zero
       opdcpl(:,0) = zero

       do i=1,nlvcpl
         tti(i) = gstcpl(i) * exp(-elkcpl(i) / max(temp,500.0d0))
       end do
       do i=1,nlvcpl
         xrecpl(i,0) = tti(i) / SUM(tti(1:nlvcpl))
       end do

c azote ionise

       eminpl(:,0) = zero
       opdnpl(:,0) = zero

       do i=1,nlvnpl
         tti(i) = gstnpl(i) * exp(-elknpl(i) / max(temp,500.0d0))
       end do
       do i=1,nlvnpl
         xrenpl(i,0) = tti(i) / SUM(tti(1:nlvnpl))
       end do

c oxygene ionise

       emiopl(:,0) = zero
       opdopl(:,0) = zero

       do i=1,nlvopl
         tti(i) = gstopl(i) * exp(-elkopl(i) / max(temp,500.0d0))
       end do
       do i=1,nlvopl
         xreopl(i,0) = tti(i) / SUM(tti(1:nlvopl))
       end do

c ionised sulfur

       emispl(:,0) = zero
       opdspl(:,0) = zero

       do i=1,nlvspl
         tti(i) = gstspl(i) * exp(-elkspl(i) / max(temp,500.0d0))
       end do
       do i=1,nlvspl
         xrespl(i,0) = tti(i) / SUM(tti(1:nlvspl))
       end do

c silicium ionise

       emisip(:,0) = zero
       opdsip(:,0) = zero

       do i=1,nlvsip
         tti(i) = gstsip(i) * exp(-elksip(i) / max(temp,500.0d0))
       end do
       do i=1,nlvsip
         xresip(i,0) = tti(i) / SUM(tti(1:nlvsip))
       end do

c  CO:

       fnpart = zero
       do j=1,nlvco0
         ttetl(j) = gstco0(j) * exp(-elkco0(j) / temp)
         fnpart = fnpart + ttetl(j)
       end do
       do j=1,nlvco0
         xreco0(j,0) = ttetl(j) / fnpart
       end do
       cdtc0 = zero
       cdtc0_o = zero
       do j=1,ntrco0
         emico0(j,0) = zero
         opdco0(j,0) = zero
       end do

c  C*O:

       fnpart = zero
       do j=1,nlvco1
         ttetl(j) = gstco1(j) * exp(-elkco1(j) / temp)
         fnpart = fnpart + ttetl(j)
       end do
       do j=1,nlvco1
         xreco1(j,0) = ttetl(j) / fnpart
       end do
       cdtc1 = zero
       cdtc1_o = zero
       do j=1,ntrco1
         emico1(j,0) = zero
         opdco1(j,0) = zero
       end do

c  CO*:

       fnpart = zero
       do j=1,nlvco2
         ttetl(j) = gstco2(j) * exp(-elkco2(j) / temp)
         fnpart = fnpart + ttetl(j)
       end do
       do j=1,nlvco2
         xreco2(j,0) = ttetl(j) / fnpart
       end do
       cdtc2 = zero
       cdtc2_o = zero
       do j=1,ntrco2
         emico2(j,0) = zero
         opdco2(j,0) = zero
       end do

c  C*O*:

       fnpart = zero
       do j=1,nlvco3
         ttetl(j) = gstco3(j) * exp(-elkco3(j) / temp)
         fnpart = fnpart + ttetl(j)
       end do
       do j=1,nlvco3
         xreco3(j,0) = ttetl(j) / fnpart
       end do
       cdtc3 = zero
       cdtc3_o = zero
       do j=1,ntrco3
         emico3(j,0) = zero
         opdco3(j,0) = zero
       end do

c  CS:

       fnpart = zero
       do j=1,nlvcs0
         ttetl(j) = gstcs0(j) * exp(-elkcs0(j) / temp)
         fnpart = fnpart + ttetl(j)
       end do
       do j=1,nlvcs0
         xrecs0(j,0) = ttetl(j) / fnpart
       end do
       do j=1,ntrcs0
         emics0(j,0) = zero
         opdcs0(j,0) = zero
       end do

c HCO+: -----------------------------------------------------------
c On calcule le fonction de partition
c on entre les valeurs initiales des pop dans les niveaux, des
c emissivites initiales et des proba d'absorption(?) aux points 1
c et 0 du nuage

       fnpart = zero
       do  j=1,nlvhcop
         ttetl(j) = gsthcop(j) * exp(-elkhcop(j) / temp)
         fnpart = fnpart + ttetl(j)
       enddo
       do  j=1,nlvhcop
         xrehcop(j,0) = ttetl(j) / fnpart
       enddo
       do  j=1,ntrhcop
         emihcop(j,0) = zero
         opdhcop(j,0) = zero
       enddo

c  champ de Mathis:

c      nwlgin = 45
c      do iwlg=1,nwlgin
c        wlang(iwlg) = wlnMMP(iwlg)
c        ray(iwlg) = rayMMP(iwlg) * 1.0d-17
c      end do

       return
       end

c      %%%%%%%%%%%%%%%
       subroutine ETIN
c      %%%%%%%%%%%%%%%

c  Resolution de l'equation de transfert par devoloppement sur une
c             base de fonction de Legendre.

c  Calcul d apres B.P. Flannery, W. Roberge, G.B. Rybicki
c      Ap J, 236, 598-608, (1980)
c      Attention: faute de signe dans les formules (17), (21) et (22) !

c  Extension au cas non symetrique - JLB 17 IV 2002
c  Extension a un albedo variable  - JLB 1 VIII 2003

       use constantes
       use auxiliaries
       use inoutput
       use RF_TOOLS
       USE STR_DATA
       USE PROFIL
       use transfer

       implicit double precision (a-h,o-z)

       real (kind=dp), dimension(:), allocatable   :: hl, wr, wi, sigl
       real (kind=dp), dimension(:,:,:), allocatable :: rlm
       real (kind=dp), dimension(:,:), allocatable :: aam, bbm
       real (kind=dp), dimension(:,:), allocatable :: bim
       real (kind=dp), dimension(:,:), allocatable :: pli
       integer, dimension(:), allocatable          :: indx
       real (kind=dp), allocatable, dimension(:)   :: xmunl          ! -1 puissance l
       real (kind=dp), allocatable, dimension (:)  :: amp            ! Legendre amplitudes

       integer                                     :: n
       real (kind=dp)                              :: coef2

c Parametres de la routine LAPACK dstevd :

       CHARACTER (LEN=1)                           :: JOBZ  = 'N'    ! Eigenvalues only
       integer                                     :: LWORK
       integer                                     :: LIWORK

c Parametres de la routine LAPACK dgesvx :

       CHARACTER (LEN=1)                           :: FACT  = 'E'    ! Equilibrate system
       CHARACTER (LEN=1)                           :: TRANS = 'N'    ! Do not transpose
       integer                                     :: nrhs = 1
       CHARACTER (LEN=1)                           :: EQUED          ! Kind of equilibration
       real (kind=dp), dimension(:), allocatable   :: R, C           ! Equilibration factors
       real (kind=dp), dimension(:), allocatable   :: FERR, BERR     ! Errors (estimates)
       real (kind=dp), dimension(:), allocatable   :: WORK           ! Real work array
       integer, dimension(:), allocatable          :: IWORK          ! Integer work array
       real (kind=dp)                              :: RCOND          ! Condition number
       integer                                     :: info
       real (kind=dp), dimension(:,:), allocatable :: aamf, bbmf
       real (kind=dp)                              :: opdesc = 1.0e99_dp

c  Used for computing effective equivalent intensity

       type(rf_type) :: rf_tmp, rf_tmp1
       real (kind=dp), allocatable, dimension (:)   :: xkm         ! Legendre absissa
       real (kind=dp), allocatable, dimension (:)   :: o_xkm       ! "old" Legendre absissa
c integrated eigenvalue of transfer equation
       real (kind=dp), allocatable, dimension (:,:) :: xkmI
       real (kind=dp), allocatable, dimension (:,:) :: atmkm
       real (kind=dp), allocatable, dimension (:,:) :: atmkp

       real (kind=dp), dimension(:), allocatable  :: p_0
       real (kind=dp), dimension(:), allocatable  :: q_0

       real (kind=dp)                             :: aux1, aux2
       real (kind=dp)                             :: aux3, aux4

c  Actuellement, les zeros des polynomes de Legendre ne sont
c     inclus que jusqu'a L = 19 => M = 10

c---------------------------------------------------------------------------

       coef2 = tautav * cdunit / rrr
       nl = lleg + 1
       mm = nl / 2

       allocate (hl(0:lleg))
       allocate (p_0(0:lleg))
       allocate (q_0(0:lleg))
       allocate (rlm(0:lleg,-mleg:mleg,0:npo))
       allocate (sigl(0:lleg), pli(0:lleg,-mleg:mleg))
       allocate (bim(-mleg:mleg,-mleg:mleg))
       allocate (aam(lleg+1,lleg+1), bbm(lleg+1,1))
       allocate (aamf(lleg+1,lleg+1), bbmf(lleg+1,1))
       allocate (R(lleg+1), C(lleg+1))
       allocate (FERR(1), BERR(1))
       LWORK = 4*lleg+4
       allocate (WORK(LWORK))
       LIWORK = lleg+1
       allocate (IWORK(LIWORK))
       allocate (wr(lleg+1), wi(lleg+1))
       allocate (indx(lleg+1))
       allocate (xkm(-mleg:mleg))
       allocate (o_xkm(-mleg:mleg))
       allocate (xmunl(0:lleg))
       allocate (amp(-mleg:mleg))
       allocate (xkmI(mleg,0:npo))
       allocate (atmkm(mleg,0:npo))
       allocate (atmkp(mleg,0:npo))

       if ( ALLOCATED(g_absI)) then
         deallocate (g_absI)
         deallocate (g_absL)
         deallocate (abspm, abspp)
         deallocate (bbspm, bbspp)
       endif
       allocate (g_absI(0:npo,0:nwlg))
       allocate (g_absL(0:npo,0:nwlg))
       allocate (abspm(0:npo,0:nwlg))
       allocate (abspp(0:npo,0:nwlg))
       allocate (bbspm(0:npo,0:nwlg))
       allocate (bbspp(0:npo,0:nwlg))

c   Calcul des polynomes de Legendre aux points -mui et +mui

       pli = 0.0_DP
       do i=1,mm
c  P0 et P1: Num Rec p246
         pli(0,-i) = un
         pli(0,i) = un
         ymui = xmui(mm,i)
         pli(1,-i) = -ymui
         pli(1,i) = ymui
         do l=1,lleg-1
c  Recurence: Num Rec p 172
           pli(l+1,-i) = (-ymui*(deux*l+un) * pli(l,-i)
     1              - l * pli(l-1,-i)) / (l+un)
           pli(l+1,i) = (ymui*(deux*l+un) * pli(l,i)
     1              - l * pli(l-1,i)) / (l+un)
         end do
       end do

       p_0 = 0.0_dp
       q_0 = 0.0_dp
       p_0(0) = 1.0_dp
       q_0(1) = p_0(0) / 2.0_dp
       do l=1,lleg/2
         p_0(2*l) = - (2.0_dp*l-1) * p_0(2*l-2) / (2.0_dp*l)
         q_0(2*l+1) = p_0(2*l) / (2.0_dp*l+2.0_dp)
       end do

c      do l=0,lleg
c        do m=-mm,mm
c          print *, "  pli:", l, m, pli(l,m)
c        end do
c      end do

c  uvspec.out is the total absorption through the cloud. multiply by
c             incident radition field to compute spectrum.

       fichier = TRIM(out_dir)//TRIM(modele)//'.uv'
       open (37, file=fichier, status="unknown")
       fichier = TRIM(out_dir)//TRIM(modele)//'.flin'
       open (60, file=fichier, status="unknown")

c  Initialise illuminating star parameters, if any...
c  geom is (1 - cos(theta_0)). A second order development
c       is adequate in most cases.

       if (d_sour /= 0.0_DP) then
         if (srcpp(1:2) == 'F_') then
           nbpt = 1
           wlo = wlth0
           wl = rf_wl%Val(wl_pointer(rf_exter%Ord(2)))
         else
           call STAR (srcpp, teff, rstar)
         endif
         geom = 0.5_dp * (rstar/(d_sour*pccm))**2
       endif

c  This is just (-1)**l

       xmunl(0) = un
       do l=1,lleg
         xmunl(l) = - xmunl(l-1)
       end do

c  sigl = gg**l (Formule 48, Henyey-Greenstein phase function)
       sigl = 0.0_DP
       sigl(0) = un
       do l=1,lleg
         sigl(l) = sigl(l-1) * gg
       end do

c Compute absorption by gas

       g_absI = 0.0_DP
       call ABSGAZ

c Then solve for each wavelength

       do iwlg=0,nwlg

         amp = 0.0_DP
         xkmI = 0.0_dp
         atmkm = 0.0_dp
         atmkp = 0.0_dp
         hl = 0.0_DP
         rlm = 0.0_DP
         bim = 0.0_DP
         aam = 0.0_DP
         aamf = 0.0_DP
         bbm = 0.0_DP
         bbmf = 0.0_DP
         wr = 0.0_DP
         wi = 0.0_DP
         indx = 0

         xkm = 0.0_DP
         o_xkm = 0.0_DP

         dust_ext = rf_abspou%Val(iwlg) / coef2

c  loop over optical depth

         do n=0,npo

           albed = alb / (1.0_dp
     1           + g_absL(n,iwlg) / (dust_ext * dens_o(n)))

c  construction des hl:

c  hl: Formule A2
           hl(0) = un - albed
           do l=1,lleg
             hl(l) = (deux*l+un) * (un - albed * sigl(l))
           end do

c  Formule A3:
           do i=1,lleg
             wi(i) = i / sqrt(hl(i-1)*hl(i))
           end do

c  Compute eigenvalues (tridiagonal symetric matrix)
c  20 Aug 2003 - replace HQR by LAPACK routine

           wr = 0.0_DP
           call DSTEVD(JOBZ, nl, wr, wi, aam, nl, WORK, LWORK, IWORK,
     1                 LIWORK, info )

c  Rearangement des valeurs propres par taille decroissante.

           wi = wr
           do i=1,nl
             wr(i) = wi(nl+1-i)
           end do

c  On obtient les km = 1 / lamdam par valeurs croissantes
           do m=1,mm
             xkm(m) = un / wr(m)
             xkm(-m) = -xkm(m)
           end do

c          do m=-mm,mm
c            print *, "xkm:", m, xkm(m)
c          end do

c  Calcul des Rlm

           do m=1,mm
c  Formule A9 de Roberge 1983
             rlm(0,m,n) = un
             rlm(1,m,n) = (un - albed) / xkm(m)
             rlm(0,-m,n) = un
             rlm(1,-m,n) = -(un - albed) / xkm(m)
             do l=2,lleg
               rlm(l,m,n) = (hl(l-1) * rlm(l-1,m,n)
     1                    - (l-1) * xkm(m) * rlm(l-2,m,n))
     2                    / (l*xkm(m))
               rlm(l,-m,n) = xmunl(l) * rlm(l,m,n)
             end do
           end do

c          do l=0,lleg
c            do m=-mm,mm
c              print *, "  rlm:", l, m, rlm(l,m,n)
c            end do
c          end do

c Integrate xkm. Simple Simpson rule.

           if (n /= 0) then
             do m=1,mm
               xkmI(m,n) = xkmI(m,n-1)
     1              + 0.5_dp * (tau_o(n) - tau_o(n-1))
     2              * (coef2 * (xkm(m) * g_absL(n,iwlg) / dens_o(n)
     3              + o_xkm(m) * g_absL(n-1,iwlg) / dens_o(n-1))
     4              + rf_abspou%Val(iwlg) * (xkm(m) + o_xkm(m)))
             end do
           endif

           o_xkm = xkm

         end do

         do n=0,npo
           do m=1,mm
             atmkm(m,n) = exp(-xkmI(m,n))
             atmkp(m,n) = exp(-(xkmI(m,npo)-xkmI(m,n)+opdinf))
           end do
         end do

c  Calcul des Am (conditions aux limites)

c  Compute total absorption by dust + gas through whole cloud

         x2 = taumax * rf_abspou%Val(iwlg) + g_absI(npo,iwlg)
         write (37,"(1p,3e17.7E3)") rf_wl%Val(iwlg), exp(-x2), x2

c  Construction des Bim
c  Pour eviter les exponentielles positive, le systeme est reformule
c       suivant Roberge, Ap J 275, 292 (1983)
c       Formules (19) et (23)

         do m=1,mm
           emkmi0 = atmkm(m,npo)
           emkmiT = exp(-(xkmI(m,npo)+opdinf))
           do i=1,mm
             aux1 = 0.0d0
             aux2 = 0.0d0
             aux3 = 0.0d0
             aux4 = 0.0d0
             do l=0,lleg
               aux1 = aux1 + (deux*l+un) * rlm(l,-m,0) * pli(l,-i)
               aux2 = aux2 + (deux*l+un) * rlm(l,m,0) * pli(l,-i)
               aux3 = aux3 + (deux*l+un) * rlm(l,-m,npo) * pli(l,i)
               aux4 = aux4 + (deux*l+un) * rlm(l,m,npo) * pli(l,i)
             end do
             bim(-i,-m) = aux1
             bim(-i,m) = aux2 * emkmi0
             bim(i,-m) = aux3 * emkmiT
             bim(i,m) = aux4
           end do
         end do

c        do i=-mm,mm
c          do m=-mm,mm
c            print *, " bim:", i, m, bim(i,m)
c          end do
c        end do

c  Formule 39 (...et utilisation de LAPACK)

         aam = 0.0_DP
         aamf = 0.0_DP
         bbm = 0.0_DP
         bbmf = 0.0_DP

         aam(1:mm,1:mm) = bim(-mm:-1,-mm:-1)
         do m=1,mm
           do i=1,mm
             aam(i,mm+m) = bim(-mm-1+i,m)
             aam(mm+i,m) = bim(i,-mm-1+m)
           end do
         end do
         aam(mm+1:nl,mm+1:nl) = bim(1:mm,1:mm)

c        do i=1,nl
c          do m=1,nl
c            print *, "  aam:", i, m, aam(i,m)
c          end do
c        end do

c  Le flux de Draine est pris comme reference

         bbm(1:mm,1) = radm0
         bbm(mm+1:2*mm,1) = radp0

c  On ajoute la contribution eventuelle d'une source plan parallele
c  Attention a la conversion d'unite pour lambda : 4 fois en cm et 1 fois en Angstrom...
c  B_lambda is converted to an energy density (factor 4 pi / c)
c  before division by the unscaled isotropic field.

         if (d_sour < 0.0_DP) then

           wlth = rf_wl%Val(iwlg)

           if (srcpp(1:2) == 'F_') then

             if (wlth >
     1           rf_wl%Val(wl_pointer(rf_exter%Ord(nbpt+1)))) then
               nbpt = nbpt + 1
               wlo = wl
               wl = rf_wl%Val(wl_pointer(rf_exter%Ord(nbpt+1)))
             endif
             if (wlo == wlth) then
               bbstar = rf_exter%Val(nbpt)
             else if (wl == wlth) then
               bbstar = rf_exter%Val(nbpt+1)
             else
               bbstar = rf_exter%Val(nbpt)
     1                + (rf_exter%Val(nbpt+1)-rf_exter%Val(nbpt))
     2                * (wlth - wlo) / (wl - wlo)
             endif
           else
             wlth = wlth * 1.0e-8_DP
             bbstar = 1.0e-8_DP*8.0_DP*xpi*xhp*clum / wlth**5
             bbstar = bbstar / (exp(hcsurk/(teff*wlth)) - 1.0_DP)
           endif

           f_inc = (bbstar / rf_draypr%Val(iwlg)) - 1.0_dp

           fluxin = 2.0_dp * geom * f_inc
     2            / (2.0_dp - xmui(mm,mm) - xmui(mm,mm-1))

           bbm(1,1) = radm0 + fluxin

           write (60, '(1p,3e15.6)') wlth, bbstar, f_inc

         else if (d_sour > 0.0_DP) then

           wlth = rf_wl%Val(iwlg) * 1.0e-8_DP

           bbstar = 1.0e-8_DP*8.0_DP*xpi*xhp*clum / wlth**5
           bbstar = bbstar / (exp(hcsurk/(teff*wlth)) - 1.0_DP)
           f_inc = (bbstar / rf_draypr%Val(iwlg)) - 1.0_dp

           fluxin = 2.0_dp * geom * f_inc
     2            / (2.0_dp - xmui(mm,mm) - xmui(mm,mm-1))


           bbm(2*mm,1) = radp0 + fluxin

         endif

         call dgesvx (FACT, TRANS, nl, nrhs, aam, nl, aamf, nl, indx,
     1                EQUED, R, C,  bbm, nl, bbmf, nl, RCOND, FERR,
     2                BERR, WORK, IWORK, info)

         amp(-mm:-1) = bbmf(1:mm,1)
         amp(1:mm) = bbmf(mm+1:nl,1)

c  Compute radiation field energy density attenuation here and not in stv2a.f
c  Do we compute correctly over half-space? YES !

         do n=0,npo
           aux1 = 0.0_dp
           aux2 = 0.0_dp
           do m=1,mm
             aux3 = 0.0_dp
             do l=lleg/2,0,-1
               ll = 2 * l + 1
               aux3 = aux3 + (2*ll+1) * q_0(ll) * rlm(ll,m,n)
             end do
             aux1 = aux1
     1             + amp(-m) * atmkm(m,n) * (1.0_dp + aux3)
     2             + amp(m)  * atmkp(m,n) * (1.0_dp - aux3)
             aux2 = aux2
     1             + amp(-m) * atmkm(m,n) * (1.0_dp - aux3)
     2             + amp(m)  * atmkp(m,n) * (1.0_dp + aux3)
           end do
           abspm(n,iwlg) = 0.5_dp * aux1
           abspp(n,iwlg) = 0.5_dp * aux2
         end do

c  Compute escape probabilities here

         do n=0,npo

           if (isoneside == 0) then
             aam = 0.0_DP
             bim = 0.0_dp
             do m=1,mm
               do i=1,mm
                 aux1 = 0.0d0
                 do l=0,lleg
                   aux1 = aux1 + (deux*l+un) * rlm(l,-m,n) * pli(l,-i)
                 end do
                 bim(-i,-m) = aux1
               end do
             end do

             bbm = 0.0_DP
             aamf = 0.0_DP
             bbmf = 0.0_DP

             bbm(1:mm,1) = 1.0_dp
             aam(1:mm,1:mm) = bim(-mm:-1,-mm:-1)

             call dgesvx (FACT, TRANS, mm, nrhs, aam, nl, aamf, nl,
     1                    indx, EQUED, R, C,  bbm, nl, bbmf, nl, RCOND,
     2                    FERR, BERR, WORK, IWORK, info)

             aux1 = 0.0_dp
             do m=mleg,1,-1
               aux1 = aux1 + bbmf(m,1) * atmkp(m,n)
             end do
             bbspp(n,iwlg) = aux1
           else
             bbspp(n,iwlg) = 0.0_dp
           endif

           aam = 0.0_DP
           do m=1,mm
             do i=1,mm
               aux1 = 0.0d0
               do l=0,lleg
                 aux1 = aux1 + (deux*l+un) * rlm(l,m,n) * pli(l,i)
               end do
               aam(i,m) = aux1
             end do
           end do

c          aamf = 0.0_DP
c          bbm = 0.0_DP
c          bbmf = 0.0_DP

           bbm(1:mm,1)      = 1.0_dp

           call dgesvx (FACT, TRANS, mm, nrhs, aam, nl, aamf, nl, indx,
     1                  EQUED, R, C,  bbm, nl, bbmf, nl, RCOND, FERR,
     2                  BERR, WORK, IWORK, info)

           aux1 = 0.0_dp
           do m=mleg,1,-1
             aux1 = aux1 + bbmf(m,1) * atmkm(m,n)
           end do
           bbspm(n,iwlg) = aux1

         end do

         enor_m = bbspm(0,iwlg)
         enor_p = bbspp(npo,iwlg)
         if (enor_p == 0.0_dp) then
           enor_p = 1.0_dp
         endif

         do n=0,npo
           bbspm(n,iwlg) = bbspm(n,iwlg) / enor_m
           bbspp(n,iwlg) = bbspp(n,iwlg) / enor_p
c          wlth = rf_wl%Val(iwlg) * 1.0e-8_DP
c          write (49,'(1p,e13.4E3,e15.6E3,2e13.4E3)') tau_o(n), wlth
c    1                            , bbspm(n,iwlg), bbspp(n,iwlg)
         end do
c        write (49,'(" ")')

       end do                             ! loop over wavelength

       close (37)
       close (60)

       deallocate (hl, xmunl)
       deallocate (p_0, q_0)
       deallocate (rlm)
       deallocate (sigl, pli)
       deallocate (bim)
       deallocate (aam, bbm)
       deallocate (aamf, bbmf)
       deallocate (wr, wi)
       deallocate (indx)
       deallocate (FERR, BERR)
       deallocate (R, C)
       deallocate (WORK)
       deallocate (IWORK)
       deallocate (xkm, xkmI, amp)
       deallocate (o_xkm)
       deallocate (atmkm, atmkp)

c 20 V 2003 - Compute here the effective radiation field scaling

       call rf_create(rf_tmp,1,2)
       call rf_locate(rf_wl, wlth0, i1)
       rf_tmp%Val(1) = 1.0_dp
       rf_tmp%Ord(1) = rf_wl%Ord(i1)
       print *, " "
       print *, " Standard Draine field in free space:"

       call rf_create(rf_tmp1,0,nwlg)
       rf_tmp1%Val = rf_wl%Val * rf_draypr%Val

c A - cut-off at 6 eV:

       call rf_locate(rf_wl, wl_6ev, i2)
       rf_tmp%Val(2) = 1.0_dp
       rf_tmp%Ord(2) = rf_wl%Ord(i2)
c      print *, rf_tmp%Ord(1), rf_tmp%Ord(2)
       int_rad0 = sect_int(rf_tmp, rf_draypr, wl_pointer, wl_6ev)
       flu_rad0 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_6ev)
     1          / (1.0e8_dp * xhp)

       write (iscrn,'("  U0:",1pe12.4," erg cm-3,     "
     1               ,"from 912 to",0pf9.2," Angstrom")')
     1      int_rad0, wl_6ev
       write (iscrn,'("  F0:",1pe12.4," phot cm-2 s-1,"
     1               ,"from 912 to",0pf9.2," Angstrom")')
     1      flu_rad0, wl_6ev
       write (iwrit2,'("  U0:",1pe12.4," erg cm-3,     "
     1               ,"from 912 to",0pf9.2," Angstrom")')
     1      int_rad0, wl_6ev
       write (iwrit2,'("  F0:",1pe12.4," phot cm-2 s-1,"
     1               ,"from 912 to",0pf9.2," Angstrom")')
     1      flu_rad0, wl_6ev

c B - cut-off at 2400 A:

       call rf_locate(rf_wl, wl_hab, i2)
       rf_tmp%Val(2) = 1.0_dp
       rf_tmp%Ord(2) = rf_wl%Ord(i2)
c      print *, rf_tmp%Ord(1), rf_tmp%Ord(2)
       int_rad0 = sect_int(rf_tmp, rf_draypr, wl_pointer, wl_hab)
       flu_rad0 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_hab)
     1          / (1.0e8_dp * xhp)

       write (iscrn,'("  U0:",1pe12.4," erg cm-3,     "
     1               ,"from 912 to",0pf9.2," Angstrom")')
     1      int_rad0, wl_hab
       write (iscrn,'("  F0:",1pe12.4," phot cm-2 s-1,"
     1               ,"from 912 to",0pf9.2," Angstrom")')
     1      flu_rad0, wl_hab
       write (iwrit2,'("  U0:",1pe12.4," erg cm-3,     "
     1               ,"from 912 to",0pf9.2," Angstrom")')
     1      int_rad0, wl_hab
       write (iwrit2,'("  F0:",1pe12.4," phot cm-2 s-1,"
     1               ,"from 912 to",0pf9.2," Angstrom")')
     1      flu_rad0, wl_hab
       print *, " "
       print *, " Scaled Draine field at cloud edge"

       int_rad1 = 0.0_dp
       int_rad2 = 0.0_dp
       if (isoneside == 1) then
         do iwlg=0,nwlg
           rf_dray%Val(iwlg) = rf_draypr%Val(iwlg)
     1                       * (abspm(0,iwlg) + abspp(0,iwlg))
         end do
         rf_tmp1%Val = rf_wl%Val * rf_dray%Val

         call rf_locate(rf_wl, wl_cut_C, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad1 = sect_int(rf_tmp, rf_dray, wl_pointer, wl_cut_C)
         flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_cut_C)
     1          / (1.0e8_dp * xhp)
         write (iscrn,'("  U(C):",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad1, wl_cut_C
         write (iscrn,'("  F(C):",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad1, wl_cut_C
         write (iwrit2,'("  U(C):",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad1, wl_cut_C
         write (iwrit2,'("  F(C):",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad1, wl_cut_C

         call rf_locate(rf_wl, wl_6ev, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad1 = sect_int(rf_tmp, rf_dray, wl_pointer, wl_6ev)
         flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_6ev)
     1          / (1.0e8_dp * xhp)
         write (iscrn,'("  U1:",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad1, wl_6ev
         write (iscrn,'("  F1:",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad1, wl_6ev
         write (iwrit2,'("  U1:",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad1, wl_6ev
         write (iwrit2,'("  F1:",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad1, wl_6ev

         call rf_locate(rf_wl, wl_hab, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad1 = sect_int(rf_tmp, rf_dray, wl_pointer, wl_hab)
         flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_hab)
     1          / (1.0e8_dp * xhp)
         write (iscrn,'("  U1:",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad1, wl_hab
         write (iscrn,'("  F1:",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad1, wl_hab
         write (iwrit2,'("  U1:",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad1, wl_hab
         write (iwrit2,'("  F1:",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad1, wl_hab

         radm = int_rad1 / int_rad0
         rfield(0,:) = rf_dray%Val(:)
       else
         do iwlg=0,nwlg
           rf_dray%Val(iwlg) = rf_draypr%Val(iwlg)
     1                       * (abspm(npo,iwlg) + abspp(npo,iwlg))
         end do
         rf_tmp1%Val = rf_wl%Val * rf_dray%Val

         call rf_locate(rf_wl, wl_6ev, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad2 = sect_int(rf_tmp, rf_dray, wl_pointer, wl_6ev)
         flu_rad2 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_6ev)
     1          / (1.0e8_dp * xhp)
         write (iscrn,'("  U2:",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad2, wl_6ev
         write (iscrn,'("  F2:",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad2, wl_6ev
         write (iwrit2,'("  U2:",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad2, wl_6ev
         write (iwrit2,'("  F2:",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad2, wl_6ev

         call rf_locate(rf_wl, wl_hab, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad2 = sect_int(rf_tmp, rf_dray, wl_pointer, wl_hab)
         flu_rad2 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_hab)
     1          / (1.0e8_dp * xhp)
         write (iscrn,'("  U2:",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad2, wl_hab
         write (iscrn,'("  F2:",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad2, wl_hab
         write (iwrit2,'("  U2:",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad2, wl_hab
         write (iwrit2,'("  F2:",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad2, wl_hab

         radp = int_rad2 / int_rad0
         rfield(n_imp,:) = rf_dray%Val(:)

         do iwlg=0,nwlg
           rf_dray%Val(iwlg) = rf_draypr%Val(iwlg)
     1                       * (abspm(0,iwlg) + abspp(0,iwlg))
         end do
         rf_tmp1%Val = rf_wl%Val * rf_dray%Val

         call rf_locate(rf_wl, wl_6ev, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad1 = sect_int(rf_tmp, rf_dray, wl_pointer, wl_6ev)
         flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_6ev)
     1          / (1.0e8_dp * xhp)
         write (iscrn,'("  U1:",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad1, wl_6ev
         write (iscrn,'("  F1:",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad1, wl_6ev
         write (iwrit2,'("  U1:",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad1, wl_6ev
         write (iwrit2,'("  F1:",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad1, wl_6ev

         call rf_locate(rf_wl, wl_hab, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad1 = sect_int(rf_tmp, rf_dray, wl_pointer, wl_hab)
         flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_hab)
     1          / (1.0e8_dp * xhp)
         write (iscrn,'("  U1:",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad1, wl_hab
         write (iscrn,'("  F1:",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad1, wl_hab
         write (iwrit2,'("  U1:",1pe12.4," erg cm-3,     "
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        int_rad1, wl_hab
         write (iwrit2,'("  F1:",1pe12.4," phot cm-2 s-1,"
     1                 ,"from 912 to",0pf9.2," Angstrom")')
     1        flu_rad1, wl_hab

         radm = int_rad1 / int_rad0
         rfield(0,:) = rf_dray%Val(:)
       endif

       print *, " "
       write (iscrn,'("  G_0m :",1pe12.4," (in Habing units)")')
     1        int_rad1/5.6e-14_dp
       write (iscrn,'("  G_0p :",1pe12.4," (in Habing units)")')
     1        int_rad2/5.6e-14_dp
       write(iwrit2,'(/," U0:",1pe13.4, " erg cm-3")') int_rad0
       write(iwrit2,'(" G_0m =", 1pe12.3, " in Habing units",/)')
     1                                       int_rad1/5.6e-14_dp
       write(iwrit2,'(" G_0p =", 1pe12.3, " in Habing units",/)')
     1                                       int_rad2/5.6e-14_dp

c set here wavelength threshold for integrated intensity.

       call rf_locate(rf_wl, wl_6ev, i2)
       rf_tmp%Val(2) = 1.0_dp
       rf_tmp%Ord(2) = rf_wl%Ord(i2)
       int_rad(0) = sect_int(rf_tmp, rf_dray, wl_pointer, wl_6ev)

       print *, " "
       print *, " Effective Radiation field strength AT the edge:"
       write (iscrn,'("  left :",1pe12.4," (Draine)")') radm
       write (iscrn,'(" right :",1pe12.4," (Draine)")') radp
       write (iwrit2,'("  left :",1pe12.4," (Draine)")') radm
       write (iwrit2,'(" right :",1pe12.4," (Draine)")') radp
       print *, " "

       call rf_free (rf_tmp)
       call rf_free (rf_tmp1)

       return
       end

c-------------------------------------------------------------------

      subroutine STAR (srcpp, teff, rstar)

!  Most data from "Allen's Astrophysical Quantities" 4th ed (A. Cox Ed.)
!  Sect 15.3.1 (p 388).

!  Some additional data from :
!  Effective temperature are from Lang, "Astrophysical Formulae",  p564
!  Some radii from Lang p 546 - Note inconsistencies between quoted temp...

! Note a better representation (continuous) can be derived from :
!      De Jager & Nieuwenhuijzen, A & A 177, 217 (1987)

!  Specific stars are easily added... ;-)

        USE constantes

        implicit none

        CHARACTER (LEN=35), INTENT(in) :: srcpp       ! Type of star
        REAL (KIND=DP), INTENT(out)    :: teff         ! Effective Temperature (K)
        REAL (KIND=DP), INTENT(out)    :: rstar        ! Stellar radius (cm)

        select case (TRIM(ADJUSTL(srcpp)))
! a - Main sequence V
        case ("O 3", "O 3 V")
          teff = 50000.0_DP            ! Estimate
          rstar = 12.0_DP * rsun       ! Estimate
        case ("O 5", "O 5 V")
          teff = 42000.0_DP            ! Allen
          rstar = 12.0_DP * rsun       ! Allen
        case ("O 8", "O 8 V")
          teff = 37000.0_DP            ! Van den Ancker et al.
          rstar = 10.0_DP * rsun       ! ???
        case ("B 0", "B 0 V")
          teff = 30000.0_DP            ! Allen
          rstar = 7.4_DP * rsun        ! Allen
        case ("B 1", "B 1 V")
          teff = 24200.0_DP
          rstar = 7.2_DP * rsun        ! Lang p 546 (eta Orioni)
        case ("NGC7023")
          teff = 20000.0_DP
          rstar = 7.35_DP * rsun       ! ref: C. Joblin (march 2004)
        case ("B 3", "B 3 V")
          teff = 17000.0_DP
          rstar = 7.2_DP * rsun        ! From Mb = 4.3 (NGC7023) cf Lang p 565
        case ("B 5", "B 5 V")
          teff = 15200.0_DP            ! Allen
          rstar = 3.9_DP * rsun        ! Allen
        case ("B 8", "B 8 V")
          teff = 11400.0_DP            ! Allen
          rstar = 3.0_DP * rsun        ! Allen
        case ("B 9", "B 9 IV")         ! Estimation from De Jager A&A 177 (1987)
          teff = 11300.0_DP            ! and cf Lang
          rstar = 2.55_DP * rsun
        case ("A 0", "A 0 V")
          teff = 9790.0_DP             ! Allen
          rstar = 2.4_DP * rsun        ! Allen
        case ("A 1", "A 1 V")
          teff = 10200.0_DP
          rstar = 2.4_DP * rsun        ! Lang p 546 (Sirius)
        case ("A 5", "A 5 V")
          teff = 8180.0_DP             ! Allen
          rstar = 1.7_DP * rsun        ! Allen
        case ("G 2", "G 2 V")
          teff = 5780.0_DP
          rstar = rsun                 ! Sun
        case ("M 5", "M 5 V")
          teff = 5780.0_DP
          rstar = 0.5_DP * rsun        ! Lang p 546 (Barnard's star)
! b - Super Giants I
        case ("B 5 I")
          teff = 13600.0_DP            ! Allen
          rstar = 50.0_DP * rsun       ! Allen
        case ("A 0 I")
          teff = 9980.0_DP             ! Allen
          rstar = 60.0_DP * rsun       ! Allen
        case ("A 5 I")
          teff = 8610.0_DP             ! Allen
          rstar = 60.0_DP * rsun       ! Allen
        case DEFAULT

          teff = 24200.0_DP
          rstar = 7.2 * rsun           ! Lang p 546 (eta Orioni)
          print *, "  WARNING ! Unknown stellar type"
          print *, "            Change in subroutine STAR"
          print *, "            Using B 1"
          print *, "            teff =", teff, ", rstar =", rstar

        end select

        return

      end subroutine STAR

c-------------------------------------------------------------------

c      %%%%%%%%%%%%%%%%%
       subroutine ABSGAZ
c      %%%%%%%%%%%%%%%%%

c  Gaz absorption coefficient in lines
c  JLB - 22 V 2003

       use constantes
       use auxiliaries
       use RF_TOOLS
       USE STR_DATA
       USE PROFIL
       USE transfer

       implicit none

       integer                            :: n, i, j
       real (kind=dp)                     :: wl0, nu0, fosc, dvie
       real (kind=dp)                     :: nu
       real (kind=dp)                     :: dopl, coef1, coef2, aux
       real (kind=dp)                              :: avoi
       real (kind=dp), dimension(:), allocatable   :: xwl, yvoi
       integer                            :: nvl, njl

       ALLOCATE (xwl(0:nwlg), yvoi(0:nwlg))
       g_absI = 0.0_dp
       g_absL = 0.0_dp
       coef1 = sqrt(xpi) * clum * e2smc2
       coef2 = tautav * cdunit / rrr

c  Outer loop on position in cloud

       do n=0,npo

c  Middle loop on lines
c  A - Atomic hydrogen

         if (jlyman == 1) then
         do i=1,nraih
           wl0 = praih(i,2) * 1.0e-8_dp
           nu0 = clum / wl0
           fosc = praih(i,1)
           dvie = praih(i,3)

c  dopl is in frequency unit
           dopl = sqrt(vturb*vturb + 2.0_dp * xk * tgaz_o(n) / xmh)
     1          / wl0
           avoi = dvie / (4.0_dp * xpi * dopl)
           do j=0,nwlg
             nu = 1.0e8_dp * clum / rf_wl%Val(j)
             xwl(j) = (nu0 - nu) / dopl
           end do

           call humlik (nwlg+1, xwl, avoi, yvoi)

           aux = coef1 * fosc / dopl
           do j=0,nwlg
             g_absL(n,j) = g_absL(n,j) + yvoi(j) * aux
     1                   * abh_o(n)
           end do

         end do
         endif

c  B - Molecular hydrogen (Lyman band)

         do i=1,nrh2bt
           nvl = irah2b(i,1)
           njl = irah2b(i,2)
           if (nvl > 0 .OR. njl >= jfgkh2) then
             cycle
           endif
           wl0 = prah2b(i,2) * 1.0e-8_dp
           nu0 = clum / wl0
           fosc = prah2b(i,1)
           dvie = prah2b(i,3)

c  dopl is in frequency unit
           dopl = sqrt(vturb*vturb + 2.0_dp * xk * tgaz_o(n)
     1          / (2.0_dp*xmh)) / wl0
           avoi = dvie / (4.0_dp * xpi * dopl)
           do j=0,nwlg
             nu = 1.0e8_dp * clum / rf_wl%Val(j)
             xwl(j) = (nu0 - nu) / dopl
           end do

           call humlik (nwlg+1, xwl, avoi, yvoi)

           aux = coef1 * fosc / dopl
           do j=0,nwlg
             g_absL(n,j) = g_absL(n,j) + yvoi(j) * aux
     1                  * abh2_o(n) * xh2_o(njl,n)
           end do

         end do

c  C - Molecular hydrogen (Werner band)

         do i=1,nrh2ct
           nvl = irah2c(i,1)
           njl = irah2c(i,2)
           if (nvl > 0 .OR. njl >= jfgkh2) then
             cycle
           endif
           wl0 = prah2c(i,2) * 1.0e-8_dp
           nu0 = clum / wl0
           fosc = prah2c(i,1)
           dvie = prah2c(i,3)

c  dopl is in frequency unit
           dopl = sqrt(vturb*vturb + 2.0_dp * xk * tgaz_o(n)
     1          / (2.0_dp*xmh)) / wl0
           avoi = dvie / (4.0_dp * xpi * dopl)
           do j=0,nwlg
             nu = 1.0e8_dp * clum / rf_wl%Val(j)
             xwl(j) = (nu0 - nu) / dopl
           end do

           call humlik (nwlg+1, xwl, avoi, yvoi)

           aux = coef1 * fosc / dopl
           do j=0,nwlg
             g_absL(n,j) = g_absL(n,j) + yvoi(j) * aux
     1                  * abh2_o(n) * xh2_o(njl,n)
           end do

         end do
       end do

       do n=1,npo
         g_absI(n,:) = g_absI(n-1,:)
     1               + 0.5_dp * coef2 * (tau_o(n) - tau_o(n-1))
     2               * (g_absL(n,:) / dens_o(n)
     3               + g_absL(n-1,:) / dens_o(n-1))
       end do

       DEALLOCATE (xwl, yvoi)

       return
       end

c-------------------------------------------------------------------

       subroutine humlik (n, x, y, k)

       use constantes

       real (kind=dp) r0, r1
c      parameter (r0 = 146.7, r1 = 14.67)
c  rr must <= 5.0
c      parameter (r0 = 1.51*exp(1.144*rr), r1 = 1.60*exp(0.554*rr))
       parameter (r0 = 460.4_dp, r1 = 25.5_dp)

       integer n
       real (kind=dp) x(0:n-1)
       real (kind=dp) y
       real (kind=dp) k(0:n-1)

       real (kind=dp) rrtpi
       parameter (rrtpi = 0.564189583547756_dp)
       real (kind=dp) y0, y0py0, y0q
       parameter (y0 = 1.5_dp, y0py0 = y0+y0, y0q = y0*y0)
       real (kind=dp) c(0:5), s(0:5), t(0:5)
       save c, s, t
       data c / 1.0117281_dp, -0.75197147_dp, 0.012557727_dp,
     1          0.010022008_dp, -0.00024206814_dp,
     2          0.00000050084806_dp /
       data s / 1.393237_dp, 0.23115241_dp, -0.15535147_dp,
     1          0.0062183662_dp, 0.000091908299_dp,
     2          -0.00000062752596_dp /
       data t / 0.31424038_dp, 0.94778839_dp, 1.5976826_dp,
     1          2.2795071_dp, 3.0206370_dp, 3.8897249_dp /

       integer i, j
       integer rg1, rg2, rg3
       real (kind=dp) abx, xq, yq, yrrtpi
       real (kind=dp) xlim0, xlim1, xlim2, xlim3, xlim4
       real (kind=dp) a0, d0, d2, e0, e2, e4, h0, h2, h4, h6
       real (kind=dp) p0, p2, p4, p6, p8, z0, z2, z4, z6, z8
       real (kind=dp) xp(0:5), xm(0:5), yp(0:5), ym(0:5)
       real (kind=dp) mq(0:5), pq(0:5), mf(0:5), pf(0:5)
       real (kind=dp) d, yf, ypy0, ypy0q

c-----------------------------------------------------------------------

       rg1 = 1
       rg2 = 1
       rg3 = 1
       yq = y * y
       yrrtpi = y * rrtpi

       xlim0 = r0 - y
       xlim1 = r1 - y
       xlim3 = 3.097_dp*y - 0.45_dp

c      xlim0 = 15100.0 + y * (40.0 + y * 3.6)
c      xlim1 = 164.0 - y * (4.3 + y * 1.8)
c      xlim3 = 5.76 * yq

       xlim2 = 6.8_dp - y
       xlim4 = 18.1_dp * y + 1.65_dp
       if (y <= 1.0e-6_dp) then
         xlim1 = xlim0
         xlim2 = xlim0
       endif

       do i=0,n-1
         abx = abs(x(i))
         xq = abx * abx

         if (abx > xlim0) then
           k(i) = yrrtpi / (xq + yq)

         else if (abx > xlim1) then
           if (rg1 /= 0) then
             rg1 = 0
             a0 = yq + 0.5_dp
             d0 = a0 * a0
             d2 = yq + yq - 1.0_dp
           endif
           d = rrtpi / (d0 + xq * (d2 + xq))
           k(i) = d * y * (a0 + xq)

         else if (abx > xlim2) then
           if (rg2 /= 0) then
             rg2 = 0
             h0 = 0.5625_dp + yq * (4.5_dp + yq * (10.5_dp
     1                      + yq * (6.0_dp + yq)))
             h2 = -4.5_dp + yq * (9.0_dp + yq * (6.0_dp + yq * 4.0_dp))
             h4 = 10.5_dp - yq * (6.0_dp - yq * 6.0_dp)
             h6 = -6.0_dp + yq * 4.0_dp
             e0 = 1.875_dp + yq * (8.25_dp + yq * (5.5_dp + yq))
             e2 = 5.25_dp + yq * (1.0_dp + yq * 3.0_dp)
             e4 = 0.75_dp * h6
           endif
           d = rrtpi / (h0 + xq * (h2 + xq * (h4 + xq * (h6 + xq))))
           k(i) = d * y * (e0 + xq * (e2 + xq * (e4 + xq)))

         else if (abx < xlim3) then
           if (rg3 /= 0) then
             z0 = 272.1014_dp + y * (1280.829_dp + y * (2802.870_dp
     1                        + y * (3764.966_dp + y * (3447.629_dp
     2                        + y * (2256.981_dp + y * (1074.409_dp
     3                        + y * (369.1989_dp + y * (88.26741_dp
     4                        + y * (13.39880_dp + y)))))))))
             z2 = 211.678_dp  + y * (902.3066_dp + y * (1758.336_dp
     1                        + y * (2037.310_dp + y * (1549.675_dp
     2                        + y * (793.4273_dp + y * (266.2987_dp
     3                        + y * (53.59518_dp + y * 5.0_dp)))))))
             z4 = 78.86585_dp + y * (308.1852_dp + y * (497.3014_dp
     1                        + y * (479.2576_dp + y * (269.2916_dp
     2                        + y * (80.39278_dp + y * 10.0_dp)))))
             z6 = 22.03523_dp + y * (55.02933_dp + y * (92.75679_dp
     1                        + y * (53.59518_dp + y * 10.0_dp)))
             z8 = 1.496460_dp + y * (13.39880_dp + y * 5.0_dp)
             p0 = 153.5168_dp + y * (549.3954_dp + y * (919.4955_dp
     1                        + y * (946.8970_dp + y * (662.8097_dp
     2                        + y * (328.2151_dp + y * (115.3772_dp
     3                        + y * (27.93941_dp + y * (4.264678_dp
     4                        + y * 0.3183291_dp))))))))
             p2 = -34.16955_dp + y * (-1.322256_dp + y * (124.5975_dp
     1                        + y * (189.7730_dp  + y * (139.4665_dp
     2                        + y * (56.81652_dp  + y * (12.79458_dp
     3                        + y * 1.2733163_dp))))))
             p4 = 2.584042_dp + y * (10.46332_dp + y * (24.01655_dp
     1                        + y * (29.81482_dp + y * (12.79568_dp
     2                        + y * 1.9099744_dp))))
             p6 = -0.07272979_dp + y * (0.9377051_dp + y * (4.266322_dp
     1                        + y * 1.273316_dp))
             p8 = 0.0005480304_dp + y * 0.3183291_dp
           endif
           d = 1.7724538_dp / (z0 + xq * (z2 + xq * (z4 + xq
     1       * (z6 + xq * (z8 + xq)))))
           k(i ) = d * (p0 + xq * (p2 + xq * (p4 + xq
     1           * (p6 + xq * p8))))

         else
           ypy0 = y + y0
           ypy0q = ypy0 * ypy0
           k(i) = 0.0_dp
           do j=0,5
             d = x(i) - t(j)
             mq(j) = d * d
             mf(j) = 1.0_dp / (mq(j) + ypy0q)
             xm(j) = mf(j) * d
             ym(j) = mf(j) * ypy0
             d = x(i) + t(j)
             pq(j) = d * d
             pf(j) = 1.0_dp / (pq(j) + ypy0q)
             xp(j) = pf(j) * d
             yp(j) = pf(j) * ypy0
           end do

           if (abx < xlim4) then
             do j=0,5
               k(i) = k(i) + c(j) * (ym(j) + yp(j))
     1                     - s(j) * (xm(j) - xp(j))
             end do

           else
             yf = y + y0py0
             do j=0,5
               k(i) = k(i) + (c(j) * (mq(j) * mf(j) - y0 * ym(j))
     1                     + s(j) * yf * xm(j)) / (mq(j) + y0q)
     2                     + (c(j) * (pq(j) * pf(j) - y0 * yp(j))
     3                     - s(j) * yf * xp(j)) / (pq(j) + y0q)
             end do
             k(i) = y * k(i) + exp(-xq)

           endif
         endif
       end do

       end
