
! Copyright (C) 2002 - Observatoire de Paris/Meudon
!                      Universite Paris 7
!                      CNRS (France)
!                      Contact : Jacques.Lebourlot@obspm.fr
! 
! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License version 2
! as published by the Free Software Foundation.
! 
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


MODULE STR_DATA

! Structure and transfer related quantities for:
!     CO, C*O, CO*, C*O*, CS, HCO+
!     C, N, O, S, Si, C+, N+, O+, S+, Si+

  use constantes

! Dumb quantities used in DEJONG

  REAL (kind=DP), PUBLIC, DIMENSION (njco)          :: opd       ! Optical depth ("direct": towards left side)
  REAL (kind=DP), PUBLIC, DIMENSION (njco)          :: opdr      ! Optical depth ("reverse": towards right side)
  REAL (kind=DP), PUBLIC, DIMENSION (njco)          :: aij       ! Einstein coefficient
  REAL (kind=DP), PUBLIC, DIMENSION (njco)          :: wlk       ! Wavelength
  REAL (kind=DP), PUBLIC, DIMENSION (njco)          :: emi       ! Emissivity
  REAL (kind=DP), PUBLIC, DIMENSION (njco+1)        :: xrepre    ! Relative abundances at "previous" point
  REAL (kind=DP), PUBLIC, DIMENSION (njco+1)        :: xreici    ! Relative abundances at "current" point
  REAL (kind=DP), PUBLIC, DIMENSION (njco+1)        :: gst       ! Statistical weight
  REAL (kind=DP), PUBLIC, DIMENSION (njco+1)        :: elk       ! Line energy
  REAL (kind=DP), PUBLIC, DIMENSION (njco+1,njco+1) :: eqcol     ! Collisional equilibrium
  INTEGER, PUBLIC, DIMENSION (njco)                 :: iup       ! Upper level index
  INTEGER, PUBLIC, DIMENSION (njco)                 :: jlo       ! Lower level index
  INTEGER, PUBLIC                                   :: ntr       ! Number of transitions
  INTEGER, PUBLIC                                   :: nlv       ! Number of levels

! Atoms and atomic ions are as follow:
!           1:   cat  : atomic carbon    (C I)
!           2:   nat  : atomic nitrogen  (N I)
!           3:   oat  : atomic oxygen    (O I)
!           4:   sat  : atomic sulfur    (S I)
!           5:   siat : atomic silicium  (Si I)
!           6:   cpl  : ionised carbon   (C II)
!           7:   npl  : ionised nitrogen (N II)
!           8:   opl  : ionised oxygen   (O II)
!           9:   spl  : ionised sulfur   (S II)
!          10:   sip  : ionised silicium (Si II)

! Cooling and heating rates

  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: reftot    ! Total cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: chotot    ! Total heating

  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refcat    ! C cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refnat    ! N cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refoat    ! O cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refsat    ! S cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refsiat   ! Si cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refcpl    ! C+ cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refnpl    ! N+ cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refopl    ! O+ cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refspl    ! S+ cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refsip    ! Si+ cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: rfjh2t    ! H2 total cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: rfjhdt    ! HD total cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refco0    ! CO cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refco1    ! 13CO cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refco2    ! C18O cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refco3    ! 13C18O cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refcs0    ! CS cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refh2o    ! H2O cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refoh     ! OH cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: reffep    ! Fe+ cooling
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: refhcop   ! HCO+ cooling

  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: chophg    ! Photoelectric effect on grains
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: chochi    ! Chemical exothermicity
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: choeqg    ! Gaz-grains equilibrium
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: chopht    ! Direct Photoprocesses
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: chorc     ! Cosmic rays
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: chophs    ! Secondary photons
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: choh2g    ! H2 formation on grains
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: chohdg    ! HD formation on grains
  REAL (kind=DP), PUBLIC, DIMENSION (0:noptm)       :: chotur    ! Disspation of turbulence

! This is the minimum collision rate (used when nothing else is known)

  REAL (kind=DP), PUBLIC, PARAMETER                   :: colr_min = 1.0e-30_DP

! CO 0 (C12 et O16)
! donnees du 28 septembre 2000 a partir du JPL

  INTEGER, PUBLIC, PARAMETER                 :: nlvco0 = 31      ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvco0) :: gstco0           ! Statistical weight
  DATA gstco0                                 /  1.0_DP,  3.0_DP,  5.0_DP,  7.0_DP,  9.0_DP &
                                              , 11.0_DP, 13.0_DP, 15.0_DP, 17.0_DP, 19.0_DP &
                                              , 21.0_DP, 23.0_DP, 25.0_DP, 27.0_DP, 29.0_DP &
                                              , 31.0_DP, 33.0_DP, 35.0_DP, 37.0_DP, 39.0_DP &
                                              , 41.0_DP, 43.0_DP, 45.0_DP, 47.0_DP, 49.0_DP &
                                              , 51.0_DP, 53.0_DP, 55.0_DP, 57.0_DP, 59.0_DP &
                                              , 61.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvco0) :: elkco0           ! Level energy
  DATA elkco0                                 /   0.000_DP,    5.532_DP,   16.596_DP,   33.192_DP &
                                             ,   55.318_DP,   82.974_DP,  116.160_DP,  154.872_DP &
                                             ,  199.112_DP,  248.875_DP,  304.162_DP,  364.968_DP &
                                             ,  431.293_DP,  503.134_DP,  580.487_DP,  663.350_DP &
                                             ,  751.721_DP,  845.594_DP,  944.968_DP, 1049.837_DP &
                                             , 1160.198_DP, 1276.047_DP, 1397.380_DP, 1524.191_DP &
                                             , 1656.476_DP, 1794.230_DP, 1937.447_DP, 2086.122_DP &
                                             , 2240.250_DP, 2399.824_DP, 2564.839_DP /

  REAL (kind=DP), PUBLIC                     :: amco0 = 28.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(njco+1,0:noptm) :: xreco0    ! CO levels relative abundances
  REAL (KIND=DP), PUBLIC, DIMENSION(njco,0:noptm)   :: emico0    ! CO Emissivity
  REAL (kind=DP), PUBLIC, DIMENSION (0:njco,0:njco) :: eqrco0    ! CO radiative equlibrium

  INTEGER, PUBLIC, PARAMETER                 :: ntrco0 = 30      ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntrco0) :: aijco0           ! Transition probability
  DATA aijco0                                 / 7.165598E-08_DP, 6.878974E-07_DP, 2.487486E-06_DP &
                                              , 6.112985E-06_DP, 1.220879E-05_DP, 2.141903E-05_DP &
                                              , 3.437722E-05_DP, 5.165579E-05_DP, 7.400161E-05_DP &
                                              , 1.021826E-04_DP, 1.365310E-04_DP, 1.778280E-04_DP &
                                              , 2.266856E-04_DP, 2.837192E-04_DP, 3.495736E-04_DP &
                                              , 4.248763E-04_DP, 5.101879E-04_DP, 6.062004E-04_DP &
                                              , 7.134401E-04_DP, 8.325689E-04_DP, 9.641728E-04_DP &
                                              , 1.108853E-03_DP, 1.267150E-03_DP, 1.439707E-03_DP &
                                              , 1.627083E-03_DP, 1.829820E-03_DP, 2.048529E-03_DP &
                                              , 2.283784E-03_DP, 2.536060E-03_DP, 2.806008E-03_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntrco0) :: wlkco0           ! Energy of transition
  DATA wlkco0                                 /   5.532_DP,  11.064_DP,  16.596_DP,  22.126_DP,  27.656_DP &
                                              ,  33.186_DP,  38.712_DP,  44.240_DP,  49.763_DP,  55.287_DP &
                                              ,  60.806_DP,  66.325_DP,  71.841_DP,  77.353_DP,  82.863_DP &
                                              ,  88.371_DP,  93.873_DP,  99.374_DP, 104.869_DP, 110.361_DP &
                                              , 115.849_DP, 121.333_DP, 126.811_DP, 132.285_DP, 137.754_DP &
                                              , 143.217_DP, 148.675_DP, 154.128_DP, 159.574_DP, 165.015_DP /

  INTEGER, PUBLIC, DIMENSION (ntrco0)        :: iupco0           ! Upper state of transition
  DATA iupco0                                 /  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16 &
                                              , 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 /

  INTEGER, PUBLIC, DIMENSION (ntrco0)        :: jloco0           ! Lower state of transition 
  DATA jloco0                                 /  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 &
                                              , 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30 /


  REAL (KIND=DP), PUBLIC, DIMENSION (1:njco)        :: cdodj1    ! CO collision rate J -> J-1
  REAL (KIND=DP), PUBLIC, DIMENSION (0:njco-1)      :: cupdj1    ! CO collision rate J -> J+1
  REAL (KIND=DP), PUBLIC, DIMENSION (2:njco)        :: cdodj2    ! CO collision rate J -> J-2
  REAL (KIND=DP), PUBLIC, DIMENSION (0:njco-2)      :: cupdj2    ! CO collision rate J -> J+2
  REAL (KIND=DP), PUBLIC, DIMENSION (3:njco)        :: cdodj3    ! CO collision rate J -> J-3
  REAL (KIND=DP), PUBLIC, DIMENSION (0:njco-3)      :: cupdj3    ! CO collision rate J -> J+3
  REAL (KIND=DP), PUBLIC, DIMENSION (4:njco)        :: cdodj4    ! CO collision rate J -> J-4
  REAL (KIND=DP), PUBLIC, DIMENSION (0:njco-4)      :: cupdj4    ! CO collision rate J -> J+4

! C*O 1 (C13 et O16)
! donnees du 28 septembre 2000 a partir du JPL

  INTEGER, PUBLIC, PARAMETER                 :: nlvco1 = 31      ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvco1) :: gstco1           ! Statistical weight
  DATA gstco1                                 /  1.0_DP,  3.0_DP,  5.0_DP,  7.0_DP,  9.0_DP &
                                              , 11.0_DP, 13.0_DP, 15.0_DP, 17.0_DP, 19.0_DP &
                                              , 21.0_DP, 23.0_DP, 25.0_DP, 27.0_DP, 29.0_DP &
                                              , 31.0_DP, 33.0_DP, 35.0_DP, 37.0_DP, 39.0_DP &
                                              , 41.0_DP, 43.0_DP, 45.0_DP, 47.0_DP, 49.0_DP &
                                              , 51.0_DP, 53.0_DP, 55.0_DP, 57.0_DP, 59.0_DP &
                                              , 61.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvco1) :: elkco1           ! Level energy
  DATA elkco1                                 /    0.000_DP,    5.289_DP,   15.866_DP,   31.732_DP &
                                              ,   52.885_DP,   79.325_DP,  111.051_DP,  148.062_DP &
                                              ,  190.356_DP,  237.932_DP,  290.788_DP,  348.923_DP &
                                              ,  412.333_DP,  481.017_DP,  554.973_DP,  634.197_DP &
                                              ,  718.686_DP,  808.438_DP,  903.449_DP, 1003.717_DP &
                                              , 1109.236_DP, 1220.004_DP, 1336.015_DP, 1457.267_DP &
                                              , 1583.754_DP, 1715.473_DP, 1852.417_DP, 1994.583_DP &
                                              , 2141.964_DP, 2294.556_DP, 2452.353_DP /

  REAL (kind=DP), PUBLIC                     :: amco1 = 29.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(njco+1,0:noptm) :: xreco1    ! 13CO levels relative abundances
  REAL (KIND=DP), PUBLIC, DIMENSION(njco,0:noptm)   :: emico1    ! 13CO Emissivity
  REAL (kind=DP), PUBLIC, DIMENSION (0:njco,0:njco) :: eqrco1    ! 13CO radiative equlibrium

  INTEGER, PUBLIC, PARAMETER                 :: ntrco1 = 30      ! Number of transitions

! donnees du 28 septembre 2000 a partir de la formule de A en
! fonction du moment dipolaire et en prenant le moment dip
! relatif a CO soit 0.1098 Debyes
! (rem: JPL donne mom dip CO1=0.11046 Debyes) (FLP)
  REAL (kind=DP), PUBLIC, DIMENSION (ntrco1) :: aijco1           ! Transition probability
  DATA aijco1                                 / 6.262195E-08_DP, 6.010003E-07_DP, 2.173465E-06_DP &
                                              , 5.341467E-06_DP, 1.066814E-05_DP, 1.871462E-05_DP &
                                              , 3.004183E-05_DP, 4.520688E-05_DP, 6.477089E-05_DP &
                                              , 8.928752E-05_DP, 1.193177E-04_DP, 1.553966E-04_DP &
                                              , 1.980950E-04_DP, 2.479576E-04_DP, 3.055111E-04_DP &
                                              , 3.713075E-04_DP, 4.459030E-04_DP, 5.298093E-04_DP &
                                              , 6.235958E-04_DP, 7.277216E-04_DP, 8.427930E-04_DP &
                                              , 9.692478E-04_DP, 1.107704E-03_DP, 1.258578E-03_DP &
                                              , 1.422467E-03_DP, 1.599756E-03_DP, 1.791082E-03_DP &
                                              , 1.996801E-03_DP, 2.217525E-03_DP, 2.453662E-03_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntrco1) :: wlkco1           ! Energy of transition
  DATA wlkco1                                 /   5.289_DP,  10.577_DP,  15.866_DP,  21.153_DP,  26.440_DP &
                                              ,  31.726_DP,  37.011_DP,  42.294_DP,  47.576_DP,  52.856_DP &
                                              ,  58.135_DP,  63.410_DP,  68.684_DP,  73.956_DP,  79.224_DP &
                                              ,  84.489_DP,  89.752_DP,  95.011_DP, 100.268_DP, 105.519_DP &
                                              , 110.768_DP, 116.011_DP, 121.252_DP, 126.487_DP, 131.719_DP &
                                              , 136.944_DP, 142.166_DP, 147.381_DP, 152.592_DP, 157.797_DP /

  INTEGER, PUBLIC, DIMENSION (ntrco1)        :: iupco1           ! Upper state of transition
  DATA iupco1                                 /  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16 &
                                              , 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 /

  INTEGER, PUBLIC, DIMENSION (ntrco1)        :: jloco1           ! Lower state of transition 
  DATA jloco1                                 /  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 &
                                              , 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30 /

! CO* 2 (C12 et O18)
! donnees du 28 septembre 2000 a partir du JPL

  INTEGER, PUBLIC, PARAMETER                 :: nlvco2 = 31      ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvco2) :: gstco2           ! Statistical weight
  DATA gstco2                                 /  1.0_DP,  3.0_DP,  5.0_DP,  7.0_DP,  9.0_DP &
                                              , 11.0_DP, 13.0_DP, 15.0_DP, 17.0_DP, 19.0_DP &
                                              , 21.0_DP, 23.0_DP, 25.0_DP, 27.0_DP, 29.0_DP &
                                              , 31.0_DP, 33.0_DP, 35.0_DP, 37.0_DP, 39.0_DP &
                                              , 41.0_DP, 43.0_DP, 45.0_DP, 47.0_DP, 49.0_DP &
                                              , 51.0_DP, 53.0_DP, 55.0_DP, 57.0_DP, 59.0_DP &
                                              , 61.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvco2) :: elkco2           ! Level energy
  DATA elkco2                                 /    0.000_DP,    5.269_DP,   15.806_DP,   31.611_DP &
                                              ,   52.684_DP,   79.024_DP,  110.629_DP,  147.499_DP &
                                              ,  189.632_DP,  237.027_DP,  289.683_DP,  347.596_DP &
                                              ,  410.765_DP,  479.189_DP,  552.863_DP,  631.786_DP &
                                              ,  715.955_DP,  805.366_DP,  900.017_DP,  999.903_DP &
                                              , 1105.022_DP, 1215.369_DP, 1330.941_DP, 1451.733_DP &
                                              , 1577.741_DP, 1708.960_DP, 1845.386_DP, 1987.013_DP &
                                              , 2133.837_DP, 2285.851_DP, 2443.051_DP /

  REAL (kind=DP), PUBLIC                     :: amco2 = 30.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(njco+1,0:noptm) :: xreco2    ! C18O levels relative abundances
  REAL (KIND=DP), PUBLIC, DIMENSION(njco,0:noptm)   :: emico2    ! C18O Emissivity
  REAL (kind=DP), PUBLIC, DIMENSION (0:njco,0:njco) :: eqrco2    ! C18O radiative equlibrium

  INTEGER, PUBLIC, PARAMETER                 :: ntrco2 = 30      ! Number of transitions
! donnees du 28 septembre 2000 en utilisant la formule de A
! a partir du moment dipolaire. Le moment dipolaire employe est celui
! de CO - (Le JPL done toutefois pour C12O18 0.11079 Debyes) (FLP)
  REAL (kind=DP), PUBLIC, DIMENSION (ntrco2) :: aijco2           ! Transition probability
  DATA aijco2                                 / 6.191423E-08_DP, 5.942074E-07_DP, 2.148493E-06_DP &
                                              , 5.281092E-06_DP, 1.054755E-05_DP, 1.850131E-05_DP &
                                              , 2.969979E-05_DP, 4.469257E-05_DP, 6.403444E-05_DP &
                                              , 8.827779E-05_DP, 1.179560E-04_DP, 1.536315E-04_DP &
                                              , 1.958539E-04_DP, 2.451320E-04_DP, 3.020421E-04_DP &
                                              , 3.671045E-04_DP, 4.408399E-04_DP, 5.238097E-04_DP &
                                              , 6.164956E-04_DP, 7.194771E-04_DP, 8.332198E-04_DP &
                                              , 9.582861E-04_DP, 1.095145E-03_DP, 1.244333E-03_DP &
                                              , 1.406330E-03_DP, 1.581671E-03_DP, 1.770787E-03_DP &
                                              , 1.974247E-03_DP, 2.192421E-03_DP, 2.425918E-03_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntrco2) :: wlkco2           ! Energy of transition
  DATA wlkco2                                 /   5.269_DP,  10.537_DP,  15.805_DP,  21.073_DP,  26.340_DP &
                                              ,  31.605_DP,  36.870_DP,  42.133_DP,  47.395_DP,  52.656_DP &
                                              ,  57.913_DP,  63.169_DP,  68.424_DP,  73.674_DP,  78.923_DP &
                                              ,  84.169_DP,  89.411_DP,  94.651_DP,  99.886_DP, 105.119_DP &
                                              , 110.347_DP, 115.572_DP, 120.792_DP, 126.008_DP, 131.219_DP &
                                              , 136.426_DP, 141.627_DP, 146.824_DP, 152.014_DP, 157.200_DP /

  INTEGER, PUBLIC, DIMENSION (ntrco2)        :: iupco2           ! Upper state of transition
  DATA iupco2                                 /  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16 &
                                              , 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 /

  INTEGER, PUBLIC, DIMENSION (ntrco2)        :: jloco2           ! Lower state of transition 
  DATA jloco2                                 /  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 &
                                              , 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30 /

! C*O* 3 (C13 et O18)
! donnees du 28 septembre 2000 avec le moment dipol de CO

  INTEGER, PUBLIC, PARAMETER                 :: nlvco3 = 31      ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvco3) :: gstco3           ! Statistical weight
  DATA gstco3                                 /  1.0_DP,  3.0_DP,  5.0_DP,  7.0_DP,  9.0_DP &
                                              , 11.0_DP, 13.0_DP, 15.0_DP, 17.0_DP, 19.0_DP &
                                              , 21.0_DP, 23.0_DP, 25.0_DP, 27.0_DP, 29.0_DP &
                                              , 31.0_DP, 33.0_DP, 35.0_DP, 37.0_DP, 39.0_DP &
                                              , 41.0_DP, 43.0_DP, 45.0_DP, 47.0_DP, 49.0_DP &
                                              , 51.0_DP, 53.0_DP, 55.0_DP, 57.0_DP, 59.0_DP &
                                              , 61.0_DP /

! donnees du 28 septembre 2000 a partir des coeff de Dunham
! ref: Guelachvili, J Molecular Spect, 98, 64 (1983)
  REAL (kind=DP), PUBLIC, DIMENSION (nlvco3) :: elkco3           ! Level energy
  DATA elkco3                                 /    0.000_DP,    5.025_DP,   15.076_DP,   30.151_DP &
                                              ,   50.251_DP,   75.374_DP,  105.520_DP,  140.687_DP &
                                              ,  180.875_DP,  226.082_DP,  276.307_DP,  331.547_DP &
                                              ,  391.801_DP,  457.067_DP,  527.342_DP,  602.624_DP &
                                              ,  682.911_DP,  768.199_DP,  858.486_DP,  953.769_DP &
                                              , 1054.044_DP, 1159.307_DP, 1269.556_DP, 1384.786_DP &
                                              , 1504.993_DP, 1630.173_DP, 1760.322_DP, 1895.436_DP &
                                              , 2035.509_DP, 2180.536_DP, 2330.513_DP /

  REAL (kind=DP), PUBLIC                     :: amco3 = 31.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(njco+1,0:noptm) :: xreco3    ! 13C18O levels relative abundances
  REAL (KIND=DP), PUBLIC, DIMENSION(njco,0:noptm)   :: emico3    ! 13C18O Emissivity
  REAL (kind=DP), PUBLIC, DIMENSION (0:njco,0:njco) :: eqrco3    ! 13C18O radiative equlibrium

  INTEGER, PUBLIC, PARAMETER                 :: ntrco3 = 30      ! Number of transitions
  REAL (kind=DP), PUBLIC, DIMENSION (ntrco3) :: aijco3           ! Transition probability
  DATA aijco3                                 / 5.370492E-08_DP, 5.157212E-07_DP, 1.864328E-06_DP &
                                              , 4.582820E-06_DP, 9.152062E-06_DP, 1.605551E-05_DP &
                                              , 2.577151E-05_DP, 3.878443E-05_DP, 5.556907E-05_DP &
                                              , 7.660687E-05_DP, 1.023654E-04_DP, 1.333294E-04_DP &
                                              , 1.699682E-04_DP, 2.127452E-04_DP, 2.621381E-04_DP &
                                              , 3.186170E-04_DP, 3.826237E-04_DP, 4.546462E-04_DP &
                                              , 5.351338E-04_DP, 6.245271E-04_DP, 7.232779E-04_DP &
                                              , 8.318812E-04_DP, 9.507226E-04_DP, 1.080268E-03_DP &
                                              , 1.220961E-03_DP, 1.373242E-03_DP, 1.537550E-03_DP &
                                              , 1.714248E-03_DP, 1.903793E-03_DP, 2.106651E-03_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntrco3) :: wlkco3           ! Energy of transition
  DATA wlkco3                                 /   5.025_DP,  10.051_DP,  15.075_DP,  20.100_DP,  25.123_DP &
                                              ,  30.146_DP,  35.167_DP,  40.188_DP,  45.207_DP,  50.225_DP &
                                              ,  55.240_DP,  60.254_DP,  65.266_DP,  70.275_DP,  75.282_DP &
                                              ,  80.287_DP,  85.288_DP,  90.287_DP,  95.283_DP, 100.275_DP &
                                              , 105.263_DP, 110.249_DP, 115.230_DP, 120.207_DP, 125.180_DP &
                                              , 130.149_DP, 135.114_DP, 140.073_DP, 145.027_DP, 149.977_DP /

  INTEGER, PUBLIC, DIMENSION (ntrco3)        :: iupco3           ! Upper state of transition
  DATA iupco3                                 /  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16 &
                                              , 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 /

  INTEGER, PUBLIC, DIMENSION (ntrco3)        :: jloco3           ! Lower state of transition 
  DATA jloco3                                 /  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 &
                                              , 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30 /

! CS

  INTEGER, PUBLIC, PARAMETER                 :: nlvcs0 = 21      ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvcs0) :: gstcs0           ! Statisctical weight
  DATA gstcs0                                 /  1.0_DP,  3.0_DP,  5.0_DP,  7.0_DP,  9.0_DP &
                                              , 11.0_DP, 13.0_DP, 15.0_DP, 17.0_DP, 19.0_DP &
                                              , 21.0_DP, 23.0_DP, 25.0_DP, 27.0_DP, 29.0_DP &
                                              , 31.0_DP, 33.0_DP, 35.0_DP, 37.0_DP, 39.0_DP &
                                              , 41.0_DP /
 
  REAL (kind=DP), PUBLIC, DIMENSION (nlvcs0) :: elkcs0           ! Level energy
  DATA elkcs0                                 /  0.000_DP,  2.355_DP,  7.066_DP, 14.132_DP, 23.553_DP &
                                              , 35.330_DP, 49.462_DP, 65.950_DP, 84.793_DP, 105.99_DP &
                                              , 129.54_DP, 155.45_DP, 183.72_DP, 214.33_DP, 247.31_DP &
                                              , 282.64_DP, 320.33_DP, 360.37_DP, 381.57_DP, 447.52_DP &
                                              , 494.62_DP /

  REAL (kind=DP), PUBLIC                     :: amcs0 = 44.1_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(njcs+1,0:noptm) :: xrecs0    ! CS levels relative abundances
  REAL (KIND=DP), PUBLIC, DIMENSION(njcs,0:noptm)   :: emics0    ! CS Emissivity

  INTEGER, PUBLIC, PARAMETER                 :: ntrcs0 = 20      ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntrcs0) :: aijcs0           ! Transition probability
  DATA aijcs0                                 / 1.753e-6_DP, 1.683e-5_DP, 6.087e-5_DP, 1.496e-4_DP &
                                              , 2.989e-4_DP, 5.244e-4_DP, 8.420e-4_DP, 1.267e-3_DP &
                                              , 1.816e-3_DP, 2.505e-3_DP, 3.348e-3_DP, 4.363e-3_DP &
                                              , 5.564e-3_DP, 6.968e-3_DP, 8.590e-3_DP, 1.045e-2_DP &
                                              , 1.255e-2_DP, 1.492e-2_DP, 1.758e-2_DP, 2.053e-2_DP /
 
  REAL (kind=DP), PUBLIC, DIMENSION (ntrcs0) :: wlkcs0           ! Energy of transition
  DATA wlkcs0                                 /  2.355_DP,  4.711_DP,  7.066_DP,  9.421_DP, 11.777_DP &
                                              , 14.132_DP, 16.487_DP, 18.843_DP, 21.198_DP, 23.554_DP &
                                              , 25.909_DP, 28.264_DP, 30.620_DP, 32.975_DP, 35.330_DP &
                                              , 37.686_DP, 40.041_DP, 42.396_DP, 44.752_DP, 47.107_DP /
 
  INTEGER, PUBLIC, DIMENSION (ntrcs0)        :: iupcs0           ! Upper state of transition
  DATA iupcs0                                 /  2,  3,  4,  5,  6,  7,  8,  9, 10, 11 &
                                              , 12, 13, 14, 15, 16, 17, 18, 19, 20, 21 /

  INTEGER, PUBLIC, DIMENSION (ntrcs0)        :: jlocs0           ! Lower state of transition 
  DATA jlocs0                                 /  1,  2,  3,  4,  5,  6,  7,  8,  9, 10 &
                                              , 11, 12, 13, 14, 15, 16, 17, 18, 19, 20 /

  REAL (KIND=DP), PUBLIC, DIMENSION(4,210)   :: cdocs            ! CS collision rates
  INTEGER, PUBLIC                            :: ntccs            ! number of CS collisional transitions included

! HCO+

  INTEGER, PUBLIC, PARAMETER                  :: nlvhcop = 21      ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvhcop) :: gsthcop           ! Statisctical weight
  DATA gsthcop                                 /  1.0_DP,  3.0_DP,  5.0_DP,  7.0_DP,  9.0_DP &
                                               , 11.0_DP, 13.0_DP, 15.0_DP, 17.0_DP, 19.0_DP &
                                               , 21.0_DP, 23.0_DP, 25.0_DP, 27.0_DP, 29.0_DP &
                                               , 31.0_DP, 33.0_DP, 35.0_DP, 37.0_DP, 39.0_DP &
                                               , 41.0_DP /
 
  REAL (kind=DP), PUBLIC, DIMENSION (nlvhcop) :: elkhcop           ! Level energy
  DATA elkhcop                                 /   0.000_DP,   4.280_DP,  12.841_DP,  25.682_DP &
                                               ,  42.802_DP,  64.202_DP,  89.881_DP, 119.838_DP &
                                               , 154.073_DP, 192.584_DP, 235.372_DP, 282.435_DP &
                                               , 333.772_DP, 389.382_DP, 449.263_DP, 513.415_DP &
                                               , 581.836_DP, 654.524_DP, 731.477_DP, 812.695_DP &
                                               , 898.174_DP /

  REAL (kind=DP), PUBLIC                      :: amhcop = 29.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(njhcop+1,0:noptm)    :: xrehcop   ! HCO+ levels relative abundances
  REAL (KIND=DP), PUBLIC, DIMENSION(njhcop,0:noptm)      :: emihcop   ! HCO+ Emissivity

  INTEGER, PUBLIC, PARAMETER                  :: ntrhcop = 20      ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntrhcop) :: aijhcop           ! Transition probability
  DATA aijhcop                                 / 2.887e-5_DP, 2.771e-4_DP, 1.002e-3_DP, 2.462e-3_DP &
                                               , 4.917e-3_DP, 8.625e-3_DP, 1.384e-2_DP, 2.083e-2_DP &
                                               , 2.983e-2_DP, 4.110e-2_DP, 5.490e-2_DP, 7.149e-2_DP &
                                               , 9.109e-2_DP, 0.114_DP, 0.140_DP, 0.170_DP, 0.205_DP &
                                               , 0.243_DP, 0.286_DP, 0.333_DP /
 
  REAL (kind=DP), PUBLIC, DIMENSION (ntrhcop) :: wlkhcop           ! Energy of transition
  DATA wlkhcop                                 /  4.280_DP,  8.561_DP, 12.841_DP, 17.120_DP &
                                               , 21.400_DP, 25.679_DP, 29.957_DP, 34.236_DP &
                                               , 38.512_DP, 42.788_DP, 47.063_DP, 51.337_DP &
                                               , 55.610_DP, 59.881_DP, 64.152_DP, 68.421_DP &
                                               , 72.688_DP, 76.954_DP, 81.217_DP, 85.479_DP /
 
  INTEGER, PUBLIC, DIMENSION (ntrhcop)        :: iuphcop           ! Upper state of transition
  DATA iuphcop                                 /  2,  3,  4,  5,  6,  7,  8,  9, 10, 11 &
                                               , 12, 13, 14, 15, 16, 17, 18, 19, 20, 21/
 
  INTEGER, PUBLIC, DIMENSION (ntrhcop)        :: jlohcop           ! Lower state of transition 
  DATA jlohcop                                 /  1,  2,  3,  4,  5,  6,  7,  8,  9, 10 &
                                               , 11, 12, 13, 14, 15, 16, 17, 18, 19, 20/

  REAL (KIND=DP), PUBLIC, DIMENSION(4,0:njhcop,0:njhcop) :: colhcoph2 ! HCO+ collision rates (with H2)

! C
! Energy levels.
!        1 - 3P J = 0
!        2 - 3P J = 1
!        3 - 3P J = 2
!        4 - 1D J = 2
!        5 - 1S J = 0

  INTEGER, PUBLIC, PARAMETER                 :: nlvcat = 5       ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvcat) :: gstcat           ! Statistical weight
  DATA gstcat                                 / 1.0_DP, 3.0_DP, 5.0_DP, 5.0_DP, 1.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvcat) :: elkcat           ! Level energy
  DATA elkcat                                 / 0.0_DP, 23.5969_DP, 62.4454_DP &
                                              , 14665.507_DP, 31147.902_DP /

  REAL (kind=DP), PUBLIC                     :: amcat = 12.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(nlvcat,0:noptm) :: xrecat    ! C levels relative abundances

! Transitions
!       1: 2-1 -> 609.75 micron
!       2: 3-2 -> 370.37 micron
!       3: 3-1 -> 230.41 micron
!       4: 4-3 -> 9850.26 Angstrom
!       5: 4-2 -> 9824.13 Angstrom
!       6: 4-1 -> 9808.32 Angstrom
!       7: 5-4 -> 8727.13 Angstrom
!       8: 5-3 -> 4627.346 Angstrom
!       9: 5-2 -> 4621.570 Angstrom
!      10: 5-1

  INTEGER, PUBLIC, PARAMETER                 :: ntrcat = 10      ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntrcat) :: wlkcat           ! Energy of transition
  DATA wlkcat                                 /    23.5969_DP,    38.8485_DP,    62.4454_DP &
                                              , 14603.0616_DP, 14641.9101_DP, 14665.507_DP &
                                              , 16482.395_DP,  31085.4566_DP, 31124.3051_DP &
                                              , 31147.902_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntrcat) :: aijcat           ! Transition probability
  DATA aijcat                                 / 7.93e-8_DP, 2.65e-7_DP, 1.71e-14_DP &
                                              , 2.44e-4_DP, 8.21e-5_DP, 7.77e-8_DP &
                                              , 5.28e-1_DP, 2.00e-5_DP, 2.71e-3_DP &
                                              , 0.00e-0_DP /

  INTEGER, PUBLIC, DIMENSION (ntrcat)        :: iupcat           ! Upper state of transition
  DATA iupcat                                 / 2, 3, 3, 4, 4, 4, 5, 5, 5, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrcat)        :: jlocat           ! Lower state of transition 
  DATA jlocat                                 / 1, 2, 1, 3, 2, 1, 4, 3, 2, 1 /

  REAL (KIND=DP), PUBLIC, DIMENSION(ntrcat,0:noptm) :: emicat    ! C Emissivity

! N
! Energy levels.
!        1 - 4So J = 3/2
!        2 - 2Do J = 5/2
!        3 - 2Do J = 3/2
!        4 - 2Po J = 1/2
!        5 - 2Po J = 3/2

  INTEGER, PUBLIC, PARAMETER                 :: nlvnat = 5       ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvnat) :: gstnat           ! Statistical weight
  DATA gstnat                                 / 4.0_DP, 6.0_DP, 4.0_DP, 2.0_DP, 4.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvnat) :: elknat           ! Level energy
  DATA elknat                                 / 0.0_DP, 27660.82_DP, 27673.36_DP &
                                              , 41494.431_DP, 41494.986_DP /

  REAL (kind=DP), PUBLIC                     :: amnat = 14.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(nlvnat,0:noptm) :: xrenat    ! N levels relative abundances

! Transitions
!       1: 3-2 -> 1.147 mm (261.21 GHz)
!       2: 4-3 -> 1.04104 micron
!       3: 5-3 -> 1.04100 micron
!       4: 4-2 -> 1.04010 micron
!       5: 5-2 -> 1.04006 micron
!       6: 2-1 -> 5200.257 Angstrom
!       7: 3-1 -> 5197.902 Angstrom
!       8: 4-1 -> 3466.543 Angstrom
!       9: 5-1 -> 3466.497 Angstrom
!      10: 5-4

  INTEGER, PUBLIC, PARAMETER                 :: ntrnat = 10      ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntrnat) :: wlknat           ! Energy of transition
  DATA wlknat                                 /    12.540_DP, 13821.071_DP, 13821.626_DP &
                                              , 13833.611_DP, 13834.166_DP, 27660.82_DP &
                                              , 27673.360_DP, 41494.431_DP, 41494.986_DP &
                                              ,     0.555_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntrnat) :: aijnat           ! Transition probability
  DATA aijnat                                 / 1.27e-8_DP, 5.29e-2_DP, 2.76e-2_DP &
                                              , 3.45e-2_DP, 6.14e-2_DP, 7.27e-6_DP &
                                              , 2.02e-5_DP, 2.71e-3_DP, 6.58e-3_DP &
                                              , 0.00e-0_DP /

  INTEGER, PUBLIC, DIMENSION (ntrnat)        :: iupnat           ! Upper state of transition
  DATA iupnat                                 / 3, 4, 5, 4, 5, 2, 3, 4, 5, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrnat)        :: jlonat           ! Lower state of transition 
  DATA jlonat                                 / 2, 3, 3, 2, 2, 1, 1, 1, 1, 4 /

  REAL (KIND=DP), PUBLIC, DIMENSION(ntrnat,0:noptm) :: eminat    ! N Emissivity

! O

! 9 VII 2003 - First attempt at unifying atomic datas with the shock model
! Added 2 levels to O structure:

! Energy levels.
!        1 - 3P J = 2
!        2 - 3P J = 1
!        3 - 3P J = 0
!        4 - 1D J = 2
!        5 - 1S J = 0

  INTEGER, PUBLIC, PARAMETER                 :: nlvoat = 5       ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvoat) :: gstoat           ! Statistical weight
  DATA gstoat                                 / 5.0_DP, 3.0_DP, 1.0_DP, 5.0_DP, 1.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvoat) :: elkoat           ! Level energy
  DATA elkoat                                 / 0.0_DP, 227.717_DP, 326.582_DP &
                                              , 22831.226_DP, 48621.932_DP /

  REAL (kind=DP), PUBLIC                     :: amoat = 16.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(nlvoat,0:noptm) :: xreoat    ! O levels relative abundances

! Transitions
!       1: 3-2 -> 145.53 micron
!       2: 2-1 -> 63.19  micron
!       3: 3-1 -> 44.06  micron
!       4: 4-3 -> 6391.73 Angstrom
!       5: 4-2 -> 6363.78 Angstrom
!       6: 4-1 -> 6300.30 Angstrom
!       7: 5-4 -> 5577.34 Angstrom
!       8: 5-2 -> 2972.29 Angstrom
!       9: 5-1 -> 2958.37 Angstrom
!      10: 5-3

  INTEGER, PUBLIC, PARAMETER                 :: ntroat = 10      ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntroat) :: wlkoat           ! Energy of transition
  DATA wlkoat                                 / 98.865_DP, 227.717_DP, 326.582_DP, &
                                                22504.644_DP, 22603.509_DP, 22831.226_DP, &
                                                25790.706_DP, 48394.215_DP, 48621.932_DP, 48295.35_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntroat) :: aijoat           ! Transition probability
  DATA aijoat                                 / 1.66e-5_dp, 8.46e-5_dp, 1.10e-10_dp, &
                                                1.20e-6_dp, 2.17e-3_dp, 6.71e-3_dp, &
                                                1.02e-0_dp, 7.83e-2_dp, 2.87e-4_dp, 0.0e0_DP /

  INTEGER, PUBLIC, DIMENSION (ntroat)        :: iupoat           ! Upper state of transition
  DATA iupoat                                 / 3, 2, 3, 4, 4, 4, 5, 5, 5, 5 /

  INTEGER, PUBLIC, DIMENSION (ntroat)        :: jlooat           ! Lower state of transition 
  DATA jlooat                                 / 2, 1, 1, 3, 2, 1, 4, 2, 1, 3 /

  REAL (KIND=DP), PUBLIC, DIMENSION(ntroat,0:noptm) :: emioat    ! O Emissivity

! S
! Energy levels.
!        1 - 3P J = 2
!        2 - 3P J = 1
!        3 - 3P J = 0
!        4 - 1D J = 2
!        5 - 1S J = 0

  INTEGER, PUBLIC, PARAMETER                 :: nlvsat = 5       ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvsat) :: gstsat           ! Statistical weight
  DATA gstsat                                 / 5.0_DP, 3.0_DP, 1.0_DP, 5.0_DP, 1.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvsat) :: elksat           ! Level energy
  DATA elksat                                 /     0.0_DP,    569.86_DP, 825.37_DP &
                                              , 13292.83_DP, 31913.28_DP /

  REAL (kind=DP), PUBLIC                     :: amsat = 32.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(nlvsat,0:noptm) :: xresat    ! S levels relative abundances

! Transitions
!       1: 3-2 -> 56.31 micron
!       2: 2-1 -> 25.25 micron
!       3: 3-1 -> 17.43 micron
!       4: 4-3 -> 1.15407 micron
!       5: 4-2 -> 1.13089 micron
!       6: 4-1 -> 1.08241 micron
!       7: 5-4 -> 7725.05 Angstrom
!       8: 5-2 -> 4589.26 Angstrom
!       9: 5-1 -> 4507.31 Angstrom
!      10: 5-3

  INTEGER, PUBLIC, PARAMETER                 :: ntrsat = 10      ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntrsat) :: wlksat           ! Energy of transition
  DATA wlksat                                 /   255.51_DP,   569.86_DP,   825.37_DP &
                                              , 12467.46_DP, 12722.97_DP, 13292.83_DP &
                                              , 18620.45_DP, 31343.42_DP, 31913.28_DP &
                                              , 31087.91_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntrsat) :: aijsat           ! Transition probability
  DATA aijsat                                 / 3.02e-4_DP, 1.39e-3_DP, 6.71e-8_DP &
                                              , 3.84e-6_DP, 8.16e-3_DP, 2.78e-2_DP &
                                              , 1.53e-0_DP, 3.50e-1_DP, 8.23e-3_DP &
                                              , 0.00e-0_DP /

  INTEGER, PUBLIC, DIMENSION (ntrsat)        :: iupsat           ! Upper state of transition
  DATA iupsat                                 / 3, 2, 3, 4, 4, 4, 5, 5, 5, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrsat)        :: jlosat           ! Lower state of transition 
  DATA jlosat                                 / 2, 1, 1, 3, 2, 1, 4, 2, 1, 3 /

  REAL (KIND=DP), PUBLIC, DIMENSION(ntrsat,0:noptm) :: emisat    ! S Emissivity

! Si
! Energy levels.
!        1 - 3P J = 0
!        2 - 3P J = 1
!        3 - 3P J = 2
!        4 - 1D J = 2
!        5 - 1S J = 0

  INTEGER, PUBLIC, PARAMETER                  :: nlvsiat = 5       ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvsiat) :: gstsiat           ! Statistical weight
  DATA gstsiat                                 / 1.0_DP, 3.0_DP, 5.0_DP, 5.0_DP, 1.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvsiat) :: elksiat           ! Level energy
  DATA elksiat                                 /    0.000_DP,   110.951_DP, 321.086_DP &
                                               , 9062.998_DP, 22149.939_DP /

  REAL (kind=DP), PUBLIC                     :: amsiat = 28.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(nlvsiat,0:noptm) :: xresiat    ! Si levels relative abundances

! Transitions
!       1: 2-1 -> 129.682 micron
!       2: 3-2 -> 68.472 micron
!       3: 3-1 -> 44.812 micron
!       4: 4-3 -> 1.6459 micron
!       5: 4-2 -> 1.6073 micron
!       6: 4-1 -> 1.5876 micron
!       7: 5-4 -> 1.0994 micron
!       8: 5-3 -> 6589.61 Angstrom
!       9: 5-2 -> 6526.78 Angstrom
!      10: 5-1

  INTEGER, PUBLIC, PARAMETER                  :: ntrsiat = 10      ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntrsiat) :: wlksiat           ! Energy of transition
  DATA wlksiat                                 /   110.951_DP,   210.135_DP,   321.086_DP &
                                               ,  8741.912_DP,  8952.047_DP,  9062.998_DP &
                                               , 13086.941_DP, 21828.853_DP, 22038.988_DP &
                                               , 21828.853_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntrsiat) :: aijsiat           ! Transition probability
  DATA aijsiat                                 / 8.25e-6_DP, 4.21e-5_DP, 3.56e-13_DP &
                                               , 2.25e-3_DP, 7.93e-4_DP, 4.70e-7_DP &
                                               , 1.14e-0_DP, 9.02e-4_DP, 3.13e-2_DP &
                                               , 0.0e0_DP /

  INTEGER, PUBLIC, DIMENSION (ntrsiat)        :: iupsiat           ! Upper state of transition
  DATA iupsiat                                 / 2, 3, 3, 4, 4, 4, 5, 5, 5, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrsiat)        :: jlosiat           ! Lower state of transition 
  DATA jlosiat                                 / 1, 2, 1, 3, 2, 1, 4, 3, 2, 1 /

  REAL (KIND=DP), PUBLIC, DIMENSION(ntrsiat,0:noptm) :: emisiat    ! Si Emissivity

! C+

! Energy levels.
!        1 - 2P J = 1/2
!        2 - 2P J = 3/2
!        3 - 4P J = 1/2
!        4 - 4P J = 3/2
!        5 - 4P J = 5/2

  INTEGER, PUBLIC, PARAMETER                 :: nlvcpl = 5       ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvcpl) :: gstcpl           ! Statistical weight
  DATA gstcpl                                 / 2.0_DP, 4.0_DP, 2.0_DP, 4.0_DP, 6.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvcpl) :: elkcpl           ! Level energy
  DATA elkcpl                                 / 0.0_DP, 91.25_DP, 61874.63_DP, 61906.28_DP &
                                              , 61947.00_DP /

  REAL (kind=DP), PUBLIC                     :: amcpl = 12.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(nlvcpl,0:noptm) :: xrecpl    ! C+ levels relative abundance

! Transitions
!       1: 2-1 -> 157.68 micron
!       2: 3-2 -> 2328.12 Angstrom
!       3: 4-2 -> 2326.93 Angstrom
!       4: 5-2 -> 2325.40 Angstrom
!       5: 3-1 -> 2324.69 Angstrom
!       6: 4-1 -> 2323.50 Angstrom
!       7: 5-1 ->

  INTEGER, PUBLIC, PARAMETER                 :: ntrcpl = 7       ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntrcpl) :: aijcpl           ! Transition probability
  DATA aijcpl                                 / 2.29e-6_DP, 6.55e+1_DP, 5.24e+0_DP, &
                                                4.32e+1_DP, 5.53e+1_DP, 1.71e-0_DP, 0.0e0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntrcpl) :: wlkcpl           ! Energy of transition
  DATA wlkcpl                                 / 91.25_DP, 61783.38_DP, 61815.03_DP, &
                                                61855.75_DP, 61874.63_DP, 61906.28_DP, 61947.00_DP /

  INTEGER, PUBLIC, DIMENSION (ntrcpl)        :: iupcpl           ! Upper state of transition
  DATA iupcpl                                 / 2, 3, 4, 5, 3, 4, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrcpl)        :: jlocpl           ! Lower state of transition 
  DATA jlocpl                                 / 1, 2, 2, 2, 1, 1, 1 /

  REAL (KIND=DP), PUBLIC, DIMENSION(ntrcpl,0:noptm) :: emicpl    ! C+ Emissivity

! N+
! Energy levels.
!        1 - 3P J = 0
!        2 - 3P J = 1
!        3 - 3P J = 2
!        4 - 1D J = 2
!        5 - 1S J = 0

  INTEGER, PUBLIC, PARAMETER                 :: nlvnpl = 5       ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvnpl) :: gstnpl           ! Statistical weight
  DATA gstnpl                                 / 1.0_DP, 3.0_DP, 5.0_DP, 5.0_DP, 1.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvnpl) :: elknpl           ! Level energy
  DATA elknpl                                 /     0.0_DP,     70.07_DP, 188.20_DP &
                                              , 22037.48_DP, 47033.77_DP /

  REAL (kind=DP), PUBLIC                     :: amnpl = 14.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(nlvnpl,0:noptm) :: xrenpl    ! N+ levels relative abundances

! Transitions
!       1: 2-1 -> 205.34 micron
!       2: 3-2 -> 121.80 micron
!       3: 3-1 -> 76.45 micron
!       4: 4-3 -> 6583.45 Angstrom
!       5: 4-2 -> 6548.05 Angstrom
!       6: 4-1 -> 6527.23 Angstrom
!       7: 5-4 -> 5754.59 Angstrom
!       8: 5-3 -> 3070.55 Angstrom
!       9: 5-2 -> 3062.83 Angstrom
!      10: 5-1 ->

  INTEGER, PUBLIC, PARAMETER                 :: ntrnpl = 10      ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntrnpl) :: wlknpl           ! Energy of transition
  DATA wlknpl                                 /    70.07_DP,   118.13_DP,   188.20_DP &
                                              , 21849.28_DP, 21967.41_DP, 22037.48_DP &
                                              , 24996.29_DP, 46845.57_DP, 46963.70_DP &
                                              , 47033.77_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntrnpl) :: aijnpl           ! Transition probability
  DATA aijnpl                                 / 2.08e-6_DP, 7.46e-6_DP, 1.16e-12_DP &
                                              , 2.99e-3_DP, 1.01e-3_DP, 5.35e-7_DP &
                                              , 1.12e-0_DP, 1.51e-4_DP, 3.38e-2_DP &
                                              , 0.0e0_DP /

  INTEGER, PUBLIC, DIMENSION (ntrnpl)        :: iupnpl           ! Upper state of transition
  DATA iupnpl                                 / 2, 3, 3, 4, 4, 4, 5, 5, 5, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrnpl)        :: jlonpl           ! Lower state of transition 
  DATA jlonpl                                 / 1, 2, 1, 3, 2, 1, 4, 3, 2, 1 /

  REAL (KIND=DP), PUBLIC, DIMENSION(ntrnpl,0:noptm) :: eminpl    ! N+ Emissivity

! O+
! Energy levels.
!        1 - 4So J = 3/2
!        2 - 2Do J = 5/2
!        3 - 2Do J = 3/2
!        4 - 2Po J = 3/2
!        5 - 2Po J = 1/2

  INTEGER, PUBLIC, PARAMETER                 :: nlvopl = 5       ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvopl) :: gstopl           ! Statistical weight
  DATA gstopl                                 / 4.0_DP, 6.0_DP, 4.0_DP, 4.0_DP, 2.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvopl) :: elkopl           ! Level energy
  DATA elkopl                                 /     0.0_DP,  38575.94_DP, 38604.75_DP &
                                              , 58226.77_DP, 58229.63_DP /

  REAL (kind=DP), PUBLIC                     :: amopl = 16.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(nlvopl,0:noptm) :: xreopl    ! O+ levels relative abundances

! Transitions
!       1: 5-4 -> 5.03 mm (59.66 GHz)
!       2: 3-2 -> 0.4995 mm (600.18 GHz)
!       3: 4-3 -> 7330.73 Angstrom
!       4: 5-3 -> 7329.67 Angstrom
!       5: 4-2 -> 7319.99 Angstrom
!       6: 5-2 -> 7318.92 Angstrom
!       7: 2-1 -> 3728.815 Angstrom
!       8: 3-1 -> 3726.032 Angstrom
!       9: 4-1 -> 2470.341 Angstrom
!      10: 5-1 -> 2470.219 Angstrom

  INTEGER, PUBLIC, PARAMETER                 :: ntropl = 10      ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntropl) :: wlkopl           ! Energy of transition
  DATA wlkopl                                 / 2.86_DP, 28.81_DP, 19622.02_DP &
                                              , 19624.88_DP, 19650.83_DP, 19653.69_DP &
                                              , 38575.94_DP, 38604.75_DP, 58226.77_DP &
                                              , 58229.63_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntropl) :: aijopl           ! Transition probability
  DATA aijopl                                 / 2.08e-11_DP, 1.20e-7_DP, 6.14e-2_DP &
                                              , 1.02e-1_DP, 1.17e-1_DP, 6.15e-2_DP &
                                              , 3.82e-5_DP, 1.65e-4_DP, 5.64e-2_DP &
                                              , 2.32e-2_DP /

  INTEGER, PUBLIC, DIMENSION (ntropl)        :: iupopl           ! Upper state of transition
  DATA iupopl                                 / 5, 3, 4, 5, 4, 5, 2, 3, 4, 5 /

  INTEGER, PUBLIC, DIMENSION (ntropl)        :: jloopl           ! Lower state of transition 
  DATA jloopl                                 / 4, 2, 3, 3, 2, 2, 1, 1, 1, 1 /

  REAL (KIND=DP), PUBLIC, DIMENSION(ntropl,0:noptm) :: emiopl    ! O+ Emissivity

! S+

! Energy levels.
!        1 - 4So J = 3/2
!        2 - 2Do J = 3/2
!        3 - 2Do J = 5/2
!        4 - 2Po J = 1/2
!        5 - 2Po J = 3/2

  INTEGER, PUBLIC, PARAMETER                 :: nlvspl = 5       ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvspl) :: gstspl           ! Statistical weight
  DATA gstspl                                 / 4.0_DP, 4.0_DP, 6.0_DP, 2.0_DP, 4.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvspl) :: elkspl           ! Level energy
  DATA elkspl                                 / 0.0_DP, 21370.92_DP, 21416.66_DP &
                                              , 35287.17_DP, 35354.38_DP /

  REAL (kind=DP), PUBLIC                     :: amspl = 32.0_DP  ! Mass (amu)

  REAL (KIND=DP), PUBLIC, DIMENSION(nlvspl,0:noptm) :: xrespl    ! C+ levels relative abundance

! Transitions
!       1: 3-2 -> 0.31456 mm (953.04 GHz)
!       2: 5-4 -> 0.21409 mm (1400.33 GHz)
!       3: 4-3 -> 1.0373 micron
!       4: 4-2 -> 1.0339 micron
!       5: 5-3 -> 1.0323 micron
!       6: 5-2 -> 1.0290 micron
!       7: 2-1 -> 6730.82 Angstrom
!       8: 3-1 -> 6716.44 Angstrom
!       9: 4-1 -> 4076.35 Angstrom
!      10: 5-1 -> 4068.60 Angstrom

  INTEGER, PUBLIC, PARAMETER                 :: ntrspl = 10      ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntrspl) :: wlkspl           ! Energy of transition
  DATA wlkspl                                 / 45.74_DP, 67.21_DP, 13870.51_DP &
                                              , 13916.25_DP, 13937.72_DP, 13983.46_DP &
                                              , 21370.92_DP, 21416.66_DP, 35287.17_DP &
                                              , 35354.38_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntrspl) :: aijspl           ! Transition probability
  DATA aijspl                                 / 3.35e-7_DP, 1.03e-6_DP, 7.79e-2 &
                                              , 1.63e-1_DP, 1.79e-1_DP, 1.33e-1_DP &
                                              , 8.82e-4_DP, 2.60e-4_DP, 9.06e-2_DP &
                                              , 2.25e-1_DP /

  INTEGER, PUBLIC, DIMENSION (ntrspl)        :: iupspl           ! Upper state of transition
  DATA iupspl                                 / 3, 5, 4, 4, 5, 5, 2, 3, 4, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrspl)        :: jlospl           ! Lower state of transition 
  DATA jlospl                                 / 2, 4, 3, 2, 3, 2, 1, 1, 1, 1 /

  REAL (KIND=DP), PUBLIC, DIMENSION(ntrspl,0:noptm) :: emispl    ! C+ Emissivity

! Si+
! Energy levels.
!        1 - 2P J = 1/2
!        2 - 2P J = 3/2
!        3 - 4P J = 1/2
!        4 - 4P J = 3/2
!        5 - 4P J = 5/2

  INTEGER, PUBLIC, PARAMETER                 :: nlvsip = 5       ! Number of levels

  REAL (kind=DP), PUBLIC, DIMENSION (nlvsip) :: gstsip           ! Statistical weight
  DATA gstsip                                 / 2.0_DP, 4.0_DP, 2.0_DP, 4.0_DP, 6.0_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (nlvsip) :: elksip           ! Level energy
  DATA elksip                                 /     0.0_DP,    413.29_DP, 61617.06_DP &
                                              , 61772.93_DP, 62025.14_DP /

  REAL (kind=DP), PUBLIC                     :: amsip = 28.0_DP  ! Mass (amu)

! Transitions
!       1: 2-1 -> 34.814 micron
!       2: 3-2 -> 2350.17 Angstrom
!       3: 4-2 -> 2344.20 Angstrom
!       4: 5-2 -> 2334.60 Angstrom
!       5: 3-1 -> 2334.40 Angstrom
!       6: 4-1 -> 2328.52 Angstrom

  REAL (KIND=DP), PUBLIC, DIMENSION(nlvsip,0:noptm) :: xresip    ! Si+ levels relative abundances

  INTEGER, PUBLIC, PARAMETER                 :: ntrsip = 6       ! Number of transitions

  REAL (kind=DP), PUBLIC, DIMENSION (ntrsip) :: wlksip           ! Energy of transition
  DATA wlksip                                 /   413.29_DP, 61203.77_DP, 61359.64_DP &
                                              , 61611.85_DP, 61617.06_DP, 61772.93_DP /

  REAL (kind=DP), PUBLIC, DIMENSION (ntrsip) :: aijsip           ! Transition probability
  DATA aijsip                                 / 2.13e-4_DP, 4.9e+3_DP, 1.7e+3_DP &
                                              , 2.7e+3_DP,  6.3e+3_DP, 2.0e+1_DP /

  INTEGER, PUBLIC, DIMENSION (ntrsip)        :: iupsip           ! Upper state of transition
  DATA iupsip                                 / 2, 3, 4, 5, 3, 4 /

  INTEGER, PUBLIC, DIMENSION (ntrsip)        :: jlosip           ! Lower state of transition 
  DATA jlosip                                 / 1, 2, 2, 2, 1, 1 /

  REAL (KIND=DP), PUBLIC, DIMENSION(ntrsip,0:noptm) :: emisip    ! Si+ Emissivity

END MODULE STR_DATA
