
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

c      %%%%%%%%%%%%%%%%%%%%%%%%
       subroutine FGKTR2 (iopt)
c      %%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE SP_DATA
       USE H2_DATA
       USE HD_DATA
       USE STR_DATA
       USE PROFIL

c---------------------------------------------------------------------
c       calcul de l absorption par les raies methode FGK
c       Nuage eclaire des deux cotes
c       Le champ de rayonnement est defini dans stv1y.f90
c       Le developpement en polynomes de Legendre est fait dans stv1c.f
c---------------------------------------------------------------------

        implicit double precision (a-h, o-z)

c Inutile de transmettre TOUS les niveaux pour l'interpolation

        real (KIND=DP) cdh2f(nlevh2)
        real (KIND=DP) cdhdf(nlevhd)
        real (KIND=DP) cdc0f(0:njco)
        real (KIND=DP) cdc1f(0:njco)
        real (KIND=DP) cdc2f(0:njco)
        real (KIND=DP) cdc3f(0:njco)
        real (KIND=DP) cdh2r(nlevh2)
        real (KIND=DP) cdhdr(nlevhd)
        real (KIND=DP) cdc0r(0:njco)
        real (KIND=DP) cdc1r(0:njco)
        real (KIND=DP) cdc2r(0:njco)
        real (KIND=DP) cdc3r(0:njco)
        real (KIND=DP) cdtau(0:noptm)
        real (KIND=DP) cdinf

        real (KIND=DP) interpcd, ress
        real (KIND=DP) ajdf, ajrf
        real (KIND=DP) vt2, aux1, aux2
        real (KIND=DP) nu0, nu

        real (KIND=DP)                    :: avoi
        real (KIND=DP), dimension(:), allocatable :: xwl, yvoi, xsum
        real (kind=DP), dimension(0:nwlg) :: absp1, absp2
        real (kind=DP), dimension(0:nwlg) :: bbsp1, bbsp2

        TYPE (RF_TYPE)  :: rf_tmp

        save

        data  ff1, ff2, ff3   / 3.02d0, 1.0d3, 6.4d-2 /

c  Point d'interpolation dans la grille precedente (inutile de le refaire a chaque fois)

         do j=1,npo
           if (tau(iopt) <= tau_o(j)) then
             icd_o = j-1
             exit
           endif
         end do

         raptau = (tau(iopt) - tau_o(icd_o))
     1           / (tau_o(icd_o+1) - tau_o(icd_o))

c   calcul de l attenuation du champ de rayonnement entre le
c   bord du nuage et le point courant
c   Voir Flannery, Roberge, Rybicki, Ap J, 236, 598 (1980)
c   abspm and abpspp computed in stv1c.f

       vt2 = vturb * vturb

       absp1 = 0.0_DP
       absp2 = 0.0_DP
       bbsp1 = 0.0_DP
       bbsp2 = 0.0_DP
       do iwlg=0,nwlg
         absp1(iwlg) = interpcd(raptau, abspm(0:npo,iwlg), icd_o, npo)
         absp2(iwlg) = interpcd(raptau, abspp(0:npo,iwlg), icd_o, npo)
         rf_dray%Val(iwlg) = rf_draypr%Val(iwlg)
     1                     * (absp1(iwlg) + absp2(iwlg))
         bbsp1(iwlg) = interpcd(raptau, bbspm(0:npo,iwlg), icd_o, npo)
         bbsp2(iwlg) = interpcd(raptau, bbspp(0:npo,iwlg), icd_o, npo)
       end do

       call rf_create(rf_tmp, rf_wl%lower, rf_wl%upper)
       rf_tmp%Ord = rf_dray%Ord
       do mm=1,npg
         rf_tmp%Val = rf_dray%Val
         call sect_pri(sigabs(mm), rf_tmp, wl_pointer, si_, nwlg)
         si_QU(mm,:) = si_
         rf_tmp%Val = rf_tmp%Val * rf_wl%Val
         call sect_pri(sigabs(mm), rf_tmp, wl_pointer, si_, nwlg)
         si_QUl(mm,:) = si_
         rf_tmp%Val = rf_tmp%Val * rf_wl%Val
         call sect_pri(sigabs(mm), rf_tmp, wl_pointer, si_, nwlg)
         si_QUl2(mm,:) = si_
       end do
       call rf_free (rf_tmp)

c     *********************************************************************
c               ct1= pi e^2/mc = 2.654057d-2
c                  =(section d absorption totale integree sur les frequences)
c                    en cm^2
c               vturb donne en cm s-1
c               ct2= 2k/mua   = 1.662902e8  en cgs
c               bdop:  dimension vitesse cm s-1 a
c      *****************************************************************

c  CDUNIT proportion gaz poussiere: NH = CDUNIT * E(B-V)
c  RRR proportion indice de couleur extinction: Av = RRR * E(B-V)

       detier = deux / trois
       ct1 = xpi * xqe2 / xme / clum
       ct2 = deux * xk / amu
       ct3 = cdunit * tautav / rrr

c  calcul de la colonne densite de h2 pour chaque niveau
c    ponderee par la largeur doppler

c  If "one side", then the true optical depth towards "dark" side is high

       if (isoneside == 0) then
         cdinf = 1.0_dp
       else
         cdinf = 10.0_dp
       endif

c  rapport de la largeur doppler turbulente a la largeur doppler totale
c    independant de la raie car lambda se simplifie!

       do lev=1,l0h2
         cdtau(0:npo) = cdth2_o(lev,:)
         ress = interpcd(raptau, cdtau(0:npo), icd_o, npo)
         cdh2f(lev) = ress
         cdh2r(lev) = cdinf * cdtau(npo) - ress
       end do

c  calcul de la colonne densite de hd pour chaque niveau
c    ponderee par la largeur doppler

       if (itrhd .ne. 0) then
         do lev=1,l0hd
           cdtau(0:npo) = cdthd_o(lev,:)
           ress = interpcd(raptau, cdtau(0:npo), icd_o, npo)
           cdhdf(lev) = ress
           cdhdr(lev) = cdinf * cdtau(npo) - ress
         end do
       endif

c  calcul de la colonne densite de co pour chaque j

       do j=0,njco
         cdtau(0:npo) = cdtc0_o(j,:)
         ress = interpcd(raptau, cdtau(0:npo), icd_o, npo)
         cdc0f(j) = ress
         cdc0r(j) = cdinf * cdtau(npo) - ress
       end do

c  calcul de la colonne densite de co1 pour chaque j

       if (i_c13 .ne. 0) then

         do j=0,njco
           cdtau(0:npo) = cdtc1_o(j,:)
           ress = interpcd(raptau, cdtau(0:npo), icd_o, npo)
           cdc1f(j) = ress
           cdc1r(j) = cdinf * cdtau(npo) - ress
         end do

       endif

c  calcul de la colonne densite de co2 pour chaque j

       if (i_o18 .ne. 0) then

         do j=0,njco
           cdtau(0:npo) = cdtc2_o(j,:)
           ress = interpcd(raptau, cdtau(0:npo), icd_o, npo)
           cdc2f(j) = ress
           cdc2r(j) = cdinf * cdtau(npo) - ress
         end do

       endif

c  calcul de la colonne densite de co3 pour chaque j

       if (i_c13 .ne. 0 .and. i_o18 .ne. 0) then

         do j=0,njco
           cdtau(0:npo) = cdtc3_o(j,:)
           ress = interpcd(raptau, cdtau(0:npo), icd_o, npo)
           cdc3f(j) = ress
           cdc3r(j) = cdinf * cdtau(npo) - ress
         end do

       endif

c  rappel:
c     irah2(bc) 2:       v du niveau bas.
c               3:       j du niveau bas.
c               4:       v du niveau haut.
c               5:       delta j (jhaut = jbas + delj)
c     prah2(bc) 1:       force d oscillateur (verifier dimension).
c               2:       longueur d onde (en a).
c               3:       inverse de la duree de vie du niveau haut.
c               4:       probabilite de discoc. du niveau haut.

c  calcul des profils de raies de h2.

       bdop = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amh2)

       do i=1,nrh2bt
         njl = irah2b(i,2)
         nvl = irah2b(i,1)
         lev = level(nvl,njl)
         nju = njl + irah2b(i,4)
         xlam0 = prah2b(i,2)
         gama = prah2b(i,3) / (quatre * xpi)
         betas = bdop / xlam0

         if (nvl > 0 .OR. njl >= jfgkh2) then
           betas0 = 1.0d8 * vturb / xlam0
           r     = gama / (sqpi * betas)
           t1    = ff1 * (ff2 * r) ** (-ff3)
           opt0  = ct1 * prah2b(i,1) / (sqpi * betas0)
           taudf = opt0 * cdh2f(lev)
           u1f   = sqrt(taudf * r) / t1
           ajrf  = (r / t1) / sqrt(u1f * u1f + xpi / quatre)
           taudfr = opt0 * cdh2r(lev)
           u1fr   = sqrt(taudfr * r) / t1
           ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / quatre)

           fgkrf(i) = min(ajdf(taudf) + ajrf, un)
           fgkrfr(i) = min(ajdf(taudfr) + ajrfr, un)
         else
           nu0 = 1.0e8_dp * clum / xlam0
           avoi = prah2b(i,3) / (quatre * xpi * betas)
           wlmin = 1.0e8_dp * clum / (nu0 + wlrwid * betas)
           wlmax = 1.0e8_dp * clum / (nu0 - wlrwid * betas)
           call rf_locate (rf_wl, wlmin, ilmin)
           call rf_locate (rf_wl, wlmax, ilmax)
           nbil = ilmax-ilmin
           allocate (xwl(0:nbil), yvoi(0:nbil), xsum(0:nbil))
           do j=0,nbil
             nu = 1.0e8_dp * clum / rf_wl%Val(j+ilmin)
             xwl(j) = (nu0 - nu) / betas
           end do

           call humlik (nbil+1, xwl, avoi, yvoi)

           xsum = yvoi * bbsp1(ilmin:ilmax)
           call simpint(xwl, xsum, nbil, tata)
           fgkrf(i) = tata

           xsum = yvoi * bbsp2(ilmin:ilmax)
           call simpint(xwl, xsum, nbil, tata)
           fgkrfr(i) = tata

           deallocate (xwl, yvoi, xsum)
         endif

       end do

       krai = nrh2bt
 
       do i=1,nrh2ct
         njl = irah2c(i,2)
         nvl = irah2c(i,1)
         lev = level(nvl,njl)
         nju = njl + irah2c(i,4)
         xlam0 = prah2c(i,2)
         gama = prah2c(i,3) / (quatre * xpi)
         betas = bdop / xlam0
         ikh2 = krai + i

         if (nvl > 0 .OR. njl >= jfgkh2) then
           betas0 = 1.0d8 * vturb / xlam0
           r     = gama / (sqpi * betas)
           t1    = ff1 * (ff2 * r) ** (-ff3)
           opt0  = ct1 * prah2c(i,1) / (sqpi * betas0)
           taudf = opt0 * cdh2f(lev)
           u1f   = sqrt(taudf * r) / t1
           ajrf  = (r / t1) / sqrt(u1f * u1f + xpi / quatre)
           taudfr = opt0 * cdh2r(lev)
           u1fr   = sqrt(taudfr * r) / t1
           ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / quatre)

c          ikh2 = krai + i                 ! FLP le 8 VII 2004
           fgkrf(ikh2) = min(ajdf(taudf) + ajrf, un)
           fgkrfr(ikh2) = min(ajdf(taudfr) + ajrfr, un)
         else
           nu0 = 1.0e8_dp * clum / xlam0
           avoi = prah2c(i,3) / (quatre * xpi * betas)
           wlmin = 1.0e8_dp * clum / (nu0 + wlrwid * betas)
           wlmax = 1.0e8_dp * clum / (nu0 - wlrwid * betas)
           call rf_locate (rf_wl, wlmin, ilmin)
           call rf_locate (rf_wl, wlmax, ilmax)
           nbil = ilmax-ilmin
           allocate (xwl(0:nbil), yvoi(0:nbil), xsum(0:nbil))
           do j=0,nbil
             nu = 1.0e8_dp * clum / rf_wl%Val(j+ilmin)
             xwl(j) = (nu0 - nu) / betas
           end do

           call humlik (nbil+1, xwl, avoi, yvoi)

           xsum = yvoi * bbsp1(ilmin:ilmax)
           call simpint(xwl, xsum, nbil, tata)
           fgkrf(ikh2) = tata

           xsum = yvoi * bbsp2(ilmin:ilmax)
           call simpint(xwl, xsum, nbil, tata)
           fgkrfr(ikh2) = tata

           deallocate (xwl, yvoi, xsum)
         endif

       end do

       krai = krai + nrh2ct
 
c-------  rappel: ----------------------------------------------------
c     irahd(bc) 1:       v du niveau bas.
c               2:       j du niveau bas.
c               3:       v du niveau haut.
c               4:       delta j (jhaut = jbas + delj)
c     prahd(bc) 1:       force d oscillateur (verifier dimension).
c               2:       longueur d onde (en a).
c               3:       inverse de la duree de vie du niveau haut.
c               4:       probabilite de discoc. du niveau haut.
 
c  calcul des profils de raies de hd.
 
       if (itrhd .ne. 0) then
 
       bdop = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amhd)
 
       do i=1,nrhdbt
         njl = irahdb(i,2)
         nvl = irahdb(i,1)
         lev = levhd(nvl,njl)
         nju = njl + irahdb(i,4)
         xlam0 = prahdb(i,2)
         gama = prahdb(i,3) / (quatre * xpi)
         betas = bdop / xlam0
         betas0 = 1.0d8 * vturb / xlam0
         r     = gama / (sqpi * betas)
         t1    = ff1 * (ff2 * r) ** (-ff3)
         opt0  = ct1 * prahdb(i,1) / (sqpi * betas0)
c        taudf = opt0 * cdthd(lev,iopt)
         taudf = opt0 * cdhdf(lev)
         u1f   = sqrt(taudf * r) / t1
         ajrf  = (r / t1) / sqrt(u1f * u1f + xpi / quatre)
         taudfr = opt0 * cdhdr(lev)
         u1fr   = sqrt(taudfr * r) / t1
         ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / quatre)
 
         ikhd = krai + i
         fgkrf(ikhd) = min(ajdf(taudf) + ajrf, un)
         fgkrfr(ikhd) = min(ajdf(taudfr) + ajrfr, un)
 
         end do
 
         krai = krai + nrhdbt

       do i=1,nrhdct
         njl = irahdc(i,2)
         nvl = irahdc(i,1)
         lev = levhd(nvl,njl)
         nju = njl + irahdc(i,4)
         xlam0 = prahdc(i,2)
         gama = prahdc(i,3) / (quatre * xpi)
         betas = bdop / xlam0
         betas0 = 1.0d8 * vturb / xlam0
         r     = gama / (sqpi * betas)
         t1    = ff1 * (ff2 * r) ** (-ff3)
         opt0  = ct1 * prahdc(i,1) / (sqpi * betas0)
c        taudf = opt0 * cdthd(lev,iopt)
         taudf = opt0 * cdhdf(lev)
         u1f   = sqrt(taudf * r) / t1
         ajrf  = (r / t1) / sqrt(u1f * u1f + xpi / quatre)
         taudfr = opt0 * cdhdr(lev)
         u1fr   = sqrt(taudfr * r) / t1
         ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / quatre)
 
         ikhd = krai + i
         fgkrf(ikhd) = min(ajdf(taudf) + ajrf, un)
         fgkrfr(ikhd) = min(ajdf(taudfr) + ajrfr, un)
 
         end do
 
         krai = krai + nrhdct

       endif

c  calcul des profils de raies de co.

       if (itrco0 .ne. 0) then

       bdop  = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amco0)

       do i=1, nrco0
         njl   = irco0(i)
         gama  = prco0(i,3) / (quatre * xpi)
         xlam0 = prco0(i,2)

         betas = bdop / xlam0
         betas0 = 1.0d8 * vturb / xlam0
         r     = gama / (sqpi * betas)
         t1    = ff1 * (ff2*r) ** (-ff3)
         opt0  = ct1 * prco0(i,1) / (sqpi * betas0)
c        taudf = opt0 * cdtc0(njl,iopt)
         taudf = opt0 * cdc0f(njl)
         u1f   = sqrt(taudf * r) / t1
         ajrf  = (r / t1) / sqrt(u1f * u1f + xpi / quatre)
         taudfr = opt0 * cdc0r(njl)
         u1fr   = sqrt(taudfr * r) / t1
         ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / quatre)
c
         ikco = krai + i
         fgkrf(ikco) = min(ajdf(taudf) + ajrf, un)
         fgkrfr(ikco) = min(ajdf(taudfr) + ajrfr, un)
       end do

       krai = krai + nrco0

       endif

c  calcul des profils de raies de co1.

       if (itrco1 .ne. 0) then

       bdop = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amco1)

       do i=1, nrco1
         njl   = irco1(i)
         gama  = prco1(i,3) / (quatre * xpi)
         xlam0  = prco1(i,2)

         betas  = bdop / xlam0
         betas0 = 1.0d8 * vturb / xlam0
         r      = gama / (sqpi*betas)
         t1     = ff1 * (ff2*r) ** (-ff3)
         opt0   = ct1 * prco1(i,1) / (sqpi * betas0)
c        taudf  = opt0 * cdtc1(njl,iopt)
         taudf  = opt0 * cdc1f(njl)
         u1f    = sqrt(taudf * r) / t1
         ajrf   = (r / t1) / sqrt(u1f * u1f + xpi / quatre)
         taudfr = opt0 * cdc1r(njl)
         u1fr   = sqrt(taudfr * r) / t1
         ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / quatre)

         ikco1 = krai + i
         fgkrf(ikco1) = min(ajdf(taudf) + ajrf, un)
         fgkrfr(ikco1) = min(ajdf(taudfr) + ajrfr, un)
       end do

       krai = krai + nrco1

       endif

c  calcul des profils de raies de co2.

       if (itrco2 .ne. 0) then

       bdop = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amco2)
 
       do i=1, nrco2
         njl   = irco2(i)
         gama  = prco2(i,3) / (quatre * xpi)
         xlam0  = prco2(i,2)

         betas  = bdop / xlam0
         betas0 = 1.0d8 * vturb / xlam0
         r      = gama / (sqpi*betas)
         t1     = ff1 * (ff2*r) ** (-ff3)
         opt0   = ct1 * prco2(i,1) / (sqpi * betas0)
c        taudf  = opt0 * cdtc2(njl,iopt)
         taudf  = opt0 * cdc2f(njl)
         u1f    = sqrt(taudf * r) / t1
         ajrf   = (r / t1) / sqrt(u1f * u1f + xpi / quatre)
         taudfr = opt0 * cdc2r(njl)
         u1fr   = sqrt(taudfr * r) / t1
         ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / quatre)

         ikco2 = krai + i
         fgkrf(ikco2) = min(ajdf(taudf) + ajrf, un)
         fgkrfr(ikco2) = min(ajdf(taudfr) + ajrfr, un)
       end do

       krai = krai + nrco2

       endif

c  calcul des profils de raies de co3.

       if (itrco3 .ne. 0) then

       bdop = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amco3)
 
       do i=1, nrco3
         njl   = irco3(i)
         gama  = prco3(i,3) / (quatre * xpi)
         xlam0  = prco3(i,2)

         betas  = bdop / xlam0
         betas0 = 1.0d8 * vturb / xlam0
         r      = gama / (sqpi*betas)
         t1     = ff1 * (ff2*r) ** (-ff3)
         opt0   = ct1 * prco3(i,1) / (sqpi * betas0)
c        taudf  = opt0 * cdtc3(njl,iopt)
         taudf  = opt0 * cdc3f(njl)
         u1f    = sqrt(taudf * r) / t1
         ajrf   = (r / t1) / sqrt(u1f * u1f + xpi / quatre)
         taudfr = opt0 * cdc3r(njl)
         u1fr   = sqrt(taudfr * r) / t1
         ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / quatre)

         ikco3 = krai + i
         fgkrf(ikco3) = min(ajdf(taudf) + ajrf, un)
         fgkrfr(ikco3) = min(ajdf(taudfr) + ajrfr, un)
       end do

       endif

c  calcul du champ de rayonnement dans la couche

       do iwlg=1,nrh2bt
         nvl = irah2b(iwlg,1)
         njl = irah2b(iwlg,2)
         if (nvl > 0 .OR. njl >= jfgkh2) then
           xlam0 = fgkwl(iwlg)
           call rf_locate(rf_wl, xlam0, jwl)
           rapwl = (xlam0 - rf_wl%Val(jwl))
     1           / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
           aux1 = absp1(jwl) + (absp1(jwl+1) - absp1(jwl)) * rapwl
           aux2 = absp2(jwl) + (absp2(jwl+1) - absp2(jwl)) * rapwl
           fgkgrd(iwlg) = aux1 * fgkrf(iwlg) + aux2 * fgkrfr(iwlg)
         endif
       end do
       do iwlg=nrh2bt+1,nrh2bt+nrh2ct
         nvl = irah2c(iwlg-nrh2bt,1)
         njl = irah2c(iwlg-nrh2bt,2)
         if (nvl > 0 .OR. njl >= jfgkh2) then
           xlam0 = fgkwl(iwlg)
           call rf_locate(rf_wl, xlam0, jwl)
           rapwl = (xlam0 - rf_wl%Val(jwl))
     1           / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
           aux1 = absp1(jwl) + (absp1(jwl+1) - absp1(jwl)) * rapwl
           aux2 = absp2(jwl) + (absp2(jwl+1) - absp2(jwl)) * rapwl
           fgkgrd(iwlg) = aux1 * fgkrf(iwlg) + aux2 * fgkrfr(iwlg)
         endif
       end do
       do iwlg=nrh2bt+nrh2ct+1,mfgk
         xlam0 = fgkwl(iwlg)
         call rf_locate(rf_wl, xlam0, jwl)
         rapwl = (xlam0 - rf_wl%Val(jwl))
     1         / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
         aux1 = absp1(jwl) + (absp1(jwl+1) - absp1(jwl)) * rapwl
         aux2 = absp2(jwl) + (absp2(jwl+1) - absp2(jwl)) * rapwl
         fgkgrd(iwlg) = aux1 * fgkrf(iwlg) + aux2 * fgkrfr(iwlg)
       end do

       return
       end

c      %%%%%%%%%%%%%%%%%%%%%%%%
       subroutine FGKDAT (iopt)
c      %%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE SP_DATA
       USE H2_DATA
       USE HD_DATA
       USE STR_DATA
       USE PROFIL

c       preparation des donnees pour l equilibre de h2, hd, co
c       et calcule du taux de photodissociation de h2, hd, co.
c       et des isotopes

        implicit double precision (a-h,o-z)

        TYPE (RF_TYPE)  :: rf_tmp, rf_tmp1

        real (kind=dp)  :: coefb
        real (KIND=DP), dimension(:), allocatable :: xwl, yvoi, xsum
        real (KIND=DP) nu0, nu

        save

c  coef numerique: pi e**2 / (me * h * c**2) * 1.0e-16 (pour les Angstrom)

        coefb = 1.0e-16_dp * xpi * xqe2 / (xme * xhp * clum*clum)
        vt2 = vturb * vturb

c  Calcul du flux de photons (utilise dans la chimie)
        call rf_create(rf_tmp, 1, 2)
        call rf_locate(rf_wl, wl_6ev, i2)
        rf_tmp%Val(2) = 1.0_dp
        rf_tmp%Ord(2) = rf_wl%Ord(i2)
        int_rad(iopt) = sect_int(rf_tmp, rf_dray, wl_pointer, wl_6ev)
        call rf_free (rf_tmp)

        call rf_create(rf_tmp1, rf_wl%lower, rf_wl%upper)
c       rf_tmp1%Val = rf_dray%Val
c       int_rad(iopt) = rf_integration (rf_wl,rf_tmp1)
        rf_tmp1%Val = rf_dray%Val * rf_wl%Val
        fluph(iopt) = rf_integration (rf_wl,rf_tmp1) * 1.0d-8 / xhp
        call rf_free (rf_tmp1)

c  calcul du taux de photoionisation de C et S
c    section efficace d ionisation de C (en cm2): sicion = 1.6d-17
c  d apres Hofmann et al A & A 126, 415
c    section efficace d ionisation de S d'apres TopBase (C.N.)

c  Recalcul avec les nouvelles variables - 14 Jan au 12 Fev 2002

        CALL rf_create (rf_tmp1, 0, nwlg)
        rf_tmp1%Val = rf_wl%Val * rf_dray%Val
        pdicat(iopt) = sect_int (rf_sig_C, rf_tmp1, wl_pointer
     1                         , wl_cut_C) / (1.0e8_DP * xhp)
        pdisat(iopt) = sect_int (rf_sig_S, rf_tmp1, wl_pointer
     1                         , wl_cut_S) / (1.0e8_DP * xhp)

        call rf_free(rf_tmp1)

c  Fin de la modif - 14 Jan au 12 Fev 2002

        ct2 = deux * xk / amu
        detier = deux / trois

c  pompage radiatif
c    traitement de h2

        evrdbc(1:l0h2,1:l0h2) = zero
        if (i_hd .ne. 0) then
          evrdhd(1:l0hd,1:l0hd) = zero
        endif

        eqrco0 = zero
        eqrco1 = zero
        eqrco2 = zero
        eqrco3 = zero

        dumh2 = zero
        dumhd = zero
        dumy = zero
        dumco = zero
        dumco1 = zero
        dumco2 = zero
        dumco3 = zero
        pdish2(iopt) = zero
        pdh2vJ(:,iopt) = zero
        sdh2vJ = zero
        pdishd(iopt) = zero
        pdico0(iopt) = zero
        pdico1(iopt) = zero
        pdico2(iopt) = zero
        pdico3(iopt) = zero

c  probabilite de dissociation de h2
c  ---------------------------------

        bdop = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amh2)

        do ik=1,nrh2bt
          nvl = irah2b(ik,1)
          njl = irah2b(ik,2)
          lev = level(nvl,njl)
          xlam0 = prah2b(ik,2)

          if (nvl > 0 .OR. njl >= jfgkh2) then
            call rf_locate(rf_wl, xlam0, jwl)
            rapwl = (xlam0 - rf_wl%Val(jwl))
     1            / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
            rfint = rf_draypr%Val(jwl)
     1            + (rf_draypr%Val(jwl+1) - rf_draypr%Val(jwl)) * rapwl
            chrai = fgkgrd(ik) * rfint
          else
            nu0 = 1.0e8_dp * clum / xlam0
            betas = bdop / xlam0
            avoi = prah2b(ik,3) / (quatre * xpi * betas)
            wlmin = 1.0e8_dp * clum / (nu0 + wlrwid * betas)
            wlmax = 1.0e8_dp * clum / (nu0 - wlrwid * betas)
            call rf_locate (rf_wl, wlmin, ilmin)
            call rf_locate (rf_wl, wlmax, ilmax)
            nbil = ilmax-ilmin
            allocate (xwl(0:nbil), yvoi(0:nbil), xsum(0:nbil))
            do j=0,nbil
              nu = 1.0e8_dp * clum / rf_wl%Val(j+ilmin)
              xwl(j) = (nu0 - nu) / betas
            end do

            call humlik (nbil+1, xwl, avoi, yvoi)

            xsum = yvoi * rf_dray%Val(ilmin:ilmax)
     1           * 1.0e8 * clum / (betas * rf_wl%Val(ilmin:ilmax)**2)
            call simpint(rf_wl%Val(ilmin:ilmax), xsum, nbil, tata)
            chrai = tata
            deallocate (xwl, yvoi, xsum)
          endif

          nvu = irah2b(ik,3)
          ndj = irah2b(ik,4)
          nju = njl + ndj

          probdi = prah2b(ik,4)
          beinst = coefb * prah2b(ik,1) * (xlam0**3)
          absorb = beinst * chrai
          disici = absorb * probdi
          dumh2 = dumh2 + disici * xreh2(lev,iopt)
          pdh2vJ(lev,iopt) = pdh2vJ(lev,iopt)
     1                     + disici * xreh2(lev,iopt)
          if (iopt == 0) then
            sdh2vJ(lev) = sdh2vJ(lev) + beinst * probdi
          endif

          ! calcul explicite des populations de la bande B
          if ((nvu .le. nvbc) .and. (nju .le. njbc)) then
            h2bnua(nvu,nju,iopt) = h2bnua(nvu,nju,iopt) +
     1                            absorb * xreh2(lev,iopt)
          endif

c  construction de la matrice d equilibre radiatif

          evrdbc(lev,lev) = evrdbc(lev,lev) + absorb

          dumy = absorb * (un - probdi)
          if (mod(njl,2) == 0) then
            evrdbc(levpc,lev) = evrdbc(levpc,lev)
     1                       - dumy * bhmbx(nvu,nju,levpc)
          else
            evrdbc(levic,lev) = evrdbc(levic,lev)
     1                        - dumy * bhmbx(nvu,nju,levic)
          endif
        end do

        !fin du calcul de la pop des niveaux hauts
        do ivu=0,nvbc
          do iju=0, njbc
             h2bnua(ivu,iju,iopt) = h2bnua(ivu,iju,iopt)
     1                                / tmunh2bc(1,ivu,iju)
          enddo
        enddo

        krai = nrh2bt
 
        do ik=1,nrh2ct
          ikh2 = krai + ik
          nvl = irah2c(ik,1)
          njl = irah2c(ik,2)
          lev = level(nvl,njl)
          xlam0 = prah2c(ik,2)

          if (nvl > 0 .OR. njl >= jfgkh2) then
            call rf_locate(rf_wl, xlam0, jwl)
            rapwl = (xlam0 - rf_wl%Val(jwl))
     1            / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
            rfint = rf_draypr%Val(jwl)
     1            + (rf_draypr%Val(jwl+1) - rf_draypr%Val(jwl)) * rapwl
            chrai = fgkgrd(ikh2) * rfint
          else
            nu0 = 1.0e8_dp * clum / xlam0
            betas = bdop / xlam0
            avoi = prah2c(ik,3) / (quatre * xpi * betas)
            wlmin = 1.0e8_dp * clum / (nu0 + wlrwid * betas)
            wlmax = 1.0e8_dp * clum / (nu0 - wlrwid * betas)
            call rf_locate (rf_wl, wlmin, ilmin)
            call rf_locate (rf_wl, wlmax, ilmax)
            nbil = ilmax-ilmin
            allocate (xwl(0:nbil), yvoi(0:nbil), xsum(0:nbil))
            do j=0,nbil
              nu = 1.0e8_dp * clum / rf_wl%Val(j+ilmin)
              xwl(j) = (nu0 - nu) / betas
            end do

            call humlik (nbil+1, xwl, avoi, yvoi)

            xsum = yvoi * rf_dray%Val(ilmin:ilmax)
     1           * 1.0e8 * clum / (betas * rf_wl%Val(ilmin:ilmax)**2)
            call simpint(rf_wl%Val(ilmin:ilmax), xsum, nbil, tata)
            chrai = tata
            deallocate (xwl, yvoi, xsum)
          endif

          nvu = irah2c(ik,3)
          ndj = irah2c(ik,4)
          nju = njl + ndj

          probdi = prah2c(ik,4)
          beinst = coefb * prah2c(ik,1) * (xlam0**3)
          absorb = beinst * chrai
          disici = absorb * probdi
          dumh2 = dumh2 + disici * xreh2(lev,iopt)
          pdh2vJ(lev,iopt) = pdh2vJ(lev,iopt)
     1                     + disici * xreh2(lev,iopt)
          if (iopt == 0) then
            sdh2vJ(lev) = sdh2vJ(lev) + beinst * probdi
          endif

          ! calcul explicite des populations de la bande C
          if ((nvu .le. nvbc) .and. (nju .le. njbc)) then
            h2cnua(nvu,nju,iopt) = h2cnua(nvu,nju,iopt) +
     1                            absorb * xreh2(lev,iopt)
          endif

c  construction de la matrice d equilibre radiatif

          evrdbc(lev,lev) = evrdbc(lev,lev) + absorb

          dumy = absorb * (un - probdi)
          if (mod(njl,2) == 0) then
            evrdbc(levpc,lev) = evrdbc(levpc,lev)
     1                        - dumy * bhmcx(nvu,nju,levpc)
          else
            evrdbc(levic,lev) = evrdbc(levic,lev)
     1                        - dumy * bhmcx(nvu,nju,levic)
          endif

        end do

        !fin du calcul de la pop des niveaux hauts
        do ivu=0,nvbc
          do iju=0, njbc
            if (tmunh2bc(2,ivu,iju) .gt. 1.0e-30_dp) then
             h2cnua(ivu,iju,iopt) = h2cnua(ivu,iju,iopt)
     1                              / tmunh2bc(2,ivu,iju)
            else
             h2cnua(ivu,iju,iopt) = -1.0d0
            endif
          enddo
        enddo

        pdish2(iopt) = dumh2

        krai = krai + nrh2ct
 
c  probabilite de dissociation de hd
c  ---------------------------------
 
        if (itrhd .ne. 0) then
 
        do 33 ik=1,nrhdbt
          ikhd = krai + ik
          nvl = irahdb(ik,1)
          njl = irahdb(ik,2)
          lev = levhd(nvl,njl)
          xlam0 = prahdb(ik,2)
          call rf_locate(rf_wl, xlam0, jwl)
          rapwl = (xlam0 - rf_wl%Val(jwl))
     1          / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
          rfint = rf_draypr%Val(jwl)
     1          + (rf_draypr%Val(jwl+1) - rf_draypr%Val(jwl)) * rapwl
          chrai = fgkgrd(ikhd) * rfint
          if (lev .gt. l0hd .or. chrai .le. zero) goto 33
 
          nvu = irahdb(ik,3)
          ndj = irahdb(ik,4)
          nju = njl + ndj
 
          probdi = prahdb(ik,4)
          beinst = coefb * prahdb(ik,1) * (xlam0**3)
          absorb = beinst * chrai
          disici = absorb * probdi
          dumhd = dumhd + disici * xrehd(lev,iopt)
 
c  construction de la matrice d equilibre radiatif
 
          evrdhd(lev,lev) = evrdhd(lev,lev) + absorb

          do 44 levl=1,l0hd
            nj2 = njhd(levl)
            if (nj2 .le. mjhd) then
              dumy = absorb * (un - probdi)
     1             * bhdbx(nvu,nju,levl)
              evrdhd(levl,lev) = evrdhd(levl,lev) - dumy
            endif
 44           continue
 
 33         continue
 
        krai = krai + nrhdbt

        do 34 ik=1,nrhdct
          ikhd = krai + ik
          nvl = irahdc(ik,1)
          njl = irahdc(ik,2)
          lev = levhd(nvl,njl)
          xlam0 = prahdc(ik,2)
          call rf_locate(rf_wl, xlam0, jwl)
          rapwl = (xlam0 - rf_wl%Val(jwl))
     1          / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
          rfint = rf_draypr%Val(jwl)
     1          + (rf_draypr%Val(jwl+1) - rf_draypr%Val(jwl)) * rapwl
          chrai = fgkgrd(ikhd) * rfint
          if (lev .gt. l0hd .or. chrai .le. zero) goto 34
 
          nvu = irahdc(ik,3)
          ndj = irahdc(ik,4)
          nju = njl + ndj
 
          probdi = prahdc(ik,4)
          beinst = coefb * prahdc(ik,1) * (xlam0**3)
          absorb = beinst * chrai
          disici = absorb * probdi
          dumhd = dumhd + disici * xrehd(lev,iopt)
 
c  construction de la matrice d equilibre radiatif

          evrdhd(lev,lev) = evrdhd(lev,lev) + absorb
 
          do 45 levl=1,l0hd
            nj2 = njhd(levl)
            if (nj2 .le. mjhd) then
             if (ndj .eq. 0) then
              dumy = absorb * (un - probdi)
     1             * bhdcmx(nvu,nju,levl)
              evrdhd(levl,lev) = evrdhd(levl,lev) - dumy
             else
              dumy = absorb * (un - probdi)
     1             * bhdcpx(nvu,nju,levl)
              evrdhd(levl,lev) = evrdhd(levl,lev) - dumy
            endif
            endif
 45           continue
 
 34           continue
 
        pdishd(iopt) = dumhd
 
        krai = krai + nrhdct

        endif
c  probabilite de dissociation de co
c  ---------------------------------

        do ik=1,nrco0
          njl = irco0(ik)
          probdi = prco0(ik,4)
          xlam0 = prco0(ik,2)

c  coeff num: (pi*e**2)/(m*h*c**2)=1.336d14
c     * 1.0d-16 (pour tenir compte des longueurs d onde en a).

          beinst = coefb * prco0(ik,1) * (xlam0**3)

          ikco = krai + ik
          call rf_locate(rf_wl, xlam0, jwl)
          rapwl = (xlam0 - rf_wl%Val(jwl))
     1          / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
          rfint = rf_draypr%Val(jwl)
     1          + (rf_draypr%Val(jwl+1) - rf_draypr%Val(jwl)) * rapwl
          chrai = fgkgrd(ikco) * rfint

          absorb = beinst * chrai
          dumco = dumco + absorb * probdi * xreco0(njl+1,iopt)
          eqrco0(njl,njl) = eqrco0(njl,njl) + absorb * probdi
        end do

c introduction de la protection de co par h2 (Evelyne)
c         dumco = dumco * 0.8457d0
c    1          * exp(-5.6d-22*sum(cdth2(1:l0h2,iopt)))

        pdico0(iopt) = dumco

        krai = krai + nrco0

c  probabilite de dissociation de co1
c  ---------------------------------

        if (itrco1 .ne. 0) then

        do ik=1,nrco1
          njl = irco1(ik)
          probdi = prco1(ik,4)
          xlam0 = prco1(ik,2)

          beinst = coefb * prco1(ik,1) * (xlam0**3)

          ikco1 = krai + ik
          call rf_locate(rf_wl, xlam0, jwl)
          rapwl = (xlam0 - rf_wl%Val(jwl))
     1          / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
          rfint = rf_draypr%Val(jwl)
     1          + (rf_draypr%Val(jwl+1) - rf_draypr%Val(jwl)) * rapwl
          chrai = fgkgrd(ikco1) * rfint

          absorb = beinst * chrai
          dumco1 = dumco1 + absorb * probdi * xreco1(njl+1,iopt)
          eqrco1(njl,njl) = eqrco1(njl,njl) + absorb * probdi
        end do

        pdico1(iopt) = dumco1

        krai = krai + nrco1

        endif

c  probabilite de dissociation de co2
c  ---------------------------------

        if (itrco2 .ne. 0) then

        do ik=1,nrco2
          njl = irco2(ik)
          probdi = prco2(ik,4)
          xlam0 = prco2(ik,2)

          beinst = coefb * prco2(ik,1) * (xlam0**3)

          ikco2 = krai + ik
          call rf_locate(rf_wl, xlam0, jwl)
          rapwl = (xlam0 - rf_wl%Val(jwl))
     1          / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
          rfint = rf_draypr%Val(jwl)
     1          + (rf_draypr%Val(jwl+1) - rf_draypr%Val(jwl)) * rapwl
          chrai = fgkgrd(ikco2) * rfint

          absorb = beinst * chrai
          dumco2 = dumco2 + absorb * probdi * xreco2(njl+1,iopt)
          eqrco2(njl,njl) = eqrco2(njl,njl) + absorb * probdi
        end do

        pdico2(iopt) = dumco2

        krai = krai + nrco2

        endif

c  probabilite de dissociation de co3
c  ---------------------------------

        if (itrco3 .ne. 0) then

        do ik=1,nrco3
          njl = irco3(ik)
          probdi = prco3(ik,4)
          xlam0 = prco3(ik,2)

          beinst = coefb * prco3(ik,1) * (xlam0**3)

          ikco3 = krai + ik
          call rf_locate(rf_wl, xlam0, jwl)
          rapwl = (xlam0 - rf_wl%Val(jwl))
     1          / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
          rfint = rf_draypr%Val(jwl)
     1          + (rf_draypr%Val(jwl+1) - rf_draypr%Val(jwl)) * rapwl
          chrai = fgkgrd(ikco3) * rfint

          absorb = beinst * chrai
          dumco3 = dumco3 + absorb * probdi * xreco3(njl+1,iopt)
          eqrco3(njl,njl) = eqrco3(njl,njl) + absorb * probdi
        end do

        pdico3(iopt) = dumco3

        endif

        return
        end

c--------------------------------------------------------------------------

c  Fonction d'interpolation, utilisee pour determinee la colonne densitee
c           entre le point courant et le centre du nuage

       double precision function interpcd(rapp, cd_o, i, npo)

         use constantes
         integer i, npo
         real (KIND=DP) rapp, cd_o(0:npo)

         real (KIND=DP) cd_c

         cd_c = cd_o(i) + (cd_o(i+1) - cd_o(i)) * rapp
         interpcd = cd_c

       return
       end

c--------------------------------------------------------------------------

       double precision function ajdf(taudf)

         use constantes
         real (KIND=DP) taudf

         if (taudf .lt. 2.0d0) then
           ajdf = exp(-2.0d0 * taudf / 3.0d0)
         else if (taudf .lt. 10.0d0) then
           ajdf = 0.638d0 * taudf**(-1.25d0)
         else if (taudf .lt. 1.0d2) then
           ajdf = 0.505d0 * taudf**(-1.15d0)
         else
           ajdf = 0.344d0 * taudf**(-1.0667d0)
         endif

       return
       end

c--------------------------------------------------------------------------

       subroutine simpint(x, y, n, som)

c  Compute the sum of y over x by Simpson rule
c  with uneven spacing of the x's

c  Formula established by Pierre Hily-Blant - 8 Sept 2003

c  The sum is then normalised by sqrt(pi)

       use constantes
       implicit none

       integer, intent (in)                        :: n 
       real (kind=dp), intent (in), dimension(0:n) :: x, y
       real (kind=dp), intent (out)                :: som

       integer        :: i
       real (kind=dp) :: x1, x2, x3, y1, y2, y3

       real (kind=dp) :: aux1
       real (kind=dp) :: t1, t2, t3
       real (kind=dp) :: a31, a12, a23

       som = 0.0_dp
       x3 = x(0)
       y3 = y(0)
       do i=1,n-1,2
         x1 = x3
         x2 = x(i)
         x3 = x(i+1)
         y1 = y3
         y2 = y(i)
         y3 = y(i+1)

         a12 = x1 - x2
         a23 = x2 - x3
         a31 = x3 - x1

         aux1 = a12*a12 + a23*a23 + x2 * (x2-x1-x3) + x1*x3
         t1 = -y1 * (3.0_dp * a12*a12 - aux1) / (6.0_dp * a12)
         t2 =  y2 * a31*a31*a31 / (6.0_dp * a12 * a23)
         t3 = -y3 * (3.0_dp * a23*a23 - aux1) / (6.0_dp * a23)

         som = som + t1 + t2 + t3
       end do
       som = som / sqrt(xpi)

       return
       end

