
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine CHIMIJ (iopt, itn)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE H2_DATA
       USE SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c--------1---------2---------3---------4---------5---------6---------7--
c   calcul des termes sources des equations d evolution chimique
c   et des termes sources dus aux reactions chimiques intervenant
c   dans les equations mhd.   nn = nbre de neutres
c                                   ni = nbre d ions
c                                   nspec = nbre total d especes+electr
c                                   neqt = nbre de reactions

c---------------------------------------------------------------------

        implicit double precision (a-h, o-z)

        dimension indy(0:nes)
        real (kind=dp), dimension(0:natom) :: ratiab
        real (kind=dp)                     :: ratlim = 1.0_dp

c  xkneut contient les taux de neutralisation des ions sur les grains
c    suivant Bakes & Tielens, Ap. J., 427, 822, (1994)

       real (KIND=DP) xkneut(nes)

       common / cofbth / bilthg, xkneut

c  recherche des especes les plus abondantes:
c    la conservation des elements se fait (pour chaque atome)
c    sur l espece la plus abondante. En cas de conflit (une espece
c    renferme deux elements (par ex; CO)), on prend la deuxieme
c    espece la plus abondante.

c    en fin de boucle 12,
c    LCON(K2): indice de l espece utilisee pour conserver l element K2.

        do 12 k2 = 1,natom-1
          top1 = zero
          top2 = zero
          icx(k2,1) = 0
          icx(k2,2) = 0

          do 11 l = 1,nspec-1
            dum = ael(l,k2)
            if (dum .eq. zero) goto 11
            if (lcon(l) .eq. k2) then
              lcon(l) = 0
            endif
            if (ab(l)*dum .gt. top1) then
              top2 = top1
              top1 = ab(l) * dum
              icx(k2,2) = icx(k2,1)
              icx(k2,1) = l
            else if (ab(l)*dum .gt.top2) then
              top2 = ab(l) * dum
              icx(k2,2) = l
            endif
   11     continue
          if (top1 .ne. zero) then
            ratiab(k2) = top2 / top1
          endif
   12   continue

        do 14 k2 = 1,natom-1
          ici1 = icx(k2,1)
          ici2 = icx(k2,2)
          if (ici1 .eq. 0) goto 14

          if (ratiab(k2) .gt. ratlim) then
            if (ael(ici1,natom) .eq. 1
     1          .and. ael(ici2,natom) .eq. 0) then
              jjj = ici1
              ici1 = ici2
              ici2 = jjj
            endif
          endif

          icc = 0
          do i=1,k2-1
            if (lcon(ici1) .eq. i) icc = i
         end do

          if (icc .eq. 0) then
            lcon(ici1) = k2
          else
            if (ratiab(k2) .gt. ratiab(icc)) then
              lcon(ici2) = k2
            else
              lcon(ici1) = k2
              lcon(icx(icc,2)) = icc
            endif
          endif

   14   continue

c  initialisations

        xkmat = zero

        indy = 0

C----DENSITE TOTALE (CM-3) DES ESPECES COLLEES SUR LES GRAINS
      abongr = zero
      do i=nn+ni+nim+1,nn+ni+nim+nng
        abongr = abongr + ab(i)
      end do

c   Extinction par l'autre cote du nuage

      avr = Avmax - av

c   Fraction de H2 (pour le flux de photons secondaires)

      frach2 = ab(i_h2) / (ab(i_h) + ab(i_h2))

C---Hydrogenation:
      xkhydr = max(abongr, bsitgr * densh(iopt))

C----IR1 ET IR2 SONT LES DEUX reactANTS DE LA reactION
C    IP1 ET IP2 LES DEUX PREMIERS PRODUITS

C chemical reactions are reordered:
C (0) formation of H2 and HD on grains [Old Style] (nh2for)
C (1) cosmic ray processes (ncrion)
C (2) secondary photon processes (npsion)
C (3) radiative association  (nasrad)
C (4) gas phase reactions (nrest)
C (6) Endothermal reactions with H2 (nh2endo) (NEW!)
C (5) photoreactions avec fit exterieur (nphot_f)
C (7) photoreactions avec integration directe (nphot_i)
C (8) 3 body reactions (n3body)
C (101...) exceptions (ncpart)

C (11) reactions on grain surface (ngsurf)
C (12) photoreactions on grain surface (ngphot)
C (13) adsorption on to grains (ngads)
C (14) neutralization (ngneut)
C (15) explosive desorption from grains (nggdes)
C (16) cosmic ray induced desorption from grains (ncrdes)
C (17) photodesorption from grains (nphdes)
C (18) evaporation from grains (ngevap)

C-------------------------------------------------------------------------
C  ESPECES SPECifIQUES : NSPEC + 1 : ELECTRON
C                        NSPEC + 2 : PHOTON
C                        NSPEC + 3 : CRP
C                        NSPEC + 4 : GRAIN

C       --------------------------
C-- 1 - reactIONS EN PHASE GAZEUSE
C       --------------------------

C-------------------------------------------------------------------------
C (0) formation of H2 and HD on grains [Old Style] (nh2for)  (cm3 s-1)
C-------------------------------------------------------------------------

      do i=1,nh2for

        ir1 = react(i,1)
        ir2 = react(i,2)
        iprod = react(i,4)

        rate = signgr * vmaxw * sqrt(temp / mol(ir1))
        rate = rate * gamm(i) * densh(iopt) / (istiny + ab(i_h))

        kkk = indy(iprod) + 1
        indy(iprod) = kkk
        xkmat(iprod,kkk) = rate

        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

        kkk = indy(ir2) + 1
        indy(ir2) = kkk
        xkmat(ir2,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (1) cosmic ray processes (ncrion)   (s-1)
C-------------------------------------------------------------------------
      ideb = nh2for + 1
      ifin = nh2for + ncrion
      do i=ideb,ifin

        rate = gamm(i) * zeta

        ik = react(i,0)
        do j=4,ik+1
          iprod = react(i,j)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate
        end do

        ir1 = react(i,1)
        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (2) secondary photon processes (npsion)   (s-1)
C-------------------------------------------------------------------------

c  27 II 2003 - Test to use UMIST form of secondary photons reactions
c               Needed for Serena Viti chemistry

      i_UMIST = 0

      ideb = ifin + 1
      ifin = ifin + npsion
      do i=ideb,ifin

        if (i_UMIST == 1) then
          rate = gamm(i) * frach2 * zeta * bet(i) / (un - alb)
        else
          rate = gamm(i) * frach2 * zeta * (temp/300.0d0)**alpha(i)
        endif

        ik = react(i,0)
        do j=4,ik+1
          iprod = react(i,j)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate
        end do

        ir1 = react(i,1)
        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (3) radiative association  (nasrad)   (cm3 s-1)
C-------------------------------------------------------------------------
      ideb = ifin + 1
      ifin = ifin + nasrad
      do i=ideb,ifin

        rate = gamm(i) * (temp/300.0d0)**alpha(i)
     1       * exp(-bet(i)/temp)

        iprod = react(i,4)
        kkk = indy(iprod) + 1
        indy(iprod) = kkk
        xkmat(iprod,kkk) = rate

        ir1 = react(i,1)
        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

        ir2 = react(i,2)
        kkk = indy(ir2) + 1
        indy(ir2) = kkk
        xkmat(ir2,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (4) gas phase reactions (nrest)   (cm3 s-1)
C-------------------------------------------------------------------------
      ideb = ifin + 1
      ifin = ifin + nrest
      do i=ideb,ifin

        if (bet(i) < 0.0_dp) then
          expon = exp(-bet(i)/max(temp,200.0_dp))
        else
          expon = exp(-bet(i)/temp)
        endif
        rate = gamm(i) * (temp/300.0d0)**alpha(i)
     1      * expon
c    1      * exp(-bet(i)/temp)

        ik = react(i,0)
        do j=4,ik+1
          iprod = react(i,j)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate
        end do

        ir1 = react(i,1)
        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

        ir2 = react(i,2)
        kkk = indy(ir2) + 1
        indy(ir2) = kkk
        xkmat(ir2,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (6) Endothermal reactions with H2 (nh2endo)   (cm3 s-1)
C-------------------------------------------------------------------------
      ideb = ifin + 1
      ifin = ifin + nh2endo
      do i=ideb,ifin

        rate = 0.0_DP
        do lev=1,l0h2
          endo_rxn = max(bet(i) - elhm(lev), 0.0_DP)
          rat_l = gamm(i) * (temp/300.0d0)**alpha(i)
     1        * exp(-endo_rxn/temp)
          rate = rate + rat_l * xreh2(lev,iopt)
        end do

        ik = react(i,0)
        do j=4,ik+1
          iprod = react(i,j)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate
        end do

        ir1 = react(i,1)
        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

        ir2 = react(i,2)
        kkk = indy(ir2) + 1
        indy(ir2) = kkk
        xkmat(ir2,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (5) photoreactions (external fit) (nphot_f)    (s-1)
C-------------------------------------------------------------------------
      ideb = ifin + 1
      ifin = ifin + nphot_f
      do i=ideb,ifin

        ir1 = react(i,1)
c  Le coefficient effectif (radm ou radp) a ete calcule dans le main
        rate1 = gamm(i) * exp(-bet(i) * av) * radm
        rate2 = gamm(i) * exp(-bet(i) * avr) * radp
        rate = rate1 + rate2

        ik = react(i,0)
        do j=4,ik+1
          iprod = react(i,j)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate
        end do

        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (7) photoreactions (nphot_i)    (s-1)
C-------------------------------------------------------------------------
      ideb = ifin + 1
      ifin = ifin + nphot_i
      do i=ideb,ifin

        ir1 = react(i,1)
        if (ir1 .eq. i_h2 .and. itrh2 .ne. 0) then
          rate = pdish2(iopt)
        else if (ir1 .eq. i_hd .and. itrhd .ne. 0) then
          rate = pdishd(iopt)
        else if (ir1 .eq. i_co0 .and. itrco0 .ne. 0) then
          rate = pdico0(iopt)
        else if (ir1 .eq. i_co1 .and. itrco1 .ne. 0) then
          rate = pdico1(iopt)
        else if (ir1 .eq. i_co2 .and. itrco2 .ne. 0) then
          rate = pdico2(iopt)
        else if (ir1 .eq. i_co3 .and. itrco3 .ne. 0) then
          rate = pdico3(iopt)
        else if (ir1 .eq. i_c .or. ir1 .eq. i_c13) then
          rate = pdicat(iopt)
        else if (ir1 .eq. i_s) then
          rate = pdisat(iopt)
        else
          print *, "  Error in chemistry!"
          stop
        endif

        ik = react(i,0)
        do j=4,ik+1
          iprod = react(i,j)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate
        end do

        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (8) 3 body reactions (n3body)
C-------------------------------------------------------------------------

      ideb = ifin + 1
      ifin = ifin + n3body
      do i=ideb,ifin

        ir1 = react(i,1)
        ir2 = react(i,2)
        ir3 = react(i,3)
        expon = exp(-bet(i)/temp)
        rate = gamm(i) * (temp/300.0d0)**alpha(i)
     1      * expon

        ik = react(i,0)
        do j=4,ik+1
          iprod = react(i,j)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate
        end do

        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate
        kkk = indy(ir2) + 1
        indy(ir2) = kkk
        xkmat(ir2,kkk) = - rate
        kkk = indy(ir3) + 1
        indy(ir3) = kkk
        xkmat(ir3,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (101...) exceptions (ncpart)
C-------------------------------------------------------------------------
      ideb = ifin + 1
      ifin = ifin + ncpart
      do i=ideb,ifin
        ir1 = react(i,1)
        ir2 = react(i,2)

c  101: N+ + H2:
        if   (ir1 .eq. i_h2 .and. ir2 .eq. i_np) then
          rate = gamm(i) * (h2para * exp(-bet(i) / temp) + h2orth)
     1         * ((temp / 300.0d0)**alpha(i))

c  102: H3+ + HD:
          else  if (ir1 .eq. i_h2 .and. ir2 .eq. i_h2dp
     1      .and. temp.ge. 20.d0)   then
          rate = gamm(i) / 10**(62.481 *temp**(-0.94871d0))
         else if  (ir1 .eq. i_h2 .and. ir2 .eq. i_h2dp
     1      .and. temp.lt. 20.d0) then
           rate = gamm(i) / (6.3407d35
     1         * temp **(-25.203d0))

c  103: O+ + H:
         else if   (ir1 .eq. i_h .and. ir2 .eq. i_op) then
          rate = (gamm(i) + alpha(i) * temp) * exp(-bet(i) / temp)

c  104: H+ + O:
         else if   (ir1 .eq. i_o .and. ir2 .eq.i_hp) then
          rate = (gamm(i) + alpha(i) * temp) * exp(-bet(i) / temp)

c  105: reaction exp(-T/E)  (formation de H- : Black)
         else if (itype(i) .eq. 105) then
          rate = gamm(i) * (temp/300.0)**alpha(i) * exp(-temp/bet(i))
        else
          rate = zero
        endif

        iprod = react(i,4)
        kkk = indy(iprod) + 1
        indy(iprod) = kkk
        xkmat(iprod,kkk) = rate

        iprod = react(i,5)
        kkk = indy(iprod) + 1
        indy(iprod) = kkk
        xkmat(iprod,kkk) = rate

        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

        kkk = indy(ir2) + 1
        indy(ir2) = kkk
        xkmat(ir2,kkk) = - rate

      end do

C       -------------------------
C-- 2 - REACTIONS AVEC LES GRAINS
C       -------------------------

C-------------------------------------------------------------------------
C (11) reactions on grain surface (ngsurf)
c      (y compris formation de H2)
C-------------------------------------------------------------------------
      ideb = ifin + 1
      ifin = ifin + ngsurf
      do i=ideb,ifin

        ir1 = react(i,1)
        ir2 = react(i,2)

        rate = gamm(i) * (tm1h(ir1) + tm1h(ir2)) / xkhydr

        ik = react(i,0)
        do j=4,ik+1
          iprod = react(i,j)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate
        end do

        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

        kkk = indy(ir2) + 1
        indy(ir2) = kkk
        xkmat(ir2,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (12) photoreactions on grain surface (ngphot)
C-------------------------------------------------------------------------
      ideb = ifin + 1
      ifin = ifin + ngphot
      do i=ideb,ifin

        rate1 = gamm(i) * exp(-bet(i)*av) * radm
        rate2 = gamm(i) * exp(-bet(i)*avr) * radp
        rate = rate1 + rate2

        ik = react(i,0)
        do j=4,ik+1
          iprod = react(i,j)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate
        end do

        ir1 = react(i,1)
        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

        ir2 = react(i,2)
        kkk = indy(ir2) + 1
        indy(ir2) = kkk
        xkmat(ir2,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (13) adsorption on to grains (ngads)
C-------------------------------------------------------------------------
      if (istic .eq. 1) then
        stickH = gamm(iadhgr)

      else if (istic .eq. 2) then
c  Sticking bidouille a partir de Sternberg et Dalgarno pour
c     avoir a peu pres 1 a basse temperature
        t2 = temp / 100.0d0
        stickH = 2.5d0
     1        / (un + 0.4d0 * sqrt(tdust(1,iopt) + t2) + 0.2d0 * t2)

      else if (istic .eq. 3) then
c  Sticking pompe dans Andersson & Wannier Ap J 402 585, 1993
        stickH = un / (temp/102.0d0 + un)**2

      else if (istic .eq. 4 .and. temp .ge. 10.0d0) then
c  Sticking sauce ' Pineau des forets & Flower, mnras
c       stickH = un / (temp * temp * 1.0d-4 + un)
        stickH = un * sqrt(10.d0) / sqrt(temp)
      else if (istic .eq. 4 .and. temp. lt. 10.d0) then
         stickH = un

      else if (istic .eq. 5) then
c  Sticking sauce JLB
        stickH = un / (temp * temp * 1.0d-4 + un)

      endif

      gamm(iadhgr) = stickH
      if (i_d .ne. 0) then
        gamm(iaddgr) = stickH
      endif

      ideb = ifin + 1
      ifin = ifin + ngads
      do i=ideb,ifin
        ir1 = react(i,1)
        if (itype(i) .eq. 13) then
c  Adsorption "ordinaire"

          rate = signgr * vmaxw * sqrt(temp / mol(ir1))
          rate = rate * gamm(i) * densh(iopt)

          iprod = react(i,4)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate

          kkk = indy(ir1) + 1
          indy(ir1) = kkk
          xkmat(ir1,kkk) = - rate

        else if (itype(i) .eq. 113) then
c  Adsorption de H2, taux "reel"

          ratef = vmaxw * sqrt(temp / mol(i_h2))
     1          * densh(iopt) * gamm(i) / quatre

          iprod = react(i,4)
          rate = ratef * surfgr(1+iprod-i_h2gr)

          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate

          kkk = indy(i_h2) + 1
          indy(i_h2) = kkk
          xkmat(i_h2,kkk) = - rate

        else if (itype(i) .eq. 123) then
c  Adsorption de H2 "pseudo-reaction" limitante

          rate = vmaxw * sqrt(temp / mol(i_h2))
     1          * gamm(i) * dsite*dsite / quatre

          iprod = react(i,4)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate

          ir2 = react(i,2)
          kkk = indy(ir2) + 1
          indy(ir2) = kkk
          xkmat(ir2,kkk) = - rate

        end if

      end do

C-------------------------------------------------------------------------
C (14) neutralization (ngneut)
C-------------------------------------------------------------------------
      ideb = ifin + 1
      ifin = ifin + ngneut

      if (ngneut > 0) then

        do i=ideb,ifin
          ir1 = react(i,1)
          rxigr(ir1,iopt) = zero
        end do


        do mm=1,npg
          call fbth(temp, sigabs(mm), iopt, itn, mm)
          j = 0
          do i=ideb,ifin
            ir1 = react(i,1)
            j = j + 1
            rxigr(ir1,iopt) = rxigr(ir1,iopt) + xkneut(j) * wwmg(mm)
          end do
        end do

      endif

      do i=ideb,ifin

        ir1 = react(i,1)
        rate = rxigr(ir1,iopt)

        iprod = react(i,4)
        kkk = indy(iprod) + 1
        indy(iprod) = kkk
        xkmat(iprod,kkk) = rate

        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (15) explosive desorption from grains (nggdes)
c    ref: Shalabiea & Greenberg 1994
C-------------------------------------------------------------------------

c On suppose le taux constant (i.e. independant de la taille)

      ideb = ifin + 1
      ifin = ifin + nggdes
      do i=ideb,ifin

        ir1 = react(i,1)
        rat1 = gamm(i) * rdeso * densh(iopt)
        rat2 = gamm(i) * 3.0d4 * zeta * (signgr / 7.0d-21)
        rate = rat1 + rat2

        if (itype(i) .eq. 15) then
          iprod = react(i,4)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate

          kkk = indy(ir1) + 1
          indy(ir1) = kkk
          xkmat(ir1,kkk) = - rate
        else if (ir1 .eq. i_h2gr) then
          do mm=1,npg
            iprod = react(i,4)
            kkk = indy(iprod) + 1
            indy(iprod) = kkk
            rateh = rate * surfgr(mm)
     1            * densh(iopt) / (dsite*dsite)
            xkmat(iprod,kkk) = rateh

            ir2 = ir1 + (mm-1)
            kkk = indy(ir2) + 1
            indy(ir2) = kkk
            xkmat(ir2,kkk) = - rate
          end do
        else
          print *, "  Not yet included, use type 15, not 115"
          stop
        end if

      end do

C-------------------------------------------------------------------------
C (16) cosmic ray induced desorption from grains (ncrdes)
c      (sans les photons secondaires)
C-------------------------------------------------------------------------
      ideb = ifin + 1
      ifin = ifin + ncrdes
      do i=ideb,ifin

c       rate = gamm(i) * fphsec * frach2 * quatre * signgr / xkhydr
c       rate = rate * densh(iopt)
        rate = zero

        iprod = react(i,4)
        kkk = indy(iprod) + 1
        indy(iprod) = kkk
        xkmat(iprod,kkk) = rate

        ir1 = react(i,1)
        kkk = indy(ir1) + 1
        indy(ir1) = kkk
        xkmat(ir1,kkk) = - rate

      end do

C-------------------------------------------------------------------------
C (17) photodesorption from grains (nphdes)
c      (les photons secondaires sont isotropes, pas les photons incidents)
C-------------------------------------------------------------------------
c  flux de photons secondaires due aux rayons cosmiques:
c     1E3 cm-2 s-1 (Roberge, in Dalgarno's book)
c      fphsec = 1.0d3 * (zeta / 1.0d-17) = 1.0d3 * fmrc

c  flux de Mathis et al de 911 a 2400 A (avec F(l) = 5e-17 erg cm-3 A-1)
c  flupho = 1e8 photon cm-2 s-1      (Tielens & Hagen, 1982, A&A 114, 245)
c     flupho = 1.0d8
c     flupho = flupho * RAD * exp(-2.0d0 * AV)
c     print *, flupho, fluph

      ideb = ifin + 1
      ifin = ifin + nphdes
      do i=ideb,ifin

        ir1 = react(i,1)

        if (itype(i) .eq. 17) then
          rate = gamm(i) * fphsec * frach2 * quatre * signgr / xkhydr
          rate = rate + gamm(i) * fluph(iopt) * signgr / xkhydr
          rate = rate * densh(iopt)
          iprod = react(i,4)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate

          kkk = indy(ir1) + 1
          indy(ir1) = kkk
          xkmat(ir1,kkk) = - rate

        else if (itype(i) .eq. 117) then

          ratef = gamm(i) * densh(iopt) / xkhydr
          ratef = ratef * (frach2 * fphsec + fluph(iopt) / quatre)

          iprod = react(i,4)
          rate = ratef * surfgr(1+ir1-i_h2gr)

          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate

          kkk = indy(ir1) + 1
          indy(ir1) = kkk
          xkmat(ir1,kkk) = - rate

        else
          print *, "  Not yet included, use type 17, not 117"
          stop
        end if

      end do

C-------------------------------------------------------------------------
C (18) evaporation from grains (ngevap)
C-------------------------------------------------------------------------
      ideb = ifin + 1
      ifin = ifin + ngevap
      do i=ideb,ifin

        ir1 = react(i,1)
        if (itype(i) .eq. 18) then
          rate = gamm(i) * sqrt(deux*xk*bet(i)/(xmh*mol(ir1)))
     1         / (xpi * dsite)
          rate = rate * exp(-bet(i)/tdust(1,iopt))

          iprod = react(i,4)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate

          kkk = indy(ir1) + 1
          indy(ir1) = kkk
          xkmat(ir1,kkk) = - rate

        else if (itype(i) .eq. 118) then

          rate = gamm(i) * sqrt(deux*xk*bet(i)/(xmh*mol(ir1)))
     1         / (xpi * dsite)
          rate = rate * exp(-bet(i)/tdust(1+ir1-i_h2gr,iopt))

          iprod = react(i,4)
          kkk = indy(iprod) + 1
          indy(iprod) = kkk
          xkmat(iprod,kkk) = rate

          kkk = indy(ir1) + 1
          indy(ir1) = kkk
          xkmat(ir1,kkk) = - rate

        else
          print *, "  Not yet included, use type 18, not 118"
          stop
        end if

      end do

C-------------------------------------------------------------------------

        return
        end
