
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%
       subroutine MNEWT (iopt)
c      %%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE STR_DATA
       USE CHEM_DATA
       USE SP_DATA
       USE PROFIL

c      voir numerical recipe p 272.

       implicit double precision (a-h,o-z)

c  Numerical parameters (unused outside MNEWT, used to be defined as parameter)

       INTEGER         :: ntr1 = 25         ! Max number of global iteration in N-R algorithm
                                            !     1 global iter = change conservation equation
       INTEGER         :: ntr2 = 3          ! Max number of iteration with the same
                                            !     conservation equation
       REAL (KIND=DP)  :: tolx = 1.0e-3_DP  ! Total relative error for chemistry

       real (kind=dp), allocatable, dimension (:,:) :: am, bbmn
       character (LEN=8) :: one_spe
       real (kind=dp), dimension (nes) :: abold, abt
       real (kind=dp), dimension(0:natom) :: ratiab
       real (kind=dp)                     :: ratlim = 1.0_dp
       dimension indx(nes)

       allocate (am(nspec,nspec), bbmn(nspec,1))

       abold(1:nspec) = ab(1:nspec)

       abt = 0
       do i=1,nspec
          duma = densh(iopt)
          do j=1,natom-1
            kspec = ael(i,j)
            if (kspec == 0) then
              cycle
            else
              duma = min(duma,densh(iopt)*abtot(j)/kspec)
            endif
          end do
          abt(i) = duma
       end do

       do 300 k1=1,ntr1

       damp = 0.7d0

       do 200 k2=1,ntr2

c  usrfun calcule le Jacobien et la fonction

         call USRFUN (iopt, am, bbmn, nspec)

         abold(1:nspec) = ab(1:nspec)

c  Utilisation de LAPACK

         nrhs = 1
         call dgesv (nspec, nrhs, am, nspec, indx, bbmn, nspec, info)

         dum = sum(abs(bbmn(1:nspec,1)))
         if (dum .gt. densh(iopt)) then
           bbmn(1:nspec,1) = bbmn(1:nspec,1) * densh(iopt) / dum
         endif

         errx = zero

         bad = zero
         mauvai = 0
         do i=1,nspec
           one_spe = speci(i)
           if (ab(i) > max(istiny,-bbmn(i,1))
     1         .OR. bbmn(i,1) > 0.0_dp) then
             tt1 = abs(bbmn(i,1)) / (abold(i) + istiny)
             if (ab(i) > 1.0e10_dp*istiny) then
               errx = errx + tt1
               if (bad .lt. tt1) then
                 bad = tt1
                 mauvai = i
               endif
             endif

c  *** correction: robustesse. ***
             if (lcon(i) .eq. 0) then
               ab(i) = ab(i) + bbmn(i,1) * damp
             else
               ab(i) = ab(i) + bbmn(i,1)
             endif
             if (ab(i) .gt. abt(i)) then
               ab(i) = (abt(i) + abold(i)) * half
             else if (ab(i) .lt. zero) then
               ab(i) = abold(i) / dix
             endif
           else
             ab(i) = istiny
           endif
         end do

         if (mod(k2,2) .eq. 1) then
           do i=1,nspec
             ab(i) = (ab(i) + abold(i)) * half
           end do
         endif
c  *** fin robustesse et oscillations ***

         abelec = sum(ab(nn+1:nspec-1) * ael(nn+1:nspec-1,natom))

         errx = errx / damp
         if (errx .le. tolx .and. mod(k2,2) .eq. 0) then
           deallocate (am, bbmn)
           return
         endif
  200  continue

c  modification de l equation de conservation
c     en cas de non convergence

       do 220 i = 1,natom-1
         top1 = zero
         top2 = zero
         icx(i,1) = 0
         icx(i,2) = 0

         do 210 l = 1,nspec-1
           dum = ael(l,i)
           if (dum .eq. zero) goto 210
           if (lcon(l) .eq. i) then
             lcon(l) = 0
           endif
           ddumm = ab(l) * dum
           if (ddumm .gt. top1) then
             top2 = top1
             top1 = ddumm
             icx(i,2) = icx(i,1)
             icx(i,1) = l
           else if (ddumm .gt. top2) then
             top2 = ddumm
             icx(i,2) = l
           endif
  210    continue
         if (top1 .ne. zero) then
           ratiab(i) = top2 / top1
         endif
  220  continue

       do 221 i = 1,natom-1
         ici1 = icx(i,1)
         ici2 = icx(i,2)
         if (ici1 .eq. 0) goto 221

c  Patch recombinaison
         if (ratiab(i) .gt. ratlim) then
           if (ael(ici1,natom) .eq. 1
     1         .and. ael(ici2,natom) .eq. 0) then
             jjj = ici1
             ici1 = ici2
             ici2 = jjj
           endif
         endif

         icc = 0
         do 211 ii = 1,i-1
           if (lcon(ici1) .eq. ii) icc = ii
  211    continue

         if (icc .eq. 0) then
           lcon(ici1) = i
         else
           if (ratiab(i) .gt. ratiab(icc)) then
             lcon(ici2) = i
           else
             lcon(ici1) = i
             lcon(icx(icc,2)) = icc
           endif
         endif

  221  continue

  300  continue

       k2 = k2 - 1
c      ratlim = 0.35d0
       ratlim = 0.20d0

       deallocate (am, bbmn)

       return
       end

c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine USRFUN (iopt, am, bbmn, nsp)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE STR_DATA
       USE SP_DATA
       USE CHEM_DATA
       USE PROFIL

c      calcule les valeurs de la fonction (-beta) et ses derivees
c      partielles (alpha) au point ab, avec equations de conservation.

       implicit double precision (a-h,o-z)

       real (kind=dp), dimension (nsp,nsp) :: am
       real (kind=dp), dimension (nsp,1) :: bbmn

       dimension ab1(nqm), ab2(nqm), ab3(nqm)
       dimension j1(nqm), j2(nqm), j3(nqm)
       dimension dumb(0:nes)

c                     debut des ordres executables:

c  Les taux sont maintenant a jour. Le systeme est donc forme
c      exclusivement de polynomes.

c    Si lcon(i) = 0, Il s agit d une equation ordinaire:
c               d[Xi] / dt = 0
c    Si lcon(i) = j =/ 0, c est une equation de conservation
c        de l atome j.

       do 200 i=1,nspec-1

         jatom = lcon(i)

         if (jatom .eq. 0) then
           nrxi = ikmat(i,0)

           mm = 0
           do k=1,nrxi
             ind = ikmat(i,k)
             j1(k) = react(ind,1)
             j2(k) = react(ind,2)
             j3(k) = react(ind,3)
             ab1(k) = ab(j1(k))
             ab2(k) = ab(j2(k))
             ab3(k) = ab(j3(k))
           end do

c             calcul des residus

           duma = sum(xkmat(i,1:nrxi) * ab1(1:nrxi) * ab2(1:nrxi)
     1          * ab3(1:nrxi))

           bbmn(i,1) = - duma

c   calcul des derivees partielles

           dumb(1:nspec) = zero

           do k=1,nrxi
             dumb(j1(k)) = dumb(j1(k)) + xkmat(i,k) * ab2(k) * ab3(k)
             dumb(j2(k)) = dumb(j2(k)) + xkmat(i,k) * ab1(k) * ab3(k)
             dumb(j3(k)) = dumb(j3(k)) + xkmat(i,k) * ab1(k) * ab2(k)
           end do

           am(i,1:nspec) = dumb(1:nspec)

         else

           duma = abtot(jatom) * densh(iopt)
           do j=1,nspec-1
             kspec = ael(j,jatom)
             duma = duma - kspec * ab(j)
             am(i,j) = - kspec
           end do

           bbmn(i,1) = - duma
           am(i,nspec) = zero

         endif

  200  continue

         duma = ab(nspec)
         do j=1,nspec-1
           kspec = ael(j,natom)
           duma = duma - kspec * ab(j)
           am(nspec,j) = kspec
         end do

         bbmn(i,1) = duma
         am(nspec,nspec) = - un

       return
       end

