
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine EQLH2V (ttry, iopt)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       use RF_TOOLS
       use H2_DATA
       use SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c  ecrit la matrice d equilibre des niveaux de rotation
c  et vibration de h2 (jusqu a l0h2)et resout le systeme

       implicit double precision (a-h,o-z)

       dimension create(nlevh2)
       dimension aev(nlevh2,nlevh2), indx(nlevh2)
       dimension bev(nlevh2,1)

       real (KIND=DP), DIMENSION (:), ALLOCATABLE :: t_res
       real (KIND=DP), DIMENSION (:), ALLOCATABLE :: frac_c

c  TIMH2 contient les donnees necessaires au calcul des
c      temps caracteristiques de H2 (en s-1 cm-3 a chaque fois).
c      TIMH2(iopt,1) : FABRIC : fabrication de H2 par la chimie gazeuse.
c      TIMH2(iopt,2) : DESTRU : destruction de H2 par la chimie gazeuse.
c      TIMH2(iopt,3) : RATEGR : fabrication de H2 sur les grains.
c      TIMH2(iopt,4) : DESPHO : destruction de H2 par les photons.
c      TIMH2(iopt,5) :        : Liberation de H2 adsorbe sur les grains

       dimension ph2(0:nvxh2,0:njh2)

c      dimension TOTAL(nlevh2)

       IF ( .NOT. ALLOCATED(t_res)) THEN
         ALLOCATE (t_res(npg), frac_c(npg))
       ENDIF

       xnprot = ab(nn+1)

       evcolh(1:l0h2,1:l0h2) = zero
       evltot(1:l0h2,1:l0h2) = zero
       create(1:l0h2) = zero

c  Processus collisionnels
c      La matrice evcolh est remplie directement dans COLH2V

       call COLH2V (xnprot, ttry, iopt)

c  transitions quadrupolaires:
c  La matrice evlqua est initialisee dans init1 directement.

c  termes d origine purement chimique
c      on supprime donc la reaction de photodissociation

       fabric = zero
       destru = zero

c  taux de fabrication et de destruction de h2
c      en excluant la formation sur les grains
c                  la photodissociation
c                  les reactions d'adsortion-desorption
c                  les reactions endothermiques traitees specifiquement (type 6)

       nrxi = ikmat(i_h2,0)

       do 50 k=1,nrxi
         ind = ikmat(i_h2,k)
         if (ind .eq. ifh2gr .or.
     1       ind .eq. idh2ph .or.
     2       itype(ind) .gt. 110 .or.
     3       itype(ind) == 6) goto 50
         k1 = react(ind,1)
         k2 = react(ind,2)
         k3 = react(ind,3)
         taux = xkmat(i_h2,k)
         if (taux .gt. zero) then
           fabric = fabric + taux * ab(k1) * ab(k2) * ab(k3)
         else
           destru = destru + taux * ab(k1) * ab(k2) * ab(k3)
         endif
   50  continue

c  formation de h2 sur les grains:

       rate = gamm(ifh2gr) * deux * tm1h(i_hgr) / xkhydr

       timh21(iopt) = fabric
       timh22(iopt) = - destru
       timh23(iopt) = rate * ab(i_hgr) * ab(i_hgr)
       timh24(iopt) = pdish2(iopt) * ab(i_h2)

c  liberation de h2 adsorbe
c  Les dernieres reactions sont  artificielles (donc exclues)
c  La numerotation est la meme pour toutes les "fractions"

       nrxigr = ikmat(i_h2gr,0)

       t_25(1:npg,iopt) = zero
       dd1 = zero
       dd2 = zero
       do mm=1,npg
         do k=1,nrxigr
           ind = ikmat(i_h2gr+mm-1,k)
           ity = itype(ind)
           if (ity == 115 .or.
     1         ity == 117 .or.
     2         ity == 118) then
             taux = xkmat(i_h2gr+mm-1,k)
             t_25(mm,iopt) = t_25(mm,iopt) - taux * ab(i_h2gr+mm-1)
           else if (ity == 113) then
             taux = xkmat(i_h2gr+mm-1,k)
             dd1 = dd1 + taux * ab(i_h2)
c            destru = destru - dd1
           else if (ity == 123) then
             taux = xkmat(i_h2gr+mm-1,k)
             dd2 = dd2 + taux * ab(i_h2) * ab(i_h2gr+mm-1)
c            destru = destru - dd2
           endif
         end do
       end do

       dd1 = dd1 + dd2

c      print *, fabric, rate, ab(i_hgr)
c      print *, destru, pdish2(iopt), ab(i_h2)

       dumy = zero
c      do lev=1,nlevh2
c  10 mai 2004 ?
       do lev=1,l0h2
         ij = njlev(lev)
         iv = nvlev(lev)
c        print *, iv, ij, pophm(iv,ij), elhm(lev)
         dumy = dumy + pophm(iv,ij)
       end do
c      print *, '  normalisation de pophm:', dumy
       echpop = dumy

c  8 Juillet 93, introduction de l excitation
c     par les electrons secondaires (CR)
c     Source: A Dalgarno, private comm. (Calculs de R. Gredel)

c   Correction du rapport ortho / para pour l excitation
c    par les electrons secondaires:

       corpar = h2para / paretl
       corort = h2orth / ortetl
       crnorm = zeta * densh(iopt) * ab(i_h2) / (ab(i_h) + ab(i_h2))

       do lev=1,l0h2
         ij = njlev(lev)
         jjj = mod(ij,2)
         corosp = jjj * corort + (1 - jjj) * corpar
         iv = nvlev(lev)
         dumy = zero
         dumy2 = zero
         do levu=l0h2+1,nlevh2
           iju = njlev(levu)
           ivu = nvlev(levu)
           dumy = dumy + pophm(ivu,iju) * cashm(levu,lev)
     1          / echpop
           dumy2 = dumy2 + entry(ivu,iju) * cashm(levu,lev)
         end do
         create(lev) = (dumy + pophm(iv,ij) / echpop)
     1               * rate * ab(i_hgr) * ab(i_hgr)
     2               + (dumy2 + entry(iv,ij)) * corosp
     3               * crnorm
       end do

c...............................................................................

c  hypothese de repartition:

c  on utilise ici l hypothese ad hoc:
c      ici: 2/3 sur j = 0, 1/3 sur j = 1
c      car rxn h3+ + x -> h2 + xh+ dominent

c  Test: Le H2 adsorbe sur les grains est converti
c        en para-H2 (J = 0) en un temps caracteristique
c        t_o2p = 1 minute ou 1 heure

       t_o2p = 60.0d0
c      t_o2p = 3600.0d0
       if (t_25(1,iopt) .ne. 0.0d0) then
         do mm=1,npg
           t_res(mm) = ab(i_h2gr+mm-1) / t_25(mm,iopt)
         end do
         frac_c = exp(-t_o2p/t_res)
       else
         frac_c = zero
       endif

       leva = level(0,0)
       create(leva) = create(leva) + deux * fabric / trois
     1        + sum(t_25(1:npg,iopt) * (h2para + frac_c * h2orth))
       levb = level(0,1)
       create(levb) = create(levb) + fabric / trois
     1        + sum(t_25(1:npg,iopt) * (1.0d0 - frac_c) * h2orth)

c  ou: tout sur j = 0

c      leva = level(0,0)
c      create(leva) = create(leva) + fabric

c...............................................................................

c  construction de la matrice totale:

       do i2=1,l0h2
         do i1=1,l0h2
           evltot(i1,i2) = (evcolh(i1,i2) + evlqua(i1,i2)
     1                   + evrdbc(i1,i2)) * ab(i_h2)
         end do
       end do

       do ii=1,l0h2
         evltot(ii,ii) = evltot(ii,ii) - destru + dd1
       end do

c Ajout des reactions endothermiques (destruction directe d'un niveau particulier)

       do k=1,nrxi
         ind = ikmat(i_h2,k)
         if (itype(ind) /= 6) cycle
         k1 = react(ind,1)
         k2 = react(ind,2)
         do ii=1,l0h2
           endo_rxn = max(bet(ind) - elhm(ii), 0.0_DP)
           rat_l = gamm(ind) * (ttry/300.0d0)**alpha(ind)
     1         * exp(-endo_rxn/ttry)
           evltot(ii,ii) = evltot(ii,ii) + rat_l * ab(k1) * ab(k2)
         end do
       end do

c   On soustrait de la diagonale les excitations electroniques
c     dues aux RC (electrons secondaires)
c     popetl, correspond a la repartition de H2 supposee par
c     Gredel (ETL a 30 K au 8 juillet 93)
c  10 mai 2004 : a refaire

c      do 104 jj=0,njh2
c        lev = level(0,jj)
c        if (lev .gt. l0h2) goto 104
c        jjj = mod(njlev(lev),2)
c        corosp = jjj * corort + (1 - jjj) * corpar
c        evltot(lev,lev) = evltot(lev,lev)
c    1                   + (popetl(jj) / xreh2(lev,iopt))
c    2                   * sentry * corosp * crnorm
c 104  continue

c   ****** DEBUG seulement

c       do 702 jcol=1,l0h2
c         jj = njlev(jcol)
c         TOTAL(jcol) = zero
c         do 701 jlin=1,l0h2
c           TOTAL(jcol) = TOTAL(jcol) + evcolh(jlin,jcol)
c  701    continue
c         if (abs(TOTAL(jcol)) .ge. 1.0e-15) then
c           print *, jcol, '  ', TOTAL(jcol)
c         endif
c  702  continue

c       do 712 jcol=1,l0h2
c         jj = njlev(jcol)
c         TOTAL(jcol) = zero
c         do 711 jlin=1,l0h2
c           TOTAL(jcol) = TOTAL(jcol) + evlqua(jlin,jcol)
c  711    continue
c         if (abs(TOTAL(jcol)) .ge. 1.0e-15) then
c           print *, jcol, '  ', TOTAL(jcol)
c         endif
c  712  continue

c       dumy = zero
c       do 722 jcol=1,l0h2
c         jj = njlev(jcol)
c         TOTAL(jcol) = zero
c         do 721 jlin=1,l0h2
c           TOTAL(jcol) = TOTAL(jcol) + evrdbc(jlin,jcol)
c  721    continue
cc        print *, jcol, '  ', TOTAL(jcol)
c         dumy = dumy + TOTAL(jcol) * xreh2(jcol,iopt)
c  722  continue
cc      print *, dumy, pdish2(iopt)

c  ****** fin DEBUG

       aev(1:mlevh2,1:mlevh2) = evltot(levinc,levinc)
       bev(1:mlevh2,1) = create(levinc)

c   Resolution: Utilisation de LAPACK

         nrhs = 1
         call dgesv (mlevh2, nrhs, aev, nlevh2, indx, bev, nlevh2, info)

       ceqh2 = zero
       do ii=1,mlevh2
         ceqh2 = ceqh2 + bev(ii,1)
       end do

       do ii=1,mlevh2
         bev(ii,1) = bev(ii,1) / ceqh2
       end do

       ceqh2 = ceqh2 - un
       if (abs(ceqh2) .gt. 1.0d-2) then
         write(iscrn,1001) ceqh2
       endif

c      print *, '  conservation H2: ', ceqh2

       do jv=0,nvxh2
         do jj=0,njh2
           ph2(jv,jj) = zero
         end do
       end do
       j2 = 0
       h2para = zero
       h2orth = zero
       do 140 lev=1,l0h2
         jj = njlev(lev)
         j2 = j2 + 1
         jv = nvlev(lev)
         xreh2(lev,iopt) = bev(j2,1)
         ph2(jv,jj) = bev(j2,1)
         if (mod(jj,2) .eq. 0) then
           h2para = h2para + bev(j2,1)
         else
           h2orth = h2orth + bev(j2,1)
         endif
  140  continue
c      print *, "  O/P:", h2para, h2orth, h2orth/h2para

 1001  format (5x,'normalisation H2:',1pd15.6)

       return
       end

c---------------------------------------------------------------------

c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine COLH2V (xnprot, ttry, iopt)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       use H2_DATA
       use SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c---------------------------------------------------------------------
c      xnprot is no. density of protons (cm-3)
c      ttry is temperature of neutral fluid

c      called from sub. EQLH2V
c
c      Les taux de collision H2 + H viennent de Martin & Mandy.
c          Fits refaits par Jacques Le Bourlot (juillet 1996)
c
c          ou
c
c          De D. Flower (cf MNRAS)
c
c      Les taux de collision H2 + He, H2 + H2o, H2 + H2p
c          viennent de D. Flower
c
c---------------------------------------------------------------------

       implicit double precision (a-h,o-z)

c  donnees de gerlich pour le transfert ortho-para par h+.

       dimension gerlic(30)

c  Avec prescription de David pour l'alternance ortho/ para (3/1)

       data gerlic / 2.024d-10, 9.575d-10, 2.729d-10, 7.910d-10
     1       , 2.246d-10, 6.223d-10, 1.828d-10, 4.975d-10, 1.484d-10
     2       , 3.000d-10, 1.000d-10, 3.000d-10, 1.000d-10, 3.000d-10
     3       , 1.000d-10, 3.000d-10, 1.000d-10, 3.000d-10, 1.000d-10
     4       , 3.000d-10, 1.000d-10, 3.000d-10, 1.000d-10, 3.000d-10
     5       , 1.000d-10, 3.000d-10, 1.000d-10, 3.000d-10, 1.000d-10
     6       , 3.000d-10 /

       data tddh2 / 4500.0d0 /

c   pour l'instant, les collisions avec H2 et He sont limitees a v = 0
c     On peut facilement deduire des estimations a partir des collisions avec h
c     Rappel c: desexcitation, d: excitation
c            h2h: H2 + H
c            h2h2: H2 + H2
c            h2he: H2 + He
c            ophp: H2 + H+ (ortho - para)

       if (ichh2 == 0) then
         nqc = 4
       else
         nqc = 3
       endif
       if (ichh2 .le. 1) then
         xchh2 = zero
       else
         xchh2 = un
       endif

       do 20 levu=2,l0h2
         ju = njlev(levu)
         ivu = nvlev(levu)

         do 10 levl=1,levu-1
           jl = njlev(levl)
           ivl = nvlev(levl)

c  ctod ext le facteur de Boltzmann permettant de calculer le
c     taux de d'excitation a partir du taux dde desexcitation.

           ndej = abs(ju - jl)
           fac = (elhm(levu) - elhm(levl)) / ttry
           ctod = exp(-fac) * (deux*ju+un) / (deux*jl+un)
           if (mod(ju,2) .eq. 1) then
             ctod = ctod * trois
           endif
           if (mod(jl,2) .eq. 1) then
             ctod = ctod / trois
           endif

c  Fit des taux de collision H2 + H. La variable reduite tdd
c      est "decalee" d'une valeur differente pour les
c      collisions reactive (delta(J) impair) et non reactives.
c      Ceci permet une forme asymptotique differente a basse
c      temperature.
c      En revanche, elle est la meme pour toutes les especes 
c         (contrairement a HD)

           if (mod(ndej,2) .eq. 0) then
             decal = 1.0d0
           else
             decal = 0.3d0
           endif
           tdd = decal + ttry * 1.0d-3
           tdd2 = tdd*tdd

c  a- Collisions reactives (toujours possibles)
c     et non-reactives sont melangees dans Martin & Mandy.

           gamma = qh1h2(1,levu,levl)
     1           + qh1h2(2,levu,levl) / tdd
     2           + qh1h2(3,levu,levl) / tdd2
           if (nqc == 4) then
             gamma = gamma +  qh1h2(4,levu,levl) * tdd
           endif

           ch2h_nr = 10.0d0**gamma
           ch2h_r  = 0.0d0

c  Ajout des collisions reactives (regle de David) si ichh2 = 3

           if (ichh2 .eq. 3 .and. tau(iopt) .le. 1.0d0) then

             if (mod(ndej,2) .eq. 0) then
               ch2h_r = 10.0d0**gamma *
     1                exp(-max(0.0d0,(3900.0d0
     2              - elhm(levu) + elhm(levl))/ttry))
             else
               if (jl .gt. 0) then
                 lev1 = level(ivl,jl-1)
               else
                 lev1 = level(ivl,jl+1)
               endif
               lev2 = level(ivl,jl+1)
               if (lev2 .eq. 0) then
                 lev2 = level(ivl,jl-1)
               endif
               ga1 = qh1h2(1,levu,lev1)
     1             + qh1h2(2,levu,lev1) / tdd
     2             + qh1h2(3,levu,lev1) / tdd2
               if (nqc == 4) then
                 ga1 = ga1 + qh1h2(4,levu,lev1) * tdd
               endif
               ga2 = qh1h2(1,levu,lev2)
     1             + qh1h2(2,levu,lev2) / tdd
     2             + qh1h2(3,levu,lev2) / tdd2
               if (nqc == 4) then
                 ga2 = ga2 + qh1h2(4,levu,lev2) * tdd
               endif
               ch2h_r = (10.0d0**ga1 + 10.0d0**ga2) * 0.5 *
     1                exp(-max(0.0d0,(3900.0d0
     2              - elhm(levu) + elhm(levl))/ttry))

               if (mod(ju,2) .eq. 1) then
                 ch2h_r = ch2h_r / 3.0d0
               endif

             endif
           endif

c  c2- Transfert ortho-para avec H d'apres Schofield
c      Plus prescription de David  (seulement dqns delta v = 0)

           if (ichh2 .ge. 2 .and.
     1       ivu .eq. ivl .and. ndej .lt. 3) then
             if (mod(ju,2) .eq. 1 .and. mod(ndej,2) .eq. 1) then
               ch2h_r = 8.0d-11 * exp(-3900.0d0/ttry) / 3.0d0
             else
               ch2h_r = 8.0d-11 * exp(-3900.0d0/ttry)
             endif
           endif

c          if (ichh2 .le. 1) then
c            ch2h = ch2h_nr
c          else
c            ch2h = ch2h_nr + ch2h_r
c          endif

           ch2h = ch2h_nr + xchh2 * ch2h_r

c  Experience de scaling

c          ch2h = ch2h * 0.3

c  He - H2

           gamma = qheh2(1,levu,levl)
     1           + qheh2(2,levu,levl) / tdd
     2           + qheh2(3,levu,levl) / tdd2

           ch2he = 10.0d0**gamma

c  pH2 - H2

           gamma = qph2h2(1,levu,levl)
     1           + qph2h2(2,levu,levl) / tdd
     2           + qph2h2(3,levu,levl) / tdd2

           ch2ph2 = 10.0d0**gamma

c  oH2 - H2

           gamma = qoh2h2(1,levu,levl)
     1           + qoh2h2(2,levu,levl) / tdd
     2           + qoh2h2(3,levu,levl) / tdd2

           ch2oh2 = 10.0d0**gamma

c  c- Transfert ortho-para avec les ions (Gerlich). Seulement dans v = 0

c          if (ivu .eq. 0 .and. ndej .eq. 1) then
           if (jl .eq. ju-1) then
             cophp = gerlic(ju)
           else
             cophp = zero
           endif

c  e- On recolle les morceaux:

           c = ch2ph2 * ab(i_h2) * h2para
     1       + ch2oh2 * ab(i_h2) * h2orth
     2       + ch2he * ab(i_he)
     3       + cophp * (xnprot + ab(i_h3p))
     4       + ch2h * ab(i_h)
           d = c * ctod

           evcolh(levl,levu) = evcolh(levl,levu) - c
           evcolh(levu,levu) = evcolh(levu,levu) + c
           evcolh(levu,levl) = evcolh(levu,levl) - d
           evcolh(levl,levl) = evcolh(levl,levl) + d

   10    continue

   20  continue

       return
       end

