
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine EQLHDV (ttry, iopt, inithd)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE HD_DATA
       USE SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c  ecrit la matrice d equilibre des niveaux de rotation
c  et vibration de hd (jusqu a l0hd) et resout le systeme

       implicit double precision (a-h,o-z)

       dimension evltot(nlevhd,nlevhd), create(nlevhd)
       dimension aev(nlevhd,nlevhd), bev(nlevhd,1),
     1           indx(nlevhd)

c  TIMHD contient les donnees necessaires au calcul des
c      temps caracteristiques de HD (en s-1 cm-3 a chaque fois).
c      TIMHD(iopt,1) : FABRIC : fabrication de HD par la chimie gazeuse.
c      TIMHD(iopt,2) : DESTRU : destruction de HD par la chimie gazeuse.
c      TIMHD(iopt,3) : RATEGR : fabrication de HD sur les grains.
c      TIMHD(iopt,4) : DESPHO : destruction de HD par les photons.

       dimension phd(0:nvxhd,0:njthd)

c      dimension TOTAL(nlevhd)

       evcohd(1:l0hd,1:l0hd) = zero
       evltot(1:l0hd,1:l0hd) = zero
       create(1:l0hd) = zero

c  Processus collisionnels
c      La matrice evcohd est remplie directement dans COLHDV

       call COLHD (ttry)

c  transitions dipolaires et quadrupolaires:
c  La matrice evdihd est initialisee dans init1 directement.

c  termes d origine purement chimique
c      on supprime donc la reaction de photodissociation

       fabric = zero
       destru = zero

c  taux de fabrication et de destruction de hd
c      en excluant la formation sur les grains (k=1)

       nrxi = ikmat(i_hd,0)

       do 50 k=1,nrxi
         ind = ikmat(i_hd,k)
         if (ind .eq. ifhdgr .or.
     1       ind .eq. idhdph) goto 50
         k1 = react(ind,1)
         k2 = react(ind,2)
         k3 = react(ind,3)
         taux = xkmat(i_hd,k)
         if (taux .gt. zero) then
           fabric = fabric + taux * ab(k1) * ab(k2)
     1            * ab(k3)
         else
           destru = destru + taux * ab(k1) * ab(k2)
     1            * ab(k3)
         endif
   50  continue

c--------------------------------------
c  formation de hd sur les grains:
c--------------------------------------

       rate = gamm(ifhdgr) * (tm1h(i_dgr) + tm1h(i_hgr)) / xkhydr
       timhd3(iopt) = rate * ab(i_hgr) * ab(i_dgr)
       rateff = rate * ab(i_hgr) * ab(i_dgr)

       timhd1(iopt) = fabric
       timhd2(iopt) = - destru
       timhd4(iopt) = pdishd(iopt) * ab(i_hd)

c      print *, fabric, rate, ab(i_hgr), ab(i_dgr)
c      print *, destru, pdishd(iopt), ab(i_hd)

       dumy = zero
       do lev=1,nlevhd
         ij = njhd(lev)
         if (ij .le. mjhd) then
           iv = nvhd(lev)
c          print *, iv, ij, pophd(iv,ij), elhd(lev)
           dumy = dumy + pophd(iv,ij)
         endif
       end do
c      print *, '  normalisation de pophd:', dumy
       echpod = dumy


       do lev=1,l0hd
         ij = njhd(lev)
         if (ij .le. mjhd) then
           iv = nvhd(lev)
           dumy = zero
           do levu=l0hd+1,nlevhd
             iju = njhd(levu)
             if (iju .le. mjhd) then
               ivu = nvhd(levu)
               dumy = dumy + pophd(ivu,iju) * cashd(levu,lev)
     1              / echpod
             endif
           end do
           create(lev) = (dumy + pophd(iv,ij) / echpod)
     1                 * rateff
         endif
       end do

c...............................................................................

c  hypothese de repartition:
c  si equipartition, il faut utiliser

c      fabric = fabric / (mjhd + 1)

c  et le rajouter a tous les create.

c      do jj=0,mjhd
c        lev = levhd(0,jj)
c        create(lev) = create(lev) + fabric
c      end do


c  ou: tout sur j = 0

      leva = levhd(0,0)
      create(leva) = create(leva) + fabric

c...............................................................................

c  construction de la matrice totale:

       do i2=1,l0hd
         do i1=1,l0hd
           evltot(i1,i2) = (evcohd(i1,i2) + evdihd(i1,i2)
     1                   + evrdhd(i1,i2)) * ab(i_hd)
         end do
       end do

       do ii=1,l0hd
         evltot(ii,ii) = evltot(ii,ii) - destru
       end do


c   ****** DEBUG seulement

c      do 702 jcol=1,l0hd
c        jj = njhd(jcol)
c        if (jj .gt. mjhd) goto 702
c        TOTAL(jcol) = zero
c        do 701 jlin=1,l0hd
c          TOTAL(jcol) = TOTAL(jcol) + evcohd(jlin,jcol)
c 701    continue
c        if (abs(TOTAL(jcol)) .ge. 1.0e-15) then
c          print *, jcol, '  ', TOTAL(jcol)
c        endif
c 702  continue
c      pause

c      do 712 jcol=1,l0hd
c        jj = njhd(jcol)
c        if (jj .gt. mjhd) goto 712
c        TOTAL(jcol) = zero
c        do 711 jlin=1,l0hd
c          TOTAL(jcol) = TOTAL(jcol) + evdihd(jlin,jcol)
c 711    continue
c        print *, jcol, '  ', TOTAL(jcol)
c 712  continue
c      pause

c      dumy = zero
c      do 722 jcol=1,l0hd
c        jj = njhd(jcol)
c        if (jj .gt. mjhd) goto 722
c        TOTAL(jcol) = zero
c        do 721 jlin=1,l0hd
c          TOTAL(jcol) = TOTAL(jcol) + evrdhd(jlin,jcol)
c 721    continue
c        print *, jcol, '  ', TOTAL(jcol)
c        dumy = dumy + TOTAL(jcol) * xrehd(jcol,iopt)
c 722  continue
c      print *, dumy, pdishd(iopt)

c      pause

c  ****** fin DEBUG


c  Preparation de la resolution:

       j2 = 0
       do 112 i2=1,l0hd
         jj = njhd(i2)
         if (jj .gt. mjhd) goto 112
         j2 = j2 + 1
         j1 = 0
         do 111 i1=1,l0hd
           jj = njhd(i1)
           if (jj .gt. mjhd) goto 111
           j1 = j1 + 1
           aev(j1,j2) = evltot(i1,i2)
  111    continue
         bev(j2,1) = create(i2)
  112  continue

       nat = j2

c   Resolution: Utilisation de LAPACK

         nrhs = 1
         call dgesv (l0hd, nrhs, aev, nlevhd, indx, bev, nlevhd, info)

       ceqhd = zero
       do ii=1,nat
         ceqhd = ceqhd + bev(ii,1)
       end do

       do ii=1,nat
         bev(ii,1) = bev(ii,1) / ceqhd
       end do

       ceqhd = ceqhd - un
       if (abs(ceqhd) .gt. 1.0d-2) then
         write(iscrn,1001) ceqhd
       endif

c      print *, '  conservation HD: ', ceqhd

       phd = zero
       j2 = 0
       do 140 lev=1,l0hd
         jj = njhd(lev)
         if (jj .gt. mjhd) goto 140
         j2 = j2 + 1
         jv = nvhd(lev)
         xrehd(lev,iopt) = bev(j2,1)
         phd(jv,jj) = bev(j2,1)
  140  continue

 1001  format (5x,'normalisation HD:',1pd15.6)

       return
       end

c-------------------------------------------------------------

c       %%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLHD (ttry)
c       %%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE HD_DATA
       USE H2_DATA
       USE SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c--------------------------------------------------
c        ttry  : temperature du nuage dans la couche
c--------------------------------------------------

       implicit double precision (a-h, o-z)

c  Attention: evcohd(levl,levu) < 0
c             evcohd(levu,levu) > 0

c     Nouveaux fits (Jacques (1999))
       deche = 1.0d0 / 3.0d0
       tddhe = deche + ttry * 1.0d-3
       dech1 = 2.0d0 / 3.0d0
       tddh1 = dech1 + ttry * 1.0d-3
       dech2 = 0.5d0
       tddh2 = dech2 + ttry * 1.0d-3

       do ju=1,mjhd    
         do jl=0,ju-1

            levu = levhd(0,ju)
            levl = levhd(0,jl)
            fac = (elhd(levu) - elhd(levl)) / ttry
            ctod = exp(-fac) * (deux*ju+un) / (deux*jl+un)

c Collision HD - He

            chdhe = 10.0d0**(qhehd(1,levu,levl)
     1                     + qhehd(2,levu,levl) / tddhe
     2                     + qhehd(3,levu,levl) / (tddhe*tddhe))

c Collision HD - H
 
            chdh = 10.0d0**(qh1hd(1,levu,levl)
     1                    + qh1hd(2,levu,levl) / tddh1
     2                    + qh1hd(3,levu,levl) / (tddh1*tddh1))

c Collision HD - H2 para
 
            chdph2 = 10.0d0**(qph2hd(1,levu,levl)
     1                    + qph2hd(2,levu,levl) / tddh2
     2                    + qph2hd(3,levu,levl) / (tddh2*tddh2))

C collisions HD - H2 ortho

            chdoh2 = 10.0d0**(qoh2hd(1,levu,levl)
     1                      + qoh2hd(2,levu,levl) / tddh2
     2                      + qoh2hd(3,levu,levl) / (tddh2*tddh2))

            c = chdhe * ab(i_he)
     1        + chdh * ab(i_h)
     2        + chdph2 * ab(i_h2) * h2para
     3        + chdoh2 * ab(i_h2) * h2orth
            d = c * ctod

            evcohd(levl,levu) = evcohd(levl,levu) - c
            evcohd(levu,levu) = evcohd(levu,levu) + c
            evcohd(levu,levl) = evcohd(levu,levl) - d
            evcohd(levl,levl) = evcohd(levl,levl) + d

           end do
         end do

c  Collisions avec les electrons -------------------------------------
c     cf Dickinson et al.  A&A 54,645(1977)       

       dip = 8.4d-4
       betas = 11600.0d0 / ttry
       b0 = 45.655d0
       do levl=1,l0hd-1
         nj = njhd(levl)
         if (nj .lt. mjhd) then
           nv = nvhd(levl)
           levu = levhd(nv,nj+1)
           deltaE = 2.48d-04 * b0 * (nj+1)
           c = 9.08d03 / (b0*(nj+1))
           taux = (3.56d-06 * (dip**2) / sqrt(ttry))
     1          * (nj+1) / (2*nj+1)
           taux = taux * exp(-betas*deltaE)
           taux = taux * log(c*deltaE+(c/betas)
     1          * exp(-0.577d0 / (un+deux*betas*deltaE)))
           taux = taux * ab(nspec)
           evcohd(levu,levl) = evcohd(levu,levl) - taux
           evcohd(levl,levl) = evcohd(levl,levl) + taux
           fac = (elhd(levu) - elhd(levl)) / ttry
           ctod = exp(fac) * (deux*nj+un) / (deux*nj+trois)
           taux = taux * ctod
           evcohd(levl,levu) = evcohd(levl,levu) - taux
           evcohd(levu,levu) = evcohd(levu,levu) + taux
         endif
       end do

       return
       end

