
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine FROID (ttry, iopt)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE SP_DATA
       USE H2_DATA
       USE HD_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c  calcul de l excitation de O, C, C+
c  et du taux de refroidissement par les raies de structure fine
c  et les raies de rotation des molecules
c  les probabilites d echappement sont calculees suivant FGK
c  fits refait par David Flower

c  les taux d excitation collisionnels de C  O et C+ ont ete refites
c  a partir des donnees de collision initiales:
c  ref.1 Launay et Roueff A & A 56, 289 (1977)
c  ref.2 Jaquet, Staemmler, Smith & Flower, J.Phys.B (1991) in press
c  ref.3 Monteiro & Flower, MNRAS 228, 101 (1987)
c  ref.4 Schroder, Staemmler, Smith, Flower & Jaquet (1991)
c  ref.5 Staemmler, Flower(1991)  J.Phys.B (1991) in press
c        ou Lavendy et al ???????
c  ref.6 Launay et Roueff J.Phys.B 10, 879 (1977)
c  ref.7 Flower et Launay, J.Phys.B 10,3673 (1977)
c  ref.8 Hayes & Nussbaumer AA 134, 193(1984)
c      Hayes & Nussbaumer AA 139, 233(1984)
c      ou  Keenan, Lennon, Johnson & Kingston MNRAS 220,577 (1986) ???
c      Flower et Launay, MNRAS 214, 271 (1985)
c  ...

       implicit double precision (a-h,o-z)

       REAL (KIND=DP), dimension(0:noptm)   :: tem_opd
       REAL (KIND=DP) :: interpcd
       REAL (KIND=DP) :: vt2

       save

c  Pour la premire couche...

       if (iopt > 0) then
         jopt = iopt - 1
       else
         jopt = 0
       endif

c     *********************************************************************
c     ct1= pi e^2/mc = 2.654057d-2
c        =(section d absorption totale integree sur les frequences) en cm^2
c     ct2= 2 k / amu   = 1.662902e8  en cgs
c     bdop:  dimension vitesse cm s-1 a
c     hcsurk = h c / k
c     ******************************************************************

       ct1 = xpi * xqe2 / xme / clum
       ct2 = deux * xk / amu
       vt2 = vturb * vturb

       xnhmp = h2para * ab(i_h2)
       xnhmo = h2orth * ab(i_h2)

c  Il faut toujours appeler FROID avant CHAUD pour que la
c     mise a zero suivante soit faite correctement.

       reftot(iopt) = zero
       chotot(iopt) = zero

c  I-refroidissement par H2:
c  ------------------------

c   ATTENTION: evrdbc et evlqua sont NEGATIFS (voir PREDAT et EQLH2V)

       rfjh2t(iopt) = zero

c   13 decembre 1991 le bilan global ne doit inclure que des processus
c radiatifs

c  27 Jan 93: Il faut recalculer l equilibre de H2 si la temperature varie trop!

        call EQLH2V (ttry, iopt)

c   rfjh2t fait le bilan du chauffage ou refroidissement par h2

       tempo1 = zero
       do 20 lev2=1,l0h2
         nju = njlev(lev2)
         tempo2 = zero
         enerhi = elhm(lev2)
         do lev1=1,l0h2
           tempo2 = tempo2 + (enerhi - elhm(lev1))
     1       * (evlqua(lev1,lev2) + evrdbc(lev1,lev2))
         end do
c        print *, tempo2, xreh2(lev2,iopt)
         tempo1 = tempo1 + tempo2 * xreh2(lev2,iopt)
   20  continue
c      tempo3 = sum(xreh2(:,iopt) * (elhm*sum(evlqua+evrdbc, dim=1)
c    1        - matmul(elhm,evlqua+evrdbc)))

       rfjh2t(iopt) = - tempo1 * ab(i_h2) * xk

c      print *, "  ref H2:", rfjh2t(iopt)

c  I bis - refroidissement par HD:
c  ------------------------
 
c   ATTENTION: evrdhd et evdihd sont NEGATIFS (voir PREDAT et EQLHDV)
 
       rfjhdt(iopt) = zero
 
       if (i_hd .ne. 0) then

         call EQLHDV (ttry, iopt, inithd)
 
c   rfjhdt fait le bilan du chauffage ou refroidissement par hd
 
         tempo1 = zero
         do 21 lev2=1,l0hd
           nju = njhd(lev2)
           if (nju .gt. mjhd) goto 21
           tempo2 = zero
           enerhi = elhd(lev2)
           do 11 lev1=1,l0hd
             tempo2 = tempo2 + (enerhi - elhd(lev1))
     1         * (evdihd(lev1,lev2) + evrdhd(lev1,lev2))
   11      continue
           tempo1 = tempo1 + tempo2 * xrehd(lev2,iopt)
   21    continue
 
         rfjhdt(iopt) = - tempo1 * ab(i_hd) * xk
 
c        print *, rfjhdt(iopt)

       endif

c  A partir d'ici, la possibilite de laisser des photons s'echapper
c    par la face opposee est incluse.

c  Point d'interpolation dans la grille precedente (inutile de le refaire a chaque fois)
 
         do j=1,npo
           if (tau(iopt) < tau_o(j)) then
             icd_o = j-1
             exit
           endif
         end do
 
         raptau = (tau(iopt) - tau_o(icd_o))
     1           / (tau_o(icd_o+1) - tau_o(icd_o))

         do j=1,npo
c          if (tau_o(j) == tau_imp(n_imp-10)) then
           if (tau_o(j) == tau_imp(n_imp-1)) then
             jcd_o = j
             exit
           endif
         end do

         raptin = (max(2.0_dp*tau_o(npo),100.0_dp) - tau_o(jcd_o))
     1          / (tau_o(npo) - tau_o(jcd_o))

c  refroidissement par co
c  ----------------------

        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amco0)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amco0)

        fabchi = zero
        deschi = zero
        nrxi = ikmat(i_co0,0)
c  La photodestruction est traitee de facon detaillee dans eqrco0

        do 600 k=1,nrxi
          ind = ikmat(i_co0,k)
          if (ind .eq. idco0p .and. itrco0 .ne. 0) goto 600
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_co0,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
  600   continue

       timco1(iopt) = fabchi
       timco2(iopt) = - deschi
       timco3(iopt) = pdico0(iopt) * ab(i_co0)

        ntr = ntrco0
        nlv = nlvco0

        do i=1,ntrco0
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdco0_o(i,npo) - opdco0_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdco0(i,jopt)
          aij(i) = aijco0(i)
          wlk(i) = wlkco0(i)
          iup(i) = iupco0(i)
          jlo(i) = jloco0(i)
          opdr(i) = opdco0_o(i,npo) + opdin
     1            - interpcd(raptau, opdco0_o(i,0:npo), icd_o, npo)
        end do

        do ju=1,nlvco0
          gst(ju) = gstco0(ju)
          elk(ju) = elkco0(ju)
          xrepre(ju) = xreco0(ju,jopt)
          xreici(ju) = xreco0(ju,jopt)
        end do

        call COLCO0 (ttry)

        abpre = abnua(i_co0,jopt)
        abici = abnua(i_co0,iopt)

        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refco0(iopt) = zero
        do i=1,ntrco0
          opdco0(i,iopt) = opd(i)
          emico0(i,iopt) = emi(i)
          refco0(iopt) = refco0(iopt) + emi(i)
        end do

        do ju=1,nlvco0
          xreco0(ju,iopt) = xreici(ju)
        end do

        fabco = fabchi
        desco = deschi

c  refroidissement par c*o
c  -----------------------

        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amco1)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amco1)

        if (itrco1 .ne. 0) then
          fabchi = zero
          deschi = zero
           nrxi = ikmat(i_co1,0)

          do 601 k=1,nrxi
            ind = ikmat(i_co1,k)
            if (ind .eq. idco1p) goto 601
            k1 = react(ind,1)
            k2 = react(ind,2)
            k3 = react(ind,3)
            taux = xkmat(i_co1,k)
            if (taux .gt. zero) then
              fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
            else
              deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
            endif
  601     continue
        else
          fabchi = fabco / 90.0d0
          deschi = desco / 90.0d0
        endif

        ntr = ntrco1
        nlv = nlvco1

        do i=1,ntrco1
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdco1_o(i,npo) - opdco1_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdco1(i,jopt)
          aij(i) = aijco1(i)
          wlk(i) = wlkco1(i)
          iup(i) = iupco1(i)
          jlo(i) = jloco1(i)
          opdr(i) = opdco1_o(i,npo) + opdin
     1            - interpcd(raptau, opdco1_o(i,0:npo), icd_o, npo)
        end do

        do ju=1,nlvco1
          gst(ju) = gstco1(ju)
          elk(ju) = elkco1(ju)
          xrepre(ju) = xreco1(ju,jopt)
          xreici(ju) = xreco1(ju,jopt)
        end do

        call COLCO1 (ttry)

        if (itrco1 .ne. 0) then
          abpre = abnua(i_co1,jopt)
          abici = abnua(i_co1,iopt)
        else
          abpre = abnua(i_co0,jopt) / 90.0d0
          abici = abnua(i_co0,iopt) / 90.0d0
        endif

        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refco1(iopt) = zero
        do i=1,ntrco1
          opdco1(i,iopt) = opd(i)
          emico1(i,iopt) = emi(i)
          refco1(iopt) = refco1(iopt) + emi(i)
        end do

        do ju=1,nlvco1
          xreco1(ju,iopt) = xreici(ju)
        end do

c  refroidissement par co*
c  -----------------------


        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amco2)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amco2)

        if (itrco2 .ne. 0) then
          fabchi = zero
          deschi = zero
           nrxi = ikmat(i_co2,0)

          do 602 k=1,nrxi
            ind = ikmat(i_co2,k)
            if (ind .eq. idco2p) goto 602
            k1 = react(ind,1)
            k2 = react(ind,2)
            k3 = react(ind,3)
            taux = xkmat(i_co2,k)
            if (taux .gt. zero) then
              fabchi = fabchi + taux * ab(k1) * ab(k2)
     1               * ab(k3)
            else
              deschi = deschi + taux * ab(k1) * ab(k2)
     1               * ab(k3)
            endif
  602     continue
        else
          fabchi = fabco / 499.0d0
          deschi = desco / 499.0d0
        endif

        ntr = ntrco2
        nlv = nlvco2

        do i=1,ntrco2
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdco2_o(i,npo) - opdco2_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdco2(i,jopt)
          aij(i) = aijco2(i)
          wlk(i) = wlkco2(i)
          iup(i) = iupco2(i)
          jlo(i) = jloco2(i)
          opdr(i) = opdco2_o(i,npo) + opdin
     1            - interpcd(raptau, opdco2_o(i,0:npo), icd_o, npo)
        end do

        do ju=1,nlvco2
          gst(ju) = gstco2(ju)
          elk(ju) = elkco2(ju)
          xrepre(ju) = xreco2(ju,jopt)
          xreici(ju) = xreco2(ju,jopt)
        end do

        call COLCO2 (ttry)

        if (itrco2 .ne. 0) then
          abpre = abnua(i_co2,jopt)
          abici = abnua(i_co2,iopt)
        else
          abpre = abnua(i_co0,jopt) / 499.0d0
          abici = abnua(i_co0,iopt) / 499.0d0
        endif

        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refco2(iopt) = zero
        do i=1,ntrco2
          opdco2(i,iopt) = opd(i)
          emico2(i,iopt) = emi(i)
          refco2(iopt) = refco2(iopt) + emi(i)
        end do

        do ju=1,nlvco2
          xreco2(ju,iopt) = xreici(ju)
        end do

c  refroidissement par c*o*
c  ------------------------

        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amco3)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amco3)

        if (itrco3 .ne. 0) then
          fabchi = zero
          deschi = zero
           nrxi = ikmat(i_co3,0)

          do 603 k=1,nrxi
            ind = ikmat(i_co3,k)
            if (ind .eq. idco3p) goto 603
            k1 = react(ind,1)
            k2 = react(ind,2)
            k3 = react(ind,3)
            taux = xkmat(i_co3,k)
            if (taux .gt. zero) then
              fabchi = fabchi + taux * ab(k1) * ab(k2)
     1               * ab(k3)
            else
              deschi = deschi + taux * ab(k1) * ab(k2)
     1               * ab(k3)
            endif
  603     continue
        else
          fabchi = fabco / (90.0d0 * 499.0d0)
          deschi = desco / (90.0d0 * 499.0d0)
        endif

        ntr = ntrco3
        nlv = nlvco3

        do i=1,ntrco3
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdco3_o(i,npo) - opdco3_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdco3(i,jopt)
          aij(i) = aijco3(i)
          wlk(i) = wlkco3(i)
          iup(i) = iupco3(i)
          jlo(i) = jloco3(i)
          opdr(i) = opdco3_o(i,npo) + opdin
     1            - interpcd(raptau, opdco3_o(i,0:npo), icd_o, npo)
        end do

        do ju=1,nlvco3
          gst(ju) = gstco3(ju)
          elk(ju) = elkco3(ju)
          xrepre(ju) = xreco3(ju,jopt)
          xreici(ju) = xreco3(ju,jopt)
        end do

        call COLCO3 (ttry)

        if (itrco3 .ne. 0) then
          abpre = abnua(i_co3,jopt)
          abici = abnua(i_co3,iopt)
        else
          abpre = abnua(i_co0,jopt) / (90.0d0 * 499.0d0)
          abici = abnua(i_co0,iopt) / (90.0d0 * 499.0d0)
        endif

        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refco3(iopt) = zero
        do i=1,ntrco3
          opdco3(i,iopt) = opd(i)
          emico3(i,iopt) = emi(i)
          refco3(iopt) = refco3(iopt) + emi(i)
        end do

        do ju=1,nlvco3
          xreco3(ju,iopt) = xreici(ju)
        end do

c  Excitation de C:
c  ----------------

        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amcat)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amcat)

        fabchi = zero
        deschi = zero
        nrxi = ikmat(i_c,0)

        do k=1,nrxi
          ind = ikmat(i_c,k)
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_c,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
        end do

        ntr = ntrcat
        nlv = nlvcat

        do i=1,ntrcat
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdcat_o(i,npo) - opdcat_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdcat(i,jopt)
          aij(i) = aijcat(i)
          wlk(i) = wlkcat(i)
          iup(i) = iupcat(i)
          jlo(i) = jlocat(i)
          opdr(i) = opdcat_o(i,npo) + opdin
     1            - interpcd(raptau, opdcat_o(i,0:npo), icd_o, npo)
        end do

        do ju=1,nlvcat
          gst(ju) = gstcat(ju)
          elk(ju) = elkcat(ju)
          xrepre(ju) = xrecat(ju,jopt)
          xreici(ju) = xrecat(ju,jopt)
        end do

        call COLCAT (ttry, xnhmp, xnhmo)

        abpre = abnua(i_c,jopt)
        abici = abnua(i_c,iopt)

        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refcat(iopt) = zero
        do i=1,ntrcat
          opdcat(i,iopt) = opd(i)
          emicat(i,iopt) = emi(i)
          refcat(iopt) = refcat(iopt) + emi(i)
        end do

        do ju=1,nlvcat
          xrecat(ju,iopt) = xreici(ju)
        end do

c  Excitation de N:
c  ----------------

        if (i_n /= 0) then
        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amnat)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amnat)

        fabchi = zero
        deschi = zero
        nrxi = ikmat(i_n,0)

        do k=1,nrxi
          ind = ikmat(i_n,k)
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_n,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
        end do

        ntr = ntrnat
        nlv = nlvnat

        do i=1,ntrnat
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdnat_o(i,npo) - opdnat_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdnat(i,jopt)
          aij(i) = aijnat(i)
          wlk(i) = wlknat(i)
          iup(i) = iupnat(i)
          jlo(i) = jlonat(i)
          opdr(i) = opdnat_o(i,npo) + opdin
     1            - interpcd(raptau, opdnat_o(i,0:npo), icd_o, npo)
        end do

        do ju=1,nlvnat
          gst(ju) = gstnat(ju)
          elk(ju) = elknat(ju)
          xrepre(ju) = xrenat(ju,jopt)
          xreici(ju) = xrenat(ju,jopt)
        end do

        call COLNAT (ttry, xnhmp, xnhmo)

        abpre = abnua(i_n,jopt)
        abici = abnua(i_n,iopt)

        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refnat(iopt) = zero
        do i=1,ntrnat
          opdnat(i,iopt) = opd(i)
          eminat(i,iopt) = emi(i)
          refnat(iopt) = refnat(iopt) + emi(i)
        end do

        do ju=1,nlvnat
          xrenat(ju,iopt) = xreici(ju)
        end do
        endif

c  Excitation de O:
c  ----------------

        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amoat)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amoat)

        fabchi = zero
        deschi = zero
        nrxi = ikmat(i_o,0)

        do k=1,nrxi
          ind = ikmat(i_o,k)
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_o,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
        end do

        ntr = ntroat
        nlv = nlvoat

        do i=1,ntroat
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdoat_o(i,npo) - opdoat_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdoat(i,jopt)
          aij(i) = aijoat(i)
          wlk(i) = wlkoat(i)
          iup(i) = iupoat(i)
          jlo(i) = jlooat(i)
          tem_opd(0:npo) =  opdoat_o(i,0:npo)
          opdr(i) = opdoat_o(i,npo) + opdin
     1            - interpcd(raptau, tem_opd(0:npo), icd_o, npo)
        end do

        do ju=1,nlvoat
          gst(ju) = gstoat(ju)
          elk(ju) = elkoat(ju)
          xrepre(ju) = xreoat(ju,jopt)
          xreici(ju) = xreoat(ju,jopt)
        end do

        call COLOAT (ttry, xnhmp, xnhmo)

        abpre = abnua(i_o,jopt)
        abici = abnua(i_o,iopt)


        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refoat(iopt) = zero
        do i=1,ntroat
          opdoat(i,iopt) = opd(i)
          emioat(i,iopt) = emi(i)
          refoat(iopt) = refoat(iopt) + emi(i)
        end do

        do ju=1,nlvoat
          xreoat(ju,iopt) = xreici(ju)
        end do

c  Excitation de S:
c  ----------------

        if (i_s /= 0) then
        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amsat)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amsat)

        fabchi = zero
        deschi = zero
        nrxi = ikmat(i_s,0)

        do k=1,nrxi
          ind = ikmat(i_s,k)
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_s,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
        end do

        ntr = ntrsat
        nlv = nlvsat

        do i=1,ntrsat
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdsat_o(i,npo) - opdsat_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdsat(i,jopt)
          aij(i) = aijsat(i)
          wlk(i) = wlksat(i)
          iup(i) = iupsat(i)
          jlo(i) = jlosat(i)
          tem_opd(0:npo) =  opdsat_o(i,0:npo)
          opdr(i) = opdsat_o(i,npo) + opdin
     1            - interpcd(raptau, tem_opd(0:npo), icd_o, npo)
        end do

        do ju=1,nlvsat
          gst(ju) = gstsat(ju)
          elk(ju) = elksat(ju)
          xrepre(ju) = xresat(ju,jopt)
          xreici(ju) = xresat(ju,jopt)
        end do

        call COLSAT (ttry, xnhmp, xnhmo)

        abpre = abnua(i_s,jopt)
        abici = abnua(i_s,iopt)


        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refsat(iopt) = zero
        do i=1,ntrsat
          opdsat(i,iopt) = opd(i)
          emisat(i,iopt) = emi(i)
          refsat(iopt) = refsat(iopt) + emi(i)
        end do

        do ju=1,nlvsat
          xresat(ju,iopt) = xreici(ju)
        end do
        endif

c  Excitation de Si:
c  ----------------

        if (i_si /= 0) then
        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amsiat)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amsiat)

        fabchi = zero
        deschi = zero
        nrxi = ikmat(i_si,0)

        do k=1,nrxi
          ind = ikmat(i_si,k)
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_si,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
        end do

        ntr = ntrsiat
        nlv = nlvsiat

        do i=1,ntrsiat
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdsiat_o(i,npo) - opdsiat_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdsiat(i,jopt)
          aij(i) = aijsiat(i)
          wlk(i) = wlksiat(i)
          iup(i) = iupsiat(i)
          jlo(i) = jlosiat(i)
          tem_opd(0:npo) =  opdsiat_o(i,0:npo)
          opdr(i) = opdsiat_o(i,npo) + opdin
     1            - interpcd(raptau, tem_opd(0:npo), icd_o, npo)
        end do

        do ju=1,nlvsiat
          gst(ju) = gstsiat(ju)
          elk(ju) = elksiat(ju)
          xrepre(ju) = xresiat(ju,jopt)
          xreici(ju) = xresiat(ju,jopt)
        end do

        call COLSiAT (ttry, xnhmp, xnhmo)

        abpre = abnua(i_si,jopt)
        abici = abnua(i_si,iopt)


        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refsiat(iopt) = zero
        do i=1,ntrsiat
          opdsiat(i,iopt) = opd(i)
          emisiat(i,iopt) = emi(i)
          refsiat(iopt) = refsiat(iopt) + emi(i)
        end do

        do ju=1,nlvsiat
          xresiat(ju,iopt) = xreici(ju)
        end do
        endif

c  Excitation de C+
c  ----------------

        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amcpl)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amcpl)

        fabchi = zero
        deschi = zero
        nrxi = ikmat(i_cp,0)

        do k=1,nrxi
          ind = ikmat(i_cp,k)
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_cp,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
        end do

        ntr = ntrcpl
        nlv = nlvcpl

        do i=1,ntrcpl
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdcpl_o(i,npo) - opdcpl_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdcpl(i,jopt)
          aij(i) = aijcpl(i)
          wlk(i) = wlkcpl(i)
          iup(i) = iupcpl(i)
          jlo(i) = jlocpl(i)
          opdr(i) = opdcpl_o(i,npo) + opdin
     1            - interpcd(raptau, opdcpl_o(i,0:npo), icd_o, npo)
        end do

        do ju=1,nlvcpl
          gst(ju) = gstcpl(ju)
          elk(ju) = elkcpl(ju)
          xrepre(ju) = xrecpl(ju,jopt)
          xreici(ju) = xrecpl(ju,jopt)
        end do

        call COLCPL (ttry, xnhmp, xnhmo)

        abpre = abnua(i_cp,jopt)
        abici = abnua(i_cp,iopt)

        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refcpl(iopt) = zero
        do i=1,ntrcpl
          opdcpl(i,iopt) = opd(i)
          emicpl(i,iopt) = emi(i)
          refcpl(iopt) = refcpl(iopt) + emi(i)
        end do

        do ju=1,nlvcpl
          xrecpl(ju,iopt) = xreici(ju)
        end do

c  Excitation de N+
c  ----------------

        if (i_np /= 0) then
        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amnpl)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amnpl)

        fabchi = zero
        deschi = zero
        nrxi = ikmat(i_np,0)

        do k=1,nrxi
          ind = ikmat(i_np,k)
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_np,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
        end do

        ntr = ntrnpl
        nlv = nlvnpl

        do i=1,ntrnpl
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdnpl_o(i,npo) - opdnpl_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdnpl(i,jopt)
          aij(i) = aijnpl(i)
          wlk(i) = wlknpl(i)
          iup(i) = iupnpl(i)
          jlo(i) = jlonpl(i)
          opdr(i) = opdnpl_o(i,npo) + opdin
     1            - interpcd(raptau, opdnpl_o(i,0:npo), icd_o, npo)
        end do

        do ju=1,nlvnpl
          gst(ju) = gstnpl(ju)
          elk(ju) = elknpl(ju)
          xrepre(ju) = xrenpl(ju,jopt)
          xreici(ju) = xrenpl(ju,jopt)
        end do

        call COLNPL (ttry, xnhmp, xnhmo)

        abpre = abnua(i_np,jopt)
        abici = abnua(i_np,iopt)

        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refnpl(iopt) = zero
        do i=1,ntrnpl
          opdnpl(i,iopt) = opd(i)
          eminpl(i,iopt) = emi(i)
          refnpl(iopt) = refnpl(iopt) + emi(i)
        end do

        do ju=1,nlvnpl
          xrenpl(ju,iopt) = xreici(ju)
        end do
        endif

c  Excitation de O+
c  ----------------

        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amopl)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amopl)

        fabchi = zero
        deschi = zero
        nrxi = ikmat(i_op,0)

        do k=1,nrxi
          ind = ikmat(i_op,k)
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_op,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
        end do

        ntr = ntropl
        nlv = nlvopl

        do i=1,ntropl
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdopl_o(i,npo) - opdopl_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdopl(i,jopt)
          aij(i) = aijopl(i)
          wlk(i) = wlkopl(i)
          iup(i) = iupopl(i)
          jlo(i) = jloopl(i)
          opdr(i) = opdopl_o(i,npo) + opdin
     1            - interpcd(raptau, opdopl_o(i,0:npo), icd_o, npo)
        end do

        do ju=1,nlvopl
          gst(ju) = gstopl(ju)
          elk(ju) = elkopl(ju)
          xrepre(ju) = xreopl(ju,jopt)
          xreici(ju) = xreopl(ju,jopt)
        end do

        call COLOPL (ttry, xnhmp, xnhmo)

        abpre = abnua(i_op,jopt)
        abici = abnua(i_op,iopt)

        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refopl(iopt) = zero
        do i=1,ntropl
          opdopl(i,iopt) = opd(i)
          emiopl(i,iopt) = emi(i)
          refopl(iopt) = refopl(iopt) + emi(i)
        end do

        do ju=1,nlvopl
          xreopl(ju,iopt) = xreici(ju)
        end do

c  Excitation de S+
c  ----------------

        if (i_sp /= 0) then

        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amspl)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amspl)

        fabchi = zero
        deschi = zero
        nrxi = ikmat(i_sp,0)

        do k=1,nrxi
          ind = ikmat(i_sp,k)
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_sp,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
        end do

        ntr = ntrspl
        nlv = nlvspl

        do i=1,ntrspl
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdspl_o(i,npo) - opdspl_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdspl(i,jopt)
          aij(i) = aijspl(i)
          wlk(i) = wlkspl(i)
          iup(i) = iupspl(i)
          jlo(i) = jlospl(i)
          opdr(i) = opdspl_o(i,npo) + opdin
     1            - interpcd(raptau, opdspl_o(i,0:npo), icd_o, npo)
        end do

        do ju=1,nlvspl
          gst(ju) = gstspl(ju)
          elk(ju) = elkspl(ju)
          xrepre(ju) = xrespl(ju,jopt)
          xreici(ju) = xrespl(ju,jopt)
        end do

        call COLSPL (ttry, xnhmp, xnhmo)

        abpre = abnua(i_sp,jopt)
        abici = abnua(i_sp,iopt)

        call DEJONG (iopt, velici, velpre, abpre, abici
     1              , fabchi, deschi, ttry)

        refspl(iopt) = zero
        do i=1,ntrspl
          opdspl(i,iopt) = opd(i)
          emispl(i,iopt) = emi(i)
          refspl(iopt) = refspl(iopt) + emi(i)
        end do

        do ju=1,nlvspl
          xrespl(ju,iopt) = xreici(ju)
        end do

        endif

c  Excitation de Si+
c  -----------------

        if (i_sip .ne. 0) then

        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amsip)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amsip)

        fabchi = zero
        deschi = zero
        nrxi = ikmat(i_sip,0)

        do k=1,nrxi
          ind = ikmat(i_sip,k)
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_sip,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
        end do

        ntr = ntrsip
        nlv = nlvsip

        do i=1,ntrsip
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdsip_o(i,npo) - opdsip_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdsip(i,jopt)
          aij(i) = aijsip(i)
          wlk(i) = wlksip(i)
          iup(i) = iupsip(i)
          jlo(i) = jlosip(i)
          opdr(i) = opdsip_o(i,npo) + opdin
     1            - interpcd(raptau, opdsip_o(i,0:npo), icd_o, npo)
        end do

        do ju=1,nlvsip
          gst(ju) = gstsip(ju)
          elk(ju) = elksip(ju)
          xrepre(ju) = xresip(ju,jopt)
          xreici(ju) = xresip(ju,jopt)
        end do

        call COLSIP (ttry, xnhmp, xnhmo)

        abpre = abnua(i_sip,jopt)
        abici = abnua(i_sip,iopt)

        call DEJONG (iopt, velici, velpre, abpre
     1             , abici, fabchi, deschi, ttry)

        refsip(iopt) = zero
        do i=1,ntrsip
          opdsip(i,iopt) = opd(i)
          emisip(i,iopt) = emi(i)
          refsip(iopt) = refsip(iopt) + emi(i)
        end do

        do ju=1,nlvsip
          xresip(ju,iopt) = xreici(ju)
        end do

        endif

c  VII-refroidissement par excitation de h2o et oh
c  -----------------------------------------------
c  d apres Guillaume

       refh2o(iopt) = 1.0d-26 * ttry * exp(-35.0d0 / ttry)
     1          * ab(i_h2o) * (ab(i_h) + ab(i_h2) / sqrt(deux))

       refoh(iopt) = 1.0d-26 * ttry * exp(-120.0d0 / ttry)
     1         * ab(i_oh) * (dix * ab(i_h) + ab(i_h2) / sqrt(deux))

       reffep(iopt) = 1.1d-22 * (exp(-554.0d0/ttry) + 1.4d0
     1         * exp(-961.0d0/ttry)) * ab(i_fep) * (ab(i_h) + ab(i_h2))
c  refroidissement par les electrons (Dalgarno et Mc Cray)
       reffep(iopt) = reffep(iopt)
     1              + 1.1d-18 * ab(i_fep) * (exp(-554.d0/ttry)
     2              + 1.3d0 * exp(-961.d0/ttry))
     3              * ab(nspec) / sqrt(ttry)

c  Refroidissement par cs
c  ----------------------

        if (i_cs .ne. 0) then

        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amcs0)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amcs0)

        fabchi = zero
        deschi = zero
        nrxi = ikmat(i_cs,0)

        do  k=1,nrxi
          ind = ikmat(i_cs,k)
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_cs,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
        end do

        ntr = ntrcs0
        nlv = nlvcs0

        do i=1,ntrcs0
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdcs0_o(i,npo) - opdcs0_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdcs0(i,jopt)
          aij(i) = aijcs0(i)
          wlk(i) = wlkcs0(i)
          iup(i) = iupcs0(i)
          jlo(i) = jlocs0(i)
          opdr(i) = opdcs0_o(i,npo) + opdin
     1            - interpcd(raptau, opdcs0_o(i,0:npo), icd_o, npo)
        end do

        do ju=1,nlvcs0
          gst(ju) = gstcs0(ju)
          elk(ju) = elkcs0(ju)
          xrepre(ju) = xrecs0(ju,jopt)
          xreici(ju) = xrecs0(ju,jopt)
        end do

        call COLCS0 (ttry)

        abpre = abnua(i_cs,jopt)
        abici = abnua(i_cs,iopt)

        call DEJONG (iopt, velici, velpre, abpre
     1             , abici, fabchi, deschi, ttry)

        refcs0(iopt) = zero
        do i=1,ntrcs0
          opdcs0(i,iopt) = opd(i)
          emics0(i,iopt) = emi(i)
          refcs0(iopt) = refcs0(iopt) + emi(i)
        end do

        do ju=1,nlvcs0
          xrecs0(ju,iopt) = xreici(ju)
        end do

        endif

c  Refroidissement par HCO+
c  ------------------------
 
        if (i_hcop .ne. 0) then
 
        velpre = sqrt(vt2 + ct2 * tgaz(jopt) / amhcop)
        velici = sqrt(vt2 + ct2 * tgaz(iopt) / amhcop)
 
        fabchi = zero
        deschi = zero
 
        nrxi = ikmat(i_hcop,0) !nbre de reaction concernant hcop
        do  k=1,nrxi
          ind = ikmat(i_hcop,k)
          k1 = react(ind,1)
          k2 = react(ind,2)
          k3 = react(ind,3)
          taux = xkmat(i_hcop,k)
          if (taux .gt. zero) then
            fabchi = fabchi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          else
            deschi = deschi + taux * ab(k1) * ab(k2)
     1             * ab(k3)
          endif
        end do
 
        ntr = ntrhcop !plus haut des niveaux bas possible cpte a partir de 1
        nlv = nlvhcop !niveau haut max comptes a partir de 1

        do i=1,ntrhcop
          if (isoneside == 0) then
            opdin = 0.0_dp
          else
            opdin = (opdhcop_o(i,npo) - opdhcop_o(i,jcd_o)) * raptin
          endif
          opd(i) = opdhcop(i,jopt)
          aij(i) = aijhcop(i)
          wlk(i) = wlkhcop(i)
          iup(i) = iuphcop(i)
          jlo(i) = jlohcop(i)
          opdr(i) = opdhcop_o(i,npo) + opdin
     1            - interpcd(raptau, opdhcop_o(i,0:npo), icd_o, npo)
        end do
 
        do ju=1,nlvhcop
          gst(ju) = gsthcop(ju)
          elk(ju) = elkhcop(ju)
          xrepre(ju) = xrehcop(ju,jopt)
          xreici(ju) = xrehcop(ju,jopt)
        end do
 
        call COLHCOP (ttry)
 
 
        abpre = abnua(i_hcop,jopt)
        abici = abnua(i_hcop,iopt)

        call DEJONG (iopt, velici, velpre, abpre
     1             , abici, fabchi, deschi, ttry)
 
        refhcop(iopt) = zero
        do i=1,ntrhcop !nbr de transition
          opdhcop(i,iopt) = opd(i)
          emihcop(i,iopt) = emi(i)
          refhcop(iopt) = refhcop(iopt) + emi(i)
        end do
 
        do ju=1,nlvhcop
          xrehcop(ju,iopt) = xreici(ju)
        end do
 
        endif

c  refroidissement total

       reftot(iopt) = refcat(iopt) + refnat(iopt) + refoat(iopt)
     1              + refsat(iopt) + refsiat(iopt) + refcpl(iopt)
     2              + refnpl(iopt) + refopl(iopt) + refspl(iopt)
     3              + refsip(iopt) + refh2o(iopt) + refoh(iopt)
     4              + reffep(iopt) + refco1(iopt) + refco2(iopt)
     5              + refco3(iopt) + refco0(iopt) + refcs0(iopt)
     6              + refhcop(iopt)

       if (rfjh2t(iopt) .ge. zero ) then
         reftot(iopt) = reftot(iopt) + rfjh2t(iopt)
       else
         chotot(iopt) = chotot(iopt) - rfjh2t(iopt)
       endif

       if (rfjhdt(iopt) .ge. zero ) then
         reftot(iopt) = reftot(iopt) + rfjhdt(iopt)
       else
         chotot(iopt) = chotot(iopt) - rfjhdt(iopt)
       endif

       return
       end

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine DEJONG (n, velici, velpre, abpre, abici
     1                    , fabchi, deschi, ttry)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        use RF_TOOLS
        use STR_DATA
        use PROFIL

        implicit double precision (a-h,o-z)

        dimension eqmat(njco+1,njco+1), indx(njco+1)
        dimension bbab(njco+1,1)

        real (KIND=DP) yy_a(0:3,njco+1)
        real (KIND=DP) bfact(njco+1)
        real (KIND=DP) nhpre, nhici
        real (KIND=DP) :: rien = 1.0e-50_dp

c  Cette subroutine calcule les populations des niveaux
c    et le transfert dans les raies d un systeme
c    D apres de Jong, Dalgarno & Boland, A&A 91, 68 (1980)

c  La resolution est iterative:
c     1- Estimation de la profondeur optique dans chaque raie
c     2- Calcul de la probabilite d echappement.
c     3- Calcul de la fonction source.
c     4- Etablissement de la matrice d equilibre.
c     5- Resolution du systeme (population).
c     6- Test de convergeance.

c  25 avril 96: acceleration de convergence, methode de Lauer
c    (Ref fournie par Matthier Kopp, in Numerical Methods for rdiative transfer)

c  Profondeurs optiques et fonction source:

        imeth = 0

        if (n > 0) then
          m = n - 1
        else
          m = n
        endif
        deltau = tau(n) - tau(m)
        taile = (cdunit * tautav / rrr) * half * deltau
        nhpre = densh(m)
        nhici = densh(n)
        icnt = 0

        sumetl = zero
        do ju=1,nlv
          bfact(ju) = gst(ju) * exp(-elk(ju) / ttry)
          sumetl = sumetl + bfact(ju)
        end do

  100   continue

        eqmat(1:nlv,1:nlv) = zero
        bbab(1:nlv,1) = zero

        do 30 i=1,ntr
          waveln = hcsurk / wlk(i)
          ju = iup(i)
          jl = jlo(i)
          gusgl = gst(ju) / gst(jl)
          soupre = xrepre(jl) * gusgl / (xrepre(ju) + rien)
          souici = xreici(jl) * gusgl / (xreici(ju) + rien)

          delopd = (aij(i) * waveln**3 / (huit * xpi)) * taile/ sqpi
     1      * (xrepre(ju) * abpre * (soupre - un) / (nhpre * velpre)
     3       + xreici(ju) * abici * (souici - un) / (nhici * velici))
          opdtry = opd(i) + delopd

          if (opdr(i) .lt. 1.0d-2) then
            bbetar = (un - 1.17d0 * opdr(i)
     1            + 0.9126 * opdr(i) * opdr(i)) * half
          else if (opdr(i) .lt. sept) then
            bbetar = (un - exp(-2.34d0 * opdr(i))) / (4.68d0 * opdr(i))
          else
            bbetar = un / (quatre * opdr(i) * sqrt(log(opdr(i)/sqpi)))
          endif
          if (opdtry .lt. 1.0d-2) then
            bbeta = (un - 1.17d0 * opdtry
     1            + 0.9126 * opdtry * opdtry) * half
          else if (opdtry .lt. sept) then
            bbeta = (un - exp(-2.34d0 * opdtry)) / (4.68d0 * opdtry)
          else
            bbeta = un / (quatre * opdtry * sqrt(log(opdtry/sqpi)))
          endif

          if (wlk(i) < 100.0_dp) then
            bbody = un / (exp(wlk(i) / tcmb) - un)
          else
            bbody = 0.0_dp
          endif

          if (imeth .eq. 1) then

            pompe = (un - (bbeta+bbetar)) / (souici - un)
     1            + (bbeta+bbetar) * bbody
            rtuplo = aij(i) * (un + pompe)
            rtloup = aij(i) * gusgl * pompe
            eqmat(jl,ju) = eqmat(jl,ju) - rtuplo
            eqmat(ju,ju) = eqmat(ju,ju) + rtuplo
            eqmat(ju,jl) = eqmat(ju,jl) - rtloup
            eqmat(jl,jl) = eqmat(jl,jl) + rtloup

          else

            pompe = un - (souici - un) * bbody
            rtuplo = aij(i) * (bbeta+bbetar) *  pompe

            eqmat(jl,ju) = eqmat(jl,ju) - rtuplo
            eqmat(ju,ju) = eqmat(ju,ju) + rtuplo

          endif

   30   continue

        do ju=1,nlv
          do jl=1,nlv
            eqmat(ju,jl) = (eqmat(ju,jl) + eqcol(ju,jl)) * abici
          end do
          eqmat(ju,ju) = eqmat(ju,ju) - deschi
          bbab(ju,1) = fabchi * max(rien, bfact(ju) / sumetl)
        end do

c  Utilisation de LAPACK

        nrhs = 1
        call dgesv (nlv, nrhs, eqmat, nlvco0, indx, bbab, nlvco0, info)

        dumy = zero
        do ju=1,nlv
          if (bbab(ju,1) .le. zero) then
            imeth = 1
c           bbab(ju,1) = - bbab(ju,1)
            bbab(ju,1) = rien
          endif
          dumy = dumy + bbab(ju,1)
        end do

        test = zero
        do ju=1,nlv
          bbab(ju,1) = bbab(ju,1) / dumy
          test = test + abs(xreici(ju) - bbab(ju,1))
     1         / (bbab(ju,1) + rien)
        end do

        dumy = abs(dumy - un)
c       print *, dumy, test

        do ju=1,nlv
          xreici(ju) = bbab(ju,1)
        end do

        icnt = icnt + 1

c  Acceleration de convergence

        micnt = mod(4 - mod (icnt,4), 4)
        do ju=1,nlv
          yy_a(micnt,ju) = bbab(ju,1)
        end do

        if(micnt .eq. 0) then
          a1f = zero
          b1f = zero
          b2f = zero
          c1f = zero
          c2f = zero
          do ju=1,nlv
            wwt = un
c           wwt = un / abs(yy_a(0,ju))
            d0f = yy_a(0,ju) - yy_a(1,ju)
            d1f = yy_a(0,ju) - deux * yy_a(1,ju)
     1          + yy_a(2,ju)
            d2f = yy_a(0,ju) - yy_a(1,ju)
     1          - yy_a(2,ju) + yy_a(3,ju)
            a1f = a1f + wwt * d1f * d1f
            b1f = b1f + wwt * d1f * d2f
            b2f = b2f + wwt * d2f * d2f
            c1f = c1f + wwt * d0f * d1f
            c2f = c2f + wwt * d0f * d2f
          end do

          iif = log10(a1f)
          ech = 10.0d0**(-iif)
          a1f = a1f * ech
          b1f = b1f * ech
          b2f = b2f * ech
          c1f = c1f * ech
          c2f = c2f * ech

          denom = b2f*a1f - b1f*b1f
          aaf = (b2f*c1f - b1f*c2f) / denom
          bbf = (a1f*c2f - b1f*c1f) / denom
          do ju=1,nlv
            xreici(ju) = (un-aaf-bbf) * yy_a(0,ju)
     1                 + aaf * yy_a(1,ju) + bbf * yy_a(2,ju)
          end do
        endif

        if (mod(icnt,12) .eq. 0) then
          imeth = 0
        endif

c  Fin acceleration

        if (test .gt. 1.0d-4 .and. icnt .lt. 100) goto 100

        do 90 i=1,ntr
          waveln = hcsurk / wlk(i)
          ju = iup(i)
          jl = jlo(i)
          gusgl = gst(ju) / gst(jl)
          delopd = (aij(i) * waveln**3 / (huit * xpi)) * taile /sqpi
     1           * (xrepre(ju) * abpre * ((xrepre(jl) * gusgl
     2                 / xrepre(ju)) - un) / (nhpre * velpre)
     3           + xreici(ju) * abici * ((xreici(jl) * gusgl
     4                 / xreici(ju)) - un) / (nhici * velici))
          opdtry = opd(i) + delopd
          opd(i) = opdtry
          if (opdr(i) .lt. 1.0d-2) then
            bbetar = (un - 1.17d0 * opdr(i)
     1            + 0.9126 * opdr(i) * opdr(i)) * half
          else if (opdr(i) .lt. sept) then
            bbetar = (un - exp(-2.34d0 * opdr(i))) / (4.68d0 * opdr(i))
          else
            bbetar = un / (quatre * opdr(i) * sqrt(log(opdr(i)/sqpi)))
          endif
          if (opdtry .lt. 1.0d-2) then
            bbeta = (un - 1.17d0 * opdtry
     1            + 0.9126 * opdtry * opdtry) * half
          else if (opdtry .lt. sept) then
            bbeta = (un - exp(-2.34d0 * opdtry)) / (4.68d0 * opdtry)
          else
            bbeta = un / (quatre * opdtry * sqrt(log(opdtry/sqpi)))
          endif
          source = xreici(jl) * gusgl / xreici(ju) - un
c    ????????????????
          if (abs(source) .lt. 1.0d-5) then
            source = 1.0d-5
          endif
          if (wlk(i) < 100.0_dp) then
            bbody = un / (exp(wlk(i) / tcmb) - un)
          else
            bbody = 0.0_dp
          endif
          emi(i) = xk * wlk(i) * abici * xreici(ju)
     1           * (bbeta+bbetar) * aij(i)
     1           * (un - bbody * source)
   90   continue

        return
        end
