
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine CHAUD (ttry, iopt, itn)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE H2_DATA
       USE HD_DATA
       USE SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c -----------------------------------------------------------------------------
c 17 juin 2002 - JLB - Tout ce qui concerne l'interaction avec les grains
c                      devrait etre repris. Utiliser en particulier les
c                      donnees de Draine et collaborateurs.
c                      Actuellement :  proche du traitement d'Emilie Habart

c ref.1 : Hollenbach, Takahashi et Tielens (1991) ApJ 377, 192
c ref.2 : Verstraete, Leger, d'Hendecourt, Dutuit et Defourneau
c          (1990) AA 237, 436
c ref.3 : Tielens & Hollenbach(1985) ApJ 291, 722
c -----------------------------------------------------------------------------

       implicit double precision (a-h,o-z)

       real (KIND=DP) bilthg      ! chauffage par effet photo. sur les grains
       real (KIND=DP) xkneut(nes) ! taux de neutralisation des ions sur les grains
                                  ! d'apres Bakes et Tielens, ApJ, 427, 822 (1994)
       real (KIND=DP) xiig, s_gr

       common / cofbth / bilthg, xkneut

       save

       avr = Avmax - av

c ----------------------------------------------------------------------------------
c  temperature des grains
c  A partir de l'Eq. (1) de Hollenbach et al (ref.1)
c    en integrant sur NOTRE champ de rayonnement (plus de ref a Habing)
c    et en prevoyant plusieurs possibilites pour Q

c  27 juin 2002 - JLB - Les grains sont divises en 3 classes (par taille) :
c                 PAH : a < r_lin_PAH
c                 VSG : a < r_lim_VSG
c                  BG : sinon
c                 Ce calcul devrait etre refait en utilisant l'integration directe.
c ----------------------------------------------------------------------------------

c on place d'abord qqs quantites independantes du cas et du rayon

       tau100 = 4.0d-3
       ccff = 15.0d0 * hcsurk**4 / (32.0d0 * xpi**5 * xk * clum*clum)

c  td0 est la temperature des poussieres au bord
c  tdust(:,0) est initialise dans le main avant la premiere iteration
       do mm=1,npg
         xnu0 = clum / (deux * xpi * adust(mm))
         td0 = tdust(mm,0)

         s_gr = xpi * adust(mm)*adust(mm)

        xiig = sect_int(sigabs(mm), rf_dray, wl_pointer
     1                , rf_wl%Val(nwlg))
        xiig = xiig * clum * x_nc(mm) / s_gr

c formulae  (6) and (7) of ref.1 lead to
c   with modified values from Burton, Hollenbach, Tielens
c   Ap J 365, 620, (1990)
c 1 mars 2000; bug: log10 au lieu de ln

         tgrain = ccff * xnu0 * xiig + tcmb**5
     1          + 3.4d-2 * (0.423 - log(3.45d-2 * tau100 * td0))
     2          * tau100 * td0**6

         tdust(mm,iopt) = tgrain**0.2d0

c 3 Avril 2003 - Madrid
c        tdust(mm,iopt) = max(tdust(mm,iopt), 100.0_dp)
c    Temperature de grains elevee pour CRL618
c fin
       end do

c  chauffage par effet photoelectrique sur les PAH (ref.2)
c    Supprime et inclus dans l effet photoelectrique sur les
c             grains

c   Fraction de H2 (pour le flux de photons secondaires)

       frach2 = ab(i_h2) / (ab(i_h) + ab(i_h2))

c On tient compte du champ de rayonnement externe et des photons secondaires:
c    27 juin 2002 : Ce flux n'est plus inclus dans le calcul de chophe
c                   A remettre via le Champ de rayonnement!

       radsec = frach2 * fphsec / 1.0d8

c-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

c  Calcul du taux de chauffage par effet photoelectrique sur les
c    grains suivant Bakes & Tielens, Ap. J., 427, 822, (1994)
c  Ce calcul determine la distribution de charge des grains
c    en fonction de leur taille avant de faire l integration.

c     if (mod(itn,3) .eq. 1) then

c  Integration sur la distribution:

      ndeb = nh2for + ncrion + npsion + nasrad + nrest + nh2endo
     1    + nphot_f + nphot_i + n3body + ncpart + ngsurf + ngphot
     2    + ngads + 1
      nfin = ndeb + ngneut - 1

      chophe = zero
      do i=ndeb,nfin
        ir1 = react(i,1)
        rxigr(ir1,iopt) = zero
      end do

      do mm=1,npg
        call fbth(ttry, sigabs(mm), iopt, itn, mm)
c  bilthb peut etre negatif si la recombinaison des electrons
c  contribue plus que leur ejection...
c       bilthg = max(bilthg,0.0_dp)
        chophe = chophe + bilthg * wwmg(mm)
        j = 0
        do i=ndeb,nfin
          ir1 = react(i,1)
          j = j + 1
          rxigr(ir1,iopt) = rxigr(ir1,iopt) + xkneut(j) * wwmg(mm)
        end do
      end do

c 18 VII 2003 petit test
      chophg(iopt) = chophe
c     chophg(iopt) = chophe * 0.3_DP

c-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

c  chauffage par equilibre thermique avec les grains
c   Tielens & Hollenbach 1985 (ref.3) eq A19 : n'est plus utilise

c  Burke & Hollenbach 1983 Ap J 265, 223 (Eq 9)
c    4.017e-12 = 2 k sqrt(8 k / pi mH)

        densr = ab(i_h) + ab(i_h2) + ab(i_he)
        xmu = 1.4d0 * densh(iopt) / densr
        vbar = vmaxw * sqrt(ttry / xmu)
        alphat = (ab(i_h) + ab(i_h2) * 0.5_DP * sqrt(2.0_DP)) * 0.8
        choeqg(iopt) = alphat * 0.5d0 * xk * vbar * densh(iopt)
     1          * sum(surfgr(1:npg) * (tdust(1:npg,iopt)-ttry))

       xkphot = zero
       xkrc = zero
       xkphs = zero
       xkchim = zero

c  chauffage par photo-ionisation ou photo-dissociation
c  1 eV pour H2, HD, CO
c  de pour les atomes (C et S)
c  1/3 de de pour les autres (equipartition minimale)
c  Rappel les photons ont une "enthalpie" de 13.O eV

       ideb = nh2for + ncrion + npsion + nasrad + nrest + nh2endo + 1
       ifin = ideb + nphot_f - 1

       do i=ideb,ifin
         ir1 = react(i,1)
         rate1 = gamm(i) * exp(-bet(i) * av) * radm
         rate2 = gamm(i) * exp(-bet(i) * avr) * radp
         rate = (rate1 + rate2) * de(i) / trois
         xkphot = xkphot + ab(ir1) * rate
       end do

       ideb = ifin + 1
       ifin = ifin + nphot_i

       do i=ideb,ifin
         ir1 = react(i,1)

         if (ir1 .eq. i_h2 .and. itrh2 .ne. 0) then
           rate = pdish2(iopt) * de(i)
         else if (ir1 .eq. i_hd .and. itrhd .ne. 0) then
           rate = pdishd(iopt) * de(i)
         else if (ir1 .eq. i_co0 .and. itrco0 .ne. 0) then
           rate = pdico0(iopt) * de(i)
         else if (ir1 .eq. i_co1 .and. itrco1 .ne. 0) then
           rate = pdico1(iopt) * de(i)
         else if (ir1 .eq. i_co2 .and. itrco2 .ne. 0) then
           rate = pdico2(iopt) * de(i)
         else if (ir1 .eq. i_co3 .and. itrco3 .ne. 0) then
           rate = pdico3(iopt) * de(i)
         else if (ir1 .eq. i_c .or. ir1 .eq. i_c13) then
           rate = pdicat(iopt) * de(i)
         else if (ir1 .eq. i_s) then
           rate = pdisat(iopt) * de(i)
         endif
         xkphot = xkphot + ab(ir1) * rate
       end do

c  chauffage par l ionisation par les rayons cosmiques

       ideb = nh2for + 1
       ifin = nh2for + ncrion
       do i=ideb,ifin
         ir1 = react(i,1)
         rate = gamm(i) * zeta
         xkrc = xkrc + ab(ir1) * rate
       end do

c  chauffage par l ionisation par les photons secondaires

       ideb = nh2for + ncrion + 1
       ifin = ideb + npsion - 1
       do i=ideb,ifin
         ir1 = react(i,1)
         rate = gamm(i) * frach2 * zeta * (ttry / 300.0d0)**alpha(i)
         xkphs = xkphs + ab(ir1) * rate * de(i)
       end do

c  bilan thermique de la chimie

       ideb = nh2for + ncrion + npsion + nasrad + 1
       ifin = nh2for + ncrion + npsion + nasrad + nrest + nh2endo
       do i=ideb,ifin

c    taux associe a chaque reaction:

         if (de(i) .gt. 1.0d3) then
           dde = zero
           print *, i, ir1, ir2, de(i)
         else
           dde = de(i)
         endif

         ir1 = react(i,1)
         ir2 = react(i,2)
         ir3 = react(i,3)
         if (bet(i) < 0.0_dp) then
           expon = exp(-bet(i)/max(ttry,200.0_dp))
         else
           expon = exp(-bet(i)/ttry)
         endif
         rate = gamm(i) * expon
     1        * (ttry * 3.3333d-3)**alpha(i)
         xkchim = xkchim
     1          + ab(ir1) * ab(ir2) * ab(ir3) * rate * dde

       end do

c   photoionisation ou photodissociation: 1 eV pour H2, HD et CO, de(i) pour les autres
c   avec une energie moyenne de 10 eV pour les photons incidents.

       chopht(iopt) = xkphot * everg

c   4 ev par cosmique

c      chorc(iopt) = xkrc * everg * six
       chorc(iopt) = xkrc * everg * quatre

c   de(i) par photon secondaire
c   avec une energie moyenne de 10 eV pour les photons incidents.

       chophs(iopt) = xkphs * everg

c  conversion en erg du chauffage chimique

       chochi(iopt) = xkchim * everg

c  chauffage du a la formation de H2 sur les grains:

c    chaque formation libere 1/3 de l energie de formation
c    sous forme d energie cinetique

       rate = gamm(ifh2gr) * deux * tm1h(i_hgr) / xkhydr

c  Quand on tient compte de tous les v, il faut aussi inclure
c      L energie interne (v excites) qui est rayonnee
c      immediatement par cascades quadrupolaires
c      parcequ'elle est prise en compte dans le refroidissement.
c      Le premier terme du "MIN" est la somme des energies qui ne vont
c        pas dans le grain. Elle est forcement < 4.5 eV

       toto = everg * de(ifh2gr)
       choh2g(iopt) = min(emoyh2 * xk  + toto / trois, toto)

       choh2g(iopt) = choh2g(iopt) * rate * ab(i_hgr) * ab(i_hgr)

c  chauffage du a la formation de HD sur les grains:

c    chaque formation libere 1/3 de l energie de formation
c    sous forme d energie cinetique

       if (i_dgr .ne. 0) then
         rate = gamm(ifhdgr) *(tm1h(i_dgr) + tm1h(i_hgr)) / xkhydr
         rate = rate * ab(i_hgr) * ab(i_dgr)
       else
         rate = 0.0d0
       endif

       if (i_hd .ne. 0) then
         toto = everg * de(ifhdgr)
         chohdg(iopt) = min(emoyhd * xk  + toto / trois, toto)
         chohdg(iopt) = chohdg(iopt) * rate
       else
         chohdg(iopt) = zero
       endif

c   chauffage par dissipation de la turbulence:
c      Falgarone et Puget (1985)

c      fv = 0.03d0
c   On limite la taille inferieure d un clump, pour eviter la divergence de chotur
c      size = max(taille(iopt),0.1d0)
c      chotur(iopt) = 2.0d-29 * densh(iopt) * (vturb / 1.0d5)**3
c    1              * (fv / 0.03d0) / size

c   Autre proposition: (Boland & de Jong, 1984)

c      ct2 = deux * xk / amu
c      pmu = (ab(i_h) + deux * ab(i_h2) + quatre * ab(i_he))
c    1     / (ab(i_h) + ab(i_h2) + ab(i_he))
c      alp = vturb / sqrt(ct2 * ttry / pmu)
c      chotur(iopt) = 2.7d-30 * alp**3 * densh(iopt) * ttry**1.5d0
c    1              / (un + alp*alp)

c   Pour l instant:

       chotur(iopt) = zero

c      print *, ct2, pmu, alp, chotur(iopt)

c   chauffage total

       chotot(iopt) = chotot(iopt) + chophg(iopt)
c                   + chohdg(iopt)
     2              + choh2g(iopt) + chopht(iopt) + chorc(iopt)
     3              + chophs(iopt) + chochi(iopt) + chotur(iopt)

       if (choeqg(iopt) .ge. zero) then
         chotot(iopt) = chotot(iopt) + choeqg(iopt)
       else
         reftot(iopt) = reftot(iopt) - choeqg(iopt)
       endif

       return
       end

c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine fbth(temper, Q_abs_r, iopt, itn, mm)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       use SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL
       USE RF_TOOLS

       implicit none

c  Calcul du bilan de l effet photo-electrique avec
c  distribution de charge des grains en fonction de leur taille.
c  D apres Bakes & Tielens, Ap J 427, 822 (1994)
c  Ainsi que des taux de collision ions-grains

      TYPE(RF_TYPE), intent (IN) :: Q_abs_r

c     xkneut : vecteur resultat (taux de chauffage + taux de collisions)

      real (KIND=DP) xkneut(nes)

      common / cofbth / bilthg, xkneut

c     x_nc : Nombre d atomes de carbonne dans le grain.

      integer iopt, itn
      integer mm
c  Charge maximale acceptee:

      integer izm0
      parameter (izm0=10000)

c  Charges...

c     real (KIND=DP) zmin
      real (KIND=DP) zmax
      integer izmin, izmax
      integer kzmin, kzmax
      real (KIND=DP) zzmoy
      integer, DIMENSION(:), ALLOCATABLE ::  jzmin
      integer, DIMENSION(:), ALLOCATABLE ::  jzmax

c  Caracteristiques du gaz:

c     temper : Temperature (en K)

      real (KIND=DP) temper

c  Integrals of Q * U * (1 or l or l2)  over lambda

      real (KIND=DP) s_QU, s_QUl, s_QUl2

c  Les differents taux d interactions des grains et du gaz sont:
c     xjpe : taux de photoemission
c     xje  : taux d accretion des electrons
c     xji  : taux d accretion des ions (individuels)
c     xjit : taux d accretion des ions (total)
!--------------------------------------------------------
      real (KIND=DP) xjpe(-izm0:izm0), xje(-izm0:izm0)
      real (KIND=DP) xji(0:nes,-izm0:izm0), xjit(-izm0:izm0)

c  La fraction d ionisation

      real (KIND=DP) frac(-izm0:izm0)

c  Chauffage et refroidissement:

      real (KIND=DP) heat(-izm0:izm0), cool(-izm0:izm0)
      real (KIND=DP) bilth, bilthg

c     stic : sticking coeff des electrons

      real (KIND=DP) stic

c  Intermediaires de calcul

      real (KIND=DP) tauph, cte, x_nc_t
      real (KIND=DP) x_ip, thet, xjtm, xjtp, xjt0
      real (KIND=DP) xltm, xltp, xlt0, ziz
      real (KIND=DP) sup, titi, toto, tata, xsum, tier, tempo
      real (KIND=DP) cte_H, cte_J, cte_IP

c     TYPE(RF_TYPE)  :: rf_tmp
      real (KIND=DP) :: wl_cut_IP, IP_min
      integer        :: I_cut_IP

c  Indices muets

      integer i, j, iz, ndeb, nfin, ir1, izf, izd

      data stic / 1.0d0 /
      data tier / 0.333333333333333333d0 /

      save jzmin, jzmax

c------------------------------------------------------------------------


      IF ( .NOT. ALLOCATED(jzmin)) THEN
        ALLOCATE (jzmin(npg), jzmax(npg))
        jzmin = 0
        jzmax = 0
      ENDIF

      IP_min = xhp * clum * 1.0e8_dp / (rf_wl%Val(nwlg) * everg)
c     CALL rf_create (rf_tmp, 0, nwlg)
c     rf_tmp%Ord = rf_dray%Ord

c  Parametre reduit (apres Eq 9) on suppose que qi = +/- 1

      tauph = adust(mm) * xk * temper / xqe2

c  Eq 9 (debut):

      cte = sqrt(huit * xk * temper * xpi) * adust(mm)**2

      x_nc_t = x_nc(mm)**tier

c  Charges extremes (Eq 26 et 27):

      if (iopt .eq. 0 .and. itn .eq. 1) then

c  Charges can not be very negative at the edge...
c       zmin = 0.4d0 * x_nc_t - half
c       zmin =  max(-zmin,-90.0d0*tauph)
c       izmin = zmin
        izmin = -1

        zmax = (hiop/everg - 4.4_dp) * x_nc_t / 11.1d0 - half
        izmax = zmax

c  On limite l excursion possible:

        izmin = max(izmin,-izm0)
        izmax = min(izmax,izm0)

        if (iopt /= 0 .AND. jzmin(mm) < jzmax(mm)) then
          izmin = jzmin(mm) - 5
          izmax = jzmax(mm) + 5
        else if (iopt /= 0 .AND. mm > 1) then
          izmin = min(izmax-5,max(izmin,jzmin(mm-1)/2))
          izmax = max(izmin+5,min(izmax,jzmax(mm-1)+10))
        endif

        if (izmin .ge. izmax) then
          izmin = izmin - 2
          izmax = izmax + 2
        end if

      else

        izmin = jzmin(mm) - 2
        izmax = jzmax(mm) + 2

      endif

c  Pour les reaction de neutralisation...

      ndeb = nh2for + ncrion + npsion + nasrad + nrest + nh2endo
     1    + nphot_f + nphot_i + n3body + ncpart + ngsurf + ngphot
     2    + ngads + 1
      nfin = ndeb + ngneut - 1

c  Boucle sur la valeur des charges:
c  On considere separement les grains moins, neutre et plus

      cte_J = x_nc(mm) * 0.14 * bt_fy(mm) * 1.0e-8_DP / xhp
      cte_H = x_nc(mm) * 0.14 * bt_fy(mm) * 0.5_DP * clum

      izf = min(izmax,-1)

      do 100 iz=izmin,izf

c  Potentiel d ionisation (en eV) (Eq 25)

        x_ip = max(IP_min,min(4.4d0 + (iz + 0.5_dp)
     1       * 11.1d0 / x_nc_t, hiop/everg))
c       x_ip = min(4.4d0 + (iz + half) * 11.1d0 / x_nc_t, 13.6_DP)
c  cte_IP is 1 / lambda_IP (in Angstrom)
        cte_IP = x_ip * everg * 1.0e-8_DP / (xhp * clum)
        wl_cut_IP = 1.0_dp / cte_IP
c       rf_tmp%Val = 0.0_dp
        CALL rf_locate (rf_wl, wl_cut_IP, I_cut_IP)
c       rf_tmp%Val(0:I_cut_IP) = rf_dray%Val(0:I_cut_IP)

        s_QU   = si_QU(mm,I_cut_IP)
        s_QUl  = si_QUl(mm,I_cut_IP)
        s_QUl2 = si_QUl2(mm,I_cut_IP)
c       s_QU = sect_int(Q_abs_r, rf_tmp, wl_pointer, wl_cut_IP)
c       rf_tmp%Val = rf_tmp%Val * rf_wl%val
c       s_QUl = sect_int(Q_abs_r, rf_tmp, wl_pointer, wl_cut_IP)
c       rf_tmp%Val = rf_tmp%Val * rf_wl%val
c       s_QUl2 = sect_int(Q_abs_r, rf_tmp, wl_pointer, wl_cut_IP)

c  Ejection d un photo-electron (Eq 20):

        xjpe(iz) = cte_J * (s_QUl - cte_IP * s_QUl2)

c  Chauffage photo-electrique ( Eq 30):

        heat(iz) = cte_H * (s_QU - 2.0_DP * cte_IP * s_QUl
     1           + cte_IP*cte_IP * s_QUl2)
c       print *, xjpe(iz), heat(iz)

c  Collisions avec les particules du gaz. Eq 13:

        ziz = -iz
        thet = ziz / (un + un/sqrt(ziz))

c  Eq 11:

        xjtm = (un - iz / tauph)
     1       * (un + sqrt(deux/(tauph - deux * iz)))

c  Eq 12:

        xjtp = exp(-thet / tauph)
     1       * (un + un / sqrt(quatre * tauph - trois * iz))**2

c  Eq 33:

        xltm = (deux - iz / tauph) * (un + sqrt(un/(tauph - iz)))

c  Eq 34:

        xltp = exp(-thet / tauph) * (deux - iz / tauph)
     1       * (un + un / sqrt(trois /(deux*tauph) - trois * iz))

c  Eq 9 (suite) et 31:

        xje(iz) = cte * stic * ab(nspec) * xjtp / sqrt(xme)
        titi = zero
        toto = cte * xk * temper * ab(nspec) * xltp / sqrt(xme)
        tata = toto

        do i=ndeb,nfin
          ir1 = react(i,1)
          xji(ir1,iz) = cte * gamm(i) * ab(ir1)
     1              * xjtm / sqrt(xmh*mol(ir1))
          titi = titi + xji(ir1,iz)
          tata = tata + cte * gamm(i) * xk * temper
     1             * ab(ir1) * xltm / sqrt(xmh*mol(ir1))
        end do
        xjit(iz) = titi

c  Si les ions positif n adhere pas aux grains, ils ne contribuent pas
c     au refroidissement:
c  On inclut seulement la contribution des lectrons
c       cool(iz) = tata
        cool(iz) = toto

  100 continue

       if (izmin .le. 0 .and. izmax .ge. 0) then

      iz = 0

      x_ip = max(IP_min,min(4.4d0 + (iz + 0.5_dp)
     1     * 11.1d0 / x_nc_t, hiop/everg))
c     x_ip = min(4.4d0 + half * 11.1d0 / x_nc_t, 13.6_DP)
      cte_IP = x_ip * everg * 1.0e-8_DP / (xhp * clum)
      wl_cut_IP = 1.0_dp / cte_IP
c     rf_tmp%Val = 0.0_dp
      CALL rf_locate (rf_wl, wl_cut_IP, I_cut_IP)
c     rf_tmp%Val(0:I_cut_IP) = rf_dray%Val(0:I_cut_IP)

      s_QU   = si_QU(mm,I_cut_IP)
      s_QUl  = si_QUl(mm,I_cut_IP)
      s_QUl2 = si_QUl2(mm,I_cut_IP)
c     s_QU = sect_int(Q_abs_r, rf_tmp, wl_pointer, wl_cut_IP)
c     rf_tmp%Val = rf_tmp%Val * rf_wl%val
c     s_QUl = sect_int(Q_abs_r, rf_tmp, wl_pointer, wl_cut_IP)
c     rf_tmp%Val = rf_tmp%Val * rf_wl%val
c     s_QUl2 = sect_int(Q_abs_r, rf_tmp, wl_pointer, wl_cut_IP)

      xjpe(0) = cte_J * (s_QUl - cte_IP * s_QUl2)
      heat(0) = cte_H * (s_QU - 2.0_DP * cte_IP * s_QUl
     1        + cte_IP*cte_IP * s_QUl2)

      xjt0 = un + sqrt(xpi / (deux * tauph))

      xlt0 = deux + trois * sqrt(xpi/(deux*tauph)) /deux

      xje(0) = cte * ab(nspec) * xjt0 / sqrt(xme)
      titi= zero
      toto = cte * xk * temper * ab(nspec) * xlt0 / sqrt(xme)
      tata = toto

      do i=ndeb,nfin
        xji(ir1,0) = cte * gamm(i) * ab(ir1)
     1             * xjt0 / sqrt(xmh*mol(ir1))
        titi= titi+ xji(ir1,0)
        tata = tata + cte * gamm(i)* xk * temper * xlt0
     1           * ab(ir1) / sqrt(xmh*mol(ir1))
      end do
      xjit(0) = titi
c  Si les ions positif n adhere pas aux grains, ils ne contribuent pas
c     au refroidissement:
c     cool(0) = tata
      cool(0) = toto

      endif

      izd = max (izmin,1)

      do 200 iz=izd,izmax

c  Potentiel d ionisation (en eV) (Eq 25)

        x_ip = max(IP_min,min(4.4d0 + (iz + 0.5_dp)
     1       * 11.1d0 / x_nc_t, hiop/everg))
c       x_ip = min(4.4d0 + (iz + half) * 11.1d0 / x_nc_t, 13.6_DP)
        cte_IP = x_ip * everg * 1.0e-8_DP / (xhp * clum)
        wl_cut_IP = 1.0_dp / cte_IP
c       rf_tmp%Val = 0.0_dp
        CALL rf_locate (rf_wl, wl_cut_IP, I_cut_IP)
c       rf_tmp%Val(0:I_cut_IP) = rf_dray%Val(0:I_cut_IP)

        s_QU   = si_QU(mm,I_cut_IP)
        s_QUl  = si_QUl(mm,I_cut_IP)
        s_QUl2 = si_QUl2(mm,I_cut_IP)
c       s_QU = sect_int(Q_abs_r, rf_tmp, wl_pointer, wl_cut_IP)
c       rf_tmp%Val = rf_tmp%Val * rf_wl%val
c       s_QUl = sect_int(Q_abs_r, rf_tmp, wl_pointer, wl_cut_IP)
c       rf_tmp%Val = rf_tmp%Val * rf_wl%val
c       s_QUl2 = sect_int(Q_abs_r, rf_tmp, wl_pointer, wl_cut_IP)

        xjpe(iz) = cte_J * (s_QUl - cte_IP * s_QUl2)
        heat(iz) = cte_H * (s_QU - 2.0_DP * cte_IP * s_QUl
     1           + cte_IP*cte_IP * s_QUl2)

c  Eq 13:

        ziz = iz
        thet = ziz / (un + un/sqrt(ziz))

c  Eq 11:

        xjtm = (un + iz / tauph)
     1       * (un + sqrt(deux/(tauph + deux * iz)))

c  Eq 12:

        xjtp = exp(-thet / tauph)
     1       * (un + un / sqrt(quatre * tauph + trois * iz))**2

c  Eq 33:

        xltm = (deux + iz / tauph) * (un + sqrt(un/(tauph + iz)))

c  Eq 34:

        xltp = exp(-thet / tauph) * (deux + iz / tauph)
     1       * (un + un / sqrt(trois /(deux*tauph) + trois * iz))

c  Eq 9 (suite) et 31:

        xje(iz) = cte * ab(nspec) * xjtm / sqrt(xme)
        titi = zero
        toto = cte * xk * temper * ab(nspec) * xltm / sqrt(xme)
        tata = toto

        do i=ndeb,nfin
          ir1 = react(i,1)
          xji(ir1,iz) = cte * gamm(i) * ab(ir1)
     1              * xjtp / sqrt(xmh*mol(ir1))
          titi = titi + xji(ir1,iz)
          tata = tata + cte * gamm(i) * xk * temper
     1             * ab(ir1) * xltp / sqrt(xmh*mol(ir1))
        end do
        xjit(iz) = titi
c  Si les ions positif n adhere pas aux grains, ils ne contribuent pas
c     au refroidissement:
c       cool(iz) = tata
        cool(iz) = toto

  200 continue

c  Fraction d ionisation :

      sup = zero

      if (izmin .ge. 0) then

        frac(izmin) = zero

        toto = zero
        do iz=izmin+1,izmax
          tempo = (xjpe(iz-1) + xjit(iz-1)) / xje(iz)
          if (tempo .gt. 1.0d-40) then
            toto = toto + log(tempo)
          else
            toto = toto - 92.1d0
          endif
          frac(iz) = toto
          sup = max(sup,toto)
        end do

      else if (izmax .le. 0) then

        frac(izmax) = zero

        toto = zero
        do iz=izf,izmin,-1
          tempo = xje(iz+1) / (xjpe(iz) + xjit(iz))
          if (tempo .gt. 1.0d-40) then
            toto = toto + log(tempo)
          else
            toto = toto - 92.1d0
          endif
          frac(iz) = toto
          sup = max(sup,toto)
        end do

      else

        frac(0) = zero

c  Eq 6:

        toto = zero
        do iz=izd,izmax
          tempo = (xjpe(iz-1) + xjit(iz-1)) / xje(iz)
          if (tempo .gt. 1.0d-40) then
            toto = toto + log(tempo)
          else
            toto = toto - 92.1d0
          endif
          frac(iz) = toto
          sup = max(sup,toto)
        end do

c  Eq 7:

        toto = zero
        do iz=izf,izmin,-1
          tempo = xje(iz+1) / (xjpe(iz) + xjit(iz))
          if (tempo .gt. 1.0d-40) then
            toto = toto + log(tempo)
          else
            toto = toto - 92.1d0
          endif
          frac(iz) = toto
          sup = max(sup,toto)
        end do

      endif

c  Eq 8 et 3 (en partie):

      bilth = zero
      bilthg = zero
      xsum = zero
      do i=1,ngneut
        xkneut(i) = zero
      end do
      do iz = izmin,izmax
        frac(iz) = exp(frac(iz) - sup)
        xsum = xsum + frac(iz)
      end do

      if (xsum .eq. 0.0d0) then
        print *, ' Pb dans fbth, xsum = 0.0'
      endif

      kzmin = izmax
      kzmax = izmin
      zzmoy = 0
      do iz = izmin,izmax
        frac(iz) = frac(iz) / xsum
        if (frac(iz) .gt. 1.0d-5) then
          kzmin = min(kzmin,iz)
          kzmax = max(kzmax,iz)
        endif
        zzmoy = zzmoy + iz * frac(iz)
        bilth = bilth + (heat(iz) - cool(iz)) * frac(iz)
      end do
      jzmin(mm) = kzmin
      jzmax(mm) = kzmax
      zmoy(mm,iopt) = zzmoy

c  Constante de normalisation (voir notes de Guillaume)

      bilthg = bilth * agrnrm * densh(iopt)

      j = 0
      do 508 i=ndeb,nfin
        ir1 = react(i,1)
        j = j + 1
        do iz = izmin,izmax
          xkneut(j) = xkneut(j) + xji(ir1,iz) * frac(iz)
        end do
c  4 I 2006 : Bug (ER + FLP) : divide by ab(ir1)
c       xkneut(j) = xkneut(j) * agrnrm * densh(iopt)
        xkneut(j) = xkneut(j) * agrnrm * densh(iopt) / ab(ir1)
  508 continue

c     CALL rf_free (rf_tmp)

      return
      end

