
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLCAT (ttry, xnhmp, xnhmo)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE CHEM_DATA
        USE STR_DATA
        USE SP_DATA
        USE PROFIL

        implicit double precision (a-h,o-z)

        REAL (kind=DP), DIMENSION (nlvcat,nlvcat) :: ec_H, ec_He
        REAL (kind=DP), DIMENSION (nlvcat,nlvcat) :: ec_Hp, ec_e
        REAL (kind=DP), DIMENSION (nlvcat,nlvcat) :: ec_orthoH2
        REAL (kind=DP), DIMENSION (nlvcat,nlvcat) :: ec_paraH2, ec

        ec_H  = colr_min
        ec_orthoH2 = colr_min
        ec_paraH2  = colr_min
        ec_He = colr_min
        ec_Hp = colr_min
        ec_e  = colr_min

c    a-collisions avec H
c Fine structure: probably Launay & Roueff 1977 (check)

        ec_H(1,2) = 1.01e-10_DP * ttry**0.117_DP
        ec_H(1,3) = 4.49e-11_DP * ttry**0.194_DP
        ec_H(2,3) = 1.06e-10_DP * ttry**0.234_DP

c 1D - 3P
c DRF : Do not use O I + H rates

c  b-collisions avec H2
c fit from Schroder, Staemmler, Smith, Flower & Jaquet (1991)

        ec_paraH2(1,2)  = 0.80e-10_DP
        ec_orthoH2(1,2) = 0.75e-10_DP
        ec_paraH2(1,3)  = 0.90e-10_DP
        ec_orthoH2(1,3) = 3.54e-11_DP * ttry**0.167_DP
        ec_paraH2(2,3)  = 2.00e-10_DP
        ec_orthoH2(2,3) = 5.25e-11_DP * ttry**0.244_DP

c     c-collisions avec He
c Fine structure: Staemmler & Flower (1991)

        ec_He(1,2) = 8.38e-12_DP * ttry**0.159_DP
        ec_He(1,3) = 5.98e-11_DP * ttry**0.078_DP
        ec_He(2,3) = 3.68e-11_DP * ttry**0.041_DP

c collisions avec H+
c from AA 236, 515 (1990)  Roueff & Le Bourlot (fit DRF)

        ec_Hp(1,2) = 10._DP**(-10.359_DP + 0.7959_DP*log10(ttry)
     1             - 0.08748_DP*(log10(ttry))**2._DP)
        ec_Hp(1,3) = 10._DP**(-13.232_DP + 2.4171_DP*log10(ttry)
     1             - 0.29151_DP*(log10(ttry))**2._DP)
        ec_Hp(2,3) = 10._DP**(-11.290_DP + 1.7915_DP*log10(ttry)
     1             - 0.23010_DP*(log10(ttry))**2._DP)

c     d-collisions with electrons
c Should we use Pequignot 1990...
c From Pequignot & Aldrovandi, A&A 50, 141, 1976 as compiled by Mendoza (1983)

        auxt4 = 1.83e-4_DP * ttry ** 0.444_DP
        ec_e(1,4) = 8.629e-6_DP * auxt4 / gstcat(4) * gstcat(1) / 9.0_DP
        ec_e(2,4) = 8.629e-6_DP * auxt4 / gstcat(4) * gstcat(2) / 9.0_DP
        ec_e(3,4) = 8.629e-6_DP * auxt4 / gstcat(4) * gstcat(3) / 9.0_DP
        auxt4 = 9.86e-5_DP * ttry ** 0.343_DP
        ec_e(1,5) = 8.629e-6_DP * auxt4 / gstcat(5) * gstcat(1) / 9.0_DP
        ec_e(2,5) = 8.629e-6_DP * auxt4 / gstcat(5) * gstcat(2) / 9.0_DP
        ec_e(3,5) = 8.629e-6_DP * auxt4 / gstcat(5) * gstcat(3) / 9.0_DP
        auxt4 = 2.77e-3_DP
        ec_e(4,5) = 8.629e-6_DP *auxt4 / gstcat(5)

c  Bring'em together

       do jl=1,nlvcat-1
         do ju=jl+1,nlvcat
           ec(jl,ju) = ec_H(jl,ju) * ab(i_h)
     1               + ec_Hp(jl,ju) * ab(i_hp)
     2               + ec_paraH2(jl,ju) * xnhmp
     3               + ec_orthoH2(jl,ju) * xnhmo
     4               + ec_He(jl,ju) * ab(i_he)
     5               + ec_e(jl,ju) * ab(nspec)
         end do
       end do

       do ju=2,nlvcat
         do jl = 1,ju-1
           ec(ju,jl) = ec(jl,ju) * (gstcat(ju)/gstcat(jl))
     1               * exp(-(elkcat(ju)-elkcat(jl)) / ttry)
         end do
       end do

       eqcol = zero

       do jl=1,nlvcat-1
         do ju=jl+1,nlvcat
           eqcol(jl,ju) = eqcol(jl,ju) - ec(jl,ju)
           eqcol(ju,ju) = eqcol(ju,ju) + ec(jl,ju)
         end do
       end do

       do ju=2,nlvcat
         do jl = 1,ju-1
           eqcol(ju,jl) = eqcol(ju,jl) - ec(ju,jl)
           eqcol(jl,jl) = eqcol(jl,jl) + ec(ju,jl)
         end do
       end do

       return
       end

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLNAT (ttry, xnhmp, xnhmo)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE CHEM_DATA
        USE STR_DATA
        USE SP_DATA
        USE PROFIL

        implicit double precision (a-h,o-z)

        REAL (kind=DP), DIMENSION (nlvnat,nlvnat) :: ec_H, ec_He
        REAL (kind=DP), DIMENSION (nlvnat,nlvnat) :: ec_Hp, ec_e
        REAL (kind=DP), DIMENSION (nlvnat,nlvnat) :: ec_orthoH2
        REAL (kind=DP), DIMENSION (nlvnat,nlvnat) :: ec_paraH2, ec

        ec_H  = colr_min
        ec_orthoH2 = colr_min
        ec_paraH2  = colr_min
        ec_He = colr_min
        ec_Hp = colr_min
        ec_e  = colr_min

c    collisions avec H

c    collisions avec H2

c    collisions avec He

c    collisions avec H+

c    collisions with electrons
c     From compilation of Mendoza (1983)

        auxt4 = 7.92e-6_DP * ttry**0.589_DP
        ec_e(1,3) = 8.629e-6_DP * auxt4 / gstnat(3)
        auxt4 = 1.5_DP * auxt4
        ec_e(1,2) = 8.629e-6_DP * auxt4 / gstnat(2)
        auxt4 = 1.74e-6_DP * ttry**0.621_DP
        ec_e(1,4) = 8.629e-6_DP * auxt4 / gstnat(4)
        auxt4 = 2.0_DP * auxt4
        ec_e(1,5) = 8.629e-6_DP * auxt4 / gstnat(5)
        auxt4 = 4.78e-5_DP * ttry**0.431_DP
        ec_e(2,3) = 8.629e-6_DP * auxt4 / gstnat(3)
        auxt4 = 2.61e-6_DP * ttry**0.609_DP
        ec_e(4,5) = 8.629e-6_DP * auxt4 / gstnat(5)
        auxt4 = 3.59e-4_DP * ttry**0.217_DP
        ec_e(2,5) = 8.629e-6_DP * auxt4 / gstnat(5)
        auxt4 = 1.13e-4_DP * ttry**0.279_DP
        ec_e(3,5) = 8.629e-6_DP * auxt4 / gstnat(5)
        auxt4 = 6.82e-5_DP * ttry**0.301_DP
        ec_e(2,4) = 8.629e-6_DP * auxt4 / gstnat(4)
        auxt4 = 1.65e-4_DP * ttry**0.193_DP
        ec_e(3,4) = 8.629e-6_DP * auxt4 / gstnat(4)

c  Bring'em together

       do jl=1,nlvnat-1
         do ju=jl+1,nlvnat
           ec(jl,ju) = ec_H(jl,ju) * ab(i_h)
     1               + ec_Hp(jl,ju) * ab(i_hp)
     2               + ec_paraH2(jl,ju) * xnhmp
     3               + ec_orthoH2(jl,ju) * xnhmo
     4               + ec_He(jl,ju) * ab(i_he)
     5               + ec_e(jl,ju) * ab(nspec)
         end do
       end do

       do ju=2,nlvnat
         do jl = 1,ju-1
           ec(ju,jl) = ec(jl,ju) * (gstnat(ju)/gstnat(jl))
     1               * exp(-(elknat(ju)-elknat(jl)) / ttry)
         end do
       end do

       eqcol = zero

       do jl=1,nlvnat-1
         do ju=jl+1,nlvnat
           eqcol(jl,ju) = eqcol(jl,ju) - ec(jl,ju)
           eqcol(ju,ju) = eqcol(ju,ju) + ec(jl,ju)
         end do
       end do

       do ju=2,nlvnat
         do jl = 1,ju-1
           eqcol(ju,jl) = eqcol(ju,jl) - ec(ju,jl)
           eqcol(jl,jl) = eqcol(jl,jl) + ec(ju,jl)
         end do
       end do

       return
       end

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLOAT (ttry, xnhmp, xnhmo)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE CHEM_DATA
        USE STR_DATA
        USE SP_DATA
        USE PROFIL

        implicit double precision (a-h,o-z)

        REAL (kind=DP), DIMENSION (nlvoat,nlvoat) :: ec_H, ec_He
        REAL (kind=DP), DIMENSION (nlvoat,nlvoat) :: ec_Hp, ec_e
        REAL (kind=DP), DIMENSION (nlvoat,nlvoat) :: ec_orthoH2
        REAL (kind=DP), DIMENSION (nlvoat,nlvoat) :: ec_paraH2, ec

        ec_H  = colr_min
        ec_orthoH2 = colr_min
        ec_paraH2  = colr_min
        ec_He = colr_min
        ec_Hp = colr_min
        ec_e  = colr_min

c    collisions avec H
c  fit refait d apres Launay & Roueff

       ec_H(1,2) = 4.37d-12 * ttry**0.66d0
       ec_H(1,3) = 1.06d-12 * ttry**0.80d0
       ec_H(2,3) = 1.35d-11 * ttry**0.45d0

c 1D - 3P
c fit by JLB, from datas of Federman & Shipsey, Ap J, 1983, 269, 791 (using potential V2)

       ec_H(1,4) = 9.85e-14_DP * ttry**0.245_DP
       ec_H(2,4) = 9.85e-14_DP * ttry**0.245_DP
       ec_H(3,4) = 9.85e-14_DP * ttry**0.245_DP

c  collisions avec H2
c  fit from Jaquet, Staemmler, Smith & Flower, J.Phys.B (1992)

       ec_paraH2(1,2) = 3.46d-11 * ttry**0.316d0
       ec_orthoH2(1,2) = 2.70d-11 * ttry**0.362d0
       ec_paraH2(1,3) = 7.07d-11 * ttry**0.268d0
       ec_orthoH2(1,3) = 5.49d-11 * ttry**0.317d0
       ec_paraH2(2,3) = 1.44d-14 * ttry**1.109d0
       ec_orthoH2(2,3) = 4.64d-14 * ttry**0.976d0

c     collisions avec He
c fit from Monteiro & Flower, MNRAS 228, 101 (1987)

       ec_He(1,2) = 1.55d-12 * ttry**0.694d0
       ec_He(1,3) = 2.52d-12 * ttry**0.681d0
       ec_He(2,3) = 2.00d-15 * ttry**1.428d0

c    collisions avec H+
c Pequignot A&A 231, 499 (1990) + erratum A&A 313, 1026 (1996)

        if (ttry < 194.664_dp) then
          toto = 1.40e-3_dp * (ttry*1.0e-2_dp)**0.90_dp
        else if (ttry < 3686.2414_dp) then
          toto = 2.14e-2_dp * (ttry*1.0e-3_dp)**1.30_dp
        else
          toto = 2.78e-1_dp * (ttry*1.0e-4_dp)**0.87_dp
        endif
        ec_HP(1,2) = toto * 8.629e-6_dp / (sqrt(ttry) * gstoat(2))

        if (ttry < 511.9563_dp) then
          toto = 1.12e-4_dp * (ttry*1.0e-2_dp)**1.60_dp
        else if (ttry < 7510.155_dp) then
          toto = 3.90e-3_dp * (ttry*1.0e-3_dp)**1.40_dp
        else
          toto = 8.25e-2_dp * (ttry*1.0e-4_dp)**0.80_dp
        endif
        ec_Hp(1,3) = toto * 8.629e-6_dp / sqrt(ttry)

        if (ttry < 2090.2558_dp) then
          toto = 3.10e-4_dp * (ttry*1.0e-2_dp)**1.06_dp
        else
          toto = 2.29e-2_dp * (ttry*1.0e-4_dp)**0.69_dp
        endif
        ec_Hp(2,3) = toto * 8.629e-6_dp / sqrt(ttry)

c     d-collisions with electrons
c      From Pequignot, 1990, A&A, 231, 499 and Berrington, 1998, MNRAS, 293, L83

       auxt4 = 4.28e-1_DP * (ttry/1.e4_DP) ** 1.43_DP
     1       / (1.00_DP + 6.05e-1_DP * (ttry/1.e4_DP) ** 1.105_DP)
       ec_e(1,4) = 8.629e-6_DP * auxt4 / gstoat(4)
     1       / sqrt(ttry) * gstoat(1) / 9.0_DP
       ec_e(2,4) = 8.629e-6_DP * auxt4 / gstoat(4)
     1       / sqrt(ttry) * gstoat(2) / 9.0_DP
       ec_e(3,4) = 8.629e-6_DP * auxt4 / gstoat(4)
     1       / sqrt(ttry) * gstoat(3) / 9.0_DP
       auxt4 = 5.88e-2_DP * (ttry/1.e4_DP) ** 1.50_DP
     1       / (1.00_DP + 8.00e-1_DP * (ttry/1.e4_DP) ** 1.125_DP)
       ec_e(1,5) = 8.629e-6_DP * auxt4 / gstoat(5)
     1       / sqrt(ttry) * gstoat(1) / 9.0_DP
       ec_e(2,5) = 8.629e-6_DP * auxt4 / gstoat(5)
     1       / sqrt(ttry) * gstoat(2) / 9.0_DP
       ec_e(3,5) = 8.629e-6_DP * auxt4 / gstoat(5)
     1       / sqrt(ttry) * gstoat(3) / 9.0_DP
       auxt4 = 1.16e-1_DP * (ttry/1.e4_DP) ** 0.53_DP
     1       / (1.00_DP + 1.11e-1_DP * (ttry/1.e4_DP) ** 0.160_DP)
       ec_e(4,5) = 8.629e-6_DP *auxt4 / gstoat(5) / sqrt(ttry)

       auxt4 = 1.49e-4_DP * ttry ** 0.4565_DP
       ec_e(1,2) = 8.629e-6_DP * auxt4 / gstoat(2) / sqrt(ttry)
       auxt4 = 4.98e-5_DP * ttry ** 0.4955_DP
       ec_e(1,3) = 8.629e-6_DP * auxt4 / gstoat(3) / sqrt(ttry)
       auxt4 = 1.83e-9_DP * ttry ** 1.347_DP
       ec_e(2,3) = 8.629e-6_DP * auxt4 / gstoat(3) / sqrt(ttry)

c  Bring'em together

       do jl=1,nlvoat-1
         do ju=jl+1,nlvoat
           ec(jl,ju) = ec_H(jl,ju) * ab(i_h)
     1               + ec_Hp(jl,ju) * ab(i_hp)
     2               + ec_paraH2(jl,ju) * xnhmp
     3               + ec_orthoH2(jl,ju) * xnhmo
     4               + ec_He(jl,ju) * ab(i_he)
     5               + ec_e(jl,ju) * ab(nspec)
         end do
       end do

       do ju=2,nlvoat
         do jl = 1,ju-1
           ec(ju,jl) = ec(jl,ju) * (gstoat(ju)/gstoat(jl))
     1               * exp(-(elkoat(ju)-elkoat(jl)) / ttry)
         end do
       end do

       eqcol = zero

       do jl=1,nlvoat-1
         do ju=jl+1,nlvoat
           eqcol(jl,ju) = eqcol(jl,ju) - ec(jl,ju)
           eqcol(ju,ju) = eqcol(ju,ju) + ec(jl,ju)
         end do
       end do

       do ju=2,nlvoat
         do jl = 1,ju-1
           eqcol(ju,jl) = eqcol(ju,jl) - ec(ju,jl)
           eqcol(jl,jl) = eqcol(jl,jl) + ec(ju,jl)
         end do
       end do

       return
       end

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLSAT (ttry, xnhmp, xnhmo)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE CHEM_DATA
        USE STR_DATA
        USE SP_DATA
        USE PROFIL

        implicit double precision (a-h,o-z)

        REAL (kind=DP), DIMENSION (nlvsat,nlvsat) :: ec_H, ec_He
        REAL (kind=DP), DIMENSION (nlvsat,nlvsat) :: ec_Hp, ec_e
        REAL (kind=DP), DIMENSION (nlvsat,nlvsat) :: ec_orthoH2
        REAL (kind=DP), DIMENSION (nlvsat,nlvsat) :: ec_paraH2, ec

        ec_H  = colr_min
        ec_orthoH2 = colr_min
        ec_paraH2  = colr_min
        ec_He = colr_min
        ec_Hp = colr_min
        ec_e  = colr_min

c All collision rates as for O, but CT with H is excluded, as it is non-resonant

c    collisions avec H

        ec_H(1,2) = 4.37e-12_DP * ttry**0.66_DP
        ec_H(1,3) = 1.06e-12_DP * ttry**0.80_DP
        ec_H(2,3) = 1.35e-11_DP * ttry**0.45_DP

c    collisions avec H2
c    fit from Jaquet, Staemmler, Smith & Flower, J.Phys.B (1991)

        ec_paraH2(1,2) = 3.46e-11_DP * ttry**0.316_DP
        ec_orthoH2(1,2) = 2.70e-11_DP * ttry**0.362_DP
        ec_paraH2(1,3) = 7.07e-11_DP * ttry**0.268_DP
        ec_orthoH2(1,3) = 5.49e-11_DP * ttry**0.317_DP
        ec_paraH2(2,3) = 1.44e-14_DP * ttry**1.109_DP
        ec_orthoH2(2,3) = 4.64e-14_DP * ttry**0.976_DP

c    collisions avec He
c    fit from Monteiro & Flower, MNRAS 228, 101 (1987)

        ec_He(1,2) = 1.55e-12_DP * ttry**0.694_DP
        ec_He(1,3) = 2.52e-12_DP * ttry**0.681_DP
        ec_He(2,3) = 2.00e-15_DP * ttry**1.428_DP

c    collisions avec H+
c guess (Evelyne)

        ec_Hp(1,2) = 1.0e-8_DP
        ec_Hp(1,3) = 1.0e-8_DP
        ec_Hp(2,3) = 1.0e-8_DP

c    collisions with electrons
c    From Pequignot, 1990, A&A, 231, 499 and Berrington, 1998, MNRAS, 293, L83

        auxt4 = 4.28e-1_DP * (ttry/1.e4_DP) ** 1.43_DP
     1        / (1.00_DP + 6.05e-1_DP * (ttry/1.e4_DP) ** 1.105_DP)
        ec_e(1,4) = 8.629e-6_DP * auxt4 / gstsat(4)
     1        / sqrt(ttry) * gstsat(1) / 9.0_DP
        ec_e(2,4) = 8.629e-6_DP * auxt4 / gstsat(4)
     1        / sqrt(ttry) * gstsat(2) / 9.0_DP
        ec_e(3,4) = 8.629e-6_DP * auxt4 / gstsat(4)
     1        / sqrt(ttry) * gstsat(3) / 9.0_DP
        auxt4 = 5.88e-2_DP * (ttry/1.e4_DP) ** 1.50_DP
     1        / (1.00_DP + 8.00e-1_DP * (ttry/1.e4_DP) ** 1.125_DP)
        ec_e(1,5) = 8.629e-6_DP * auxt4 / gstsat(5)
     1        / sqrt(ttry) * gstsat(1) / 9.0_DP
        ec_e(2,5) = 8.629e-6_DP * auxt4 / gstsat(5)
     1        / sqrt(ttry) * gstsat(2) / 9.0_DP
        ec_e(3,5) = 8.629e-6_DP * auxt4 / gstsat(5)
     1        / sqrt(ttry) * gstsat(3) / 9.0_DP
        auxt4 = 1.16e-1_DP * (ttry/1.e4_DP) ** 0.53_DP
     1        / (1.00_DP + 1.11e-1_DP * (ttry/1.e4_DP) ** 0.160_DP)
        ec_e(4,5) = 8.629e-6_DP *auxt4 / gstsat(5) / sqrt(ttry)

        auxt4 = 1.49e-4_DP * ttry ** 0.4565_DP
        ec_e(1,2) = 8.629e-6_DP * auxt4 / gstsat(2) / sqrt(ttry)
        auxt4 = 4.98e-5_DP * ttry ** 0.4955_DP
        ec_e(1,3) = 8.629e-6_DP * auxt4 / gstsat(3) / sqrt(ttry)
        auxt4 = 1.83e-9_DP * ttry ** 1.347_DP
        ec_e(2,3) = 8.629e-6_DP * auxt4 / gstsat(3) / sqrt(ttry)

c  Bring'em together

       do jl=1,nlvsat-1
         do ju=jl+1,nlvsat
           ec(jl,ju) = ec_H(jl,ju) * ab(i_h)
     1               + ec_Hp(jl,ju) * ab(i_hp)
     2               + ec_paraH2(jl,ju) * xnhmp
     3               + ec_orthoH2(jl,ju) * xnhmo
     4               + ec_He(jl,ju) * ab(i_he)
     5               + ec_e(jl,ju) * ab(nspec)
         end do
       end do

       do ju=2,nlvsat
         do jl = 1,ju-1
           ec(ju,jl) = ec(jl,ju) * (gstsat(ju)/gstsat(jl))
     1               * exp(-(elksat(ju)-elksat(jl)) / ttry)
         end do
       end do

       eqcol = zero

       do jl=1,nlvsat-1
         do ju=jl+1,nlvsat
           eqcol(jl,ju) = eqcol(jl,ju) - ec(jl,ju)
           eqcol(ju,ju) = eqcol(ju,ju) + ec(jl,ju)
         end do
       end do

       do ju=2,nlvsat
         do jl = 1,ju-1
           eqcol(ju,jl) = eqcol(ju,jl) - ec(ju,jl)
           eqcol(jl,jl) = eqcol(jl,jl) + ec(ju,jl)
         end do
       end do

       return
       end

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLSiAT (ttry, xnhmp, xnhmo)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE CHEM_DATA
        USE STR_DATA
        USE SP_DATA
        USE PROFIL

        implicit double precision (a-h,o-z)

        REAL (kind=DP), DIMENSION (nlvsiat,nlvsiat) :: ec_H, ec_He
        REAL (kind=DP), DIMENSION (nlvsiat,nlvsiat) :: ec_Hp, ec_e
        REAL (kind=DP), DIMENSION (nlvsiat,nlvsiat) :: ec_orthoH2
        REAL (kind=DP), DIMENSION (nlvsiat,nlvsiat) :: ec_paraH2, ec

        ec_H  = colr_min
        ec_orthoH2 = colr_min
        ec_paraH2  = colr_min
        ec_He = colr_min
        ec_Hp = colr_min
        ec_e  = colr_min

c All collision rates as for C

c    collisions avec H
c    Fine structure: Launay & Roueff, 1977, A&A, 56, 289

        ec_H(1,2) = 1.01e-10_DP * ttry**0.117_DP
        ec_H(1,3) = 4.49e-11_DP * ttry**0.194_DP
        ec_H(2,3) = 1.06e-10_DP * ttry**0.234_DP

c    collisions avec H2
c    fit from Schroder, Staemmler, Smith, Flower & Jaquet (1991)

        ec_paraH2(1,2) = 0.80e-10_DP
        ec_orthoH2(1,2)= 0.75e-10_DP
        ec_paraH2(1,3) = 0.90e-10_DP
        ec_orthoH2(1,3)= 3.54e-11_DP * ttry**0.167_DP
        ec_paraH2(2,3) = 2.00e-10_DP
        ec_orthoH2(2,3)= 5.25e-11_DP * ttry**0.244_DP

c    collisions avec He
c    Fine structure: Staemmler & Flower (1991)

        ec_He(1,2) = 8.38e-12_DP * ttry**0.159_DP
        ec_He(1,3) = 5.98e-11_DP * ttry**0.078_DP
        ec_He(2,3) = 3.68e-11_DP * ttry**0.041_DP

c    collisions avec H+

        ec_Hp(1,2) = 10._DP**(-10.359_DP + 0.7959_DP*log10(ttry)
     1             - 0.08748_DP*(log10(ttry))**2._DP)
        ec_Hp(1,3) = 10._DP**(-13.232_DP + 2.4171_DP*log10(ttry)
     1             - 0.29151_DP*(log10(ttry))**2._DP)
        ec_Hp(2,3) = 10._DP**(-11.290_DP + 1.7915_DP*log10(ttry)
     1             - 0.23010_DP*(log10(ttry))**2._DP)

c    collisions with electrons
c    From Pequignot & Aldrovandi, A&A 50, 141, 1976 as compiled by Mendoza (1983)

        auxt4 = 1.83e-4_DP * ttry ** 0.444_DP
        ec_e(1,4) = 8.629e-6_DP * auxt4
     1            / gstsiat(4) * gstsiat(1) / 9.0_DP
        ec_e(2,4) = 8.629e-6_DP * auxt4
     1            / gstsiat(4) * gstsiat(2) / 9.0_DP
        ec_e(3,4) = 8.629e-6_DP * auxt4
     1            / gstsiat(4) * gstsiat(3) / 9.0_DP
        auxt4 = 9.86e-5_DP * ttry ** 0.343_DP
        ec_e(1,5) = 8.629e-6_DP * auxt4
     1            / gstsiat(5) * gstsiat(1) / 9.0_DP
        ec_e(2,5) = 8.629e-6_DP * auxt4
     1            / gstsiat(5) * gstsiat(2) / 9.0_DP
        ec_e(3,5) = 8.629e-6_DP * auxt4
     1            / gstsiat(5) * gstsiat(3) / 9.0_DP
        auxt4 = 2.77e-3_DP
        ec_e(4,5) = 8.629e-6_DP *auxt4 / gstsiat(5)

c  Bring'em together

       do jl=1,nlvsiat-1
         do ju=jl+1,nlvsiat
           ec(jl,ju) = ec_H(jl,ju) * ab(i_h)
     1               + ec_Hp(jl,ju) * ab(i_hp)
     2               + ec_paraH2(jl,ju) * xnhmp
     3               + ec_orthoH2(jl,ju) * xnhmo
     4               + ec_He(jl,ju) * ab(i_he)
     5               + ec_e(jl,ju) * ab(nspec)
         end do
       end do

       do ju=2,nlvsiat
         do jl = 1,ju-1
           ec(ju,jl) = ec(jl,ju) * (gstsiat(ju)/gstsiat(jl))
     1               * exp(-(elksiat(ju)-elksiat(jl)) / ttry)
         end do
       end do

       eqcol = zero

       do jl=1,nlvsiat-1
         do ju=jl+1,nlvsiat
           eqcol(jl,ju) = eqcol(jl,ju) - ec(jl,ju)
           eqcol(ju,ju) = eqcol(ju,ju) + ec(jl,ju)
         end do
       end do

       do ju=2,nlvsiat
         do jl = 1,ju-1
           eqcol(ju,jl) = eqcol(ju,jl) - ec(ju,jl)
           eqcol(jl,jl) = eqcol(jl,jl) + ec(ju,jl)
         end do
       end do

       return
       end

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLCPL (ttry, xnhmp, xnhmo)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE STR_DATA
        USE CHEM_DATA
        USE SP_DATA
        USE PROFIL

        implicit double precision (a-h,o-z)

        REAL (kind=DP), DIMENSION (nlvcpl,nlvcpl) :: ec_H, ec_He
        REAL (kind=DP), DIMENSION (nlvcpl,nlvcpl) :: ec_Hp, ec_e
        REAL (kind=DP), DIMENSION (nlvcpl,nlvcpl) :: ec_orthoH2
        REAL (kind=DP), DIMENSION (nlvcpl,nlvcpl) :: ec_paraH2, ec

        ec_H  = colr_min
        ec_orthoH2 = colr_min
        ec_paraH2  = colr_min
        ec_He = colr_min
        ec_Hp = colr_min
        ec_e  = colr_min

c    a-collisions avec H

        ec_H(1,2) = 8.86e-10_dp

c  b-collisions avec H2

        ec_paraH2(1,2) = 3.94e-10_dp
        ec_orthoH2(1,2) = 4.92e-10_dp

c     c-collisions avec electrons

c k21 : Hayes & Nussbaumer, 1984, A&A 134, 193
c Dec 02, CM & DRF
c k21 changed to power law fit to Wilson & Bell, 2002, MNRAS, 337, 1027
c other : Lennon et al., 1985, Ap J 294, 200

        toto = 0.65582_dp * ttry**(0.13244_DP)
        ec_e(1,2) = 8.629e-6_DP * min(toto,2.5_dp)
     1            / (4.0_DP*sqrt(ttry))
        ec_e(1,3) = 8.629e-6_DP * (0.288_DP - 5.89e-7_DP*ttry)
     1            / (2.0_DP * sqrt(ttry))
        ec_e(1,4) = 8.629e-6_DP * (0.424_DP - 8.35e-7_DP*ttry)
     1            / (4.0_DP * sqrt(ttry))
        ec_e(1,5) = 8.629e-6_DP * (0.257_DP - 3.95e-7_DP*ttry)
     1            / (6.0_DP * sqrt(ttry))
        ec_e(2,3) = 8.629e-6_DP * (0.197_DP - 3.15e-7_DP*ttry)
     1            / (2.0_DP * sqrt(ttry))
        ec_e(2,4) = 8.629e-6_DP * (0.544_DP - 9.84e-7_DP*ttry)
     1            / (4.0_DP * sqrt(ttry))
        ec_e(2,5) = 8.629e-6_DP * (1.200_DP - 2.40e-6_DP*ttry)
     1            / (6.0_DP * sqrt(ttry))

c  Bring'em together

       do jl=1,nlvcpl-1
         do ju=jl+1,nlvcpl
           ec(jl,ju) = ec_H(jl,ju) * ab(i_h)
     1               + ec_Hp(jl,ju) * ab(i_hp)
     2               + ec_paraH2(jl,ju) * xnhmp
     3               + ec_orthoH2(jl,ju) * xnhmo
     4               + ec_He(jl,ju) * ab(i_he)
     5               + ec_e(jl,ju) * ab(nspec)
         end do
       end do

       do ju=2,nlvcpl
         do jl = 1,ju-1
           ec(ju,jl) = ec(jl,ju) * (gstcpl(ju)/gstcpl(jl))
     1               * exp(-(elkcpl(ju)-elkcpl(jl)) / ttry)
         end do
       end do

       eqcol = zero

       do jl=1,nlvcpl-1
         do ju=jl+1,nlvcpl
           eqcol(jl,ju) = eqcol(jl,ju) - ec(jl,ju)
           eqcol(ju,ju) = eqcol(ju,ju) + ec(jl,ju)
         end do
       end do

       do ju=2,nlvcpl
         do jl = 1,ju-1
           eqcol(ju,jl) = eqcol(ju,jl) - ec(ju,jl)
           eqcol(jl,jl) = eqcol(jl,jl) + ec(ju,jl)
         end do
       end do

        return
        end

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLNPL (ttry, xnhmp, xnhmo)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE STR_DATA
        USE CHEM_DATA
        USE SP_DATA
        USE PROFIL

        implicit double precision (a-h,o-z)

        REAL (kind=DP), DIMENSION (nlvnpl,nlvnpl) :: ec_H, ec_He
        REAL (kind=DP), DIMENSION (nlvnpl,nlvnpl) :: ec_Hp, ec_e
        REAL (kind=DP), DIMENSION (nlvnpl,nlvnpl) :: ec_orthoH2
        REAL (kind=DP), DIMENSION (nlvnpl,nlvnpl) :: ec_paraH2, ec

        ec_H  = colr_min
        ec_orthoH2 = colr_min
        ec_paraH2  = colr_min
        ec_He = colr_min
        ec_Hp = colr_min
        ec_e  = colr_min

c    collisions avec H

c    collisions avec H2

c    collisions avec electrons
c Mendoza, 1983, International astrophysical union, symposium 103

        ec_e(1,2) =  8.629e-6_DP * (0.138_DP * ttry**0.119_DP)
     1            / (3.0_DP * sqrt(ttry))
        ec_e(1,3) =  8.629e-6_DP * (0.0834_DP * ttry**0.130_DP)
     1            / (5.0_DP * sqrt(ttry))
        ec_e(2,3) =  8.629e-6_DP * (0.359_DP * ttry**0.125_DP)
     1            / (5.0_DP * sqrt(ttry))
        ec_e(1,4) =  8.629e-6_DP * (0.203_DP * ttry**0.0414_DP)
     1            / (5.0_DP * sqrt(ttry))
        ec_e(2,4) =  8.629e-6_DP * (0.609_DP * ttry**0.0414_DP)
     1            / (5.0_DP * sqrt(ttry))
        ec_e(3,4) =  8.629e-6_DP * (1.02_DP * ttry**0.0414_DP)
     1            / (5.0_DP * sqrt(ttry))
        ec_e(4,5) =  8.629e-6_DP * (3.14_DP * ttry**(-0.142_DP))
     1            / (2.0_DP * sqrt(ttry))
        ec_e(3,5) =  8.629e-6_DP * (0.158_DP + ttry*4.26e-7_DP)
     1            / (2.0_DP * sqrt(ttry))
        ec_e(2,5) =  8.629e-6_DP * (0.0949_DP + ttry*2.56e-7_DP)
     1            / (2.0_DP * sqrt(ttry))
        ec_e(1,5) =  8.629e-6_DP * (0.0316_DP + ttry*8.52e-8_DP)
     1            / (2.0_DP * sqrt(ttry))

c  Bring'em together

       do jl=1,nlvnpl-1
         do ju=jl+1,nlvnpl
           ec(jl,ju) = ec_H(jl,ju) * ab(i_h)
     1               + ec_Hp(jl,ju) * ab(i_hp)
     2               + ec_paraH2(jl,ju) * xnhmp
     3               + ec_orthoH2(jl,ju) * xnhmo
     4               + ec_He(jl,ju) * ab(i_he)
     5               + ec_e(jl,ju) * ab(nspec)
         end do
       end do

       do ju=2,nlvnpl
         do jl = 1,ju-1
           ec(ju,jl) = ec(jl,ju) * (gstnpl(ju)/gstnpl(jl))
     1               * exp(-(elknpl(ju)-elknpl(jl)) / ttry)
         end do
       end do

       eqcol = zero

       do jl=1,nlvnpl-1
         do ju=jl+1,nlvnpl
           eqcol(jl,ju) = eqcol(jl,ju) - ec(jl,ju)
           eqcol(ju,ju) = eqcol(ju,ju) + ec(jl,ju)
         end do
       end do

       do ju=2,nlvnpl
         do jl = 1,ju-1
           eqcol(ju,jl) = eqcol(ju,jl) - ec(ju,jl)
           eqcol(jl,jl) = eqcol(jl,jl) + ec(ju,jl)
         end do
       end do

        return
        end

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLOPL (ttry, xnhmp, xnhmo)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE STR_DATA
        USE CHEM_DATA
        USE SP_DATA
        USE PROFIL

        implicit double precision (a-h,o-z)

        REAL (kind=DP), DIMENSION (nlvopl,nlvopl) :: ec_H, ec_He
        REAL (kind=DP), DIMENSION (nlvopl,nlvopl) :: ec_Hp, ec_e
        REAL (kind=DP), DIMENSION (nlvopl,nlvopl) :: ec_orthoH2
        REAL (kind=DP), DIMENSION (nlvopl,nlvopl) :: ec_paraH2, ec

        ec_H  = colr_min
        ec_orthoH2 = colr_min
        ec_paraH2  = colr_min
        ec_He = colr_min
        ec_Hp = colr_min
        ec_e  = colr_min

c    collisions avec H

c    collisions avec H2

c    collisions avec electrons

c  Bring'em together

       do jl=1,nlvopl-1
         do ju=jl+1,nlvopl
           ec(jl,ju) = ec_H(jl,ju) * ab(i_h)
     1               + ec_Hp(jl,ju) * ab(i_hp)
     2               + ec_paraH2(jl,ju) * xnhmp
     3               + ec_orthoH2(jl,ju) * xnhmo
     4               + ec_He(jl,ju) * ab(i_he)
     5               + ec_e(jl,ju) * ab(nspec)
         end do
       end do

       do ju=2,nlvopl
         do jl = 1,ju-1
           ec(ju,jl) = ec(jl,ju) * (gstopl(ju)/gstopl(jl))
     1               * exp(-(elkopl(ju)-elkopl(jl)) / ttry)
         end do
       end do

       eqcol = zero

       do jl=1,nlvopl-1
         do ju=jl+1,nlvopl
           eqcol(jl,ju) = eqcol(jl,ju) - ec(jl,ju)
           eqcol(ju,ju) = eqcol(ju,ju) + ec(jl,ju)
         end do
       end do

       do ju=2,nlvopl
         do jl = 1,ju-1
           eqcol(ju,jl) = eqcol(ju,jl) - ec(ju,jl)
           eqcol(jl,jl) = eqcol(jl,jl) + ec(ju,jl)
         end do
       end do

        return
        end

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLSPL (ttry, xnhmp, xnhmo)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE STR_DATA
        USE CHEM_DATA
        USE SP_DATA
        USE PROFIL

        implicit double precision (a-h,o-z)

        REAL (kind=DP), DIMENSION (nlvspl,nlvspl) :: ec_H, ec_He
        REAL (kind=DP), DIMENSION (nlvspl,nlvspl) :: ec_Hp, ec_e
        REAL (kind=DP), DIMENSION (nlvspl,nlvspl) :: ec_orthoH2
        REAL (kind=DP), DIMENSION (nlvspl,nlvspl) :: ec_paraH2, ec

        ec_H  = colr_min
        ec_orthoH2 = colr_min
        ec_paraH2  = colr_min
        ec_He = colr_min
        ec_Hp = colr_min
        ec_e  = colr_min

c    a-collisions avec H
c Storzer and Hollenbach, Ap J 539, 751 (2000) (guess)

       ec_H(1,2) = 1.0e-12_DP
       ec_H(1,3) = 1.0e-12_DP
       ec_H(2,3) = 1.0e-12_DP
       ec_H(1,4) = 1.0e-12_DP
       ec_H(2,4) = 1.0e-12_DP
       ec_H(3,4) = 1.0e-12_DP
       ec_H(1,5) = 1.0e-12_DP
       ec_H(2,5) = 1.0e-12_DP
       ec_H(3,5) = 1.0e-12_DP
       ec_H(4,5) = 1.0e-12_DP

c  b-collisions avec H2

c     c-collisions avec electrons

c Mendoza, 1983, International astrophysical union, symposium 103

c      ec_e(1,2) =  8.629e-6_DP * (6.4739_DP * ttry**(-0.092663_DP))
c    1           / (4.0_DP * sqrt(ttry))
c      ec_e(1,3) =  8.629e-6_DP * (9.7526_DP * ttry**(-0.093125_DP))
c    1           / (6.0_DP * sqrt(ttry))

c Storzer and Hollenbach, Ap J 539, 751 (2000)

       tt4 = ttry * 1.0e-4_DP
       ec_e(1,2) = 6.0e-8_DP * tt4**(-0.6_DP)
       ec_e(1,3) = 6.0e-8_DP * tt4**(-0.6_DP)
       ec_e(2,3) = 1.1e-7_DP * tt4**(-0.6_DP)
       ec_e(1,4) = 5.0e-8_DP * tt4**(-0.6_DP)
       ec_e(2,4) = 7.7e-8_DP * tt4**(-0.6_DP)
       ec_e(3,4) = 9.5e-8_DP * tt4**(-0.6_DP)
       ec_e(1,5) = 5.1e-8_DP * tt4**(-0.6_DP)
       ec_e(2,5) = 6.5e-8_DP * tt4**(-0.6_DP)
       ec_e(3,5) = 1.1e-7_DP * tt4**(-0.6_DP)
       ec_e(4,5) = 5.8e-8_DP * tt4**(-0.6_DP)

c  Bring'em together

       do jl=1,nlvspl-1
         do ju=jl+1,nlvspl
           ec(jl,ju) = ec_H(jl,ju) * ab(i_h)
     1               + ec_Hp(jl,ju) * ab(i_hp)
     2               + ec_paraH2(jl,ju) * xnhmp
     3               + ec_orthoH2(jl,ju) * xnhmo
     4               + ec_He(jl,ju) * ab(i_he)
     5               + ec_e(jl,ju) * ab(nspec)
         end do
       end do

       do ju=2,nlvspl
         do jl = 1,ju-1
           ec(ju,jl) = ec(jl,ju) * (gstspl(ju)/gstspl(jl))
     1               * exp(-(elkspl(ju)-elkspl(jl)) / ttry)
         end do
       end do

       eqcol = zero

       do jl=1,nlvspl-1
         do ju=jl+1,nlvspl
           eqcol(jl,ju) = eqcol(jl,ju) - ec(jl,ju)
           eqcol(ju,ju) = eqcol(ju,ju) + ec(jl,ju)
         end do
       end do

       do ju=2,nlvspl
         do jl = 1,ju-1
           eqcol(ju,jl) = eqcol(ju,jl) - ec(ju,jl)
           eqcol(jl,jl) = eqcol(jl,jl) + ec(ju,jl)
         end do
       end do

        return
        end

c       %%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLSIP (ttry, xnhmp, xnhmo)
c       %%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE STR_DATA
        USE CHEM_DATA
        USE SP_DATA
        USE PROFIL

        implicit double precision (a-h,o-z)

        REAL (kind=DP), DIMENSION (nlvsip,nlvsip) :: ec_H, ec_He
        REAL (kind=DP), DIMENSION (nlvsip,nlvsip) :: ec_Hp, ec_e
        REAL (kind=DP), DIMENSION (nlvsip,nlvsip) :: ec_orthoH2
        REAL (kind=DP), DIMENSION (nlvsip,nlvsip) :: ec_paraH2, ec

        ec_H  = colr_min
        ec_orthoH2 = colr_min
        ec_paraH2  = colr_min
        ec_He = colr_min
        ec_Hp = colr_min
        ec_e  = colr_min

c  excitation de Si+
c  d apres Roueff 90 A&A 234, 567

c    collisions avec H

        ec_H(1,2) = 6.00e-10_DP

c    collisions avec H2

        ec_paraH2(1,2)  = 3.94e-10_DP
        ec_orthoH2(1,2) = 4.92e-10_DP

c  excitation de Si+ par les electrons
c Dufton & Kingston, 1991, MNRAS 248, 827

        ec_e(1,2) = 8.629e-6_DP * 5.6_DP / (4.0_DP*sqrt(ttry))

c  Bring'em together

       do jl=1,nlvsip-1
         do ju=jl+1,nlvsip
           ec(jl,ju) = ec_H(jl,ju) * ab(i_h)
     1               + ec_Hp(jl,ju) * ab(i_hp)
     2               + ec_paraH2(jl,ju) * xnhmp
     3               + ec_orthoH2(jl,ju) * xnhmo
     4               + ec_He(jl,ju) * ab(i_he)
     5               + ec_e(jl,ju) * ab(nspec)
         end do
       end do

       do ju=2,nlvsip
         do jl = 1,ju-1
           ec(ju,jl) = ec(jl,ju) * (gstsip(ju)/gstsip(jl))
     1               * exp(-(elksip(ju)-elksip(jl)) / ttry)
         end do
       end do

       eqcol = zero

       do jl=1,nlvsip-1
         do ju=jl+1,nlvsip
           eqcol(jl,ju) = eqcol(jl,ju) - ec(jl,ju)
           eqcol(ju,ju) = eqcol(ju,ju) + ec(jl,ju)
         end do
       end do

       do ju=2,nlvsip
         do jl = 1,ju-1
           eqcol(ju,jl) = eqcol(ju,jl) - ec(ju,jl)
           eqcol(jl,jl) = eqcol(jl,jl) + ec(ju,jl)
         end do
       end do


        return
        end

c       %%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLCO0 (ttry)
c       %%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE STR_DATA
        USE SP_DATA
        USE PROFIL

c---------------------------------------------------------------------
c       ttry is temperature of neutral fluid

c       include 30 levels of co
c---------------------------------------------------------------------

        implicit double precision (a-h, o-z)

c       nouveaux taux (corrects?)
        data tcoh21, tcohe1,  tcoh1  / 1.d-10, 3.3d-11, 1.d-11 /
        data tcoh22, tcohe2,  tcoh2  / 1.d-10, 6.0d-11, 1.d-11 /
        data tcoh23, tcohe3,  tcoh3  / 1.d-11, 1.6d-11, 3.d-13 /
        data tcoh24, tcohe4,  tcoh4  / 1.d-11, 1.3d-11, 1.d-13 /

c         *************transfert deltaj = 1 *************

        cdodj1(1) = tcoh21 * ab(i_h2) / deux + tcoh1 * ab(i_h) * 0.6d0
     1             + tcohe1 * ab(i_he) * deux / trois
        do j=2,njco
          cdodj1(j) = tcoh21 * ab(i_h2) + tcoh1 * ab(i_h)
     1              + tcohe1 * ab(i_he)
        end do

        do j=0,njco-1
          delten = wlkco0(j+1)
          cupdj1(j) = cdodj1(j+1) * (2*j+3)
     1              * exp(-delten / ttry) / (2*j+1)
        end do

c       *************transfert deltaj = 2 *************

        cdodj2(2) = tcoh22 * ab(i_h2) / deux + tcoh2 * ab(i_h) * 0.5d0
     1            + tcohe2 * ab(i_he) / deux
        cdodj2(3) = tcoh22 * ab(i_h2) + tcoh2 * ab(i_h) * 0.7d0
     1            + tcohe2 * ab(i_he) / 1.33d0

        do j=4,njco
          cdodj2(j) = tcoh22 * ab(i_h2) + tcoh2 * ab(i_h)
     1              + tcohe2 * ab(i_he)
        end do

        do j=0,njco-2
          delten = wlkco0(j+2) + wlkco0(j+1)
          cupdj2(j) = cdodj2(j+2) * (2*j+5)
     1              * exp(-delten / ttry) / (2*j+1)
        end do

c       *************transfert deltaj = 3 *************

        cdodj3(3) = tcoh23 * ab(i_h2) / deux + tcoh3 * ab(i_h) / 3.0d0
     1            + tcohe3 * ab(i_he) * 0.3d0
        cdodj3(4) = tcoh23 * ab(i_h2) + tcoh3 * ab(i_h)
     1            + tcohe3 * ab(i_he) / deux
        do j=4, njco
           cdodj3(j) = tcoh23 * ab(i_h2) + tcoh3 *ab(i_h)
     1               + tcohe3 *ab(i_he)
        enddo

        do j=0, njco-3
          delten = wlkco0(j+3) + wlkco0(j+2) + wlkco0(j+1)
          cupdj3(j) = cdodj3(j+3) * (2*j+7)
     1              * exp(-delten / ttry) / (2*j+1)
        enddo


c       *************transfert deltaj = 4 ************************

        cdodj4(4) = tcoh24 * ab(i_h2) / deux + tcoh4 * ab(i_h) /deux
     1            + 0.5d-11 * ab(i_he)
        cdodj4(5) = tcoh24 * ab(i_h2) * 0.8d0 + tcoh4 * ab(i_h) * 0.8d0
     1            + 1.0d-11 * ab(i_he)
        do j=5, njco
           cdodj4(j) = tcoh24 * ab(i_h2) + tcoh4 * ab(i_h)
     1               + tcohe4 * ab(i_he)
        enddo

        do j=0, njco-4
          delten = wlkco0(j+4)+wlkco0(j+3)+wlkco0(j+2)+wlkco0(j+1)
          cupdj4(j) = cdodj4(j+4) * (2*j+9)
     1              * exp(-delten / ttry) / (2*j+1)
        enddo

c       ************ matrice d'equilibre collisionnel************

        eqcol = zero

        eqcol(1,1) = cupdj1(0) + cupdj2(0) + cupdj3(0) + cupdj4(0)
        eqcol(1,2) = - cdodj1(1)
        eqcol(1,3) = - cdodj2(2)
        eqcol(1,4) = - cdodj3(3)
        eqcol(1,5) = - cdodj4(4)

        eqcol(2,1) = - cupdj1(0)
        eqcol(2,2) = cupdj1(1) + cupdj2(1) + cupdj3(1) + cupdj4(1)
     1             + cdodj1(1)
        eqcol(2,3) = - cdodj1(2)
        eqcol(2,4) = - cdodj2(3)
        eqcol(2,5) = - cdodj3(4)
        eqcol(2,6) = - cdodj4(5)


        eqcol(3,1) = - cupdj2(0)
        eqcol(3,2) = - cupdj1(1)
        eqcol(3,3) = cupdj1(2) + cupdj2(2) + cupdj3(2) + cupdj4(2)
     1             + cdodj1(2) + cdodj2(2)
        eqcol(3,4) = - cdodj1(3)
        eqcol(3,5) = - cdodj2(4)
        eqcol(3,6) = - cdodj3(5)
        eqcol(3,7) = - cdodj4(6)

        eqcol(4,1) = - cupdj3(0)
        eqcol(4,2) = - cupdj2(1)
        eqcol(4,3) = - cupdj1(2)
        eqcol(4,4) = cupdj1(3) + cupdj2(3) + cupdj3(3) + cupdj4(3)
     1             + cdodj1(3) + cdodj2(3) + cdodj3(3)
        eqcol(4,5) = - cdodj1(4)
        eqcol(4,6) = - cdodj2(5)
        eqcol(4,7) = - cdodj3(6)
        eqcol(4,8) = - cdodj4(7)

        do ij=5,nlvco0-4
          eqcol(ij,ij-4) = - cupdj4(ij-5)
          eqcol(ij,ij-3) = - cupdj3(ij-4)
          eqcol(ij,ij-2) = - cupdj2(ij-3)
          eqcol(ij,ij-1) = - cupdj1(ij-2)
          eqcol(ij,ij) = cdodj1(ij-1) + cupdj1(ij-1)
     1                 + cdodj2(ij-1) + cupdj2(ij-1)
     2                 + cdodj3(ij-1) + cupdj3(ij-1)
     4                 + cdodj4(ij-1) + cupdj4(ij-1)
          eqcol(ij,ij+1) = - cdodj1(ij)
          eqcol(ij,ij+2) = - cdodj2(ij+1)
          eqcol(ij,ij+3) = - cdodj3(ij+2)
          eqcol(ij,ij+4) = - cdodj4(ij+3)
        end do


        eqcol(nlvco0-3,nlvco0-7) = - cupdj4(nlvco0-8)
        eqcol(nlvco0-3,nlvco0-6) = - cupdj3(nlvco0-7)
        eqcol(nlvco0-3,nlvco0-5) = - cupdj2(nlvco0-6)
        eqcol(nlvco0-3,nlvco0-4) = - cupdj1(nlvco0-5)
        eqcol(nlvco0-3,nlvco0-3) = cdodj1(nlvco0-4) + cupdj1(nlvco0-4)
     1                           + cdodj2(nlvco0-4) + cupdj2(nlvco0-4)
     2                           + cdodj3(nlvco0-4) + cupdj3(nlvco0-4)
     3                           + cdodj4(nlvco0-4)
        eqcol(nlvco0-3,nlvco0-2) = - cdodj1(nlvco0-3)
        eqcol(nlvco0-3,nlvco0-1) = - cdodj2(nlvco0-2)
        eqcol(nlvco0-3,nlvco0)   = - cdodj3(nlvco0-1)

        eqcol(nlvco0-2,nlvco0-6) = - cupdj4(nlvco0-7)
        eqcol(nlvco0-2,nlvco0-5) = - cupdj3(nlvco0-6)
        eqcol(nlvco0-2,nlvco0-4) = - cupdj2(nlvco0-5)
        eqcol(nlvco0-2,nlvco0-3) = - cupdj1(nlvco0-4)
        eqcol(nlvco0-2,nlvco0-2) = cdodj1(nlvco0-3) + cupdj1(nlvco0-3)
     1                           + cdodj2(nlvco0-3) + cupdj2(nlvco0-3)
     2                           + cdodj3(nlvco0-3) + cdodj4(nlvco0-3)
        eqcol(nlvco0-2,nlvco0-1) = - cdodj1(nlvco0-2)
        eqcol(nlvco0-2,nlvco0)   = - cdodj2(nlvco0-1)

        eqcol(nlvco0-1,nlvco0-5) = - cupdj4(nlvco0-6)
        eqcol(nlvco0-1,nlvco0-4) = - cupdj3(nlvco0-5)
        eqcol(nlvco0-1,nlvco0-3) = - cupdj2(nlvco0-4)
        eqcol(nlvco0-1,nlvco0-2) = - cupdj1(nlvco0-3)
        eqcol(nlvco0-1,nlvco0-1) = cdodj1(nlvco0-2) + cupdj1(nlvco0-2)
     1                           + cdodj2(nlvco0-2) + cdodj3(nlvco0-2)
     2                           + cdodj4(nlvco0-2)
        eqcol(nlvco0-1,nlvco0) = - cdodj1(nlvco0-1)


        eqcol(nlvco0,nlvco0-4) = - cupdj4(nlvco0-5)
        eqcol(nlvco0,nlvco0-3) = - cupdj3(nlvco0-4)
        eqcol(nlvco0,nlvco0-2) = - cupdj2(nlvco0-3)
        eqcol(nlvco0,nlvco0-1) = - cupdj1(nlvco0-2)
        eqcol(nlvco0,nlvco0)   = cdodj1(nlvco0-1) + cdodj2(nlvco0-1)
     1                         + cdodj3(nlvco0-1) + cdodj4(nlvco0-1)

        do j=1,nlvco0
          eqcol(j,j) = eqcol(j,j) + eqrco0(j-1,j-1)
        end do

        return
        end

c       %%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLCO1 (ttry)
c       %%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE STR_DATA
        USE SP_DATA
        USE PROFIL

c---------------------------------------------------------------------
c       ttry is temperature of neutral fluid

c       include 30 levels of co
c---------------------------------------------------------------------

        implicit double precision (a-h, o-z)

c       nouveaux taux (correctes?)
        data tcoh21, tcohe1,  tcoh1  / 1.d-10, 3.3d-11, 1.d-11 /
        data tcoh22, tcohe2,  tcoh2  / 1.d-10, 6.0d-11, 1.d-11 /
        data tcoh23, tcohe3,  tcoh3  / 1.d-11, 1.6d-11, 3.d-13 /
        data tcoh24, tcohe4,  tcoh4  / 1.d-11, 1.3d-11, 1.d-13 /

c         *************transfert deltaj = 1 *************

        cdodj1(1) = tcoh21 * ab(i_h2) / deux + tcoh1 * ab(i_h) * 0.6d0
     1             + tcohe1 * ab(i_he) * deux / trois
        do j=2,njco
          cdodj1(j) = tcoh21 * ab(i_h2) + tcoh1 * ab(i_h)
     1              + tcohe1 * ab(i_he)
        end do

        do j=0,njco-1
          delten = wlkco1(j+1)
          cupdj1(j) = cdodj1(j+1) * (2*j+3)
     1              * exp(-delten / ttry) / (2*j+1)
        end do

c       *************transfert deltaj = 2 *************

        cdodj2(2) = tcoh22 * ab(i_h2) / deux + tcoh2 * ab(i_h) * 0.5d0
     1            + tcohe2 * ab(i_he) / deux
        cdodj2(3) = tcoh22 * ab(i_h2) + tcoh2 * ab(i_h) * 0.7d0
     1            + tcohe2 * ab(i_he) / 1.33d0

        do j=4,njco
          cdodj2(j) = tcoh22 * ab(i_h2) + tcoh2 * ab(i_h)
     1              + tcohe2 * ab(i_he)
        end do

        do j=0,njco-2
          delten = wlkco1(j+2) + wlkco1(j+1)
          cupdj2(j) = cdodj2(j+2) * (2*j+5)
     1              * exp(-delten / ttry) / (2*j+1)
        end do

c       *************transfert deltaj = 3 *************

        cdodj3(3) = tcoh23 * ab(i_h2) / deux + tcoh3 * ab(i_h) / 3.0d0
     1            + tcohe3 * ab(i_he) * 0.3d0
        cdodj3(4) = tcoh23 * ab(i_h2) + tcoh3 * ab(i_h)
     1            + tcohe3 * ab(i_he) / deux
        do j=4, njco
           cdodj3(j) = tcoh23 * ab(i_h2) + tcoh3 * ab(i_h)
     1              + tcohe3 * ab(i_he)
        enddo

        do j=0, njco-3
          delten = wlkco1(j+3) + wlkco1(j+2) + wlkco1(j+1)
          cupdj3(j) = cdodj3(j+3) * (2*j+7)
     1              * exp(-delten / ttry) / (2*j+1)
        enddo


c       *************transfert deltaj = 4 ************************

        cdodj4(4) = tcoh24 * ab(i_h2) / deux + tcoh4 * ab(i_h) / deux
     1            + 0.5d-11 * ab(i_he)
        cdodj4(5) = tcoh24 * ab(i_h2) * 0.8d0 + tcoh4 * ab(i_h) * 0.8d0
     1            + 1.0d-11 * ab(i_he)
        do j=5, njco
           cdodj4(j) = tcoh24 * ab(i_h2) + tcoh4 * ab(i_h)
     1               + tcohe4 * ab(i_he)
        enddo

        do j=0, njco-4
          delten = wlkco1(j+4) + wlkco1(j+3) + wlkco1(j+2) + wlkco1(j+1)
          cupdj4(j) = cdodj4(j+4) * (2*j+9)
     1              * exp(-delten / ttry) / (2*j+1)
        enddo

c       ************ matrice d'equilibre collisionnel************

        eqcol = zero

        eqcol(1,1) = cupdj1(0) + cupdj2(0) + cupdj3(0) + cupdj4(0)
        eqcol(1,2) = - cdodj1(1)
        eqcol(1,3) = - cdodj2(2)
        eqcol(1,4) = - cdodj3(3)
        eqcol(1,5) = - cdodj4(4)

        eqcol(2,1) = - cupdj1(0)
        eqcol(2,2) = cupdj1(1) + cupdj2(1) + cupdj3(1) + cupdj4(1)
     1             + cdodj1(1)
        eqcol(2,3) = - cdodj1(2)
        eqcol(2,4) = - cdodj2(3)
        eqcol(2,5) = - cdodj3(4)
        eqcol(2,6) = - cdodj4(5)


        eqcol(3,1) = - cupdj2(0)
        eqcol(3,2) = - cupdj1(1)
        eqcol(3,3) = cupdj1(2) + cupdj2(2) + cupdj3(2) + cupdj4(2)
     1             + cdodj1(2) + cdodj2(2)
        eqcol(3,4) = - cdodj1(3)
        eqcol(3,5) = - cdodj2(4)
        eqcol(3,6) = - cdodj3(5)
        eqcol(3,7) = - cdodj4(6)

        eqcol(4,1) = - cupdj3(0)
        eqcol(4,2) = - cupdj2(1)
        eqcol(4,3) = - cupdj1(2)
        eqcol(4,4) = cupdj1(3) + cupdj2(3) + cupdj3(3) + cupdj4(3)
     1             + cdodj1(3) + cdodj2(3) + cdodj3(3)
        eqcol(4,5) = - cdodj1(4)
        eqcol(4,6) = - cdodj2(5)
        eqcol(4,7) = - cdodj3(6)
        eqcol(4,8) = - cdodj4(7)

        do ij=5,nlvco1-4
          eqcol(ij,ij-4) = - cupdj4(ij-5)
          eqcol(ij,ij-3) = - cupdj3(ij-4)
          eqcol(ij,ij-2) = - cupdj2(ij-3)
          eqcol(ij,ij-1) = - cupdj1(ij-2)
          eqcol(ij,ij) = cdodj1(ij-1) + cupdj1(ij-1)
     1                 + cdodj2(ij-1) + cupdj2(ij-1)
     2                 + cdodj3(ij-1) + cupdj3(ij-1)
     4                 + cdodj4(ij-1) + cupdj4(ij-1)
          eqcol(ij,ij+1) = - cdodj1(ij)
          eqcol(ij,ij+2) = - cdodj2(ij+1)
          eqcol(ij,ij+3) = - cdodj3(ij+2)
          eqcol(ij,ij+4) = - cdodj4(ij+3)
        end do


        eqcol(nlvco1-3,nlvco1-7) = - cupdj4(nlvco1-8)
        eqcol(nlvco1-3,nlvco1-6) = - cupdj3(nlvco1-7)
        eqcol(nlvco1-3,nlvco1-5) = - cupdj2(nlvco1-6)
        eqcol(nlvco1-3,nlvco1-4) = - cupdj1(nlvco1-5)
        eqcol(nlvco1-3,nlvco1-3) = cdodj1(nlvco1-4) + cupdj1(nlvco1-4)
     1                           + cdodj2(nlvco1-4) + cupdj2(nlvco1-4)
     2                           + cdodj3(nlvco1-4) + cupdj3(nlvco1-4)
     3                           + cdodj4(nlvco1-4)
        eqcol(nlvco1-3,nlvco1-2) = - cdodj1(nlvco1-3)
        eqcol(nlvco1-3,nlvco1-1) = - cdodj2(nlvco1-2)
        eqcol(nlvco1-3,nlvco1)   = - cdodj3(nlvco1-1)

        eqcol(nlvco1-2,nlvco1-6) = - cupdj4(nlvco1-7)
        eqcol(nlvco1-2,nlvco1-5) = - cupdj3(nlvco1-6)
        eqcol(nlvco1-2,nlvco1-4) = - cupdj2(nlvco1-5)
        eqcol(nlvco1-2,nlvco1-3) = - cupdj1(nlvco1-4)
        eqcol(nlvco1-2,nlvco1-2) = cdodj1(nlvco1-3) + cupdj1(nlvco1-3)
     1                           + cdodj2(nlvco1-3) + cupdj2(nlvco1-3)
     2                           + cdodj3(nlvco1-3) + cdodj4(nlvco1-3)
        eqcol(nlvco1-2,nlvco1-1) = - cdodj1(nlvco1-2)
        eqcol(nlvco1-2,nlvco1)   = - cdodj2(nlvco1-1)

        eqcol(nlvco1-1,nlvco1-5) = - cupdj4(nlvco1-6)
        eqcol(nlvco1-1,nlvco1-4) = - cupdj3(nlvco1-5)
        eqcol(nlvco1-1,nlvco1-3) = - cupdj2(nlvco1-4)
        eqcol(nlvco1-1,nlvco1-2) = - cupdj1(nlvco1-3)
        eqcol(nlvco1-1,nlvco1-1) = cdodj1(nlvco1-2) + cupdj1(nlvco1-2)
     1                           + cdodj2(nlvco1-2) + cdodj3(nlvco1-2)
     2                           + cdodj4(nlvco1-2)
        eqcol(nlvco1-1,nlvco1) = - cdodj1(nlvco1-1)


        eqcol(nlvco1,nlvco1-4) = - cupdj4(nlvco1-5)
        eqcol(nlvco1,nlvco1-3) = - cupdj3(nlvco1-4)
        eqcol(nlvco1,nlvco1-2) = - cupdj2(nlvco1-3)
        eqcol(nlvco1,nlvco1-1) = - cupdj1(nlvco1-2)
        eqcol(nlvco1,nlvco1)   = cdodj1(nlvco1-1) + cdodj2(nlvco1-1)
     1                         + cdodj3(nlvco1-1) + cdodj4(nlvco1-1)

        if (itrco1 .ne.0) then
          do j=1,nlvco1
            eqcol(j,j) = eqcol(j,j) + eqrco1(j-1,j-1)
          end do
        else
          do j=1,nlvco1
            eqcol(j,j) = eqcol(j,j) + eqrco0(j-1,j-1)
          end do
        endif

        return
        end

c       %%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLCO2 (ttry)
c       %%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE STR_DATA
        USE SP_DATA
        USE PROFIL

c---------------------------------------------------------------------
c       ttry is temperature of neutral fluid

c       include 30 levels of co
c---------------------------------------------------------------------

        implicit double precision (a-h, o-z)

c       nouveaux taux (correctes?)
        data tcoh21, tcohe1,  tcoh1  / 1.d-10, 3.3d-11, 1.d-11 /
        data tcoh22, tcohe2,  tcoh2  / 1.d-10, 6.0d-11, 1.d-11 /
        data tcoh23, tcohe3,  tcoh3  / 1.d-11, 1.6d-11, 3.d-13 /
        data tcoh24, tcohe4,  tcoh4  / 1.d-11, 1.3d-11, 1.d-13 /

c         *************transfert deltaj = 1 *************

        cdodj1(1) = tcoh21 * ab(i_h2) / deux + tcoh1 * ab(i_h) * 0.6d0
     1             + tcohe1 * ab(i_he) * deux / trois
        do j=2,njco
          cdodj1(j) = tcoh21 * ab(i_h2) + tcoh1 * ab(i_h)
     1              + tcohe1 * ab(i_he)
        end do

        do j=0,njco-1
          delten = wlkco2(j+1)
          cupdj1(j) = cdodj1(j+1) * (2*j+3)
     1              * exp(-delten / ttry) / (2*j+1)
        end do

c       *************transfert deltaj = 2 *************

        cdodj2(2) = tcoh22 * ab(i_h2) / deux + tcoh2 * ab(i_h) * 0.5d0
     1            + tcohe2 * ab(i_he) / deux
        cdodj2(3) = tcoh22 * ab(i_h2) + tcoh2 * ab(i_h) * 0.7d0
     1            + tcohe2 * ab(i_he) / 1.33d0

        do j=4,njco
          cdodj2(j) = tcoh22 * ab(i_h2) + tcoh2 * ab(i_h)
     1              + tcohe2 * ab(i_he)
        end do

        do j=0,njco-2
          delten = wlkco2(j+2) + wlkco2(j+1)
          cupdj2(j) = cdodj2(j+2) * (2*j+5)
     1              * exp(-delten / ttry) / (2*j+1)
        end do

c       *************transfert deltaj = 3 *************

        cdodj3(3) = tcoh23 * ab(i_h2) / deux + tcoh3 * ab(i_h) / 3.0d0
     1            + tcohe3 * ab(i_he) * 0.3d0
        cdodj3(4) = tcoh23 * ab(i_h2) + tcoh3 * ab(i_h)
     1            + tcohe3 * ab(i_he) / deux
        do j=4, njco
           cdodj3(j) = tcoh23 * ab(i_h2) + tcoh3 * ab(i_h)
     1               + tcohe3 * ab(i_he)
        enddo

        do j=0, njco-3
          delten = wlkco2(j+3) + wlkco2(j+2) + wlkco2(j+1)
          cupdj3(j) = cdodj3(j+3) * (2*j+7)
     1              * exp(-delten / ttry) / (2*j+1)
        enddo


c       *************transfert deltaj = 4 ************************

        cdodj4(4) = tcoh24 * ab(i_h2) / deux + tcoh4 * ab(i_h) / deux
     1            + 0.5d-11 * ab(i_he)
        cdodj4(5) = tcoh24 * ab(i_h2) * 0.8d0 + tcoh4 * ab(i_h) * 0.8d0
     1            + 1.0d-11 * ab(i_he)
        do j=5, njco
           cdodj4(j) = tcoh24 * ab(i_h2) + tcoh4 * ab(i_h)
     1               + tcohe4 * ab(i_he)
        enddo

        do j=0, njco-4
          delten = wlkco2(j+4)+wlkco2(j+3)+wlkco2(j+2)+wlkco2(j+1)
          cupdj4(j) = cdodj4(j+4) * (2*j+9)
     1              * exp(-delten / ttry) / (2*j+1)
        enddo


c       ************ matrice d'equilibre collisionnel************

        eqcol = zero

        eqcol(1,1) = cupdj1(0) + cupdj2(0) + cupdj3(0) + cupdj4(0)
        eqcol(1,2) = - cdodj1(1)
        eqcol(1,3) = - cdodj2(2)
        eqcol(1,4) = - cdodj3(3)
        eqcol(1,5) = - cdodj4(4)

        eqcol(2,1) = - cupdj1(0)
        eqcol(2,2) = cupdj1(1) + cupdj2(1) + cupdj3(1) + cupdj4(1)
     1             + cdodj1(1)
        eqcol(2,3) = - cdodj1(2)
        eqcol(2,4) = - cdodj2(3)
        eqcol(2,5) = - cdodj3(4)
        eqcol(2,6) = - cdodj4(5)


        eqcol(3,1) = - cupdj2(0)
        eqcol(3,2) = - cupdj1(1)
        eqcol(3,3) = cupdj1(2) + cupdj2(2) + cupdj3(2) + cupdj4(2)
     1             + cdodj1(2) + cdodj2(2)
        eqcol(3,4) = - cdodj1(3)
        eqcol(3,5) = - cdodj2(4)
        eqcol(3,6) = - cdodj3(5)
        eqcol(3,7) = - cdodj4(6)

        eqcol(4,1) = - cupdj3(0)
        eqcol(4,2) = - cupdj2(1)
        eqcol(4,3) = - cupdj1(2)
        eqcol(4,4) = cupdj1(3) + cupdj2(3) + cupdj3(3) + cupdj4(3)
     1             + cdodj1(3) + cdodj2(3) + cdodj3(3)
        eqcol(4,5) = - cdodj1(4)
        eqcol(4,6) = - cdodj2(5)
        eqcol(4,7) = - cdodj3(6)
        eqcol(4,8) = - cdodj4(7)

        do ij=5,nlvco2-4
          eqcol(ij,ij-4) = - cupdj4(ij-5)
          eqcol(ij,ij-3) = - cupdj3(ij-4)
          eqcol(ij,ij-2) = - cupdj2(ij-3)
          eqcol(ij,ij-1) = - cupdj1(ij-2)
          eqcol(ij,ij) = cdodj1(ij-1) + cupdj1(ij-1)
     1                 + cdodj2(ij-1) + cupdj2(ij-1)
     2                 + cdodj3(ij-1) + cupdj3(ij-1)
     4                 + cdodj4(ij-1) + cupdj4(ij-1)
          eqcol(ij,ij+1) = - cdodj1(ij)
          eqcol(ij,ij+2) = - cdodj2(ij+1)
          eqcol(ij,ij+3) = - cdodj3(ij+2)
          eqcol(ij,ij+4) = - cdodj4(ij+3)
        end do


        eqcol(nlvco2-3,nlvco2-7) = - cupdj4(nlvco2-8)
        eqcol(nlvco2-3,nlvco2-6) = - cupdj3(nlvco2-7)
        eqcol(nlvco2-3,nlvco2-5) = - cupdj2(nlvco2-6)
        eqcol(nlvco2-3,nlvco2-4) = - cupdj1(nlvco2-5)
        eqcol(nlvco2-3,nlvco2-3) = cdodj1(nlvco2-4) + cupdj1(nlvco2-4)
     1                           + cdodj2(nlvco2-4) + cupdj2(nlvco2-4)
     2                           + cdodj3(nlvco2-4) + cupdj3(nlvco2-4)
     3                           + cdodj4(nlvco2-4)
        eqcol(nlvco2-3,nlvco2-2) = - cdodj1(nlvco2-3)
        eqcol(nlvco2-3,nlvco2-1) = - cdodj2(nlvco2-2)
        eqcol(nlvco2-3,nlvco2)   = - cdodj3(nlvco2-1)

        eqcol(nlvco2-2,nlvco2-6) = - cupdj4(nlvco2-7)
        eqcol(nlvco2-2,nlvco2-5) = - cupdj3(nlvco2-6)
        eqcol(nlvco2-2,nlvco2-4) = - cupdj2(nlvco2-5)
        eqcol(nlvco2-2,nlvco2-3) = - cupdj1(nlvco2-4)
        eqcol(nlvco2-2,nlvco2-2) = cdodj1(nlvco2-3) + cupdj1(nlvco2-3)
     1                           + cdodj2(nlvco2-3) + cupdj2(nlvco2-3)
     2                           + cdodj3(nlvco2-3) + cdodj4(nlvco2-3)
        eqcol(nlvco2-2,nlvco2-1) = - cdodj1(nlvco2-2)
        eqcol(nlvco2-2,nlvco2)   = - cdodj2(nlvco2-1)

        eqcol(nlvco2-1,nlvco2-5) = - cupdj4(nlvco2-6)
        eqcol(nlvco2-1,nlvco2-4) = - cupdj3(nlvco2-5)
        eqcol(nlvco2-1,nlvco2-3) = - cupdj2(nlvco2-4)
        eqcol(nlvco2-1,nlvco2-2) = - cupdj1(nlvco2-3)
        eqcol(nlvco2-1,nlvco2-1) = cdodj1(nlvco2-2) + cupdj1(nlvco2-2)
     1                           + cdodj2(nlvco2-2) + cdodj3(nlvco2-2)
     2                           + cdodj4(nlvco2-2)
        eqcol(nlvco2-1,nlvco2) = - cdodj1(nlvco2-1)


        eqcol(nlvco2,nlvco2-4) = - cupdj4(nlvco2-5)
        eqcol(nlvco2,nlvco2-3) = - cupdj3(nlvco2-4)
        eqcol(nlvco2,nlvco2-2) = - cupdj2(nlvco2-3)
        eqcol(nlvco2,nlvco2-1) = - cupdj1(nlvco2-2)
        eqcol(nlvco2,nlvco2)   = cdodj1(nlvco2-1) + cdodj2(nlvco2-1)
     1                         + cdodj3(nlvco2-1) + cdodj4(nlvco2-1)

        if (itrco2 .ne.0) then
          do j=1,nlvco2
            eqcol(j,j) = eqcol(j,j) + eqrco2(j-1,j-1)
          end do
        else
          do j=1,nlvco2
            eqcol(j,j) = eqcol(j,j) + eqrco0(j-1,j-1)
          end do
        endif

        return
        end

c       %%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLCO3 (ttry)
c       %%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE STR_DATA
        USE SP_DATA
        USE PROFIL

c---------------------------------------------------------------------
c       ttry is temperature of neutral fluid

c       include 30 levels of co
c---------------------------------------------------------------------

        implicit double precision (a-h, o-z)

c       nouveaux taux (correctes?)
        data tcoh21, tcohe1,  tcoh1  / 1.d-10, 3.3d-11, 1.d-11 /
        data tcoh22, tcohe2,  tcoh2  / 1.d-10, 6.0d-11, 1.d-11 /
        data tcoh23, tcohe3,  tcoh3  / 1.d-11, 1.6d-11, 3.d-13 /
        data tcoh24, tcohe4,  tcoh4  / 1.d-11, 1.3d-11, 1.d-13 /

c         *************transfert deltaj = 1 *************

        cdodj1(1) = tcoh21 * ab(i_h2) / deux + tcoh1 * ab(i_h) * 0.6d0
     1             + tcohe1 * ab(i_he) * deux / trois
        do j=2,njco
          cdodj1(j) = tcoh21 * ab(i_h2) + tcoh1 * ab(i_h)
     1              + tcohe1 * ab(i_he)
        end do

        do j=0,njco-1
          delten = wlkco3(j+1)
          cupdj1(j) = cdodj1(j+1) * (2*j+3)
     1              * exp(-delten / ttry) / (2*j+1)
        end do

c       *************transfert deltaj = 2 *************

        cdodj2(2) = tcoh22 * ab(i_h2) / deux + tcoh2 * ab(i_h) * 0.5d0
     1            + tcohe2 * ab(i_he) / deux
        cdodj2(3) = tcoh22 * ab(i_h2) + tcoh2 * ab(i_h) * 0.7d0
     1            + tcohe2 * ab(i_he) / 1.33d0

        do j=4,njco
          cdodj2(j) = tcoh22 * ab(i_h2) + tcoh2 * ab(i_h)
     1              + tcohe2 * ab(i_he)
        end do

        do j=0,njco-2
          delten = wlkco3(j+2) + wlkco3(j+1)
          cupdj2(j) = cdodj2(j+2) * (2*j+5)
     1              * exp(-delten / ttry) / (2*j+1)
        end do

c       *************transfert deltaj = 3 *************

        cdodj3(3) = tcoh23 * ab(i_h2) / deux + tcoh3 * ab(i_h) / 3.0d0
     1            + tcohe3 * ab(i_he) * 0.3d0
        cdodj3(4) = tcoh23 * ab(i_h2) + tcoh3 * ab(i_h)
     1            + tcohe3 * ab(i_he) / deux
        do j=4, njco
           cdodj3(j) = tcoh23 * ab(i_h2) + tcoh3 * ab(i_h)
     1               + tcohe3 * ab(i_he)
        enddo

        do j=0, njco-3
          delten = wlkco3(j+3) + wlkco3(j+2) + wlkco3(j+1)
          cupdj3(j) = cdodj3(j+3) * (2*j+7)
     1              * exp(-delten / ttry) / (2*j+1)
        enddo


c       *************transfert deltaj = 4 ************************

        cdodj4(4) = tcoh24 * ab(i_h2) / deux + tcoh4 * ab(i_h) / deux
     1            + 0.5d-11 * ab(i_he)
        cdodj4(5) = tcoh24 * ab(i_h2) *0.8d0 + tcoh4 * ab(i_h) * 0.8d0
     1            + 1.0d-11 * ab(i_he)
        do j=5, njco
           cdodj4(j) = tcoh24 * ab(i_h2) + tcoh4 * ab(i_h)
     1               + tcohe4 * ab(i_he)
        enddo

        do j=0, njco-4
          delten = wlkco3(j+4) + wlkco3(j+3) + wlkco3(j+2) + wlkco3(j+1)
          cupdj4(j) = cdodj4(j+4) * (2*j+9)
     1              * exp(-delten / ttry) / (2*j+1)
        enddo


c       ************ matrice d'equilibre collisionnel************

        eqcol = zero

        eqcol(1,1) = cupdj1(0) + cupdj2(0) + cupdj3(0) + cupdj4(0)
        eqcol(1,2) = - cdodj1(1)
        eqcol(1,3) = - cdodj2(2)
        eqcol(1,4) = - cdodj3(3)
        eqcol(1,5) = - cdodj4(4)

        eqcol(2,1) = - cupdj1(0)
        eqcol(2,2) = cupdj1(1) + cupdj2(1) + cupdj3(1) + cupdj4(1)
     1             + cdodj1(1)
        eqcol(2,3) = - cdodj1(2)
        eqcol(2,4) = - cdodj2(3)
        eqcol(2,5) = - cdodj3(4)
        eqcol(2,6) = - cdodj4(5)


        eqcol(3,1) = - cupdj2(0)
        eqcol(3,2) = - cupdj1(1)
        eqcol(3,3) = cupdj1(2) + cupdj2(2) + cupdj3(2) + cupdj4(2)
     1             + cdodj1(2) + cdodj2(2)
        eqcol(3,4) = - cdodj1(3)
        eqcol(3,5) = - cdodj2(4)
        eqcol(3,6) = - cdodj3(5)
        eqcol(3,7) = - cdodj4(6)

        eqcol(4,1) = - cupdj3(0)
        eqcol(4,2) = - cupdj2(1)
        eqcol(4,3) = - cupdj1(2)
        eqcol(4,4) = cupdj1(3) + cupdj2(3) + cupdj3(3) + cupdj4(3)
     1             + cdodj1(3) + cdodj2(3) + cdodj3(3)
        eqcol(4,5) = - cdodj1(4)
        eqcol(4,6) = - cdodj2(5)
        eqcol(4,7) = - cdodj3(6)
        eqcol(4,8) = - cdodj4(7)

        do ij=5,nlvco3-4
          eqcol(ij,ij-4) = - cupdj4(ij-5)
          eqcol(ij,ij-3) = - cupdj3(ij-4)
          eqcol(ij,ij-2) = - cupdj2(ij-3)
          eqcol(ij,ij-1) = - cupdj1(ij-2)
          eqcol(ij,ij) = cdodj1(ij-1) + cupdj1(ij-1)
     1                 + cdodj2(ij-1) + cupdj2(ij-1)
     2                 + cdodj3(ij-1) + cupdj3(ij-1)
     4                 + cdodj4(ij-1) + cupdj4(ij-1)
          eqcol(ij,ij+1) = - cdodj1(ij)
          eqcol(ij,ij+2) = - cdodj2(ij+1)
          eqcol(ij,ij+3) = - cdodj3(ij+2)
          eqcol(ij,ij+4) = - cdodj4(ij+3)
        end do


        eqcol(nlvco3-3,nlvco3-7) = - cupdj4(nlvco3-8)
        eqcol(nlvco3-3,nlvco3-6) = - cupdj3(nlvco3-7)
        eqcol(nlvco3-3,nlvco3-5) = - cupdj2(nlvco3-6)
        eqcol(nlvco3-3,nlvco3-4) = - cupdj1(nlvco3-5)
        eqcol(nlvco3-3,nlvco3-3) = cdodj1(nlvco3-4) + cupdj1(nlvco3-4)
     1                           + cdodj2(nlvco3-4) + cupdj2(nlvco3-4)
     2                           + cdodj3(nlvco3-4) + cupdj3(nlvco3-4)
     3                           + cdodj4(nlvco3-4)
        eqcol(nlvco3-3,nlvco3-2) = - cdodj1(nlvco3-3)
        eqcol(nlvco3-3,nlvco3-1) = - cdodj2(nlvco3-2)
        eqcol(nlvco3-3,nlvco3)   = - cdodj3(nlvco3-1)

        eqcol(nlvco3-2,nlvco3-6) = - cupdj4(nlvco3-7)
        eqcol(nlvco3-2,nlvco3-5) = - cupdj3(nlvco3-6)
        eqcol(nlvco3-2,nlvco3-4) = - cupdj2(nlvco3-5)
        eqcol(nlvco3-2,nlvco3-3) = - cupdj1(nlvco3-4)
        eqcol(nlvco3-2,nlvco3-2) = cdodj1(nlvco3-3) + cupdj1(nlvco3-3)
     1                           + cdodj2(nlvco3-3) + cupdj2(nlvco3-3)
     2                           + cdodj3(nlvco3-3) + cdodj4(nlvco3-3)
        eqcol(nlvco3-2,nlvco3-1) = - cdodj1(nlvco3-2)
        eqcol(nlvco3-2,nlvco3)   = - cdodj2(nlvco3-1)

        eqcol(nlvco3-1,nlvco3-5) = - cupdj4(nlvco3-6)
        eqcol(nlvco3-1,nlvco3-4) = - cupdj3(nlvco3-5)
        eqcol(nlvco3-1,nlvco3-3) = - cupdj2(nlvco3-4)
        eqcol(nlvco3-1,nlvco3-2) = - cupdj1(nlvco3-3)
        eqcol(nlvco3-1,nlvco3-1) = cdodj1(nlvco3-2) + cupdj1(nlvco3-2)
     1                           + cdodj2(nlvco3-2) + cdodj3(nlvco3-2)
     2                           + cdodj4(nlvco3-2)
        eqcol(nlvco3-1,nlvco3) = - cdodj1(nlvco3-1)


        eqcol(nlvco3,nlvco3-4) = - cupdj4(nlvco3-5)
        eqcol(nlvco3,nlvco3-3) = - cupdj3(nlvco3-4)
        eqcol(nlvco3,nlvco3-2) = - cupdj2(nlvco3-3)
        eqcol(nlvco3,nlvco3-1) = - cupdj1(nlvco3-2)
        eqcol(nlvco3,nlvco3)   = cdodj1(nlvco3-1) + cdodj2(nlvco3-1)
     1                         + cdodj3(nlvco3-1) + cdodj4(nlvco3-1)

        if (itrco3 .ne.0) then
          do j=1,nlvco3
            eqcol(j,j) = eqcol(j,j) + eqrco3(j-1,j-1)
          end do
        else
          do j=1,nlvco3
            eqcol(j,j) = eqcol(j,j) + eqrco0(j-1,j-1)
          end do
        endif

        return
        end

c       %%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLCS0 (ttry)
c       %%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        USE STR_DATA
        USE CHEM_DATA
        USE SP_DATA
        USE PROFIL

c---------------------------------------------------------------------
c       ttry is temperature of neutral fluid

c       include  20 levels of cs
c---------------------------------------------------------------------

        implicit double precision (a-h, o-z)

        tcse1 = 3.0d-6

c-- initialisation de la matrice --
        eqcol= zero

c  On limite l'intervalle de temperature dans lequel
c     le fit est "valable"

        teffcs = max(5.0d0,ttry)
        teffcs = min(teffcs,400.0d0)

        xncol = ab(i_h2) + 0.7d0 * ab(i_he)

        do jji=1,nlvcs0-1
          do jjf = 0,jji-1
            ntc = jji * (jji-1) * 0.5_dp + jjf + 1
            ratdo = cdocs(1,ntc) * teffcs + cdocs(2,ntc)
     1            + cdocs(3,ntc) * exp(-teffcs/cdocs(4,ntc))
            ratdo = ratdo * xncol

            if ((jji-jjf) .eq. 1) then
              ratdo = ratdo + tcse1 * ab(nspec)
            endif

            energ = elkcs0(jji+1) - elkcs0(jjf+1)
            ratup = ratdo * (deux*jji+un)
     1            * exp(-energ / ttry) / (deux*jjf+1)

            eqcol(jji+1,jji+1) = eqcol(jji+1,jji+1) + ratdo
            eqcol(jji+1,jjf+1) = eqcol(jji+1,jjf+1) - ratup
            eqcol(jjf+1,jjf+1) = eqcol(jjf+1,jjf+1) + ratup
            eqcol(jjf+1,jji+1) = eqcol(jjf+1,jji+1) - ratdo

          end do
        end do

        return
        end


c       %%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLHCOP (ttry)
c       %%%%%%%%%%%%%%%%%%%%%%%%%

c---------------------------------------------------------------------
c       ttry  :temperature du gaz

c       tient compte de 20 niveaux de HCO+
c---------------------------------------------------------------------

        use constantes
        use auxiliaries
        USE STR_DATA
        USE SP_DATA
        USE PROFIL

        implicit double precision (a-h, o-z)

c-- initialisation de la matrice --
            eqcol= zero

c  On limite l'intervalle de temperature dans lequel
c     le fit est "valable" (entre 10 et 400 K)

        teffhco = max(10.0d0,ttry)
        teffhco = min(teffhco,400.0d0)

c  comme sur le modele de CS, on ne tient pas cpte du H atomique
c de toute facon, la ou il y a du HCO+, tout l'hydrogene est molecul.

c le 0.7 correspond a la vitesse relative de l'helium/ H2
        xncol = ab(i_h2) + 0.7d0 * ab(i_he)

        do 20 ju=1,njhcop
          do 21 jl=0,ju-1
            a1 = colhcoph2(1,ju,jl)
            a2 = colhcoph2(2,ju,jl)
            a3 = colhcoph2(3,ju,jl)
            a4 = colhcoph2(4,ju,jl)
            ratdo = a1 * teffhco + a2 + a3 * exp(-teffhco/a4)
            ratdo = ratdo * xncol

            energ = elkhcop(ju+1) - elkhcop(jl+1)
            ratup = ratdo * (deux*ju+un)/ (deux*jl+1)
     1          * exp(-energ / ttry)

            eqcol(ju+1,ju+1) = eqcol(ju+1,ju+1) + ratdo
            eqcol(ju+1,jl+1) = eqcol(ju+1,jl+1) - ratup
            eqcol(jl+1,jl+1) = eqcol(jl+1,jl+1) + ratup
            eqcol(jl+1,ju+1) = eqcol(jl+1,ju+1) - ratdo

 21       enddo
 20     enddo

        return
        end
